Package: hydroTSM
Type: Package
Title: Time Series Management and Analysis for Hydrological Modelling
Version: 0.8-6
Authors@R: c( person("Mauricio Zambrano-Bigiarini", email = "mzb.devel@gmail.com", role=c("aut", "cre", "cph"), comment=c(ORCID = "0000-0002-9536-643X")), person("Joschka", "Thurner", email="git@joschkathurner.de", role=c("ctb")) )
Maintainer: Mauricio Zambrano-Bigiarini <mzb.devel@gmail.com>
Description: S3 functions for management, analysis, interpolation and plotting of time series used in hydrology and related environmental sciences. In particular, this package is highly oriented to hydrological modelling tasks. The focus of this package has been put in providing a collection of tools useful for the daily work of hydrologists (although an effort was made to optimise each function as much as possible, functionality has had priority over speed). Bugs / comments / questions / collaboration of any kind are very welcomed, and in particular, datasets that can be included in this package for academic purposes.
License: GPL (>= 2)
Depends: R (>= 3.5.0), zoo (>= 1.7-2)
Imports: xts (>= 0.9-7), e1071, stats, utils, methods, graphics,
        grDevices, lattice, classInt, timechange
Suggests: knitr, rmarkdown, gridExtra
VignetteBuilder: knitr
URL: https://hzambran.github.io/hydroTSM/,
        https://github.com/hzambran/hydroTSM,
        https://CRAN.R-project.org/package=hydroTSM
MailingList: https://stat.ethz.ch/mailman/listinfo/r-sig-ecology
BugReports: https://github.com/hzambran/hydroTSM/issues
LazyLoad: yes
NeedsCompilation: no
Repository: CRAN
Packaged: 2026-04-28 01:34:48 UTC; hzambran
Author: Mauricio Zambrano-Bigiarini [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-9536-643X>),
  Joschka Thurner [ctb]
Date/Publication: 2026-04-28 07:20:03 UTC
Built: R 4.6.0; ; 2026-04-28 11:14:03 UTC; unix

importFrom("stats", time, cor, cor.test, symnum, plot.stepfun, ecdf, median, splinefun, integrate, filter, IQR, sd, start, end, window, aggregate, approxfun, fivenum, lm, quantile, rnorm, runif)
importFrom("graphics", boxplot, plot, hist, par, layout, axis, Axis, abline, lines, points, legend, box, text, strwidth, rect, polygon, pairs, panel.smooth)
importFrom("grDevices", xy.coords, palette, colorRampPalette, dev.set, dev.list, dev.new, dev.off, png, rainbow) 
importFrom("utils", read.table, write.table, write.csv, write.csv2, installed.packages, ls.str, modifyList, sessionInfo, setTxtProgressBar, txtProgressBar) 
importFrom("methods", as, hasArg, is, slot) 
importFrom("graphics", barplot, grid, axTicks, Axis, abline, axis, boxplot, hist, legend, lines, mtext, par, plot)
importFrom("classInt", classIntervals)

importFrom("zoo", is.zoo, as.Date, as.zoo, zoo, coredata, as.yearmon, rollapply, index, "time<-", na.approx, na.spline)
importFrom("xts", axTicksByTime, to.period, .index, periodicity)
#importFrom("zoo", aggregate) # aggregate is not exported from zoo
importFrom("timechange", time_force_tz)
importFrom("e1071", skewness, kurtosis)
importFrom("lattice", levelplot)
#importFrom("lattice", lattice.options, lattice.getOption, trellis.currentLayout, trellis.focus, trellis.unfocus, trellis.panelArgs)
#importFrom("grid", grid.lines, gpar)

#importFrom("maptools", readShapePoly)
#importFrom("gstat", krige, idw)
#importFrom("automap", autoKrige)
#importFrom("sp", proj4string, coordinates, "coordinates<-", "proj4string<-", spplot, CRS, spsample, "gridded<-", SpatialPoints, layout.north.arrow, layout.scale.bar, over)

#importFrom("chron", month.day.year)
exportPattern("^[^\\.]")

export(shiftyears)

S3method(annualfunction, data.frame)
S3method(annualfunction, default)
S3method(annualfunction, matrix)
S3method(annualfunction, zoo)
S3method(baseflow, zoo)
S3method(calendarHeatmap, zoo)
S3method(cmv, default)
S3method(cmv, default)
S3method(cmv, zoo) 
S3method(cmv, data.frame) 
S3method(cmv, matrix) 
S3method(daily2annual, default)
S3method(daily2annual, zoo)
S3method(daily2annual, data.frame)
S3method(daily2annual, matrix) 
S3method(daily2monthly, default)
S3method(daily2monthly, zoo)
S3method(daily2monthly, data.frame) 
S3method(daily2monthly, matrix) 
S3method(daily2weekly, default)
S3method(daily2weekly, zoo)
S3method(daily2weekly, data.frame) 
S3method(daily2weekly, matrix) 
S3method(dm2seasonal, default)
S3method(dm2seasonal, zoo)
S3method(dm2seasonal, data.frame) 
S3method(dm2seasonal, matrix) 
S3method(dwdays, default) 
#S3method(dwdays, zoo) 
S3method(dwdays, data.frame) 
S3method(dwdays, matrix)
S3method(dwi, default)  
S3method(dwi, zoo)
S3method(dwi, data.frame)
S3method(dwi, matrix) 
S3method(extract, default)
S3method(extract, zoo)
S3method(fdc, default) 
S3method(fdc, zoo) 
S3method(fdc, data.frame)
S3method(fdc, matrix) 
S3method(fdcu, default) 
#S3method(fdcu, zoo) 
S3method(fdcu, data.frame) 
S3method(fdcu, matrix)
#S3method(hydrokrige, data.frame) 
#S3method(hydrokrige, default) 
S3method(hydroplot, default)
S3method(hydroplot, zoo)
S3method(hydroplot, data.frame)
S3method(infillxy, data.frame)
S3method(infillxy, default) 
S3method(infillxy, matrix) 
S3method(istdx, data.frame) 
S3method(istdx, default)
S3method(istdx, matrix)
S3method(isComplete, zoo)
S3method(izoo2rzoo, default) 
S3method(izoo2rzoo, zoo) 
S3method(ma, default) 
S3method(ma, zoo)
S3method(monthlyfunction, default)
S3method(monthlyfunction, zoo)
S3method(monthlyfunction, data.frame)
S3method(monthlyfunction, matrix) 
S3method(plot_pq, zoo)
S3method(seasonalfunction, default) 
S3method(seasonalfunction, zoo) 
S3method(seasonalfunction, data.frame)
S3method(seasonalfunction, matrix) 
S3method(smry, Date)
S3method(smry, default) 
S3method(smry, zoo) 
S3method(smry, data.frame) 
S3method(smry, matrix) 
S3method(sname2plot, default)
S3method(sname2plot, zoo)
S3method(sname2plot, data.frame)
S3method(stdx, data.frame) 
S3method(stdx, default)
S3method(stdx, matrix) 
S3method(subdaily2daily, default)
S3method(subdaily2daily, zoo)
S3method(subdaily2daily, data.frame)
S3method(subdaily2daily, matrix)
S3method(subdaily2weekly, default)
S3method(subdaily2weekly, zoo)
S3method(subdaily2weekly, data.frame)
S3method(subdaily2weekly, matrix)
S3method(subdaily2monthly, default)
S3method(subdaily2monthly, zoo)
S3method(subdaily2monthly, data.frame)
S3method(subdaily2monthly, matrix)
S3method(subhourly2hourly, default)
S3method(subhourly2hourly, zoo)
S3method(subhourly2hourly, data.frame)
S3method(subhourly2hourly, matrix)
S3method(subhourly2nminutes, default)
S3method(subhourly2nminutes, zoo)
S3method(subhourly2nminutes, data.frame)
S3method(subhourly2nminutes, matrix)
S3method(weeklyfunction, default)
S3method(weeklyfunction, zoo)
S3method(weeklyfunction, data.frame)
S3method(weeklyfunction, matrix) 

# File subdaily2daily.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2013-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                            subdaily2daily                                    #
################################################################################
# This function transform a SUB-DAILY time series into a DAILY one

# 'x'        : usb-daily values that will be converted into daily ones.
#              class(x) must be 'xts'
# 'FUN'      : Function that have to be applied for transforming from sub-daily into 
#              daily time step. E.g., for precipitation FUN MUST be "sum"
#              For temperature and flow time series, FUN MUST be "mean"
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the monthly and annual values  are computed considering only those 
#                     values different from NA
#              FALSE: if there is AT LEAST one NA within a year, the monthly and annual 
#                     values are NA
# 'start'    : character, indicating the starting time used for aggregating sub-daily time 
#              series into daily ones. 
#              It MUST be provided in the format specified by \code{start.fmt}. \cr
#              This value is used to define the time when a new day begins (e.g., for some 
#              rain gauge stations). \cr
#              -) All the values of \code{x} with a time attribute before \code{start} are
#                 considered as belonging to the day before the one indicated in the time 
#                 attribute of those values. \cr
#              -) All the values of \code{x} with a time attribute equal to \code{start} 
#                 are considered to be equal 
#                 to \code{"00:00:00"} in the output zoo object. \cr
#              -) All the values of \code{x} with a time attribute after \code{start} are 
#                 considered as belonging to the same day as the one indicated in the time 
#                 attribute of those values. \cr
#              It is useful when the daily values start at a time different from
#              \code{"00:00:00"}. Use with caution. See examples.
# 'start.fmt': character indicating the format in which the time is provided in \code{start}.
#              By default \code{date.fmt=\%H:\%M:\%S}. See \code{format} in 
#              \code{\link[base]{as.POSIXct}}.
# 'tz'       : character, with the specification of the time zone used in both 
#              \code{x} and \code{start}. 
#              System-specific (see time zones), but \code{""} is the current time zone,
#              and \code{"GMT"} is UTC (Universal Time, Coordinated). 
#              See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
#              If \code{tz} is missing (the default), it is automatically set to the
#              time zone used in \code{time(x)}. \cr
#              This argument can be used to force using the local time zone or any other 
#              time zone instead of UTC as time zone.
subdaily2daily <-function(x, ...) UseMethod("subdaily2daily")


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 09-Apr-2013                                                         #
# Updates: 06-Dec-2019                                                         #
#          27-May-2021                                                         #
#          11-Oct-2022                                                         # 
#          30-Jul-2023 ; 03-Aug-2023                                           #
################################################################################
subdaily2daily.default <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                   start.fmt= "%H:%M:%S", tz, ...) {

  # Checking that 'x' is a zoo object
  if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo'")

  # Automatic detection of 'tz'
  #if (missing(tz)) tz <- ""
  if (missing(tz)) tz <- format(time(x), "%Z")[1]

  subdaily2daily.zoo(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, 
                     start=start, start.fmt=start.fmt, tz=tz, ...)

} # 'subdaily2daily.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Mar-2013                                                         #
# Updates: 26-Mar-2013 ; 08-Apr-2013 ; 09-Apr-2013                             #
#          29-Nov-2015 ; 01-Dec-2015                                           #
#          06-Dec-2019 ; 18-Dec-2019                                           #
#          27-May-2021                                                         #
#          11-Oct-2022                                                         #
#          30-Jul-2023 ; 31-Jul-2023 ; 03-Aug-2023                             #
#          05-Feb-2025 ; 25-Jul-2025 ; 26-Jul-2025 ; 31-Jul-2025               #
################################################################################
subdaily2daily.zoo <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                               start.fmt= "%H:%M:%S", tz, ...) {

    # testing the existence of 'na.rm' argument
    #args <- list(...)
    #exist <- "na.rm" %in% names(args)
    #exist

    # Checking that the user provied a valid class for 'x'   
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !!")

    # Checking the user provide a valid value for 'FUN'
    if ( missing(FUN) | !is.function(FUN) )
      stop("Missing argument: 'FUN' must contain a valid function for aggregating the sub-daily values")

    # Checking that 'na.rm.max' is in [0, 1]
    if ( (na.rm.max < 0) | (na.rm.max > 1) )
      stop("Invalid argument: 'na.rm.max' must be in [0, 1] !") 

    if ( !zoo::is.regular(x, strict=FALSE) )
      warning("'x' is not a regular '", sfreq(x), "' time series !. (see the 'izoo2rzoo' function)")

    # Automatic detection of 'tz'
    #if (missing(tz)) tz <- ""
    if (missing(tz)) tz <- format(time(x), "%Z")[1]

    # Analysis of days different from 00:00 to 23:59 hrs
    if ( start != "00:00:00" ) {
      # Storing the original time
      time.old <- time(x)

      # Converting the new starting time provided by the user into a POSIXct object
      startFULL <- as.POSIXct(start, format=start.fmt, tz=tz)

      # normal staring time for a day
      nstart <- as.POSIXct("00:00:00", format="%H:%M:%S", tz=tz)

      # time difference between the desired starting time 'strat' and the "normal"
      # starting time 'nstart', [s]
      delta <- difftime(startFULL, nstart, units="secs")

      # Computing teh time difference between 'start' and the "normal" starting time, [s]
      #time.new <- as.POSIXct(time.old, tz=tz) - delta
      time.new <- time.old - delta

      # Changing the time in 'x' in 'delta' seconds
      time(x)  <- time.new
    } # IF end

    # Making sure that the time serie is complete before aggregation
    # This is useful when the first element of 'x' is not given at the time defined by 'start'.
    # For example, if the first element of 'x' starts at 08:00:00 hrs, but 'start=00:00:00', 
    # what happens with all the values from 00:00:00 to 07:59:59 hrs?
    # The following lines of code makes sure that the missing elements in a day are actually 
    # considered as missing

    #st <- paste(format(start(x), "%Y-%m-%d"), "00:00:00", tz)
    #et <- paste(format(end(x), "%Y-%m-%d"), "23:59:59", tz)
    #x  <- izoo2rzoo(x, from=st, to=et, tz=tz)

    # Computing the Daily time series 
    tmp <- aggregate(x, by= function(tt) format(tt, "%Y-%m-%d"), FUN=FUN, na.rm= na.rm, ...)
    time(tmp) <- as.Date( time(tmp) )
    
    # Removing annual values in the output object for days with 
    # more than 'na.rm.max' percentage of NAs in a given day
    if ( na.rm ) {

      # Computing the percentage of missing values in each day
      na.pctg <- cmv(x, tscale="daily", start=start, start.fmt=start.fmt, tz=tz)

      # identifying days with a percentage of missing values higher than 'na.rm.max'
      na.pctg.index <- which( na.pctg > na.rm.max)

      # Setting as NA all the days with a percentage of missing values higher than 'na.rm.max'
      tmp[na.pctg.index] <- NA 
    } # IF end

    # Removing subdaily time attibute, but not the dates
    if (NCOL(tmp) == 1) {
      tmp <- zoo(as.numeric(tmp), as.Date(time(tmp), format="%Y-%m-%d") ) 
    } else tmp <- zoo::zoo(zoo::coredata(tmp), as.Date(time(tmp), format="%Y-%m-%d") ) 

    # Replacing the NaNs by 'NA.
    # mean(NA:NA, na.rm=TRUE) == NaN
    nan.index <- which(is.nan(tmp))
    if ( length(nan.index) > 0 ) tmp[nan.index] <- NA
  
    # Replacing all the Inf and -Inf by NA's
    # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
    inf.index <- which(is.infinite(tmp))
    if ( length(inf.index) > 0 ) tmp[inf.index] <- NA        

    return(tmp)

} # 'subdaily2daily.zoo' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 09-Apr-2013                                                         #
# Updates: 18-Dec-2019                                                         #
#          27-May-2021                                                         #
#          23-Aug-2022 ; 11-Oct-2022                                           #
#          25-May-2023 ; 30-Jul-2023 ; 03-Aug-2023                             #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.fmt' : character, for selecting if the result will be 'numeric' or 'zoo'. Valid values are: c('numeric', 'zoo')
# 'verbose'      : logical; if TRUE, progress messages are printed
subdaily2daily.data.frame <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                      start.fmt= "%H:%M:%S", tz, 
                                      dates=1, date.fmt="%Y-%m-%d %H:%M:%S",
				                              out.fmt="zoo",
				                              verbose=TRUE,...) {

  # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN))
      stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values !!")

  # Checking that the user provied a valid argument for 'out.fmt'
  if (is.na(match( out.fmt, c("numeric", "zoo") ) ) )
      stop("Invalid argument: 'out.fmt' must be in c('numeric', 'zoo')")

  # Checking that the user provied a valid argument for 'dates'
  if (missing(dates)) {
      stop("Missing argument: 'dates' must be provided")
  } else
     # Checking that the user provied a valid argument for 'dates'
     if (FALSE && (class(dates) %in% c("numeric", "factor", "POSIXct", "POSIXt")) )
         stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'POSIXct', 'POSIXt'")

  # Automatic detection of 'tz'
  #if (missing(tz)) tz <- ""
  if (missing(tz)) tz <- format(time(x), "%Z")[1]

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( TRUE && ( inherits(dates, "numeric") ) ) {
    tmp   <- dates
    dates <- as.POSIXct(x[, dates], format= date.fmt, tz=tz) 
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( TRUE && ( inherits(dates, "factor") ) ) dates <- as.POSIXct(dates, format= date.fmt, tz=tz) 

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( (TRUE && (class(dates) %in% c("POSIXct", "POSIXt")) ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
     
  # Transforming 'x' into a zoo object
  x <- zoo::zoo(x, dates)
  
  ##############################################################################
  
  z <- subdaily2daily.zoo(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, 
                          start=start, start.fmt=start.fmt, tz=tz, ...)
    
  if (out.fmt == "numeric") {
     snames      <- colnames(z)
     dates.lab   <- as.character(time(z))
     z           <- zoo::coredata(z)
     colnames(z) <- snames
     rownames(z) <- dates.lab        
  } # IF end

  return( z )

 } #'subdaily2daily.data.frame' END


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 09-Apr-2013                                                         #
# Updates: 18-Dec-2019                                                         #
#          27-May-2021                                                         #
#          25-May-2023                                                         #
#          30-Jul-2023 ; 03-Aug-2023                                           #
################################################################################
subdaily2daily.matrix  <- function(x, FUN, na.rm=TRUE, na.rm.max=0, 
                                   start="00:00:00", start.fmt= "%H:%M:%S", tz,
                                   dates=1, date.fmt="%Y-%m-%d %H:%M:%S",
				                           out.fmt="zoo",
                                   verbose=TRUE,...) {

   # Automatic detection of 'tz'
   #if (missing(tz)) tz <- ""
   if (missing(tz)) tz <- format(time(x), "%Z")[1]

   x <- as.data.frame(x)
   #NextMethod("daily2annual")  # I don't know why is redirecting to 'daily2monthly.default' instead of 'daily2monthly.data.frame'....
   subdaily2daily.data.frame(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, 
                             start=start, start.fmt=start.fmt, tz=tz,
                             dates=dates, date.fmt=date.fmt,
			                       out.fmt=out.fmt,
                             verbose=verbose,...)

} # 'subdaily2daily.matrix  ' END

# File monthlyfunction.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# monthlyfunction: Generic function for applying any R function to             #
#                  ALL the values in 'x' belonging to a given month            #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: May 15th, 2009;                                                     #
# Updates: 31-Aug-2009 ; 25-Jul-2011 ; 08-Aug-2011                             #
#          08-Apr-2013                                                         #
################################################################################
# 'x   '    : variable of type 'zoo' or 'data.frame', with daily or monthly frequency
# 'FUN'      : Function that will be applied to ALL the values in 'x' belonging to each one of the 12 months of the year
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the monthly values  are computed considering only those values in 'x' different from NA
#              FALSE: if there is AT LEAST one NA within a month, the FUN and monthly values are NA
monthlyfunction <- function(x, ...) UseMethod("monthlyfunction")

monthlyfunction.default <- function(x, FUN, na.rm=TRUE,...) {

    # Checking that 'x' is a zoo object
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

    monthlyfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)

} # 'monthlyfunction.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2011                                                         #
# Updates: 08-Aug-2011                                                         #
#          05-Jun-2012                                                         #
#          08-Apr-2013                                                         #
#          28-Nov-2015                                                         #
################################################################################
monthlyfunction.zoo <- function(x, FUN, na.rm=TRUE,...) {

     # Checking that the user provied a valid argument for 'FUN'
     if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")
     
     # Checking the user provide a valid value for 'x'
     if (sfreq(x) %in% c("quarterly", "annual"))
	stop("Invalid argument: 'x' is not a sub-daily, daily, weekly or monthly ts. 'x' is a ", sfreq(x), " ts" )

     # Monthly index for 'x'
     dates  <- time(x)
     m      <- as.numeric(format( dates, "%m" ))
     months <- factor( month.abb[m], levels=unique(month.abb[m]) )
     
     # 'as.numeric' is necessary for being able to change the names to the output
     totals <- aggregate(x, by= months, FUN=FUN, na.rm= na.rm, ... ) 

     # Replacing the NaNs by 'NA.
     # NaN's are obtained when using the FUN=mean with complete NA values
     nan.index          <- which(is.nan(totals))
     if ( length(nan.index) > 0 )  totals[ nan.index] <- NA
     
     # Replacing all the Inf and -Inf by NA's
     # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
     inf.index <- which(is.infinite(totals))
     if ( length(inf.index) > 0 ) totals[inf.index] <- NA

     # Giving meaningful names to the output
     if ( (is.matrix(x)) | (is.data.frame(x)) ) {
       totals <- t(totals) # For having the months' names as column names
       colnames(totals) <- levels(months)
     } #IF end

     return(totals)

} # 'monthlyfunction.zoo' end
 
 

################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2011                                                         #
# Updates: 08-Aug-2011                                                         #
#          29-May-2013 ; 03-Jun-2013                                           #
#          23-Aug-2022                                                         #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with 12 columns representing the months,
#                              and as many rows as stations are included in 'x'
#             -) "db"        : a data.frame, with 4 colums will be produced.
#                              The first column stores the ID of the station
#                              The second column stores the Year,
#                              The third column stores the ID of the station,
#                              The fourth column contains the monthly value corresponding to the year specified in the second column
# 'verbose'      : logical; if TRUE, progress messages are printed
monthlyfunction.data.frame <- function(x, FUN, na.rm=TRUE,
                                       dates=1, date.fmt="%Y-%m-%d",
                                       out.type="data.frame",
                                       verbose=TRUE,...) {

  # Checking that the user provied a valid argument for 'out.type'
  if (is.na(match( out.type, c("data.frame", "db") ) ) )
    stop("Invalid argument: 'out.type' must be in c('data.frame', 'db'")

  # Checking that the user provied a valid argument for 'FUN'
  if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
     stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt)
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt)
  
  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # Transforming 'x' into a zoo object
  x <- zoo(x, dates)
  
  ##############################################################################
  if (out.type == "data.frame") {
  
    monthlyfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)
    
  } else if (out.type == "db") {
  
        # Amount of stations in 'x'
        nstations <- ncol(x)
        
        # ID of all the stations in 'x'
        snames <- colnames(x)
      
        if (is.null(snames)) snames <- paste("V", 1:nstations, sep="")
  
        # Computing the Starting and Ending Year of the analysis
        Starting.Year <- as.numeric(format(start(x), "%Y"))
        Ending.Year   <- as.numeric(format(end(x), "%Y"))

        # Amount of Years belonging to the desired period
        nyears <- Ending.Year - Starting.Year + 1
        
        # Computing the numeric index of the resulting months
        month.index <- unique(as.numeric(format( time(x), "%m" )))
     
        # Amount of different months belonging to the desired period
        nmonths <- length(month.index)
        
        # Total amount of months belonging to the desired period
        totalmonths <- nmonths*nyears

        # Creating a vector with the names of the field that will be used for storing the results
        field.names <- c("StationID", "Year", "Month", "Value" )

        # Creating the data.frame that will store the computed averages for each subcatchment
        z <- as.data.frame(matrix(data = NA, nrow = totalmonths*nstations, ncol = 4, byrow = TRUE, dimnames = NULL) )
        
        for (j in 1:nstations) {

            if (verbose) message( "[ Station: ", format(snames[j], width=10, justify="left"),
                                  " : ", format(j, width=3, justify="left"), "/",
                                  nstations, " => ",
                                  format(round(100*j/nstations,2), width=6, justify="left"),
                                  "% ]" )

	    # Computing the annual values
	    tmp <- monthlyfunction.default(x= x[,j], FUN=FUN, na.rm=na.rm, ...)

	    # Putting the annual/monthly values in the output data.frame
            # The first column of 'x' corresponds to the Year
            row.ini <- (j-1)*totalmonths + 1
            row.fin <-  j*totalmonths

            z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'totalmonths' times
            z[row.ini:row.fin, 2] <- rep(Starting.Year:Ending.Year, each=nmonths)
            z[row.ini:row.fin, 3] <- month.abb[month.index]
            z[row.ini:row.fin, 4] <- tmp

        } # FOR end
        
        colnames(z) <- field.names
        
        return( z )
      
    } # ELSE end

 } #'monthlyfunction.data.frame' END
 
 
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2011                                                         #
# Updates: 08-Aug-2011                                                         #
#          29-May-2013                                                         #
################################################################################
monthlyfunction.matrix <- function(x, FUN, na.rm=TRUE,
                                   dates=1, date.fmt="%Y-%m-%d",
                                   out.type="data.frame",
                                   verbose=TRUE,...) {
 x <- as.data.frame(x)
 #NextMethod("monthlyfunction")
 monthlyfunction.data.frame(x=x, FUN=FUN, na.rm=na.rm,
                            dates=dates, date.fmt=date.fmt,
                            out.type=out.type,
                            verbose=verbose,...)
                                                                    
 } # 'monthlyfunction.matrix' END  

# File zoo2RHtest.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM

########################################################################
# zoo2RHtest: input file to the 'RHtest_dlyPrcp.r' script              #
########################################################################
#	                   Date: 26-Nov-2009                           #
########################################################################
# function for creating the input file to the 'RHtest_dlyPrcp.r' script,
# that test the homogeneity of climatological time series (http://ccma.seos.uvic.ca)

# 'x'        : time series that will be written. class(x) must be 'zoo'
# 'fname'    : filename of the output precipitation time series
# 'tstep.out': Character indicating the time step that have to be used for
#              writting 'x' into the output file
# 'dec'      : the string to use for decimal points in numeric or complex
#              columns: must be a single character.
# 'na'       : the string to use for missing values in the data

zoo2RHtest <- function(x, fname="pcp.txt", tstep.out="daily", dec=".", na="-999.0") {

  .Deprecated(new="trend::snh.test", package="trend")

  # Checking that 'class(x)' is 'ts' or 'zoo'
  if ( is.na( match(class(x), c("zoo") ) ) )
      stop("Invalid argument: 'class(x)' must be 'zoo'")

  # Checking 'tstep.out'
    if ( is.na( match(tstep.out, c("daily", "monthly", "annual") ) ) )
      stop("Invalid argument: 'tstep.out' must be in c('daily', monthly', 'annual')")

  pfreq <- sfreq(x)

  years  <- as.numeric(format(time(x), "%Y"))
  months <- as.numeric(format(time(x), "%m"))
  days   <- as.numeric(format(time(x), "%d"))
  values <- coredata(x)

  # Checking the user provide a valid value for 'x'
  if (is.na(match(sfreq(x), c("daily", "monthly", "annual")))) {
     stop(paste("Invalid argument: 'x' is not a daily, mothly or annual ts, it is a ", sfreq(x), " ts", sep="") ) }

  if ( tstep.out=="daily") {
    out <- data.frame(years=years, months=months, days=days, values=values)
  } else if ( tstep.out=="monthly") {
     out <- data.frame(years=years, months=months, days=0, values=values)
    } else if ( tstep.out=="monthly") {
      out <- data.frame(years=years, months=0, days=0, values=values)
      } # ELSE end

  write.table(out, file = fname, sep = " ",
              eol = "\n", na = na, dec = dec, row.names = FALSE,
              col.names = FALSE)


} # 'zoo2RHtest' end

# File sname2plot.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2020 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#  'sname2plot': Given a data.frame whose columns contains the ts              #
#                (without missing dates) of several gauging stations, it       #
#                takes the name of one gauging station and plots 9 graphs      #
#                (see 'hydroplot' description)                                 #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 17-Dic-2008                                                         #
# Updates: 19-Apr-2011 ; 10-Aug-2011                                           #
#          18-Oct-2012                                                         #
#          04-Abr-2013 : 29-May-2013 ; 03-Jun-2013                             #
#          07-Nov-2020                                                         #
################################################################################

sname2plot <-function(x, ...) UseMethod("sname2plot")


sname2plot.default <- function(x, sname, FUN, na.rm=TRUE,
                               ptype="ts+boxplot+hist",
		                       pfreq="dma",                      
                               var.type,                      
                               var.unit="units",
                               main=NULL, xlab="Time", ylab=NULL,
                               win.len1=0,
                               win.len2=0,                      
                               tick.tstep="auto",
                               lab.tstep="auto",
                               lab.fmt=NULL,
                               cex=0.3,
                               cex.main=1.3,
                               cex.lab=1.3,
                               cex.axis=1.3,
                               col=c("blue", "lightblue", "lightblue"),
                               dates=1, date.fmt="%Y-%m-%d",
                               from=NULL, to=NULL, 
                               stype="default", 
                               season.names=c("Winter", "Spring", "Summer", "Autumn"),
                               h=NULL, ...) {

    # Checking that 'x' is a zoo object
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

    hydroplot.zoo(x, sname=sname, FUN=FUN, na.rm=na.rm, ptype=ptype, pfreq=pfreq,                      
              var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,
              win.len1=win.len1, win.len2=win.len2, tick.tstep=tick.tstep, 
              lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex=cex, cex.main=cex.main, cex.lab=cex.lab,
              cex.axis=cex.axis, col=col, from=from, to=to, stype=stype, season.names=season.names, h=h, ...)

} # 'hydroplot.default' end


sname2plot.zoo <- function(x, sname, FUN, na.rm=TRUE,
                           ptype="ts+boxplot+hist",
		                   pfreq="dma",                      
                           var.type,                      
                           var.unit="units",
                           main=NULL, xlab="Time", ylab=NULL,
                           win.len1=0,
                           win.len2=0,                      
                           tick.tstep="auto",
                           lab.tstep="auto",
                           lab.fmt=NULL,
                           cex=0.3,
                           cex.main=1.3,
                           cex.lab=1.3,
                           cex.axis=1.3,
                           col=c("blue", "lightblue", "lightblue"),
                           dates=1, date.fmt="%Y-%m-%d",
                           from=NULL, to=NULL, 
                           stype="default", 
                           season.names=c("Winter", "Spring", "Summer", "Autumn"),
                           h=NULL, ...) {

  # Checking the user provides 'sname'
  if (missing(sname)) { 
    stop("Missing argument: 'sname' must be provided")
  } else
    # Checking the the station provided for the user exist within 'x'
    if ( !(sname %in% colnames(x) ) )
      stop("Invalid argument: ' The station '", sname, "' is not a column name in 'x'")

  # If monthly or annual values are required, 'FUN' or 'var.type' must be provided
  if (pfreq %in% c("dma", "ma", "dm")) {
    # Checking that the user provied a valid argument for 'var.type'
    if (missing(FUN)) {
       # If the user did not provide a title for the plots, this is created automatically
       if (missing(var.type)) {
          stop("Missing argument: 'var.type' OR 'FUN' must be provided")
       } else # If 'var.type' is provided
            # Checking that the user provied a valid argument for 'var.type'
            if (is.na(match(var.type, c("Precipitation", "Temperature", "Flow") ) ) ) {
                  stop("Invalid argument: 'var.type' must be in c('Precipitation', 'Temperature', 'Flow')")
              } else {
                     if (var.type=="Precipitation") {
                         FUN <- sum
                         if (missing(var.unit)) { var.unit <- "mm"   }
                     } else if (var.type=="Temperature") {
                              FUN <- mean
                              if (missing(var.unit)) { var.unit <- "dC" }
                            } else if (var.type=="Flow") {
                                   FUN <- mean
                                   if (missing(var.unit)) { var.unit= "m3/s" }
                              }
                     } #ELSE end
    } # IF end
  } # IF end

  if (is.na(match(ptype, c("ts", "ts+boxplot", "ts+hist", "ts+boxplot+hist"))))
        stop("'ptype' valid values are: 'ts', 'ts+boxplot', 'ts+hist', 'ts+boxplot+hist'")

  # Checking that the user provied a valid value for 'stype'   
  valid.types <- c("default", "FrenchPolynesia")    
  if (length(which(!is.na(match(stype, valid.types )))) <= 0)  
     stop("Invalid argument: 'stype' must be in c('default', 'FrenchPolynesia')")  

  # Checking that the user provied a valid argument for 'season.names'
  if ( length(season.names) != 4 )
    stop("Invalid argument: 'season.names' must have 4 elements !")    

  # 'ylab' value
  if ( missing(ylab) ) { 
     ylab <- sname
  } else if ( is.null(ylab) ) ylab <- sname
     
  ##########################################   
  ## In case 'from' and 'to' are provided  ##
  dates  <- time(x)
  ndates <- length(dates)
     
  # Checking the validity of the 'from' argument
  if (!is.null(from)) { 
     from <- as.Date(from, format=date.fmt)
     if ( !(from %in% dates) ) {
        stop("Invalid argument: 'from' is not in 'dates' ")
     } else x <- window(x, start=from)
  } # IF end

  # Checking the validity of the 'to' argument
  if (!is.null(to)) { 
     to <- as.Date(to, format=date.fmt)
     if ( !(from %in% dates) ) {
        stop("Invalid argument: 'to' is not in 'dates' ")
     } else x <- window(x, end=to)
  } # IF end

  ################
  # column index of the station identified by 'sname' within 'x'
  col.index <- which( colnames(x) == sname )

  # If the station name exists within 'x'
  if ( length(col.index) > 0 ) {

    # Slecting the time series within 'x' corresponding to the 'sname' station
    x <- x[ ,col.index]

    # 9 plots:
    # 1: Line plot with Daily time series, with 2 moving averages, specified by 'win.len1' and 'win.len2'
    # 2: Line plot with Monthly time series, with 2 moving averages, specified by 'win.len1' and 'win.len2'
    # 3: Line plot with Annual time series
    # 4: Boxplot with daily time series
    # 5: Boxplot with monthly time series
    # 6: Boxplot with annual time series
    # 7: Histogram of the daily time series
    # 8: Histogram of the monthly time series
    # 9: Histogram of the annual time series
    hydroplot(x, FUN=FUN, na.rm=na.rm, ptype=ptype, pfreq=pfreq,                      
              var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,
              win.len1=win.len1, win.len2=win.len2, tick.tstep=tick.tstep, 
              lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex=cex, cex.main=cex.main, cex.lab=cex.lab,
              cex.axis=cex.axis, col=col, stype=stype, season.names=season.names, h=h, ...)

  } else stop("The station name", sname, "does not exist in 'x'")

}  # 'sname2plot.zoo' END



################################################################################
#  'sname2plot': Given a data.frame whose columns contains the ts              #
#                (without missing dates) of several gauging stations, it       #
#                takes the name of one gauging station and plots 9 graphs      #
#                (see 'hydroplot' description)                                 #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 17-Dic-2008                                                         #
# Updates: 19-Apr-2011 ; 10-Aug-2011                                           #
#          18-Oct-2012                                                         #
#          04-Abr-2013 : 29-May-2013 ; 03-Jun-2013                             #
#          07-Nov-2020                                                         #
#          23-Aug-2022                                                         #
################################################################################
sname2plot.data.frame <- function(x, sname, FUN, na.rm=TRUE,
                                  ptype="ts+boxplot+hist",
                                  pfreq="dma",                      
                                  var.type,                      
                                  var.unit="units",
                                  main=NULL, xlab="Time", ylab=NULL,
                                  win.len1=0,
                                  win.len2=0,                      
                                  tick.tstep="auto",
                                  lab.tstep="auto",
                                  lab.fmt=NULL,
                                  cex=0.3,
                                  cex.main=1.3,
                                  cex.lab=1.3,
                                  cex.axis=1.3,
                                  col=c("blue", "lightblue", "lightblue"),
                                  dates=1, date.fmt = "%Y-%m-%d",
                                  from=NULL, to=NULL, 
                                  stype="default", 
                                  season.names=c("Winter", "Spring", "Summer", "Autumn"),
                                  h=NULL, ...) {

  
  # Checking the user provides the dates
  if ( !any( class(dates) %in% c("numeric", "factor", "character", "Date" ,"POSIXct", "POSIXlt", "POSIXt") ) )
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'character', 'Date', 'POSIXct', 'POSIXlt', 'POSIXt'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  if ( inherits(dates, "numeric") ) {
    temp  <- x[, -dates]
    dates <- as.Date(as.character(x[, dates]), format= date.fmt) # zoo::as.Date
    x     <- temp
  } # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( ( inherits(dates, "factor") ) | ( inherits(dates, "character") ) ) 
    dates <- as.Date(dates, format= date.fmt) # zoo::as.Date

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'sname' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
    stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")  

  # converting from data.frame to zoo
  x <- zoo::zoo(x, dates)     

  sname2plot.zoo(x, sname=sname, FUN=FUN, na.rm=na.rm, ptype=ptype, pfreq=pfreq,                      
                var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,
                win.len1=win.len1, win.len2=win.len2, tick.tstep=tick.tstep, 
                lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex=cex, cex.main=cex.main, cex.lab=cex.lab,
                cex.axis=cex.axis, col=col, from=from, to=to, stype=stype, season.names=season.names, h=h, ...)



}  # 'sname2plot.data.frame' END

# File mip.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                                 'mip'                                        #
################################################################################
# Given any starting and ending dates, it generates:                           #
#        1) a vector with all the months between the two dates, OR             #
#	 2) the amount of months between the two dates                               #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 2008                                                                #
# Updates: 29-May-2013                                                         #  
#          23-Aug-2022                                                         #          
################################################################################

# 'from'    : Starting date for computing the number of dyas. MUST have the date format specified by 'date.fmt'
# 'to'	    : Ending date for computing the number of dyas. MUST have the date format specified by 'date.fmt'
# 'date.fmt': Format of the dates (e.g., "%d-%m-%Y")
# out.type  : type of result that is given by this function
#		      -) type= "seq"  => a vectorial sequence with all the months within the given year
#		      -) type= "nmbr" => the number of days in the vectorial sequence with all the months within the given year
mip <- function(from, to, date.fmt="%Y-%m-%d", out.type="seq") {

     # Checking 'out.type'
     if (is.na(match(out.type, c("seq", "nmbr"))))
        stop("Invalid argument: 'out.type' must be of class 'seq' or 'nmbr'")
        
     # Converting 'from' into a Date object (if necessary)
     if ( !inherits(from, "Date") ) {
       from.bak <- from
       from     <- as.Date(from, format=date.fmt)
     } # IF end
     
     # Checking that 'from' is a valid Date object
     if (is.na(from)) 
       stop("Invalid argument: 'from' (", from.bak, 
            ") is not compatible with 'date.ftm' (", date.fmt, ") !")
     
     # Converting 'to' into a Date object (if necessary)
     if ( !inherits(to, "Date") ) {
       to.bak <- to
       to     <- as.Date(to, format=date.fmt)
     } # IF end
     
     # Checking that 'to' is a valid Date object
     if (is.na(to)) 
       stop("Invalid argument: 'to' (", to.bak, 
            ") is not compatible with 'date.ftm' (", date.fmt, ") !")
     
     # Checking that 'from' is lower or equal to 'to'
     if (to < from) stop("Invalid argument: 'from > to' (", from, " > ", to, ")")

     # Generating an Annual-regular time series of Dates.
     vec.months <- seq( from=as.Date(from, format=date.fmt), to=as.Date(to, format=date.fmt), by="months" )

     if (out.type=="seq") return(vec.months)
     else if (out.type=="nmbr") return ( length(vec.months) )

} # 'mip' END

# File rm1stchar.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2010-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# 'rm1stchar' : deletes the first n characther(s) of 'x'                       #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 01-Oct-2010                                                         #
################################################################################

# 'x'   : Character, e.g, each element may represent the name of a single gauging station
# 'n'   : numeric, indicating the number of characters that have to be removed from the beginning of 'x'
rm1stchar <- function(x, n=1) {

   if (n<0) stop("'n' must be a positive integer")

  L <- length(x)

  start.col <- n + 1

  x[start.col:length(x)] <- sapply(start.col:length(x), function(j,x) {
                      x[j] <- substr(x[j], start=start.col, stop=nchar(x[j]))
                  }, x = x)

  return(x)

} # 'rm1stchar' END

# File dip.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                                    'dip'                                     #  
################################################################################       
# Purpose: Given any starting and ending dates, it generates:                  #
#        1) a vector with all the days between the two dates, OR               #
#	 2) the amount of days between the two dates                                 #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 2008                                                                #
# Updates: 29-May-2013                                                         #
#          24-aug-2022                                                         #
################################################################################

# 'from'    : Character indicating the starting date for computing the number of dyas.
#             It MUST have the date format specified by 'date.fmt'
# 'to'	    : Character indicating the ending date for computing the number of dyas.
#             It MUST have the date format specified by 'date.fmt'
# 'date.fmt': Character indicating the date format in which you provide 'from' and 'to'. (e.g., "%d-%m-%Y")
# 'out.type': Character indicating the type of result that is given by this function.
#             Valid values are:
#		      -) type= "seq"  => a vectorial sequence with all the days within the given time period
#		      -) type= "nmbr" => the number of days in the vectorial sequence with all the days within the given time period
dip <- function(from, to, date.fmt="%Y-%m-%d", out.type="seq") {

     # Checking 'out.type'
     if (is.na(match(out.type, c("seq", "nmbr"))))
        stop("Invalid argument: 'out.type' must be of class 'seq' or 'nmbr'")
        
     # Converting 'from' into a Date object (if necessary)
     if ( !inherits(from, "Date") ) {
       from.bak <- from
       from     <- as.Date(from, format=date.fmt)
     } # IF end
     
     # Checking that 'from' is a valid Date object
     if (is.na(from)) 
       stop("Invalid argument: 'from' (", from.bak, 
            ") is not compatible with 'date.ftm' (", date.fmt, ") !")
     
     # Converting 'to' into a Date object (if necessary)
     if ( !inherits(to, "Date") ) {
       to.bak <- to
       to     <- as.Date(to, format=date.fmt)
     } # IF end
     
     # Checking that 'to' is a valid Date object
     if (is.na(to)) 
       stop("Invalid argument: 'to' (", to.bak, 
            ") is not compatible with 'date.ftm' (", date.fmt, ") !")
     
     # Checking that 'from' is lower or equal to 'to'
     if (to < from) stop("Invalid argument: 'from > to' (", from, " > ", to, ")")

     # Generating an Annual-regular time series of Dates.
     vec.days <- seq( from=from, to=to, by="days" )

     if (out.type=="seq") return(vec.days)
     else if (out.type=="nmbr") return ( length(vec.days) )

} # 'dip' END

# File istdx.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                                  'istdx'                                     #
################################################################################
# This function transforms back a standarized vector/matrix into their original#
# values, i.e., re-scales all the values in the [0,1] interval to the original #
# range of values                                                              #
#           x = re-scale(z) = z*[ zmax - zmin ] + xmin                         #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 19-Feb-2009                                                         #
################################################################################
#

# 'z'     : standarized vector or matriz to be re-scaled, all the values
#           have to be in the range [0,1]
# 'xmin'  : numeric with the minimum value(s) in the original 'x'
#           -) if 'z' is a vector, 'xmin' has to be a real
#           -) if 'z' is a matrix/data.frame, 'xmin' has to be a vector, with
#              the minimum values for each column of the original 'x'
#              in this case, the vector of minimums can be otained as:
#               xmin <-  apply(x,2,min, na.rm=TRUE)
# 'xrange'  : numeric with the range of value(s) in the original 'x'
#           -) if 'z' is a vector, 'xrange' has to be a real
#           -) if 'z' is a matrix/data.frame, 'xrange' has to be a vector, with
#              the range of values for each column of the original 'x'
#              in this case, the vector of ranges can be otained as:
#               xrange <-  apply(x, 2, range, na.rm=TRUE)
#               xrange <-  apply(xrange,2, diff, na.rm=TRUE)
# 'result': re-scaled 'x', where all the values of each column of 'x'
#           are within the original range of x values

istdx <-function(x,...) UseMethod("istdx")

istdx.default <- function(x, xmin, xrange,...) {

   z <- x*xrange + xmin

   names(z) <- names(x)

   return(as.numeric(z))

} # 'istdx.default' end


istdx.matrix <- function(x, xmin, xrange,...) {

  z <- matrix(data = NA, nrow = nrow(x), ncol = ncol(x))

  z <- sapply(1:ncol(z), function(i) {
		 z[,i] <- istdx.default( x[,i], xmin[i], xrange[i] )
		 })

  colnames(z) <- colnames(x)
  rownames(z) <- rownames(x)

  return(z)

}  # 'istdx.matrix' end


istdx.data.frame <- function(x, xmin, xrange,...) {

  x <- as.matrix(x)
  NextMethod("istdx.matrix")

}  # 'istdx.data.frame' end

# File isComplete.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2025-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# isComplete: Generic function for identifiying whether a zoo object with a    #
#             regular frequency (e.g., hourly, daily, monthly, annual) has a   #
#             value in each time step or not, from the beggining to the end    #
################################################################################
# Author     : Mauricio Zambrano-Bigiarini                                     #
# Contributor:                                                                 #
################################################################################
# It is completly based on a function with the same name included since 2024 in#
# the hydroRTS R package                                                       #
################################################################################
# Started: 26-Jul-20245                                                        #
# Updates:                                                                     #
################################################################################
# 'x'     : zoo object. The time attribute should have a regular time frequency 
#           (e.g., hourly, daily, monthly, annual)
# 'start' : Starting date/time for the time window used to subset \code{x}.
# 'end'   : Ending date/time for the time window used to subset \code{x}.
# 'format': Format used with \code{start} and \code{end}
# 'tz'    : character, indicating the time zone in which the date of \code{x} is located.

isComplete <- function(x, ...) UseMethod("isComplete")

isComplete.zoo <- function(x, tz, out.type=c("single", "all"), verbose=TRUE, ...) {
  
  # Checking the 'out.type' argument
  out.type <- match.arg(out.type)
  
  # Getting the time attribute of 'x'
  datetimes <- time(x)

  # if 'time(x)' is a yearmon object but stored as real numbers (terra way)
  # it is transformed into a real yearmon object
  if ( inherits(datetimes, "numeric") & !all( datetimes == trunc(datetimes) ) )
    datetimes <- zoo::as.yearmon(datetimes)

   # Automatic detection of 'tz'
  missingTZ <- FALSE
  if (missing(tz)) {
    missingTZ <- TRUE
    tz        <- attr(datetimes, "tzone")
  } else {
      # For the Date/Time of 'x' to be in the time zone specified by 'tz'
      tx.new     <- timechange::time_force_tz(datetimes, tz=tz)
      time(x)    <- tx.new
       datetimes <- time(x)
    } # ELSE end

  # Getting the total amount of elements in 'datetimes'
  nDateTimes <- length(datetimes)

  # Getting the starting DateTime object in 'x'
  dt.ini <- datetimes[1]

  # Getting the ending DateTime object in 'x'
  dt.end <- datetimes[nDateTimes]

  # Creating a temporal zoo object with the same DateTime as 'x'
  temp <- zoo::zoo(rep(1, nDateTimes), datetimes)

  # Checking whether 'x' is:
  # i)   stictly regular (regular, without missing vlaues), 
  # ii)  approximately regular (regular, with some missing values) or 
  # iii) not regular at all
  isStrictlyRegular <- zoo::is.regular(temp, strict=TRUE) # It fails for monthly objects with Date time attribute
  isApproximRegular <- zoo::is.regular(temp, strict=FALSE)

  if ( !isApproximRegular )         
    stop("[ 'x' does not have a regular time frequency => is not possible to assess whether it is complete or not ! ]") 

  ##############################################################################
  # Getting the time frequency of 'x'                                          #
  ##############################################################################

  # Detecting ANNUAL 2-annual, 3-annual (years as numeric integers, not as yearmon)
  if ( ( inherits(datetimes, "numeric") & all( datetimes == trunc(datetimes) ) ) & 
       !inherits(datetimes, "yearmon")
      ) {

    # years as integer numbers
    if ( all( datetimes == trunc(datetimes) ) ) { 

      dt.ini    <- as.Date(paste0(dt.ini, "-01-01"))
      dt.end    <- as.Date(paste0(dt.end, "-01-01"))
      dt.stg    <- "years"
      dt.val    <- round(median(diff(datetimes), na.rm=TRUE))
      if ( dt.val == 1 ) {
        x.freq    <- "annual"
      } else x.freq <- paste0(dt.val, "-annual")

      datetimes <- as.Date(paste0(datetimes, "-01-01"))

    } # IF end

  # Detecting MONTHLY objects (years as real numbers (not integers) or as yearmon objects)
  # Monthly objects can be: yearmon (zoo objects), numeric (terra objects) or Date (base R objects)
  } else if ( ( inherits(datetimes, "numeric") & !all( datetimes == trunc(datetimes) ) ) |
              inherits(datetimes, "yearmon") | 
              ( (hydroTSM::sfreq(temp) == "monthly") & (median(diff(as.Date(datetimes) ), na.rm=TRUE) > 28) )
            ) {

              if ( inherits(datetimes, "Date") ) { # Date objects
                dt.ini <- as.Date(dt.ini)
                dt.end <- as.Date(dt.end)
                dt.val <- round( median(diff(as.Date(datetimes) ), na.rm=TRUE) / 30 ) # division by 30 is for transforming from "days" to "months"
              } else if ( !inherits(datetimes, "yearmon") ) { # real number monthly objects
                  if ( dt.ini == trunc(dt.ini) ) {
                    dt.ini  <-  as.Date(paste0(dt.ini, "-01-01"))
                  } else dt.ini  <- as.Date( paste0( format(zoo::as.yearmon(dt.ini), "%Y-%m"), "-01") )
                  if ( dt.end == trunc(dt.end) ) {
                    dt.end  <-  as.Date(paste0(dt.end, "-01-01"))
                  } else dt.end <- as.Date( paste0( format(zoo::as.yearmon(dt.end), "%Y-%m"), "-01") )
                  datetimes <- zoo::as.yearmon(datetimes)
                  dt.val    <- round( median( diff(datetimes), na.rm=TRUE) * 12 )
                } else { # yearmon objects
                     dt.ini <- as.Date( paste0( format(dt.ini, "%Y-%m"), "-01") )
                     dt.end <- as.Date( paste0( format(dt.end, "%Y-%m"), "-01") )
                     dt.val <- round( median( diff(datetimes), na.rm=TRUE) * 12 ) 
                  } # ELSE end

              dt.stg    <- "months"
              
              if ( dt.val == 1 ) {
                x.freq    <- "monthly"
              } else x.freq <- paste0(dt.val, "-monthly")

    } else { # Detecting objects with time frequencies different from Annual and Monthly

        dt     <- diff(datetimes)
        dt.val <- median(dt, na.rm=TRUE)
        dt.stg <- attr(dt, "units")
        if (verbose) message("[ The exact time frequency of 'x' is: ", dt.val, " ", dt.stg )

        # Getting the time frequency of 'x', considering the particular case of the well known
        # hourly, daily, weekly, monthly, annual
        x.freq <- paste(dt.val, dt.stg)

        if ( (x.freq == "1 hours") | ( (dt.stg=="mins") & ( dt.val == 60 ) ) ) 
          x.freq <- "hourly"

        if ( (x.freq == "1 days") | ( (dt.stg=="hours") & ( dt.val == 24 ) ) )
          x.freq <- "daily"

        if ( (x.freq == "1 weeks") | ( (dt.stg=="days") & ( dt.val == 7 ) ) )
          x.freq <- "weekly"

        if ( (x.freq == "1 months") | ( (dt.stg=="days") & ( dt.val %in% c(28, 29, 30, 31) ) ) )
          x.freq <- "monthly"

        if ( (x.freq == "1 years") | ( (dt.stg=="days") & ( dt.val == 365 ) ) | ( (dt.stg=="months") & ( dt.val == 12 ) ) )
          x.freq <- "annual"

        # Some well known time frequencies
        if ( (x.freq == "hourly") & (dt.val > 1) ) 
          x.freq <- paste0(dt.val, "-hourly")

        if ( (x.freq == "daily") & (dt.val > 1) ) 
          x.freq <- paste0(dt.val, "-day")

        if ( (x.freq == "monthly") & (dt.val > 1) ) 
          x.freq <- paste0(dt.val, "-monthly")

        if (hydroTSM::sfreq(temp) == "quarterly") 
          x.freq <- "quarterly"

      } # ELSE end
  
  ##############################################################################
  # Getting the complete time elements that should be present in 'x' from 'dt.ini' to 'dt.end'
  ##############################################################################
  if ( (x.freq %in% c("hourly", "daily", "weekly", "monthly", "quarterly", "annual") ) ) {
    completeTime <- switch(x.freq, "hourly"    = hydroTSM::hip(from=dt.ini, to=dt.end, tz=tz),
                                   "daily"     = hydroTSM::dip(from=dt.ini, to=dt.end),
                                   "weekly"    = seq(from=as.Date(dt.ini), to=as.Date(dt.end), by="1 week"),
                                   "monthly"   = hydroTSM::mip(from=dt.ini, to=dt.end),
                                   "quarterly" = seq(from=as.Date(dt.ini), to=as.Date(dt.end), by="1 quarter"),
                                   "annual"    = hydroTSM::yip(from=dt.ini, to=dt.end)
                          )
    if ( (x.freq == "monthly") & (inherits(datetimes, "yearmon")) )
      completeTime <- zoo::as.yearmon(completeTime) 

  } else { # Identifying sub-daily time frequencies

          if ( (x.freq %in% c("minute", "hourly")) | grepl("hour", x.freq) | grepl("min", x.freq) ) {
             subdaily.ts <- TRUE
             completeTime <- seq(from=as.POSIXct(dt.ini, tz=tz), to=as.POSIXct(dt.end, tz=tz), by=x.freq )

          } else {
              subdaily.ts  <- FALSE
              x.freq       <- paste(dt.val, dt.stg)
              if ( inherits(datetimes, "numeric") & all( datetimes == trunc(datetimes) ) ) {
                completeTime <- seq(from=as.numeric(format(dt.ini, "%Y")), to=as.numeric(format(dt.end, "%Y")), by=x.freq )  
              } else completeTime <- seq(from=as.Date(dt.ini), to=as.Date(dt.end), by=x.freq )
            } # ELSE end

         } # ELSE end


  ##############################################################################
  # Identifying any eventual missing time steps 
  ##############################################################################
  missingDT.index <- which(is.na(pmatch(completeTime, datetimes))) 
  missingDT.n     <- length(missingDT.index)
  if ( missingDT.n > 0 )
    missingDateTimes <- completeTime[missingDT.index]

  ##############################################################################
  # Identifying whether 'x' is complete or not                                 #
  ##############################################################################
  out <- FALSE
  if ( isStrictlyRegular | (length(missingDT.index) == 0) ) {

    if (verbose) message("                                                       ")
    if (verbose) message("[ 'x' (", x.freq, ") is complete from '", dt.ini, "' to '", dt.end, "' (", nDateTimes, " elements) ! ]")

    out <- TRUE

  } else {  

      if ( (missingDT.n > 0) & verbose ) 
        message("[ There are ", missingDT.n, " missing date/times in 'x' (", x.freq, ") ]" )
      

      if ( (missingDT.n > 0) & verbose ) 
        message("[ Missing date/times are: ", paste(missingDateTimes, collapse=" ; "), " ]" )
          
    } # ELSE end


  # creating the final output
  if (out.type=="all") 
    out <- list(isComplete=out, NumberMissingDT=missingDT.n, missingDateTimes=missingDateTimes)
  
  return(out)
  
} # 'isComplete.SpatRaster' END

# File time2season.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2022 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# time2season : This function transforms a vector of dates or date-time into a # 
#               vector of seasons (summer, winter, autumn, spring),            #
#               considering that, by default:                                  #
#               winter = DJF: December, January, February                      #
#               spring = MAM: March, April, May                                #
#               summer = JJA: June, July, August                               #
#               autumn = SON: September, October, November                     #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 18-Mar-2009                                                         #
# Updates: 09-Aug-2011                                                         #
#          05-Apr-2013                                                         #
#          10-Mar-2020                                                         #
#          22-Oct-2022 ; 25-Oct-2022                                           #
################################################################################

# 'x'       : vector with the dates that have to be transformed. class(x) must be "Date"
# 'out.fmt' : format of the output seasons. Possible values are:
#             -) 'seasons' =>  "winter", "spring",  "summer", autumn"
#             -) 'months'  =>  "DJF", "MAM",  "JJA", SON"

# 'result': vector with the weather season to which each date in 'x' belongs

time2season <- function(x, out.fmt="months", type="default") {

 # Checking that 'class(x)==Date'
 #if ( ( !( class(x) %in% c("Date", "POSIXct", "POSIXt") ) ) && TRUE ) 
 if (!( is(x, "Date") | is(x, "POSIXct") | is(x, "POSIXt") )) 
     stop("Invalid argument: 'x' must be in c('Date', 'POSIXct', 'POSIXt') !")

 # Checking the class of out.fmt
 if (is.na(match(out.fmt, c("seasons", "months") ) ) )
     stop("Invalid argument: 'out.fmt' must be in c('seasons', 'months')")
     
 # Checking that the user provided a valid value for 'type'   
 valid.types <- c("default", "FrenchPolynesia")    
 if (length(which(!is.na(match(type, valid.types )))) <= 0)  
     stop("Invalid argument: 'type' must be in c('default', 'FrenchPolynesia')")

 ####################
 months <- format(x, "%m")
 
 if (type=="default") {
   winter <- which( months %in% c("12", "01", "02") )
   spring <- which( months %in% c("03", "04", "05") )
   summer <- which( months %in% c("06", "07", "08") )
   autumn <- which( months %in% c("09", "10", "11") ) 
 } else if (type=="FrenchPolynesia") {
   winter <- which( months %in% c("12", "01", "02", "03") )
   spring <- which( months %in% c("04", "05") )
   summer <- which( months %in% c("06", "07", "08", "09") )
   autumn <- which( months %in% c("10", "11") ) 
 } # ELSE end

 # Creation of the output, with the same length of the 'x' input
 seasons <- rep(NA, length(x))

 if (out.fmt == "seasons") {
 
    seasons[winter] <- "winter"
    seasons[spring] <- "spring"
    seasons[summer] <- "summer"
    seasons[autumn] <- "autumn"
    
 } else { # out.fmt == "months"
 
    if (type=="default") {
      seasons[winter] <- "DJF"
      seasons[spring] <- "MAM"
      seasons[summer] <- "JJA"
      seasons[autumn] <- "SON"
    } else  if (type=="FrenchPolynesia") {
       seasons[winter] <- "DJFM"
       seasons[spring] <- "AM"
       seasons[summer] <- "JJAS"
       seasons[autumn] <- "ON"
      } # IF end
   
 } # IF end

 return(seasons)

} # 'time2season' END

# From: https://github.com/tavisrudd/r_users_group_1/blob/master/calendarHeat.R

##############################################################################
 # Calendar Heatmap #
 # by #
 # Paul Bleicher #
 # an R version of a graphic from: #
 # http://stat-computing.org/dataexpo/2009/posters/wicklin-allison.pdf #
 # requires lattice, chron, grid packages #
 ##############################################################################

## calendarHeat: An R function to display time-series data as a calendar heatmap
## Copyright 2009 Humedica. All rights reserved.

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
## GNU General Public License for more details.

## You can find a copy of the GNU General Public License, Version 2 at:
## http://www.gnu.org/licenses/gpl-2.0.html

################################################################################
# calendarHeat: R function to display time-series data as a calendar heatmap   #
################################################################################
# Author of this slightly modified verison: Mauricio Zambrano-Bigiarini        #
################################################################################
# Started: 2009 ;                                                              #
# Updates: 25-Nov-2023 ; 27-Nov-2023 ; 29-Nov-2023 ; 30-Nov-2023               #
#          17-Jan-2024                                                         #
################################################################################

calendarHeatmap <- function(x, ...) UseMethod("calendarHeatmap")

calendarHeatmap.zoo <- function(x,
                            #ncolors=5,
                            #color="r2g",

                            from,
                            to,
                            date.fmt="%Y-%m-%d",

                            main="Calendar Heat Map",
                            #col= RColorBrewer::brewer.pal(n=8, name="RdYlBu"), # from Mauricio Zambrano-Bigiarini. A color palette, i.e. a vector of n contiguous colors generated by functions like rainbow, heat.colors, topo.colors, bpy.colors or one or your own making, perhaps using colorRampPalette. If none is provided, brewer.pal(n=8, name="RdYlBu") is used
                            col=colorRampPalette(c("red", "orange", "yellow", "white", "lightblue2", "deepskyblue", "blue3"), space = "Lab")(8), # from Mauricio Zambrano-Bigiarini. A color palette, i.e. a vector of n contiguous colors generated by functions like rainbow, heat.colors, topo.colors, bpy.colors or one or your own making, perhaps using colorRampPalette. If none is provided, brewer.pal(n=8, name="RdYlBu") is used
                            cuts,                                              # from Mauricio Zambrano-Bigiarini. Numeric, indicating the values used to divide the range of 'x' in the legend of colours. If not provided, it is automatically selected as a function of 'lenght(col)'
                            cuts.dec=0,                                        # from Mauricio Zambrano-Bigiarini. Number of decimal places used to present the numbers that divide the range of 'x' in the legend of colours
                            cuts.labels,                                       # from Mauricio Zambrano-Bigiarini. Character indicating the label to be used in the ccolour legend for each one of the values defined by 'cuts'. If not provided, as.character(cuts)' is used
                            cuts.style=c("fisher", "equal", "pretty", "fixed", "sd", "quantile", "kmeans", "bclust", "mzb"), # discarded becsue takes too much time or not alway provide the required number of classes: "dpih", "headtails", "hclust",  "jenks"
                            legend.title="",                                   # from Mauricio Zambrano-Bigiarini. text to be displayed above the legned of colours (e.g., showing the measurement units of the raster being displayed)
                            legend.fontsize=15,                                # from Mauricio Zambrano-Bigiarini. The size of text (in points) used in the legend 
                    
                            #varname="Values",
                            #date.fmt = "%Y-%m-%d", 

                            do.png=FALSE,
                              png.fname="mypng.png",
                              png.width=1500,
                              png.height=900,
                              png.pointsize=12,
                              png.res=90,
                            do.pdf=FALSE,
                              pdf.fname="mypdf.pdf",
                              pdf.width=11,
                              pdf.height=8.5,
                              pdf.pointsize=12,
                            ...) {

    # checking 'cuts.style'
    cuts.style <- match.arg(cuts.style)

    #require(lattice)
    #require(grid)
    #require(chron)
    dates  <- stats::time(x)
    values <- zoo::coredata(x)

    #if (class(dates) == "character" | class(dates) == "factor" ) {
    #   dates <- strptime(dates, date.fmt)
    #} # IF end

    # Checking daily time frequency
    if (sfreq(x) != "daily")
      stop("Invalid argument: 'sfreq(x)' must be 'daily', but it is '", sfreq(x), "' !")

    ####################################################################################
    # Lines 95-132 are taken and adpated from izoo2rzoo.R to check 'from' and 'to'
    ####################################################################################
  
    # If the index of 'p' is character, it is converted into a Date object
    if ( class(time(x))[1] %in% c("factor", "character") ) {
      dt <- try(as.Date(time(x), format=date.fmt))
      if("try-error" %in% class(dt) || is.na(dt)) {
        stop("Invalid argument: format of 'time(x)' is not compatible with 'date.fmt' !")
      } else time(x) <- as.Date(time(x), format=date.fmt)
    } # IF end      

    # If 'from' was given as Date, but 'x' is sub-daily
    if (!missing(from)) {
      dt <- try(as.Date(from, format=date.fmt))
      if("try-error" %in% class(dt) || is.na(dt)) {
        stop("Invalid argument: format of 'from' is not compatible with 'date.fmt' !")
      } else from <- as.Date(from, format=date.fmt)

      if (from > to) stop("Invalid argument: 'from > to' !")

      if (from > end(x)) stop("Invalid argument: 'from > end(x)' !")

      # Selecting only those data starting in 'from'
      x <- window(x, start=from)      
    } # IF end

    # If 'to' was given as Date, but 'x' is sub-daily
    if (!missing(to)) {
      dt <- try(as.Date(to, format=date.fmt))
      if("try-error" %in% class(dt) || is.na(dt)) {
        stop("Invalid argument: format of 'to' is not compatible with 'date.fmt' !")
      } else to <- as.Date(to, format=date.fmt)

      if (to < from ) stop("Invalid argument: 'to < from' !")

      if (to < start(x) ) stop("Invalid argument: 'to < start(x)' !")

      # Selecting only those data ending in 'to'
      x <- window(x, end=to)      
    } # IF end    
    ####################################################################################


    # Checking that the maximum amount of years is 6 
    if (length(x) > 2191) # 365 day/year * 6 years + 1 day in a leap year
      stop("Invalid argument: length(x)' must be less than 2191, but it is '", length(x), "' !")

    ######################################################################################
    # Lines 69-97 are from Mauricio Zambrano-Bigiarini
    ######################################################################################
    if (missing(cuts)) {
      ncuts <- length(col) + 1
      temp  <- as.numeric(values)
      temp <-  temp[!is.nan(temp)]

      if (cuts.style=="mzb") {
        probs <- seq( 0, 1, length.out=ncuts )      
        lcuts <- round( stats::quantile( temp, probs=probs, na.rm=TRUE), cuts.dec)
        if ( length(unique(lcuts)) != ncuts )
          lcuts <- .findcuts4plot(x=temp, n=ncuts)
      } else {
          lcuts <- classInt::classIntervals(temp, n=length(col), dataPrecision=cuts.dec, style = cuts.style )[["brks"]]
          lcuts <- round(lcuts, cuts.dec)
          ncuts <- length(lcuts)
          col   <- col[1:(ncuts-1)]
        }
    } else {
        if ( (length(col)+1) != length(cuts) ) {
          stop("Invalid argument: 'length(col)+1 != length(cuts)' ") 
        } else lcuts <- cuts
      } # ELSE end


    if ( missing(cuts.labels) ) {
      cuts.labels <- as.character(lcuts)
    } else {
        if (length(lcuts) != length(cuts.labels) )
          stop("Invalid argument: 'length(cuts) != length(cuts.labels)' ") 
      } # ELSE end
    ######################################################################################


    caldat   <- data.frame(value = values, dates = dates)
    min.date <- as.Date(paste(format(min(dates), "%Y"),
                        "-1-1",sep = ""))
    max.date <- as.Date(paste(format(max(dates), "%Y"),
                         "-12-31", sep = ""))
    dates.f  <- data.frame(date.seq = seq(min.date, max.date, by="days"))

    # Merge moves data by one day, avoid
    caldat <- data.frame(date.seq = seq(min.date, max.date, by="days"), value = NA)
    dates  <- as.Date(dates)
    caldat$value[match(dates, caldat$date.seq)] <- values

    caldat$dotw  <- as.numeric(format(caldat$date.seq, "%w"))
    caldat$woty  <- as.numeric(format(caldat$date.seq, "%U")) + 1
    caldat$yr    <- as.factor(format(caldat$date.seq, "%Y"))
    caldat$month <- as.numeric(format(caldat$date.seq, "%m"))
    yrs          <- as.character(unique(caldat$yr))
    d.loc        <- as.numeric()
    for (m in min(yrs):max(yrs)) {
      d.subset <- which(caldat$yr == m)
      sub.seq <- seq(1,length(d.subset))
      d.loc <- c(d.loc, sub.seq)
    }
    caldat <- cbind(caldat, seq=d.loc)

    #color styles
    r2b <- c("#0571B0", "#92C5DE", "#F7F7F7", "#F4A582", "#CA0020") #red to blue
    r2g <- c("#D61818", "#FFAE63", "#FFFFBD", "#B5E384") #red to green
    w2b <- c("#045A8D", "#2B8CBE", "#74A9CF", "#BDC9E1", "#F1EEF6") #white to blue
                
    #assign("col.sty", get(color))
    #calendar.pal <- colorRampPalette((col.sty), space = "Lab")
    def.theme    <- lattice::lattice.getOption("default.theme")

    cal.theme <- function() {
      theme <- list(
                    strip.background = list(col = "transparent"),
                    strip.border = list(col = "transparent"),
                    axis.line = list(col="transparent"),
                    par.strip.text=list(cex=0.8))
    } # 'cal.theme' END

    lattice::lattice.options(default.theme = cal.theme)

    yrs <- (unique(caldat$yr))
    nyr <- length(yrs)
    
    print(
      cal.plot <- lattice::levelplot(value~woty*dotw | yr, data=caldat,
          as.table=TRUE,
          aspect=.12,
          layout = c(1, nyr%%7),
          between = list(x=0, y=c(1,1)),
          strip=TRUE,
          main = main,
          scales = list(
          x = list(
                   at= c(seq(2.9, 52, by=4.42)),
                   labels = month.abb,
                   alternating = c(1, rep(0, (nyr-1))),
                   tck=0,
                   cex = 0.7),
         y=list(
                at = c(0, 1, 2, 3, 4, 5, 6),
                labels = c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
                           "Friday", "Saturday"),
                alternating = 1,
                cex = 0.6,
                tck=0)),
         xlim =c(0.4, 54.6),
         ylim=c(6.6,-0.6),

         #cuts= ncolors - 1,

         #col.regions = (calendar.pal(ncolors)),
         at=lcuts, col.regions=col,
         colorkey = list(title = paste0(legend.title, " \n"),
                         title.gpar=list(fontsize=legend.fontsize, font=1), #font=2 -> bold font
                         space = "right",
                         at=as.numeric(factor(lcuts)),                                     # equally spaced color bins in the legend
                         labels=list(labels=cuts.labels, at=as.numeric(factor(lcuts)), cex=0.9), # equally spaced color bins in the legend
                         col=col,
                         width = 0.6, height = 0.5                                  
                         ),

         xlab="" ,
         ylab=""
         #colorkey= list(col = calendar.pal(ncolors), width = 0.6, height = 0.5),
         #subscripts=TRUE
         ) #'levelplot' END
    ) # 'print' END

    panel.locs <- lattice::trellis.currentLayout()

    for (row in 1:nrow(panel.locs)) {
      for (column in 1:ncol(panel.locs)) {


        if (panel.locs[row, column] > 0) {

            lattice::trellis.focus("panel", row = row, column = column, highlight = FALSE)
            xyetc       <- lattice::trellis.panelArgs()
            subs        <- caldat[xyetc$subscripts,]
            dates.fsubs <- caldat[caldat$yr == unique(subs$yr),]
            y.start     <- dates.fsubs$dotw[1]
            y.end       <- dates.fsubs$dotw[nrow(dates.fsubs)]
            dates.len   <- nrow(dates.fsubs)
            adj.start   <- dates.fsubs$woty[1]

            for (k in 0:6) {
              if (k < y.start) {
                x.start <- adj.start + 0.5
              } else {
                 x.start <- adj.start - 0.5
                }
              if (k > y.end) {
                 x.finis <- dates.fsubs$woty[nrow(dates.fsubs)] - 0.5
              } else {
                  x.finis <- dates.fsubs$woty[nrow(dates.fsubs)] + 0.5
                }
              grid::grid.lines(x = c(x.start, x.finis), y = c(k -0.5, k - 0.5),
                               default.units = "native", 
                               gp=grid::gpar(col = "grey", lwd = 1))
            } # FOR end

            if (adj.start < 2) {
              grid::grid.lines(x = c( 0.5, 0.5), y = c(6.5, y.start-0.5),
                   default.units = "native", gp=grid::gpar(col = "grey", lwd = 1))
              grid::grid.lines(x = c(1.5, 1.5), y = c(6.5, -0.5), default.units = "native",
                   gp=grid::gpar(col = "grey", lwd = 1))
              grid::grid.lines(x = c(x.finis, x.finis),
                   y = c(dates.fsubs$dotw[dates.len] -0.5, -0.5), default.units = "native",
                   gp=grid::gpar(col = "grey", lwd = 1))
              if (dates.fsubs$dotw[dates.len] != 6) {
                  grid::grid.lines(x = c(x.finis + 1, x.finis + 1),
                  y = c(dates.fsubs$dotw[dates.len] -0.5, -0.5), default.units = "native",
                  gp=grid::gpar(col = "grey", lwd = 1))
              }
              grid::grid.lines(x = c(x.finis, x.finis),
                    y = c(dates.fsubs$dotw[dates.len] -0.5, -0.5), default.units = "native",
                    gp=grid::gpar(col = "grey", lwd = 1))
            } # IF end

            for (n in 1:51) {
              grid::grid.lines(x = c(n + 1.5, n + 1.5),
                               y = c(-0.5, 6.5), default.units = "native", 
                               gp=grid::gpar(col = "grey", lwd = 1))
            } # FOR end
            x.start <- adj.start - 0.5

            if (y.start > 0) {
              grid::grid.lines(x = c(x.start, x.start + 1),
                y = c(y.start - 0.5, y.start - 0.5), default.units = "native",
                gp=grid::gpar(col = "black", lwd = 1.75))
              grid::grid.lines(x = c(x.start + 1, x.start + 1),
                y = c(y.start - 0.5 , -0.5), default.units = "native",
                gp=grid::gpar(col = "black", lwd = 1.75))
              grid::grid.lines(x = c(x.start, x.start),
                y = c(y.start - 0.5, 6.5), default.units = "native",
                gp=grid::gpar(col = "black", lwd = 1.75))
             if (y.end < 6 ) {
              grid::grid.lines(x = c(x.start + 1, x.finis + 1),
               y = c(-0.5, -0.5), default.units = "native",
               gp=grid::gpar(col = "black", lwd = 1.75))
              grid::grid.lines(x = c(x.start, x.finis),
               y = c(6.5, 6.5), default.units = "native",
               gp=grid::gpar(col = "black", lwd = 1.75))
               } else {
                  grid::grid.lines(x = c(x.start + 1, x.finis),
                   y = c(-0.5, -0.5), default.units = "native",
                   gp=grid::gpar(col = "black", lwd = 1.75))
                  grid::grid.lines(x = c(x.start, x.finis),
                   y = c(6.5, 6.5), default.units = "native",
                   gp=grid::gpar(col = "black", lwd = 1.75))
                 }
             } else {
                  grid::grid.lines(x = c(x.start, x.start),
                                   y = c( - 0.5, 6.5), default.units = "native",
                                   gp=grid::gpar(col = "black", lwd = 1.75))
               } # ELSE end

             if (y.start == 0 ) {
              if (y.end < 6 ) {
              grid::grid.lines(x = c(x.start, x.finis + 1),
               y = c(-0.5, -0.5), default.units = "native",
               gp=grid::gpar(col = "black", lwd = 1.75))
              grid::grid.lines(x = c(x.start, x.finis),
               y = c(6.5, 6.5), default.units = "native",
               gp=grid::gpar(col = "black", lwd = 1.75))
               } else {
                  grid::grid.lines(x = c(x.start + 1, x.finis),
                   y = c(-0.5, -0.5), default.units = "native",
                   gp=grid::gpar(col = "black", lwd = 1.75))
                  grid::grid.lines(x = c(x.start, x.finis),
                   y = c(6.5, 6.5), default.units = "native",
                   gp=grid::gpar(col = "black", lwd = 1.75))
                 } # ELSE end
             } # IF end

            for (j in 1:12) {
               last.month <- max(dates.fsubs$seq[dates.fsubs$month == j])
               x.last.m <- dates.fsubs$woty[last.month] + 0.5
               y.last.m <- dates.fsubs$dotw[last.month] + 0.5
               grid::grid.lines(x = c(x.last.m, x.last.m), y = c(-0.5, y.last.m),
                 default.units = "native", gp=grid::gpar(col = "black", lwd = 1.75))
               if ((y.last.m) < 6) {
                  grid::grid.lines(x = c(x.last.m, x.last.m - 1), y = c(y.last.m, y.last.m),
                   default.units = "native", gp=grid::gpar(col = "black", lwd = 1.75))
                 grid::grid.lines(x = c(x.last.m - 1, x.last.m - 1), y = c(y.last.m, 6.5),
                   default.units = "native", gp=grid::gpar(col = "black", lwd = 1.75))
               } else {
                  grid::grid.lines(x = c(x.last.m, x.last.m), y = c(- 0.5, 6.5),
                   default.units = "native", gp=grid::gpar(col = "black", lwd = 1.75))
                 } # ELSE end
             } # FOR end
        } # IF end

      } # FOR end

      lattice::trellis.unfocus()

    } # FOR end

  lattice::lattice.options(default.theme = def.theme)

} # 'calendarHeatmap' END


################################################################################
###                             .findcuts4plot                                ###
################################################################################
### Author     : Mauricio Zambrano-Bigiarini                                 ###
################################################################################
### Started    :  20-Dic-2017                                                ###
### Updates    :  12-May-2018                                                ###
###               01-Aug-2019                                                ###
###               11-Jan-2020                                                ###
###               09-May-2021 ; 19-Jun-2021                                  ###
###               10-May-2022                                                ###
################################################################################

# Function used to compute suitable cutting points to be used as separators of
# different colour classes used with rasterVIS::levelplot.
# This function is usually necessary when the distribution of x is highly skewed 
# (too many zeros) to find the 'cuts' values using the normal quantiles
# dec: decimals used to round each one of the output quantiles
.findcuts4plot <- function(x, n=8, thr=0, dec=0) {

  # Finding the minimum value in 'x'
  lmin <- round( min(x, na.rm = TRUE), dec)
  
  # Finding the quantile corresponding to the first value higher than 'thr'
  probs <- seq(0.01, 1, by=0.01)
  q     <- stats::quantile(x, probs=probs, na.rm=TRUE)
  index <- which(q > thr)
  qmin  <- index[1]/100
  
  # Finding n-1 quantiles that are higher than 'thr', only for 'x > thr'
  probs <- seq(qmin, 1, length.out = n-1)
  out   <- unique( round( stats::quantile(x[ x > thr ], probs=probs, na.rm=TRUE), dec) )

  # Adding 'lmin'
  out <- c( lmin, out )

  out <- unique(out)

  # cheking length of the output
  if (length(out) < n)
    out <- seq(from=lmin, to=max(out, na.rm=TRUE), length.out=n)
  
  # Creating the output object
  return( out )
  
} # '.findcuts4plot' END


# ## Example of use: Plot financial data

# plot <- FALSE
# ## This code is not run.
# if (plot) {

# #create faux data; skip this to use data from a file or stock data
# #ndays <- 1500 #set number of days
# #dates <- as.POSIXlt(seq(Sys.Date()- ndays, Sys.Date() - 1, by="days"))
# #vals <- runif(ndays, -100, 100)

# #stock data:
# stock      <- "MSFT"
# start.date <- "2010-01-12"
# end.date   <- Sys.Date()
# quote      <- paste("http://ichart.finance.yahoo.com/table.csv?s=",
#                 stock,
#                 "&a=", substr(start.date,6,7),
#                 "&b=", substr(start.date, 9, 10),
#                 "&c=", substr(start.date, 1,4),
#                 "&d=", substr(end.date,6,7),
#                 "&e=", substr(end.date, 9, 10),
#                 "&f=", substr(end.date, 1,4),
#                 "&g=d&ignore=.csv", sep="")
# stock.data <- read.csv(quote, as.is=TRUE)

# # Plot as calendar heatmap
# calendarHeat(stock.data$Date, stock.data$Adj.Close, varname="MSFT Adjusted Close")


# }

# File monthly2annual.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                               monthly2annual                                 #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 13-May-2009                                                         #
################################################################################
# This function transform a MONTHLY regular time series into an ANNUAL one .
# It is only a wrapper to 'daily2annual' and it was not merged as unique function just for clarity

# 'x'      : Monthly zoo object that will be converted into annual ones.
#            class(x) must be 'zoo'
# 'FUN'    : Function that have to be applied for transforming from daily to monthly time step
#            For precipitation FUN MUST be "sum"
#            For temperature and flow time series, FUN MUST be "mean"
# 'na.rm'  : Logical. Should missing values be removed?
#            TRUE : the monthly and annual values  are computed considering only those values different from NA
#            FALSE: if there is AT LEAST one NA within a year, the monthly and annual values are NA
# 'out.fmt': date format for the output time series. Possible values are:
#            -) "%Y"      : only the year will be used for the time. Default option. (e.g., "1961" "1962"...)
#            -) "%Y-%m-%d": a complete date format will be used for the time. Default option. (e.g., "1961" "1962"...)

monthly2annual <-function(x,...) UseMethod("daily2annual")

#monthly2annual.default <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",...) {


#daily2annual.default(x, FUN, na.rm=na.rm, out.fmt=out.fmt )

#} # 'monthly2annual.default' end


#monthly2annual.data.frame <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",
                                      #dates, date.fmt="%Y-%m-%d",
				      #out.type="data.frame",
				      #verbose=TRUE,...) {

#daily2annual.data.frame(x, FUN, na.rm=na.rm, out.fmt=out.fmt,
                          #dates=dates, date.fmt=date.fmt, out.type=out.type,
                          #verbose=verbose )

#} # 'monthly2annual.data.frame' END

# File ma.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2010-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                               'ma'                                           #
################################################################################
# Generic Moving (sliding) Average function                                    #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 2008
# Updates: 
################################################################################
ma <-function(x, ...) UseMethod("ma")


################################################################################
# Default Moving (sliding) Average function                                    #
################################################################################
# A vector with the moving average computed using a window of 'win.len' elements

# x      : time series with n elements
# win.len: number of terms that will be considered in the mean. It have to be odd

# result : a vector with the moving average termns. The length of the resulting vector
#          is the same of 'x', but the first and last (win.len-1)/2 elements will
#          be NA.

ma.default <- function (x, win.len, FUN=mean,...) {

  if (ceiling(win.len)/2 != win.len/2)
      stop("Invalid argument: 'win.len' must be of odd")

  return( filter(x, rep(1/win.len, win.len), method="convolution", sides=2) )

} # 'ma.default' end


################################################################################
# 	ma.zoo:	Moving Average of a DAILY regular time series,                 #
#           by default using a window width =365 (Annual Average)              #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 2008                                                                #
# Updates: 29-May-2013                                                         #
################################################################################

# 'x'       : zoo variable
# 'win.len' : window width. It have to be odd
# 'FUN'     : Function that have to be applied for computing the moving average. Usually, FUN MUST be "mean"

# 'result'  : a vector with the moving average termns. The length of the resulting vector
#             is less than the length of 'x', with (win.len-1)/2 missing elements at the begining and end of 'x'
ma.zoo <- function(x, win.len, FUN=mean,... ) {

  if (ceiling(win.len)/2 != win.len/2)
      stop("Invalid argument: 'win.len' must be of odd")

  # Generating an Moving Average time series, with a window width win.len1
  return ( rollapply(x, width=win.len, FUN, by.colum=FALSE) ) # zoo::rollapply

} # 'ma.zoo' end

# File fdcu.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# fdcu: Flow Duration Curve with Uncertainty Bounds                            #
################################################################################
# Purpose: Plot the flow Duration Curve in the original time units of 'x' and  #
#          also gives the probability of exceedence of each element            #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: January 29th, 2010                                                  #
# Updates: February 04th, 2010                                                 #
#          03-May-2012                                                         #
################################################################################

# 'x'            : 'numeric', 'matrix' or 'data.frame' whose columns contains the values of the time series of observed streamflows for which the flow duration curve will be computed.
# 'lband'        : 'numeric', 'matrix' or 'data.frame' whose columns contains the values of the time series with the lower uncertainty bound of 'x'.
# 'uband'        : 'numeric', 'matrix' or 'data.frame' whose columns contains the values of the time series with the upper uncertainty bound of 'x'.
# 'sim'          : 'numeric', 'matrix' or 'data.frame' whose columns contains the values of the time series with the simulated values of 'x', for which the flow duration curve will be computed.
# 'lQ.thr'       : 'numeric', low flows separation threshold. If this value is different from 'NA', a vertical line is drawn in this value, and all the values to the left of it are deemed low flows.
# 'hQ.thr'       : 'numeric', high flows separation threshold. If this value is different from 'NA', a vertical line is drawn in this value, and all the values to the right of it are deemed high flows
# 'plot'         : 'logical'. Indicates if the flow duration curve should be plotted or not
# 'log'          : 'character', indicates which axis has to be plotted with a logarithmic scale. By default is 'y'
# 'main'         : See '?plot'. An overall title for the plot: see 'title'.
# 'xlab'         : See '?plot'. A title for the x axis: see 'title'.
# 'ylab'         : See '?plot'. A title for the y axis: see 'title'.
# 'ylim'         : See '?plot.default'.  The y limits of the plot.
# 'col'          : See '?plot.default'. The colors for lines and points.  Multiple colors can be specified so that each point can be given its own color.  If there are fewer colors than points they are recycled in the standard fashion. Lines will all be plotted in the first colour specified.
# 'pch'          : See '?plot.default'. A vector of plotting characters or symbols: see 'points'.
# 'lwd'          : See '?plot.default'. The line width, see 'par'.
# 'lty'          : See '?plot.default'. The line type, see 'par'.
# 'cex'          : See '?plot.default'. A numerical vector giving the amount by which plotting characters and symbols should be scaled relative to the default.  This works as a multiple of 'par("cex")'. 'NULL' and 'NA' are equivalent to '1.0'.  Note that this does not affect annotation
# 'leg.txt'      : vector with the names that have to be used for each column of 'x'.
# 'verbose'      : logical; if TRUE, progress messages are printed
# 'thr.shw'      : logical, indicating if the stremflow values corresponding to the user-defined thresholds 'lQ.thr' and 'hQ.thr' have to be shown in the plot.
# 'border'       : See '?polygon'. The color to draw the border of the polygon with the uncertainty bounds. The default, 'NA', means to omit borders.
# 'bands.col'    : See '?polygon'. The color for filling the polygon. The default, 'NA', is to leave polygons unfilled, unless 'density' is specified. If 'bands.density' is specified with a positive value this gives the color of the shading lines.
# 'bands.density': See '?polygon'. The density of shading lines for the polygon with the uncertainty bounds, in lines per inch.  The default value of 'NULL' means that no shading lines are drawn. A zero value of 'bands.density' means no shading nor filling whereas negative values (and 'NA') suppress shading (and so allow color filling).
# 'bands.angle'  : See '?polygon'. The slope of shading lines for the polygon with the uncertainty bounds, given as an angle in degrees (counter-clockwise).
# 'new'          : logical, if TRUE, a new plotting window is created.

fdcu <-function(x, lband, uband, ...) UseMethod("fdcu")

fdcu.default <- function (x,
                          lband,
                          uband,
                          sim=NULL,
                          lQ.thr=0.7,
                          hQ.thr=0.2,
                          plot=TRUE,
                          log="y",
                          main="Flow Duration Curve",
                          xlab="% Time flow equalled or exceeded",
                          ylab="Q, [m3/s]",
                          ylim,
                          yat=c(0.01, 0.1, 1), 
                          xat=c(0.01, 0.025, 0.05),
                          col=c("black", "red"),
                          pch=c(1, 15),
                          lwd=c(1, 0.8),
                          lty=c(1, 3),
                          cex=0.2,
                          cex.axis=1.2,
                          cex.lab=1.2,
                          leg.txt= c("Qobs", "Qsim", "95PPU"),
                          leg.cex=1,
                          leg.pos="auto",
                          verbose= TRUE,
                          thr.shw=TRUE,
                          border=NA,
                          bands.col="lightcyan",
                          bands.density=NULL,
                          bands.angle=45,
                          new=TRUE,
                          ...) {


     #checking that the user provided 'x'
     if (missing(x))  stop("Missing argument: 'x' must be provided !")

     #checking that the user provided 'lband'
     if (missing(lband))  stop("Missing argument: 'lband' must be provided !")

     #checking that the user provided 'uband'
     if (missing(uband))  stop("Missing argument: 'uband' must be provided !")

     # Function for finding the position of 'Q' within 'x'
     Qposition <- function(x, Q) {
       Q.dist  <- abs(x - Q)
       Q.index <- which.min( Q.dist )
       return(Q.index)
     } # end

	 # In case 'x', 'laband' and 'uband' or 'sim' be of 'zoo' or 'ts' class
     x     <- as.numeric(x)
	 lband <- as.numeric(lband)
	 uband <- as.numeric(uband)
     if (!missing(sim)) { sim <- as.numeric(sim)   }

     # Removing zero values when using a logarithmic scale
     if (log == "y") {

       x.zero.index     <- which(x==0)
       lband.zero.index <- which(lband==0)
       uband.zero.index <- which(uband==0)
       if (!missing(sim)) { sim.zero.index <- which(sim==0) }

       if (length(x.zero.index) > 0 ) {
        x <- x[-x.zero.index]
        if (verbose) message("[Warning: all 'x' equal to zero will not be plotted]" )
       } # IF end

       if (length(lband.zero.index) > 0 ) {
        lband <- lband[-lband.zero.index]
        if (verbose) message("[Warning: all 'lband' equal to zero will not be plotted]" )
       } # IF end

       if (length(uband.zero.index) > 0 ) {
        uband <- uband[-uband.zero.index]
        if (verbose) message("[Warning: all 'uband' equal to zero will not be plotted]" )
       } # IF end

       if (!is.null(sim)) {
            if (length(sim.zero.index) > 0 ) {
            sim <- sim[-sim.zero.index]
            if (verbose) message("[Warning: all 'sim' equal to zero will not be plotted]" )
           } # IF end
       } # IF end

     } # IF end

	 # 1) Sort 'x' in decreasing order. This is just for avoiding misleading
	 #lines when using 'type="o"' for plotting
	 x.sort     <- sort(x, decreasing=TRUE)
	 lband.sort <- sort(lband, decreasing=TRUE)
	 uband.sort <- sort(uband, decreasing=TRUE)
     if (!is.null(sim)) { sim.sort <- sort(sim, decreasing=TRUE) }

	 fdc.x     <- fdc(x.sort, log=log, plot=FALSE,...)
	 fdc.lband <- fdc(lband.sort, log=log, plot=FALSE)
	 fdc.uband <- fdc(uband.sort, log=log, plot=FALSE)
     if (!is.null(sim)) { fdc.sim <- fdc(sim.sort, log=log, plot=FALSE) }

	 #na.index <- which(is.na(x))

     # Avoiding plotting the uncertainty bands for the Na's
     #uband[na.index] <- uband[na.index-1]
     #lband[na.index] <- lband[na.index+1]

     if (plot) {

         if ( missing(ylim) ) {
          ylim <- range(lband, uband, na.rm=TRUE)
         } else {
             # In case 'ylim' is 'NA'
             if ( is.na(ylim[1]) ) { ylim[1] <- range(x, na.rm=TRUE)[1] }
             if ( is.na(ylim[2]) ) { ylim[2] <- range(x, na.rm=TRUE)[2] }
           } # ELSE end

         # Logarithmic scale can NOT start in zero
         if (log=="y") { ylim <- c(0.01, ylim[2]) }

         # Creating the 'x' values of the polygons of the bands
         t <- c( fdc.lband, rev(fdc.uband) )

         # Creating the 'y' values of the polygons of the bands
         bands <- c( lband.sort, rev(uband.sort) )

         if (new) {
             # Creating the plot, but without anything on it, for allowing the call to polygon            
             plot(fdc.x, x.sort, type="n", xaxt = "n", yaxt = "n", main=main, xlab=xlab, ylab=ylab, log=log, ylim=ylim, ...)             
         } # IF end

         # Plotting the polygons between the lower and upper bands
         polygon(t, bands, col=bands.col, density=bands.density, angle=bands.angle, border=border)

         # Ploting the OBServations over the polygon
         lines(fdc.x, x.sort, cex=cex, col=col[1], pch=pch[1], lwd=lwd[1], lty=lty[1])
         points(fdc.x, x.sort, cex=cex, col=col[1], pch=pch[1])

         # Ploting the SIMulated values over the polygon
         if (!is.null(sim)) {
           lines(fdc.sim, sim.sort, cex=cex, col=col[2], pch=pch[2], lwd=lwd[2], lty=lty[2])
           points(fdc.sim, sim.sort, cex=cex, col=col[2], pch=pch[2])
         } # IF end

         # Drawing the ticks and labels corresponding to the Y axis 
          ylabels <- pretty(ylim)
          if ( (log=="y") | (log=="xy") | (log=="yx") ) {            
            ylabels <- union( yat, ylabels )            
          } # IF end
          Axis( side = 2, at =ylabels, cex.axis=cex.axis, labels = ylabels)
          
          # Drawing the ticks and labels corresponding to the X axis
          xpos    <- seq(0.0, 1, by=0.05)
          xlabels <- seq(0.0, 1, by=0.1)
          if ( (log=="x") | (log=="xy") | (log=="yx") ) {            
            xpos    <- union( xat, xpos ) 
            xlabels <- union( xat, xlabels )            
          } # IF end
          Axis(side = 1, at = xpos, cex.axis=cex.axis, labels = FALSE)
          Axis(side = 1, at = xlabels, cex.axis=cex.axis, labels = paste(100*xlabels,"%", sep="") )

         # Drawing a legend. bty="n" => no border
         if (!is.null(leg.txt)) {

             # Drawing a legend. bty="n" => no border
             if (leg.pos=="auto")
               ifelse (log=="y", leg.pos.x <- "bottomleft", leg.pos.x <- "topright")

             if (!is.null(sim)) { #Legend with the OBS + simulations + 95PPU
              legend(x=leg.pos.x, legend=leg.txt,  #inset=0.03,
                     bty="n", cex =leg.cex, col=c(col[1], col[2], bands.col), 
                     lwd=c(lwd[1], lwd[2], 0), lty=c(lty[1], lty[2], 0), 
                     pch=c(NA,NA,15), pt.cex=3)
             } else { #Legend only with the OBS + 95PPU
              legend(x=leg.pos.x, legend=c(leg.txt[1], leg.txt[3]),  #inset=0.03,
                     bty="n", cex =leg.cex, col=c(col[1], bands.col), lwd=c(lwd[1], 0), 
                     lty=c(lty[1], 0), pch=c(NA,15), pt.cex=3)
             }# IF end

         } # IF end

          # If the user provided a value for 'lQ.thr', a vertical line is drawn
         if ( !is.na(lQ.thr) ) {
            abline(v=lQ.thr, col="grey", lty=3, lwd=2)
         } # IF end

         # If the user provided a value for 'hQ.thr', a vertical line is drawn
         if ( !is.na(hQ.thr) ) {
            abline(v=hQ.thr, col="grey", lty=3, lwd=2)
         } # IF end

         # If the user want to see the Q values corresponding to 'lQ.thr' and 'hQ.thr'
         if (thr.shw) {              
            # Drawing a legend. bty="n" => no border
            if (leg.pos=="auto")
              ifelse (log=="y", leg.pos.x <- "topright", leg.pos.x <- "bottomleft")

            # Finding the flow values corresponding to the 'lQ.thr' and 'hQ.thr' pbb of excedence
            x.lQ <- x.sort[Qposition(fdc.x, lQ.thr)]
            x.hQ <- x.sort[Qposition(fdc.x, hQ.thr)]

            legend(x=leg.pos.x, 
                   legend=c(paste("Qhigh.thr=", round(x.hQ, 2), sep=""),
                            paste("Qlow.thr=", round(x.lQ, 2), sep="") ),
                   bty="n", cex=leg.cex ) #bty="n" => no box around the legend
         } # IF end

     } # IF 'plot' end

	 #tmp <- cbind(fdc.lband, fdc.x, fdc.uband)
	 #colnames(tmp) <- c("fdc.l", "fdc.x", "fdc.u")

	 #return(tmp)

} # 'fdcu.default' END



######################################################################
# fdcu.matrix: (ONLY) Plots of Multiple Flow Duration Curves,        #
#                  with uncertainty bands                            #
######################################################################
# Started    :  January 29th, 2010                                   #
# Last updated:  April 06th, 2010                                    #
######################################################################

fdcu.matrix <- function (x,
                         lband,
                         uband,
                         sim=NULL,
                         lQ.thr=0.7,
                         hQ.thr=0.2,
                         plot=TRUE,
                         log="y",
                         main="Flow Duration Curve",
                         xlab="% Time flow equalled or exceeded",
                         ylab="Q, [m3/s]",
                         ylim,
                         yat=c(0.01, 0.1, 1), 
                         xat=c(0.01, 0.025, 0.05),
                         col=matrix(c(rep("black", ncol(x)), 
                                    palette("default")[2:(ncol(x)+1)]), byrow=FALSE, ncol=2),
                         pch=matrix(rep(c(1, 15), ncol(x)), byrow=TRUE, ncol=2),
                         lwd=matrix(rep(c(1, 0.8), ncol(x)), byrow=TRUE, ncol=2),
                         lty=matrix(rep(c(1, 3), ncol(x)), byrow=TRUE, ncol=2),
                         cex=rep(0.1, ncol(x)),
                         cex.axis=1.2,
                         cex.lab=1.2,
                         leg.txt=c("OBS", colnames(x), "95PPU"),
                         leg.cex=1,
                         leg.pos="auto",
                         verbose= TRUE,
                         thr.shw=TRUE,
                         border=rep(NA, ncol(x)),
                         bands.col=rep("lightcyan", ncol(x)),
                         bands.density=rep(NULL, ncol(x)),
                         bands.angle=rep(45, ncol(x)),
                         new=TRUE,
                         ...) {
  n <- ncol(x)

  if ( missing(ylim) ) {
    ylim <- range(lband, uband, na.rm=TRUE)
  } else {
         # In case 'ylim' is 'NA'
         if ( is.na(ylim[1]) ) { ylim[1] <- range(x, na.rm=TRUE)[1] }
         if ( is.na(ylim[2]) ) { ylim[2] <- range(x, na.rm=TRUE)[2] }
    } # ELSE end

  # Logarithmic scale can NOT start in zero
  if (log=="y") { ylim <- c(0.01, ylim[2]) }



  j <- 1 # starting column for the analysis

  if (verbose) message( paste("Column: ", format(j, width=10, justify="left"),
                      " : ", format(j, width=3, justify="left"), "/",
                      n, " => ",
                      format(round(100*j/n,2), width=6, justify="left"),
                      "%", sep="") )

  # Computing and plotting the Flow duration Curve for the first column
  if (is.null(sim)) {

      sim.exists <- FALSE
      fdcu(x=x[,1], lband=lband[,1], uband=uband[,1], lQ.thr=lQ.thr, hQ.thr=hQ.thr,
           plot=TRUE, main=main, xlab= xlab, ylab=ylab, ylim=ylim, yat=yat, xat=xat, log=log, col=col[1,], pch=pch[1,],
           lwd=lwd[1,], lty=lty[1,], cex=cex[1], cex.axis=cex.axis, cex.lab=cex.lab, 
           verbose, leg.txt=NULL, leg.cex=leg.cex, leg.pos=leg.pos, thr.shw=thr.shw, border=border[1],
           bands.col= bands.col[1], bands.density=bands.density[1], bands.angle=bands.angle[1], new=TRUE, ...)
  } else {

    sim.exists <- TRUE
    fdcu(x=x[,1], lband=lband[,1], uband=uband[,1], sim=sim[,1], lQ.thr=lQ.thr, hQ.thr=hQ.thr,
           plot=TRUE, main=main, xlab= xlab, ylab=ylab, ylim=ylim, yat=yat, xat=xat, log=log, col=col[1,], pch=pch[1,],
           lwd=lwd[1,], lty=lty[1,], cex=cex[1], cex.axis=cex.axis, cex.lab=cex.lab,
           verbose, leg.txt=NULL, leg.cex=leg.cex, leg.pos=leg.pos, thr.shw=thr.shw, border=border[1],
           bands.col= bands.col[1], bands.density=bands.density[1], bands.angle=bands.angle[1], new=TRUE, ...)
    }

  # Plotting the Flow Duration Curves
  sapply(2:n, function(j) {

         if (verbose) message( paste("Column: ", format(j, width=10, justify="left"),
                             " : ", format(j, width=3, justify="left"), "/",
                             n, " => ",
                             format(round(100*j/n,2), width=6, justify="left"),
                             "%", sep="") )

            if (!sim.exists) {
              # Computing and plotting the Flow duration Curve for the first vector
              fdcu(x=x[,j], lband=lband[,j], uband=uband[,j], lQ.thr=lQ.thr, hQ.thr=hQ.thr,
                   plot=TRUE, main=main, xlab= xlab, ylab=ylab, ylim=ylim, yat=yat, xat=xat, log=log, col=col[j,], pch=pch[j,],
                   lwd=lwd[j,], lty=lty[j,], cex=cex[j], cex.axis=cex.axis, cex.lab=cex.lab, 
                   verbose, leg.txt=NULL, leg.cex=leg.cex, leg.pos=leg.pos, thr.shw=FALSE, border=border[j],
                   bands.col= bands.col[j], bands.density=bands.density[j], bands.angle=bands.angle[j], new=FALSE, ...)
            } else {
              fdcu(x=x[,j], lband=lband[,j], uband=uband[,j], sim=sim[,j], lQ.thr=lQ.thr, hQ.thr=hQ.thr,
                   plot=TRUE, main=main, xlab= xlab, ylab=ylab, ylim=ylim, yat=yat, xat=xat, log=log, col=col[j,], pch=pch[j,],
                   lwd=lwd[j,], lty=lty[j,], cex=cex[j], cex.axis=cex.axis, cex.lab=cex.lab, 
                   verbose, leg.txt=NULL, leg.cex=leg.cex, leg.pos=leg.pos, thr.shw=FALSE, border=border[j],
                   bands.col= bands.col[j], bands.density=bands.density[j], bands.angle=bands.angle[j], new=FALSE, ...)
              } # ELSE end
    } )

    if (verbose) message("Re-plotting the 'sim' lines")

    # Re-Plotting the lines that were overdrawn by the polygons
    sapply(1:n, function(j) {

            # Ploting the line with observations over the uncertainty polygon
            tmp <- sort(x[,j], decreasing=TRUE)
            if (log == "y") {
               x.zero.index <- which(tmp==0)
               if (length(x.zero.index) > 0 ) {
                tmp <- tmp[-x.zero.index]
               } # IF end
            } # IF end

            tmp.fdc <- fdc(tmp, plot=FALSE, log=log)
            lines( tmp.fdc, tmp, cex=cex, col=col[j, 1], pch=pch[j, 1], lwd=lwd[j, 1], lty=lty[j, 1] )
            points(tmp.fdc, tmp, cex=cex, col=col[j, 1], pch=pch[j, 1])

            # Ploting the lines with simulations over the uncertainty polygon
            if (sim.exists) {
              tmp <- sort(sim[,j], decreasing=TRUE)
              if (log == "y") {
               x.zero.index <- which(tmp==0)
               if (length(x.zero.index) > 0 ) {
                tmp <- tmp[-x.zero.index]
               } # IF end
             } # IF end
            tmp.fdc <- fdc(tmp, plot=FALSE, log=log)
            lines(tmp.fdc, tmp, cex=cex, col=col[j, 2], pch=pch[j, 2], lwd=lwd[j, 2], lty=lty[j, 2])
            points(tmp.fdc, tmp, cex=cex, col=col[j, 2], pch=pch[j, 2])
            }
    } ) # sapply end

    # Drawing a legend. bty="n" => no border
    if (!is.null(leg.txt)) {

        # Legend with bold font
        par(font=2)

        # Drawing a legend. bty="n" => no border
        if (leg.pos=="auto")
          ifelse (log=="y", leg.pos.x <- "bottomleft", leg.pos.x <- "topright")

        if (!is.null(sim)) { #Legend with the OBS + SIMs + 95PPU
          legend(x=leg.pos.x, legend=leg.txt,  inset=0.01,
                 bty="n", cex=leg.cex, col=c(col[1,1], col[,2], bands.col), 
                 lwd=c(3*lwd[1,1], 3*lwd[,2], 0), lty=c(lty[1,1], lty[,2], 0),
                 pch=c(rep(NA, (ncol(x)+1)), 15), pt.cex=2.5)

        } else { #Legend only with the OBS + 95PPU
         legend(x=leg.pos.x, legend=c(leg.txt[1], leg.txt[3]),  inset=0.01,
                bty="n", cex=leg.cex, col=c(col[1,1], bands.col), 
                lwd=c(3*lwd[1,1], 0),  lty=c(lty[1,1],0), pch=c(NA,15), pt.cex=2.5)
        }# IF end

    } # IF end


} # 'fdcu.matrix' END


fdcu.data.frame <- function(x,
                         lband,
                         uband,
                         sim=NULL,
                         lQ.thr=0.7,
                         hQ.thr=0.2,
                         plot=TRUE,
                         log="y",
                         main="Flow Duration Curve",
                         xlab="% Time flow equalled or exceeded",
                         ylab="Q, [m3/s]",
                         ylim,
                         yat=c(0.01, 0.1, 1), 
                         xat=c(0.01, 0.025, 0.05),
                         col=matrix(c(rep("black", ncol(x)), 
                                    palette("default")[2:(ncol(x)+1)]), byrow=FALSE, ncol=2),
                         pch=matrix(rep(c(1, 15), ncol(x)), byrow=TRUE, ncol=2),
                         lwd=matrix(rep(c(1, 0.8), ncol(x)), byrow=TRUE, ncol=2),
                         lty=matrix(rep(c(1, 3), ncol(x)), byrow=TRUE, ncol=2),
                         cex=rep(0.1, ncol(x)),
                         cex.axis=1.2,
                         cex.lab=1.2,
                         leg.txt=c("OBS", colnames(x), "95PPU"),
                         leg.cex=1,
                         leg.pos="auto",
                         verbose= TRUE,
                         thr.shw=TRUE,
                         border=rep(NA, ncol(x)),
                         bands.col=rep("lightcyan", ncol(x)),
                         bands.density=rep(NULL, ncol(x)),
                         bands.angle=rep(45, ncol(x)),
                         new=TRUE,
                         ...) {

   x <- as.data.frame(x)

   if ( missing(ylim) ) {
    ylim <- range(lband, uband, na.rm=TRUE)
   } else {
       # In case 'ylim' is 'NA'
       if ( is.na(ylim[1]) ) { ylim[1] <- range(x, na.rm=TRUE)[1] }
       if ( is.na(ylim[2]) ) { ylim[2] <- range(x, na.rm=TRUE)[2] }
     } # ELSE end

   # Logarithmic scale can NOT start in zero
   if (log=="y") { ylim <- c(0.01, ylim[2]) }


   NextMethod("fdcu", x,
                lband,
                uband,
                sim,
                lQ.thr=lQ.thr,
                hQ.thr=hQ.thr,
                plot=plot,
                log=log,
                main=main,
                xlab=xlab,
                ylab=ylab,
                ylim=ylim,
                yat=yat,
                xat=xat,
                col=col,
                pch=pch,
                lty=lty,
                cex=cex,
                cex.axis=cex.axis, 
                cex.lab=cex.lab,
                leg.txt=leg.txt,
                leg.cex=leg.cex, 
                leg.pos=leg.pos,
                verbose= verbose,
                border=border,
                bands.col=bands.col,
                bands.density=bands.density,
                bands.angle=bands.angle,
                ...)

} # 'fdcu.data.frame' END

# ################################################################################
# # fdc.zoo: Computation and/or Plot of Multiple Flow Duration Curves,           #
# #             mainly for comparison                                            #
# ################################################################################
# # Author : Mauricio Zambrano-Bigiarini                                         #
# # Started: 03-Aug-2023                                                         #
# # Updates:                                                                     #
# ################################################################################
# fdcu.zoo <- function (x,
#                       lband,
#                       uband,
#                       sim=NULL,
#                       lQ.thr=0.7,
#                       hQ.thr=0.2,
#                       plot=TRUE,
#                       log="y",
#                       main="Flow Duration Curve",
#                       xlab="% Time flow equalled or exceeded",
#                       ylab="Q, [m3/s]",
#                       ylim,
#                       yat=c(0.01, 0.1, 1), 
#                       xat=c(0.01, 0.025, 0.05),
#                       col=matrix(c(rep("black", NCOL(x)), 
#                                  palette("default")[2:(NCOL(x)+1)]), byrow=FALSE, ncol=2),
#                       pch=matrix(rep(c(1, 15), NCOL(x)), byrow=TRUE, ncol=2),
#                       lwd=matrix(rep(c(1, 0.8), NCOL(x)), byrow=TRUE, ncol=2),
#                       lty=matrix(rep(c(1, 3), NCOL(x)), byrow=TRUE, ncol=2),
#                       cex=rep(0.1, NCOL(x)),
#                       cex.axis=1.2,
#                       cex.lab=1.2,
#                       leg.txt=c("OBS", colnames(x), "95PPU"),
#                       leg.cex=1,
#                       leg.pos="auto",
#                       verbose= TRUE,
#                       thr.shw=TRUE,
#                       border=rep(NA, NCOL(x)),
#                       bands.col=rep("lightcyan", NCOL(x)),
#                       bands.density=rep(NULL, NCOL(x)),
#                       bands.angle=rep(45, NCOL(x)),
#                       new=TRUE,
#                       ...) {

#   x <- coredata(x)

#   if ( missing(ylim) ) {
#     ylim <- range(lband, uband, na.rm=TRUE)
#   } else {
#          # In case 'ylim' is 'NA'
#          if ( is.na(ylim[1]) ) { ylim[1] <- range(x, na.rm=TRUE)[1] }
#          if ( is.na(ylim[2]) ) { ylim[2] <- range(x, na.rm=TRUE)[2] }
#     } # ELSE end

#   # Logarithmic scale can NOT start in zero
#   if (log=="y") { ylim <- c(0.01, ylim[2]) }


#   if ( is.matrix(x) | is.data.frame(x) ) {  
#     fdcu.matrix(x,
#                 lband,
#                 uband,
#                 sim,
#                 lQ.thr=lQ.thr,
#                 hQ.thr=hQ.thr,
#                 plot=plot,
#                 log=log,
#                 main=main,
#                 xlab=xlab,
#                 ylab=ylab,
#                 ylim=ylim,
#                 yat=yat,
#                 xat=xat,
#                 col=col,
#                 pch=pch,
#                 lty=lty,
#                 cex=cex,
#                 cex.axis=cex.axis, 
#                 cex.lab=cex.lab,
#                 leg.txt=leg.txt,
#                 leg.cex=leg.cex, 
#                 leg.pos=leg.pos,
#                 verbose= verbose,
#                 border=border,
#                 bands.col=bands.col,
#                 bands.density=bands.density,
#                 bands.angle=bands.angle,
#                 new=new,
#                 ...)
#   } else fdc.default(x,
#                      lband,
#                      uband,
#                      sim,
#                      lQ.thr=lQ.thr,
#                      hQ.thr=hQ.thr,
#                      plot=plot,
#                      log=log,
#                      main=main,
#                      xlab=xlab,
#                      ylab=ylab,
#                      ylim=ylim,
#                      yat=yat,
#                      xat=xat,
#                      col=col,
#                      pch=pch,
#                      lty=lty,
#                      cex=cex,
#                      cex.axis=cex.axis, 
#                      cex.lab=cex.lab,
#                      leg.txt=leg.txt,
#                      leg.cex=leg.cex, 
#                      leg.pos=leg.pos,
#                      verbose= verbose,
#                      border=border,
#                      bands.col=bands.col,
#                      bands.density=bands.density,
#                      bands.angle=bands.angle,
#                      new=new,
#                      ...)
  
  
# } # 'fdc.zoo' END

# File climograph.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
#                                 http://www.rforge.net/hydroTSM/
# Copyright 2016-2022 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# climograph: Function for drawing a climograph based on precipitation and     #
#             temperature data.                                                #
#             Precipitation and temperature data used to build the climograph  #
#             should have a monthly time frequency. If the data provided by    #
#             have a time frequency higher than monthly (i.e., daily, subdaily)#
#             the function compute the monthly mean values and then draw the   #
#             climograph                                                       #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 29-Jun-2016                                                         #
# Updates: 30-Jun-2016 ; 04-Jul-2016                                           # 
#          08-May-2017 ; 09-May-2017                                           #
#          10-Mar-2020 ; 07-Nov-2020                                           #
#          May-2022    ; 20-Jun-2022 ; 22-Aug-2022 ; 05-Oct-2022               #
#          25-May-2023 ; 26-May-2023 ; 27-May-2023                             #
################################################################################
# 'pcp'      : variable of type 'zoo' with monthly, daily or subdaily          
#              precipitation data
# 'tmean'    : variable of type 'zoo' with monthly, daily or subdaily          
#              mean temperature data
# 'tmx'      : variable of type 'zoo' with monthly, daily or subdaily          
#              maximum temperature data. 
#              ONLY used (togheter with 'tmn') when 'tmean' is missing
# 'tmn'      : variable of type 'zoo' with monthly, daily or subdaily          
#              minimum temperature data. 
#              ONLY used (togheter with 'tmx') when 'tmean' is missing
# 'date.fmt' : format in which the dates are stored in 'from' and 'to'.
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the monthly values  are computed considering only those values in 'x' different from NA
#              FALSE: if there is AT LEAST one NA within a month, the FUN and monthly values are NA
# 'pcp.solid.thr' :[OPTIONAL]. Only used when using (sub)daily precipitation and 
#                   temperature are gives as input data. \cr
#                  numeric, indicating the temperature, in degrees Celsius, used to 
#                  discriminate between solid and liquid precipitation. \cr
#                  When daily \code{tmean <= pcp.solid.thr} the precipitation for that
#                  day is considered as solid precipitation.

# 'pcp.labels'  : logical. Should monthly precipitation values to be shown above the bars?
# 'tmean.labels': logical. Should monthly mean temperature values to be shown above the lines?
# 'tmx.labels'  : logical. Should monthly maximum temperature values to be shown above the lines?
# 'tmn.labels'  : logical. Should monthly minimum temperature values to be shown above the lines?

climograph <- function(pcp, tmean, tmx, tmn, na.rm=TRUE, 
                       from, to, date.fmt="%Y-%m-%d", 
                       main="Climograph", 
                       pcp.label="Precipitation, [mm]", 
                       tmean.label="Air temperature, [\U00B0 C]",
                       start.month=1,

                       pcp.solid.thr, 

                       pcp.ylim, # if provided, used to define the range of the P axis
                       temp.ylim,# if provided, used to define the range of the Temp axis

                       pcp.col="lightblue", 
                       pcp.solid.col="skyblue2",
                       tmean.col="darkred",
                       tmn.col="blue",
                       tmx.col="red",

                       pcp.labels=TRUE,
                       tmean.labels=TRUE,
                       tmx.labels=TRUE,
                       tmn.labels=TRUE,
                       pcp.labels.cex=0.8,
                       temp.labels.cex=0.8,

                       pcp.labels.dx=c(rep(ifelse(plot.pcp.probs, -0.25,  0.0),6), 
                                       rep(ifelse(plot.pcp.probs, -0.25,  0.0),6)),
                       pcp.labels.dy=rep(2, 12),
                       temp.labels.dx=c(rep(-0.2,6), rep(0.2,6)),
                       temp.labels.dy=rep(-0.4, 12),

                       plot.pcp.probs=TRUE,
                       pcp.probs=c(0.25, 0.75),

                       plot.temp.probs=TRUE,
                       temp.probs=c(0.25, 0.75),

                       temp.probs.col=c("#3399FF", "#FF9966", "#FFCC66"), # color of tmn, tmean, tmx
                       temp.probs.alpha=0.3,

                       lat, # [OPTIONAL] numeric or character used to show the latitude for which the climograph was plotted for
                       lon  # [OPTIONAL] numeric or character used to show the longitude for which the climograph was plotted for
                       ) {


  .plotbands <- function(x, lband, uband, col="", border=NA) {
    t <- c(x, rev(x))
    bands <- c(as.numeric(lband), rev(as.numeric(uband)))
    polygon(t, bands, col=col, border=border)
  } # .plotbands END


  .shift <- function(x, imonth) {
    L <- length(x)
    if (imonth>L) stop("[ Invalid value: 'imonth' can not be larger than ", L, " !]")
    delta <- imonth-1
    index.old <- 1:L
    index.new <- index.old-delta
    neg <- which(index.new <=0)
    index.new[neg] <- index.new[neg]+L
    if (inherits(x, "zoo")) {
      x.raw    <- zoo::coredata(x)
      x.labels <- as.character(time(x))
      out        <- x.raw[match(index.old, index.new)] 
      names(out) <- x.labels[match(index.old, index.new)] 
    } else out <- x[match(index.old, index.new)] 
    return(out)
  } # .shift END


  if (missing(pcp)) {
    stop("Missing argument: 'pcp' must be provided !")
  } else {
      # Checking that 'pcp' is a zoo object
      if ( !is.zoo(pcp) ) stop("Invalid argument: 'class(pcp)' must be in c('zoo', 'xts')")

      # Checking that 'pcp' is a subdaily or monthly object
      if ( sfreq(pcp) %in% c("quarterly", "annual")) stop("Invalid argument: 'sfreq(pcp)' must be in c('minute', 'hourly', 'daily', 'weekly')")
    } # ELSE end


  # Checking that 'tmean' is provided, and if not, it computes it using 'tmx' and 'tmn'  
  if (missing(tmean)) {
    if ( (missing(tmx)) & missing(tmn) ) {
      stop("Missing argument: 'tmean' | ('tmx' & 'tmn') must be provided !")
    } else {
        # Checking that 'tmx' and 'tmn' are zoo objects
        if ( !is.zoo(tmx) ) stop("Invalid argument: 'class(tmx)' must be in c('zoo', 'xts')")
        if ( !is.zoo(tmn) ) stop("Invalid argument: 'class(tmn)' must be in c('zoo', 'xts')")

        # Checking that 'tmx' is a subdaily or monthly object
        if ( sfreq(tmx) %in% c("quarterly", "annual")) stop("Invalid argument: 'sfreq(tmx)' must be in c('minute', 'hourly', 'daily', 'weekly')")
        # Checking that 'tmn' is a subdaily or monthly object
        if ( sfreq(tmn) %in% c("quarterly", "annual")) stop("Invalid argument: 'sfreq(tmn)' must be in c('minute', 'hourly', 'daily', 'weekly')")

        # Computing 'tmean'
        if ( all.equal(time(tmn), time(tmx)) ) {
          tmean <- (tmx+tmn)/2
        } else stop("Invalid argument: 'time(tmn) != time(tmx)' !")
      } # ELSE end
  } else {
      # Checking that 'tmean'is a zoo object
      if ( !is.zoo(tmean) ) stop("Invalid argument: 'class(tmean)' must be in c('zoo', 'xts')")

      # Checking that 'tmean' is a subdaily or monthly object
      if ( sfreq(tmean) %in% c("quarterly", "annual")) stop("Invalid argument: 'sfreq(tmean)' must be in c('minute', 'hourly', 'daily', 'weekly')")
    } # ELSE end


  # Computing (sub)daily solid precipitation, only when 'pcp.solid.thr' is provided
  pcp.solid.exists <- FALSE
  if ( !missing(pcp.solid.thr) & !(sfreq(pcp) %in% c("monthly", "quarterly", "annual") ) & 
       !(sfreq(tmean) %in% c("monthly", "quarterly", "annual") ) ) {
  
    pcp.solid.exists       <- TRUE
    pcp.solid              <- pcp*NA # it keeps the time and all the elements are made equal to NA
    solid.index            <- which(tmean <= pcp.solid.thr)
    pcp.solid[solid.index] <- pcp[solid.index]
  } # IF end


  # Checking the length of 'temp.labels.dx' and 'temp.labels.dy'
  if (length(temp.labels.dx) > 12) temp.labels.dx <- temp.labels.dx[1:12]
  if (length(temp.labels.dy) > 12) temp.labels.dy <- temp.labels.dy[1:12]

  # checking 'pcp.ylim', if provided 
  if (!missing(pcp.ylim)) {
    if (length(pcp.ylim) != 2) stop("Invalid argument: 'length(pcp.ylim)' must be 2 !")
    if (pcp.ylim[2] <= 1) stop("Invalid argument: 'pcp.ylim[2]' must be larger than pcp.ylim[1] !")
  } # IF end

  # checking 'temp.ylim', if provided 
  if (!missing(temp.ylim)) {
    if (length(temp.ylim) != 2) stop("Invalid argument: 'length(temp.ylim)' must be 2 !")
    if (temp.ylim[2] <= 1) stop("Invalid argument: 'temp.ylim[2]' must be larger than temp.ylim[1] !")
  } # IF end

  ###########################################
  ## In case 'from' and 'to' are provided  ##
  dates.pcp  <- time(pcp)
  dates.temp <- time(tmean)
     
  # Checking the validity of the 'from' argument
  if (!missing(from)) { 
     from <- as.Date(from, format=date.fmt)

     if (from < dates.pcp[1])
       stop("Invalid argument: 'from' is lower than the first date in 'pcp' !")
     if (from < dates.temp[1])
       stop("Invalid argument: 'from' is lower than the first date in 'tmean' !")

     pcp   <- window(pcp  , start=from)
     tmean <- window(tmean, start=from)

     if (pcp.solid.exists) pcp.solid <- window(pcp.solid, start=from)
     if ( !missing(tmx) )  tmx       <- window(tmx      , start=from)
     if ( !missing(tmn) )  tmn       <- window(tmn      , start=from)
   } # ELSE end

  # Checking the validity of the 'to' argument
  if (!missing(to)) { 
     to <- as.Date(to, format=date.fmt)

     if (to > dates.pcp[length(pcp)])
       stop("Invalid argument: 'to' is greater than the last date in 'pcp' !")
     if (to > dates.temp[length(pcp)])
       stop("Invalid argument: 'to' is greater than the last date in 'tmean' !")

     pcp   <- window(pcp  , end=to)
     tmean <- window(tmean, end=to)

     if (pcp.solid.exists) pcp.solid <- window(pcp.solid, end=to)
     if ( !missing(tmx) )  tmx       <- window(tmx      , end=to)
     if ( !missing(tmn) )  tmn       <- window(tmn      , end=to)
   } # ELSE end


  # Detecting if 'pcp' and 'tmean', 'tmx' 'tmn' are already mean monthly values (i.e., 12 values)
  pcp.is.mean.monthly   <- FALSE
  tmean.is.mean.monthly <- FALSE
  tmx.is.mean.monthly   <- FALSE
  tmn.is.mean.monthly   <- FALSE

  if ( ( sfreq(pcp) == "monthly" ) & (length(pcp) == 12) ) {
    pcp.is.mean.monthly <- TRUE

    months     <- format(time(pcp), "%b")
    pcp        <- as.numeric(pcp)
    names(pcp) <- months

    if (start.month != 1)
      pcp <- .shift(x=pcp, imonth=start.month)

    pcp.m.avg <- pcp
    pcp.m.q1  <- pcp
    pcp.m.q2  <- pcp    
  } # IF enbd

  if ( ( sfreq(tmean) == "monthly" ) & (length(tmean) == 12) ) {
    tmean.is.mean.monthly <- TRUE

    months       <- format(time(tmean), "%b")
    tmean        <- as.numeric(tmean)
    names(tmean) <- months

    if (start.month != 1)
      tmean <- .shift(x=tmean, imonth=start.month)

    tmean.m.avg <- tmean
    tmean.m.q1  <- tmean
    tmean.m.q2  <- tmean
  } # IF end

  if (!missing(tmx)) {
    if ( ( sfreq(tmx) == "monthly" ) & (length(tmx) == 12) ) {
      tmx.is.mean.monthly <- TRUE

      months     <- format(time(tmx), "%b")
      tmx        <- as.numeric(tmx)
      names(tmx) <- months

      if (start.month != 1)
        tmx <- .shift(x=tmx, imonth=start.month)

      tmx.m.avg <- tmx
      tmx.m.q1  <- tmx
      tmx.m.q2  <- tmx
    } # IF end
  } # IF end

  if (!missing(tmn)) {
    if ( ( sfreq(tmn) == "monthly" ) & (length(tmn) == 12) ) {
      tmn.is.mean.monthly <- TRUE

      months     <- format(time(tmn), "%b")
      tmn        <- as.numeric(tmn)
      names(tmn) <- months

      if (start.month != 1)
        tmn <- .shift(x=tmn, imonth=start.month)

      tmn.m.avg <- tmn
      tmn.m.q1  <- tmn
      tmn.m.q2  <- tmn
    } # IF end
  } # IF end

  
  ###########################################
  ## In case 'pcp', 'tmean' ('tmx' and 'tmn') were not given as average monthly values
  if ( (!pcp.is.mean.monthly) & (!tmean.is.mean.monthly) ) {

    from <- time(pcp)[1]
    to   <- time(pcp)[length(pcp)]

    nyears <- yip(from=from, to=to, date.fmt="%Y-%m-%d", out.type="nmbr")

    # Computing mean monthly values of 'pcp'
    if ( (sfreq(pcp) != "monthly") | ( (sfreq(pcp) == "monthly") & ( length(pcp) > 12) ) )
      pcp.m.avg <- monthlyfunction(pcp, FUN=sum, na.rm=na.rm) / nyears

    # Computing mean monthly values of 'pcp.solid', only if it exists
    if (pcp.solid.exists){
      if ( (sfreq(pcp.solid) != "monthly") | ( (sfreq(pcp.solid) == "monthly") & ( length(pcp.solid) > 12) ) )
        pcp.solid.m.avg <- monthlyfunction(pcp.solid, FUN=sum, na.rm=na.rm) / nyears
    } # IF end

    # Computing mean monthly values of 'tmean'
    if ( (sfreq(tmean) != "monthly") | ( (sfreq(tmean) == "monthly") & ( length(tmean) > 12) ) )
      tmean.m.avg <- monthlyfunction(tmean, FUN=mean, na.rm=na.rm)

    # If provided, computing mean monthly values of 'tmx' and 'tmn'
    if ( !missing(tmx) & !missing(tmn)) {
      if ( (sfreq(tmx) != "monthly") | ( (sfreq(tmx) == "monthly") & ( length(tmx) > 12) ) ) 
        tmx.m.avg <- monthlyfunction(tmx, FUN=mean, na.rm=na.rm)

      if ( (sfreq(tmn) != "monthly") | ( (sfreq(tmn) == "monthly") & ( length(tmn) > 12) ) )
        tmn.m.avg <- monthlyfunction(tmn, FUN=mean, na.rm=na.rm)
    } # IF end

    # Shifting the monthly values when 'start.month != 1'
    if (start.month != 1) {

      pcp.m.avg         <- .shift(x=pcp.m.avg      , imonth=start.month)
      if (pcp.solid.exists) 
        pcp.solid.m.avg <- .shift(x=pcp.solid.m.avg, imonth=start.month)
      tmean.m.avg       <- .shift(x=tmean.m.avg    , imonth=start.month)
      if ( !missing(tmx) & !missing(tmn)) {
         tmx.m.avg <- .shift(x=tmx.m.avg, imonth=start.month)
         tmn.m.avg <- .shift(x=tmn.m.avg, imonth=start.month)
      } # IF end

    } # IF end

    if (plot.pcp.probs) {
      if ( sfreq(pcp) == "monthly" ) {
        pcp.m <- pcp
      } else pcp.m <- daily2monthly(pcp, FUN=sum, na.rm=na.rm) # 'subdaily2monthly' is a wrapper to 'daily2monthly'
      pcp.m.q1 <- monthlyfunction(pcp.m, FUN=quantile, probs=pcp.probs[1], na.rm=na.rm)
      pcp.m.q2 <- monthlyfunction(pcp.m, FUN=quantile, probs=pcp.probs[2], na.rm=na.rm)
      if (start.month != 1) pcp.m.q1 <- .shift(x=pcp.m.q1, imonth=start.month)
      if (start.month != 1) pcp.m.q2 <- .shift(x=pcp.m.q2, imonth=start.month)
    } # IF end

    if (plot.temp.probs) {
      temp.probs.col <- grDevices::adjustcolor(temp.probs.col, alpha.f=temp.probs.alpha)

      if ( sfreq(tmean) == "monthly" ) {
        tmean.m <- tmean
      } else tmean.m <- daily2monthly(tmean, FUN=mean, na.rm=na.rm)  # 'subdaily2monthly' is a wrapper to 'daily2monthly'
      tmean.m.q1 <- monthlyfunction(tmean.m, FUN=quantile, probs=temp.probs[1], na.rm=na.rm)
      tmean.m.q2 <- monthlyfunction(tmean.m, FUN=quantile, probs=temp.probs[2], na.rm=na.rm)
      if (start.month != 1) tmean.m.q1 <- .shift(x=tmean.m.q1, imonth=start.month)
      if (start.month != 1) tmean.m.q2 <- .shift(x=tmean.m.q2, imonth=start.month)

      if ( !missing(tmx) & !missing(tmn)) {
        if ( sfreq(tmx) == "monthly") {
          tmx.m <- tmx
        } else tmx.m <- daily2monthly(tmx, FUN=mean, na.rm=na.rm) # 'subdaily2monthly' is a wrapper to 'daily2monthly'

        if ( sfreq(tmn) == "monthly") {
          tmn.m <- tmn
        } else tmn.m <- daily2monthly(tmn, FUN=mean, na.rm=na.rm) # 'subdaily2monthly' is a wrapper to 'daily2monthly'

        tmx.m.q1 <- monthlyfunction(tmx.m, FUN=quantile, probs=temp.probs[1], na.rm=na.rm)
        tmx.m.q2 <- monthlyfunction(tmx.m, FUN=quantile, probs=temp.probs[2], na.rm=na.rm)
        tmn.m.q1 <- monthlyfunction(tmn.m, FUN=quantile, probs=temp.probs[1], na.rm=na.rm)
        tmn.m.q2 <- monthlyfunction(tmn.m, FUN=quantile, probs=temp.probs[2], na.rm=na.rm)
        if (start.month != 1) {
          tmx.m.q1 <- .shift(x=tmx.m.q1, imonth=start.month)
          tmx.m.q2 <- .shift(x=tmx.m.q2, imonth=start.month)
          tmn.m.q1 <- .shift(x=tmn.m.q1, imonth=start.month)
          tmn.m.q2 <- .shift(x=tmn.m.q2, imonth=start.month)
        } # IF end
      } # IF end
    } # IF end

  } # IF end

  #######################################
  # Drawing the climograph
  #######################################
  xlim <- c(0.5, 14.5)

  # Monthly precipitation as barplot
  if (missing(pcp.ylim)) {
    if (plot.pcp.probs) {
      ylim <- range(pretty(pcp.m.avg), pretty(pcp.m.q1), pretty(pcp.m.q2))
    } else ylim <- range(pretty(pcp.m.avg))
  } else ylim <- pcp.ylim 

  par(mar = c(7,5,3,5)) # c(bottom, left, top, right)
  x <- barplot(pcp.m.avg, col=pcp.col, xlim=xlim, ylim=ylim, ylab=pcp.label, las=1, main=main)
  if (pcp.solid.exists)
    barplot(pcp.solid.m.avg, col=pcp.solid.col, xlim=xlim, ylim=ylim, ylab=pcp.label, las=1, main=main, add=TRUE)
  # if (pcp.solid.exists){
  #   pcp.total.m.avg <- cbind(pcp.solid.m.avg, pcp.m.avg)
  #   x <- barplot(pcp.total.m.avg, col=c(pcp.solid.col, pcp.col), xlim=xlim, ylim=ylim, ylab=pcp.label, las=1, main=main)
  # } else {
  #     x <- barplot(pcp.m.avg, col=pcp.col, xlim=xlim, ylim=ylim, ylab=pcp.label, las=1, main=main)
  #   } # ELSE end

  #legend with lat and lon if they are provided
  legend.text.lab <- c("", "")
  legend.text.val <- c("", "")
  if (!missing(lat)) {
    legend.text.lab[1] <- "Lat:"
    legend.text.val[1] <- lat
  }  # IF end
  if (!missing(lon)) {
    legend.text.lab[2] <- "Lon:"
    legend.text.val[2] <- lon
  } # IF end
  legend("topright", paste(legend.text.lab, legend.text.val), bty="n",
         cex=1.2, ncol=1, title = "")

  # Adding error bars
  if (plot.pcp.probs)
    suppressWarnings( graphics::arrows(x0 = x, y0 = pcp.m.q2, y1 = pcp.m.q1, 
                                       angle=90, code=3, length=0.1) )

  grid()
  #ifelse(plot.pcp.probs, deltax <- 0.25, deltax <- 0.0)
  if (pcp.labels) text(x+pcp.labels.dx, pcp.m.avg+pcp.labels.dy, cex=pcp.labels.cex, adj=0.5, 
                       labels= round(pcp.m.avg,1), col="black" )


  # If provided, computing the ylim for the secondary temperature axis
  if (missing(temp.ylim)) {
    if ( !missing(tmx) & !missing(tmn)) {
      if (plot.temp.probs) {
        ylim <- range(#pretty(tmx.m.avg), pretty(tmean.m.avg), pretty(tmn.m.avg),
                      pretty(tmx.m.q1), pretty(tmean.m.q1), pretty(tmn.m.q1),
                      pretty(tmx.m.q2), pretty(tmean.m.q2), pretty(tmn.m.q2)
                     )
      } else ylim <- range(pretty(tmx.m.avg), pretty(tmean.m.avg), pretty(tmn.m.avg))
    } else if (plot.temp.probs) {
             ylim <- range(pretty(tmean.m.q1), pretty(tmean.m.avg), pretty(tmean.m.q2))
           } else ylim <- range(pretty(tmean.m.avg))
  } else ylim <- temp.ylim 


  # Mean temperature as line
  par(new = TRUE, xpd=TRUE)
  if (plot.temp.probs) {
    plot(x, tmean.m.avg, xlim=xlim, ylim=ylim, type="n", xlab="", ylab="", axes=FALSE)
    .plotbands(x=x, lband=tmean.m.q1, uband=tmean.m.q2, col=temp.probs.col[2], border=NA)
    lines(x, tmean.m.avg, xlim=xlim, ylim=ylim, col= tmean.col, type = "o", lwd=3, pch=15, 
          cex=1.4, bty = "n", xlab = "", ylab = "")
  } else plot(x, tmean.m.avg, xlim=xlim, ylim=ylim, col= tmean.col, type = "o", lwd=3, 
              pch=15, cex=1.4, axes = FALSE, bty = "n", xlab = "", ylab = "")
  if (tmean.labels) text(x+temp.labels.dx, tmean.m.avg+temp.labels.dy, cex=temp.labels.cex, 
                         adj=0.5, labels= round(tmean.m.avg,1), col=tmean.col )

  # If provided, tmn as line
  if (!missing(tmn)) {
    par(new = TRUE, xpd=TRUE)
    if (plot.temp.probs) {
      plot(x, tmn.m.avg, xlim=xlim, ylim=ylim, type="n", xlab="", ylab="", axes=FALSE)
      .plotbands(x=x, lband=tmn.m.q1, uband=tmn.m.q2, col=temp.probs.col[1], border=NA) 
      lines(x, tmn.m.avg, xlim=xlim, ylim=ylim, col= tmn.col, type = "o", lwd=3, pch=15, 
            cex=1.4, bty = "n", xlab = "", ylab = "")
    } else plot(x, tmn.m.avg, xlim=xlim, ylim=ylim, col= tmn.col, type = "o", lwd=3, pch=15, 
                cex=1.4, axes = FALSE, bty = "n", xlab = "", ylab = "")

    if (tmn.labels) text(x+temp.labels.dx, tmn.m.avg+temp.labels.dy, cex=temp.labels.cex, 
                         adj=0.5, labels= round(tmn.m.avg,1), col=tmn.col )
  } # IF end

  # If provided, tmx as line
  if (!missing(tmx)) {
    par(new = TRUE, xpd=TRUE)
    if (plot.temp.probs) {
      plot(x, tmx.m.avg, xlim=xlim, ylim=ylim, type="n", xlab="", ylab="", axes=FALSE)
      .plotbands(x=x, lband=tmx.m.q1, uband=tmx.m.q2, col=temp.probs.col[3], border=NA)
      lines(x, tmx.m.avg, xlim=xlim, ylim=ylim, col= tmx.col, type = "o", lwd=3, pch=15, 
            cex=1.4, bty = "n", xlab = "", ylab = "")
    } else plot(x, tmx.m.avg, xlim=xlim, ylim=ylim, col= tmx.col, type = "o", lwd=3, pch=15, 
                cex=1.4, axes = FALSE, bty = "n", xlab = "", ylab = "")
    if (tmx.labels) text(x+temp.labels.dx, tmx.m.avg+temp.labels.dy, cex=temp.labels.cex, 
                         adj=0.5, labels= round(tmx.m.avg,1), col=tmx.col )
  } # IF end


  # Plotting temperature axis on the right hand side
  if ( !missing(tmx) & !missing(tmn)) {
    axis(side=4, at = pretty(range(tmn.m.avg, tmean.m.avg, tmx.m.avg)), las=1)
  } else axis(side=4, at = pretty(range(tmean.m.avg)), las=1)
  abline(h=axTicks(side=2), col="lightpink", lty = "dotted")
  text(par("usr")[2]*1.05,mean(par("usr")[3:4]), labels= tmean.label, 
       srt=-90, xpd=TRUE, pos=4)


  # Outter box and legend
  box()
  par(xpd=TRUE)
  if ( pcp.solid.exists & !missing(tmx) & !missing(tmn) ) {
    legend("bottom", legend = c("Prec. (total)", "Prec. (solid)", "Tmn", "Tmean", "Tmx"), bty="n",
           pch=c(15, 15, 15, 15, 15), lty=c(NA, NA, 1, 1, 1), cex=1.2, 
           col=c(pcp.col, pcp.solid.col, tmn.col, tmean.col, tmx.col), ncol=5, inset=c(0.5, -0.2),
           #lty = 1:2, xjust = 1, yjust = 1,
           title = "")
  } else
      if ( !missing(tmx) & !missing(tmn)) {
        legend("bottom", legend = c("Prec.", "Tmn", "Tmean", "Tmx"), bty="n",
               pch=c(15, 15, 15, 15), lty=c(NA, 1, 1, 1), cex=1.2, col=c(pcp.col, tmn.col,
               tmean.col, tmx.col), ncol=4, inset=c(0.5, -0.2),
               #lty = 1:2, xjust = 1, yjust = 1,
               title = "")
      } else
          legend("bottom", legend = c("Precipitation", "Temperature"), bty="n",
                 pch=c(15, 15), lty=c(NA, 1), cex=1.2, col=c(pcp.col, tmean.col), ncol=2, 
                 #lty = 1:2, xjust = 1, yjust = 1,
                 inset=c(0.5, -0.2), title = "")
  
} # 'climograph' END

# File extract.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

#########################################################################
# extract: Extracts from a zoo object, all the values belonging to a    #
#             given month, year or weather season                       #
#########################################################################
# Author : Mauricio Zambrano-Bigiarini                                  #
# Started: 16-Apr-2009                                                  #
# Updates: 15-May-2009; 30-Ago-2009 ; 10-Aug-2011                       #
#########################################################################

extract <-function(x, ...) UseMethod("extract")

# This function was called 'extractzoo' up to hydroTSM 0.2-2
extractzoo <-function(x, ...) UseMethod("extract")

# 'x'    : variable of type 'zoo'
# 'trgt' : numeric or character indicating elements to extract from 'x'
#          Valid values are:
#          1) integer from 1 to 12: 'trgt' is considered as a month, and
#             all the vaues in 'x' belonging to the month specified by 'trgt'
#             will be extracted  (1=JAN, 2=FEB,...., 12=DEC)
#          2) integer > 12: 'trgt' is considered as a year, and all the
#             values in 'x' belonging to the year specified by 'trgt'
#             will be extracted
#          3) character: 'trgt' is considered as a weather season, and
#             all the values in 'x' belonging to the season specified by
#             'trgt' will be extracted. Valid values are:
#              -) "DJF": December, January, February
#              -) "MAM": March, April, May
#              -) "JJA": June, July, August
#              -) "SON": September, October, November


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 16-Apr-2009                                                         #
# Updates: 15-May-2009; 30-Ago-2009 ; 10-Aug-2011                              #
#          29-May-2013                                                         #
################################################################################
extract.default <- function(x, trgt, ...) {

  # Checking that the user provied a valid class for 'x'   
     valid.class <- c("xts", "zoo")    
     if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
        stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")

     extract.zoo(x=x, trgt=trgt, ...)

} # 'extract.default' END


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 22-Aug-2011                                                         #
# Updates: 13-Feb-2014 ; 14-Feb-2014                                           #
#          22-Aug-2022                                                         #
################################################################################
extract.zoo <- function(x, trgt, ...) {

  # Checking that the user provied a valid argument for 'trgt'
  if ( is.na( match( class(trgt), c("integer", "numeric", "character") ) ) )
      stop("Invalid argument: class('trgt') must be in c('integer', 'numeric','character')")

  # If 'trgt' is a month or a year
  if ( (inherits(trgt, "numeric") ) | ( inherits(trgt, "integer") ) ) {

    # Checking that 'trgt' is integer
    if ( !isTRUE(all.equal(trgt - trunc(trgt), rep(0,length(trgt)) ) ) )
        stop("Invalid argument: 'trgt' must be integer")

	if ( isTRUE(all.equal((trgt %in% 1:12), rep(TRUE, length(trgt)) ) )) {
	   index <- which(as.numeric(format.Date(time(x), "%m")) %in% trgt)
	} else index <- which( as.numeric(format.Date(time(x), "%Y")) %in% trgt )

  } # if END

    # if 'trgt' is a weather season
    else if ( inherits(trgt, "character") ) {
    
          seasons.default         <- c("DJF",  "MAM", "JJA",  "SON")
          seasons.FrenchPolynesia <- c("DJFM", "AM",  "JJAS", "ON")
          
          # Checking that the user provied a valid class for 'trgt'   
          valid.seasons <- valid.seasons <- union(seasons.default, seasons.FrenchPolynesia)
             
          if (length(which(!is.na(match(trgt, valid.seasons )))) <= 0)  
            stop( "Invalid argument: 'trgt' must be in 'c(", paste(valid.seasons, collapse=", "), ")'" ) 
            
          # Finding out if 'tr' belongs to 'seasons.default' or to ' seasons.FrenchPolynesia'.
          if ( trgt %in% seasons.default ) {
            season.type <- "default"
          } else if ( trgt %in% seasons.FrenchPolynesia ) {
              season.type <- "FrenchPolynesia"
            } # ELSE end

	  # Gets the season each element of 'x' belongs to 'seasons.default' or to 'seasons.FrenchPolynesia'
          seasons <- time2season(time(x), out.fmt="months", type=season.type)

          # Selects only those elements of 'x' belonging to the desired season
          index <- which(seasons == trgt)

      } # ELSE end

  if (length(index)==0)
    warning("There were no elements of 'x' within 'trgt'")

  return(x[index])

} # 'extract.zoo' END

# File hydropairs.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# 'hydropairs' : Visualization of a Correlation Matrix                         #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 29-Jul-2009                                                         #
################################################################################
# On top the (absolute) value of the correlation plus the result of the cor.test as points. 
# On botttom, the bivariate scatterplots, with a fitted line
# On diagonal, histograms (from '?pairs')

# Original idea taken from: http://addictedtor.free.fr/graphiques/graphcode.php?graph=137
# Histogram panles was taken form the R help of the original 'pairs' function

# x     : a numeric vector, matrix or data frame
# dec   : decimals places to be used for showing the correlation values

# use   : an optional character string giving a method for computing
#          covariances in the presence of missing values.  This must be
#          (an abbreviation of) one of the strings '"everything"',
#          '"all.obs"', '"complete.obs"', '"na.or.complete"', or
#          '"pairwise.complete.obs"'.

# method: a character string indicating which correlation coefficient
#          (or covariance) is to be computed.  One of '"pearson"'
#          (default), '"kendall"', or '"spearman"', can be abbreviated.

hydropairs <- function(x, dec=3, use="pairwise.complete.obs", method="pearson",... ) {

  # Checking that the user provied a valid argument for 'x'
  if ( is.na( match( class(x), c("matrix", "data.frame") ) ) )
      stop("Invalid argument: 'class(x)' must be in c('data.frame')")

  panel.cor <- function(x, y, digits=dec, prefix="", cex.cor)
    {
        usr <- par("usr"); on.exit(par(usr))
        par(usr = c(0, 1, 0, 1))

        r <- abs(cor(x, y, method= method, use= use))

        txt <- format(c(r, 0.123456789), digits=dec)[1]
        txt <- paste(prefix, txt, sep="")
        if(missing(cex.cor)) cex <- 0.8/strwidth(txt)

        test <- cor.test(x,y)
        # borrowed from printCoefmat
        Signif <- symnum(test$p.value, corr = FALSE, na = FALSE,
                      cutpoints = c(0, 0.001, 0.01, 0.05, 0.1, 1),
                      symbols = c("***", "**", "*", ".", " "))

        text(0.5, 0.5, txt, cex = cex * r)
        text(.8, .8, Signif, cex=cex, col=2)
    } # 'panel.cor' END

  panel.hist <- function(x, ...)
         {
             usr <- par("usr"); on.exit(par(usr))
             par(usr = c(usr[1:2], 0, 1.5) )
             h <- hist(x, plot = FALSE)
             breaks <- h$breaks; nB <- length(breaks)
             y <- h$counts; y <- y/max(y)
             rect(breaks[-nB], 0, breaks[-1], y, col="cyan", ...)
         } # 'panel.hist' END


  # 'font.labels' =2 : bold font for the variables
  # 'cex.labels' controls the size of the fonts
  pairs(x, lower.panel=panel.smooth, upper.panel=panel.cor,
        diag.panel=panel.hist, ...)

} # 'hydropairs' END

# File hip.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2013-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                                    'hip'                                     #  
################################################################################       
# Purpose: Given any starting and ending date/time objects, it generates:      #
#        1) a vector with all the hours between the two dates, OR              #
#	 2) the amount of hours between the two date/time objects                  #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 2008                                                                #
# Updates: 29-May-2013                                                         #
#          07-Jul-2022 ; 09-Oct-2022                                           #
#          27-Nov-2023                                                         #
################################################################################

# 'from'    : Character indicating the starting date/time object for computing the number of hours.
#             It MUST have the date/time format specified by 'date.fmt'
# 'to'	    : Character indicating the ending date/time object for computing the number of dyas.
#             It MUST have the date/time format specified by 'date.fmt'
# 'date.fmt': Character indicating the date format in which you provide 'from' and 'to'. (e.g., "%d-%m-%Y %H")
# 'out.type': Character indicating the type of result that is given by this function.
#             Valid values are:
#		      -) type= "seq"  => a vectorial sequence with all the hours within the given time period
#		      -) type= "nmbr" => the number of hours in the vectorial sequence with all the hourswithin the given time period
# 'tz'      : specification of the desired time zone yo be used. 
#             System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} (the default value) is UTC (Universal Time, Coordinated). 
#             See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
#             This argument can be used when working with subdaily zoo objects to force using the local time zone instead of GMT as time zone.
hip <- function(from, to, date.fmt="%Y-%m-%d %H", out.type="seq", tz="UTC") {

     # Checking 'out.type'
     if (is.na(match(out.type, c("seq", "nmbr"))))
        stop("Invalid argument: 'out.type' must be of class 'seq' or 'nmbr'")
        
     # Converting 'from' into a Date object (if necessary)
     if ( !(is(from, "POSIXct")) | !(is(from, "POSIXt")) ) {
       from.bak <- from
       from     <- as.POSIXct(from, format=date.fmt, tz=tz)
     } # IF end
     
     # Checking that 'from' is a valid Date object
     if (is.na(from)) 
       stop("Invalid argument: 'from' (", from.bak, 
            ") is not compatible with 'date.ftm' (", date.fmt, ") !")
     
     # Converting 'to' into a Date object (if necessary)
     if ( !(is(to, "POSIXct")) | !(is(to, "POSIXt")) ) {
       to.bak <- to
       to     <- as.POSIXct(to, format=date.fmt, tz=tz)
     } # IF end
     
     # Checking that 'to' is a valid Date object
     if (is.na(to)) 
       stop("Invalid argument: 'to' (", to.bak, 
            ") is not compatible with 'date.ftm' (", date.fmt, ") !")
     
     # Checking that 'from' is lower or equal to 'to'
     if (to < from) stop("Invalid argument: 'from > to' (", from, " > ", to, ")")

     # Generating an Annual-regular time series of Dates.
     vec.hrs <- seq( from=from, to=to, by="hours" )

     if (out.type=="seq") return(vec.hrs)
     else if (out.type=="nmbr") return ( length(vec.hrs) )

} # 'hip' END

# File daily2monthly.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#          daily2monthly                                                       #
################################################################################
# This function transform a (sub)DAILY regular time series into a MONTHLY one

# 'x'   : daily values that will be converted into monthly ones.
#         class(x) must be zoo/xts
# 'FUN' : Function that have to be applied for transforming from daily into 
#         monthly time step
#         For precipitation FUN MUST be "sum"
#         For temperature and flow time series, FUN MUST be "mean"
# 'na.rm': Logical. Should missing values be removed?
#          TRUE : the monthly and annual values  are computed considering only those values different from NA
#          FALSE: if there is AT LEAST one NA within a year, the monthly and annual values are NA

daily2monthly <- function(x, ...) UseMethod("daily2monthly")

################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: XX-XXX-2008                                                         #
# Updates: 09-Aug-2011                                                         #
#          08-Apr-013                                                          #
#          28-Jul-2023                                                         #
################################################################################
daily2monthly.default <- function(x, FUN, na.rm=TRUE, na.rm.max=0, ... ) {

     # Checking that 'x' is a zoo object
     if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !")

     daily2monthly.zoo(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, ...)

} # 'daily2monthly.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 09-Aug-2011                                                         #
# Updates: 09-Aug-2011                                                         #
#          08-Apr-2013                                                         #
#          20-Jun-2023 ; 28-Jul-2023                                           #
#          05-Feb-2025                                                         #
################################################################################
daily2monthly.zoo <- function(x, FUN, na.rm=TRUE, na.rm.max=0, ... ) {

  # Checking the user provide a valid value for 'FUN'
  if (missing(FUN))
     stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values")

  # Checking the user provide a valid value for 'x'
  if (sfreq(x) %in% c("monthly", "quarterly", "annual"))
    stop("Invalid argument: 'x' is not a (sub)daily/weekly ts. 'x' is a ", sfreq(x), " ts" )

  # Checking that 'na.rm.max' is in [0, 1]
  if ( (na.rm.max < 0) | (na.rm.max > 1) )
    stop("Invalid argument: 'na.rm.max' must be in [0, 1] !") 
      
  # Monthly index for 'x'
  dates  <- time(x)
  months <- as.Date( as.yearmon( time(x) ) ) # zoo::as.Date ; zoo::as.yearmon

  # Computing the Monthly time series 
  tmp <- aggregate( x, by=months, FUN, na.rm= na.rm, ... ) 

  # Removing monthly values in the output object for months with 
  # more than 'na.rm.max' percentage of NAs in a given month
  if ( na.rm ) {

    # Computing the percentage of missing values in each month
    na.pctg <- cmv(x, tscale="monthly")

    # identifying months with a percentage of missing values higher than 'na.rm.max'
    na.pctg.index <- which( na.pctg > na.rm.max)

    # Setting as NA all the monhts with a percentage of missing values higher than 'na.rm.max'
    tmp[na.pctg.index] <- NA 
  } # IF end
  
  # Replacing the NaNs by 'NA.
  # mean(NA:NA, na.rm=TRUE) == NaN
  nan.index <- which(is.nan(tmp))
  if ( length(nan.index) > 0 )  tmp[nan.index] <- NA 
  
  # Replacing all the Inf and -Inf by NA's
  # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
  inf.index <- which(is.infinite(tmp))
  if ( length(inf.index) > 0 ) tmp[inf.index] <- NA 

  if (NCOL(tmp) == 1) tmp <- zoo(as.numeric(tmp), time(tmp))

  return(tmp)

} # 'daily2monthly.zoo' end



################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: XX-XXX-2008                                                         #
# Updates: 09-Aug-2011                                                         #
#          04-Jun-2012                                                         #
#          29-May-2013                                                         #
#          23-Aug-2022                                                         #
#          20-Jun-2023 ; 28-Jul-2023                                           #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with as many columns as stations
#                              are included in 'x'
#             -) "db"        : a data.frame, with 4 colums will be produced.
#                              The first column stores the ID of the station,
#                              The second column stores the Year
#                              The third column stores the month
#                              The fourth colum stores the numerical values corresponding to the year and month specified in the two previous fields.
# 'out.fmt' : character, for selecting if the result will be 'numeric' or 'zoo'. Valid values are: c('numeric', 'zoo')
# 'verbose'      : logical; if TRUE, progress messages are printed
daily2monthly.data.frame <- function(x, FUN, na.rm=TRUE, na.rm.max=0,
                                     dates=1, date.fmt="%Y-%m-%d",
				                             out.type="data.frame",
				                             out.fmt="numeric",
				                             verbose=TRUE, ...) {
  
  # Checking that the user provied a valid argument for 'out.type'
  if (is.na(match( out.type, c("data.frame", "db") ) ) )
      stop("Invalid argument: 'out.type' must be in c('data.frame', 'db')")

  # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN))
      stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values !")

  # Checking that the user provied a valid argument for 'out.fmt'
  if (is.na(match( out.fmt, c("numeric", "zoo") ) ) )
      stop("Invalid argument: 'out.fmt' must be in c('numeric', 'zoo')")

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt) # zoo::as.Date
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt) # zoo::as.Date

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
     
  # Transforming 'x' into a zoo object
  x <- zoo(x, dates)
  
  ##############################################################################
  if (out.type == "data.frame") {
  
    z <- daily2monthly.zoo(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, ...)
    
    if (out.fmt == "numeric") {
       snames      <- colnames(z)
       months.lab  <- format(time(z), "%b-%Y")
       z           <- coredata(z)
       colnames(z) <- snames
       rownames(z) <- months.lab        
    } # IF end
    
  } else if (out.type == "db") {  
  
          if (verbose) message("[Starting computations...]" )

          # Amount of stations in 'x'
          nstations <- ncol(x)

          # ID of all the stations in 'x'
          snames <- colnames(x)

          # Computing the Starting and Ending Year of the analysis
          Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
          Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
          
          # Amount of Years belonging to the desired period
          nyears <- Ending.Year - Starting.Year + 1

          # Computing the amount of months with data within the desired period
          ndays   <- length(dates) # number of days in the period
          tmp     <- vector2zoo(rep(0,ndays), dates)
          tmp     <- daily2monthly.default(x= tmp, FUN=FUN, na.rm=na.rm)
          nmonths <- length(tmp)

          # Creating a vector with the names of the field that will be used for storing the results
          field.names <- c("StationID", "Year", "Month", "Value" )

          # Creating the data.frame that will store the computed averages for each subcatchment
          z <- as.data.frame(matrix(data = NA, nrow = nmonths*nstations, ncol = 4,
                            byrow = TRUE, dimnames = NULL) )
          colnames(z) <- field.names

          y = x

          for (j in 1:nstations) {

            if (verbose) message( "Station: ", format(snames[j], width=10, justify="left"),
                                  " : ",format(j, width=3, justify="left"), "/",
                                  nstations, " => ",
                                  format(round(100*j/nstations,2), width=6, justify="left"),
                                  "%" )

	        # Computing the monthly values
	        m     <- daily2monthly.zoo(x= x[,j], FUN=FUN, ..., na.rm=na.rm, na.rm.max=na.rm.max)
          dates <- time(m)
            
	        if (out.fmt == "numeric") m <- coredata(m)

	        # Putting the annual/monthly values in the output data.frame
          # The first column of 'x' corresponds to the Year
          row.ini <- (j-1)*nmonths + 1
          row.fin <-  j*nmonths

          z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeated 'nmonths' times
          z[row.ini:row.fin, 2] <- format(as.Date(dates), "%Y") # zoo::as.Date
          z[row.ini:row.fin, 3] <- format(as.Date(dates), "%b") # zoo::as.Date
          z[row.ini:row.fin, 4] <- m

        } # FOR end
        
  } # IF end

  return( z )

 } #'daily2monthly.data.frame' END


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: XX-XXX-2008                                                         #
# Updates: 09-Aug-2011                                                         #
#          29-May-2013                                                         #
#          28-Jul-2023                                                         #
################################################################################
daily2monthly.matrix  <- function(x, FUN, na.rm=TRUE, na.rm.max=0,
                                  dates=1, date.fmt="%Y-%m-%d",
				                          out.type="data.frame",
				                          out.fmt="numeric",
                                  verbose=TRUE,...) {

   x <- as.data.frame(x)
   #NextMethod("daily2annual")  # I don't know why is redirecting to 'daily2monthly.default' instead of 'daily2monthly.data.frame'....
   daily2monthly.data.frame(x=x, FUN=FUN, na.rm=na.rm, 
                            na.rm.max=na.rm.max,
                            dates=dates, date.fmt=date.fmt,
			                      out.type=out.type,
			                      out.fmt=out.fmt,
                            verbose=verbose,...)

} # 'daily2monthly.matrix  ' END

# File baseflow.R
# File plot_p_q.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
#                                 http://www.rforge.net/hydroTSM/ 
# Copyright 2012-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                               'baseflow'                                     #  
################################################################################       
# Purpose: Given a complete (without missing values) series of streamflow,     #
#          this function computes the baseflow using the filter proposed by    # 
#          Arnold & Allen in 1999                                              #
################################################################################
# References: Arnold, J. G., & Allen, P. M. (1999). Automated methods for      #
#             estimating baseflow and ground water recharge from streamflow    #
#             records. JAWRA Journal of the American Water Resources ,         #
#             Association 35(2), 411-424.                                      #
#             doi: 10.1111/j.1752-1688.1999.tb03599.x                          #
#                                                                              #
#             Ladson, A. R., Brown, R., Neal, B., & Nathan, R. (2013). A       #
#             standard approach to baseflow separation using the Lyne and      #
#             Hollick filter. Australasian Journal of Water Resources, 17(1),  #
#             25-34. doi:10.7158/13241583.2013.11465417                        #
################################################################################
# Author : Mauricio Zambrano-Bigiarini, Hector Garces-Figueroa                 #
# Started: 2013                                                                #
# Updates: 16-May-2018                                                         #
#          26-Nov-2023                                                         #
#          17-Jan-2024                                                         #
#          24-Jul-2025 (HGF) ; 25-Jul-2025                                     #
################################################################################

# 'x'       : zoo object with streamflow records. The suggested time 
#             frequency should be hourly or daily, but the algorithm will work  
#             with any time frequency
# 'beta'    : numeric representing the filter parameter. Default value is 0.925  
#             as recommended by Arnold & Allen (1999)
# 'from'       : Character indicating the starting date for creating the regularly 
#                spaced zoo object. The default value corresponds to the date of 
#                the first element of \code{x} \cr
#                It has to be in the format indicated by \code{date.fmt}. 
# 'to'         : Character indicating the ending date for creating the regularly 
#                spaced zoo object. The default value corresponds to the date of 
#                the last element of \code{x} \cr
#                It has to be in the format indicated by \code{date.fmt}.
# 'n.pass'     : Integer indicating the number times the filetr can be passed
#                over the streamflow data. Default is 3L.
# 'pass.start' : Character indicating the direction of the first filter pass.
#                Subsequent passes alternate direction (e.g., forward-backward)
#                Valid values are:
#             -) "forward" (default)
#             -) "backward"
# 'date.fmt'   : Character indicating the format in which the dates are stored in 
#                \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. 
#                See \sQuote{Details} section in \code{\link[base]{strptime}}.
#                By default, \code{date.fmt} is missing, and it is automatically 
#                set to \kbd{\%Y-\%m-\%d} when \code{time(x)} is \code{Date} 
#                object, and set to \kbd{\%Y-\%m-\%d \%H:\%M:\%S} when \code{x} 
#                is a sub-daily zoo object.
# 'tz'         : Character, with the specification of the time zone used for 
#                \code{from}, \code{to}. System-specific (see time zones), 
#                but \code{""} is the current time zone, and \code{"GMT"} is 
#                UTC (Universal Time, Coordinated). 
#                See \code{\link[base]{Sys.timezone}} and 
#                \code{\link[base]{as.POSIXct}}. \cr
#                If \code{tz} is missing (the default), it is automatically set 
#                to the time zone used in \code{time(x)}. \cr
#                This argument can be used when working with sub-daily zoo objects
#                to force using  time zones other than the local time zone for 
#                \code{from} and \code{to}. It should be used with caution, 
#                being well aware of the time zone of the data. See examples.
# 'na.fill' : Character indicating how to fill any NA present in 'x'.
#             Valid values are:
#             -) "none"   => NAs are not removed, and therefore the algorithm is 
#                            not executed
#             -) "linear" => NAs are removed by linear interpolation, using 
#                            \code{\link[zoo]{na.approx}}
#             -) "spline" => NAs are removed by spline interpolation, using 
#                            \code{\link[zoo]{na.spline}}
# 'out.type': Character indicating the type of result that is given by this function.
#             Valid values are:
#             -) "last" => only the baseflow computed after the third pass 
#                          of the filter is returned
#             -) "all " => the 3 baseflows computed after each pass of the 
#                          filter are returned in a matrix or zoo object
# 'plot'    : logical. Indicates if the baseflow should be plotted or not. 
#             If plotted, the original 'x' values are plotted as well
# 'xcol'    : character, representing the color to be used for ploting the 
#             streamflow time series
#             Only used when \code{plot=TRUE}.
# 'bfcol'   : character of lenght 3, representing the color(s) to be used for 
#             ploting the baseflow time series. The first, second and third 
#             element are used to represent the baseflow after the third, 
#             second and first pass of the filter, respectively.
#             Only used when \code{plot=TRUE}.
# 'pch'     : numeric, representing the symbols used for ploting the streamflow 
#             time series (both, the original series and the baseflow).
#             Only used when \code{plot=TRUE}.
# 'cex'     : a numerical vector giving the amount by which plotting characters 
#             and symbols should be scaled relative to the default. 
#             This works as a multiple of par("cex"). 
#             See \code{\link[graphics]{plot.default}}.
#             Only used when \code{plot=TRUE}.
# '...'     : further arguments passed to or from other methods. Not used yet.

baseflow <- function(x, ...) UseMethod("baseflow")

baseflow.zoo <- function(x, 
                         beta=0.925, 
                         from=start(x), 
                         to=end(x),
                         n.pass = 3L,
                         pass.start = c("forward", "backward"),
                         date.fmt,
                         tz,
                         na.fill=c("none", "linear", "spline"), 
                         out.type=c("last", "all"), 
                         plot=TRUE, 
                         xcol="black",
                         bfcol=c("blue", "darkcyan", "darkorange3"),
                         pch=15,
                         cex=0.3,
                         ...) {

  # Checking 'out.type'
  out.type <- match.arg(out.type)

  # Checking 'na.fill'
  na.fill <- match.arg(na.fill)

  # sampling frequency of 'x'           
  x.freq <- sfreq(x)
  
  # Checking if 'x is a sub-daily zoo object  
  subdaily.ts <- FALSE
  if (x.freq %in% c("minute","hourly") ) 
    subdaily.ts <- TRUE

  if ( (x.freq == "hourly") & (n.pass < 9) )
    warning("For hourly data 'n.pass' should be equal to 9 !! (Ladson et al., 2013) ")

  # checking 'n.pass'
  if (!inherits(n.pass, "integer")) stop("'n.pass' must be an integer !")
  if (n.pass <= 0) stop("'n.pass' must be greater than 0 !")
  
  # checking 'pass.start'
  pass.choices <- c("forward", "backward")
  pass.start <- match.arg(pass.start, pass.choices)

  # checking 'bfcol'
  if (out.type == "all" && plot && length(bfcol) != n.pass) {
    stop ("length of 'bfcol' must match 'n.pass'")
  } 

  ####################################################################################
  # Lines 154-178 are taken from izoo2rzoo.R to check 'from' and 'to'
  ####################################################################################

  # Automatic detection of 'date.fmt'
  if ( missing(date.fmt) ) {
    if ( subdaily.ts ) {
      date.fmt <- "%Y-%m-%d %H:%M:%S"
    } else date.fmt <- "%Y-%m-%d"
  } # IF end

  # Automatic detection of 'tz'
  missingTZ <- FALSE
  if (missing(tz)) {
    missingTZ <- TRUE
    tz        <- ""
  } # IF end
      
  ifelse ( grepl("%H", date.fmt, fixed=TRUE) | grepl("%M", date.fmt, fixed=TRUE) |
           grepl("%S", date.fmt, fixed=TRUE) | grepl("%I", date.fmt, fixed=TRUE) |
           grepl("%p", date.fmt, fixed=TRUE) | grepl("%X", date.fmt, fixed=TRUE), 
           subdaily.date.fmt <- TRUE, subdaily.date.fmt <- FALSE )

  # If the index of 'x' is character, it is converted into a Date object
  if ( class(time(x))[1] %in% c("factor", "character") )
    #ifelse(subdaily.date.fmt, time(x) <- as.POSIXct(time(x), format=date.fmt, tz=tz),
    ifelse(subdaily.date.fmt, time(x) <- as.POSIXct(time(x), format=date.fmt),
                              time(x) <- as.Date(time(x), format=date.fmt) )

  # If 'from' was given as Date, but 'x' is sub-daily
  if (!missing(from)) {
    if (from > to) stop("Invalid argument: 'from > to' !")

    if (from > end(x)) stop("Invalid argument: 'from > end(x)' !")

    if ( subdaily.date.fmt & !(grepl(":", from, fixed=TRUE) ) )
      from <- paste(from, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      from <- as.POSIXct(from, tz=tz)
  } # IF end

  # If 'to' was given as Date, but 'x' is sub-daily
  if (!missing(to)) {
    if (to < from ) stop("Invalid argument: 'to < from' !")

    if (to < start(x) ) stop("Invalid argument: 'to < start(x)' !")

    if ( subdaily.date.fmt & !(grepl(":", to, fixed=TRUE) ) )
      to <- paste(to, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      to <- as.POSIXct(to, tz=tz)
  } # IF end
        
  # checking that date.fmt and the sampling frequency of 'x' are compatible 
  if ( subdaily.ts ) {
    if (!subdaily.date.fmt) 
      stop("Invalid argument: 'date.fmt' (", date.fmt, 
           ") is not compatible with a sub-daily time series !!")
  } else {
           if (subdaily.date.fmt) {
             #time(x) <- as.POSIXct(time(x), tz=tz)
             time(x) <- as.POSIXct(time(x))
             warning("'date.fmt' (", date.fmt, ") is sub-daily, while 'x' is a '", 
                     x.freq, "' ts => 'time(x)=as.POSIXct(time(x), tz)'")
           } # IF end    
          } # ELSE end

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(from, format=date.fmt, tz=tz))
    #dt <-  try(as.POSIXct(from, format=date.fmt))
  } else dt <- try(as.Date(from, format=date.fmt))
  if("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'from' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      from <- as.POSIXct(from, format=date.fmt, tz=tz)
      #from <- as.POSIXct(from, format=date.fmt)
    } else from <- as.Date(from, format=date.fmt)

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(to, format=date.fmt, tz=tz))
  } else dt <- try(as.Date(to, format=date.fmt))
  if("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'to' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      to <- as.POSIXct(to, format=date.fmt, tz=tz)
    } else to <- as.Date(to, format=date.fmt)
  ####################################################################################

  # Selecting only those data within the time period between 'from' and 'to'
  x <- window(x, start=from, end=to)

  # Checking that 'x' does not have any missing value
  na.index <- which(is.na(x))
  if (length(na.index) > 0) {
    if (na.fill == "none") {
      stop("Invalid argument: 'x' has some NA values. See 'na.fill' argument !!")
    } else if (na.fill == "linear") {
        x <- zoo::na.approx(x)
      } else x <- zoo::na.spline(x)
  } # IF end

  # Storing and  then removing the possible time attribute
  HasTime <- FALSE
  if ( is.zoo(x) ) {
    HasTime <- TRUE
    xtime   <- time(x)
    x       <- zoo::coredata(x)
  } # if END

  # Initializing some parameters of the algorithm
  n  <- length(x)
  f1 <- beta
  f2 <- (1+beta) / 2

  # Initializing quickflow values 
  quickflow    <- rep(NA, n)
  quickflow <- replicate(n.pass, quickflow, simplify = FALSE)
  quickflow[[1]][1] <- x[1] / 2

  # Initializing baseflow values after the first ('baseflow1'), 
  # second ('baseflow2') and third ('baseflow3') pass of the filter
  baseflow <- rep(NA, n)
  baseflow <- replicate(n.pass, baseflow, simplify = FALSE)
  pass.type <- rep(unique(c(pass.start, pass.choices)), length.out = n.pass)

  .filter_pass <- function(l.baseflow, l.quickflow, l.type,
                           l.f1 = f1, l.f2 = f2, l.n = n) {
    if (l.type == "forward") {
      aux.n <- 2:l.n
      op    <- match.fun("-")
    } else { # type == "backward"
      aux.n <- (l.n-2) : 1
      op    <- match.fun("+")
    }

    for (i in aux.n) {
      l.quickflow[i] <- l.f1 * l.quickflow[op(i, 1)] +
                        l.f2 * (l.baseflow[i] - l.baseflow[op(i, 1)])
      if (l.quickflow[i] < 0) l.quickflow[i] <- 0
    }

    baseflow.out <- l.baseflow - l.quickflow
    neg.index <- which(baseflow.out < 0)
    if (length(neg.index) > 0) baseflow.out[neg.index] <- 0
    larger.index <- which(baseflow.out > l.baseflow)
    if (length(larger.index) > 0) {
      baseflow.out[larger.index] <- l.baseflow[larger.index]
    }
    
    out <- list(bf = baseflow.out, qf = l.quickflow)
    return(out)
  }
  # First pass of the filter
  first.pass     <- .filter_pass(l.baseflow = x,
                                 l.quickflow = quickflow[[1]],
                                 l.type = pass.type[[1]])
  baseflow[[1]]  <- first.pass[["bf"]]
  quickflow[[1]] <- first.pass[["qf"]]
  
  if (n.pass >= 2) {
    for (i in 2:n.pass) { 
      baseflow[[i]][n-1] <- baseflow[[1]][n-1] 
      filtered.data  <- .filter_pass(l.baseflow = baseflow[[i - 1]],
                                    l.quickflow = quickflow[[i - 1]],
                                    l.type = pass.type[[i]])
      baseflow[[i]]  <- filtered.data[["bf"]]
      quickflow[[i]] <- filtered.data[["qf"]]
    }
  }

  # Giving back the time attribute if the orignal 'x' object had one
  
  baseflow <- do.call(cbind, baseflow)
  if ( HasTime ) {
    x         <- zoo(x, xtime)
    baseflow <- zoo::zoo(baseflow, xtime)
  } # if END

  # Creating the output object
  if (out.type=="last") {
    out <- baseflow[, n.pass]
  } else out <- baseflow


  # Plotting, if necessary
  if (plot) {
    ylab <- "Q"
    xlab <- "Time"
    legend.text <- c("Streamflow", "Baseflow")
    if (HasTime) {
      plot(x, ylab=ylab, type="n", xaxt="n", xlab=xlab)
      drawTimeAxis(x)
      grid()
      points(x, type="o", col=xcol, pch=15, lty=1, cex=cex)
      if (out.type=="all") {
        for (i in 1:(n.pass - 1)) {
            aux.col <- bfcol[n.pass + 1 - i]
            points(1:n, baseflow[, i], type="o", col=aux.col, pch=pch, lty=1,
                   cex=cex)
          }
        legend.text <- c("Streamflow", paste("Baseflow pass", n.pass:1))
        # legend.cols <- c(xcol, bfcol[1], bfcol[2], bfcol[3])
        legend.cols <- c(xcol, bfcol)
      } else {
          legend.cols <- c(xcol, bfcol[1])
        } # ELSE end
        points(baseflow[, n.pass], type="o", col=bfcol[1], pch=pch, lty=1,
               cex=cex)
    } else { # HasTime == FALSE
        plot(1:n, x, ylab=ylab, type="n", xlab=xlab)
        grid()
        points(1:n, x, type="o", col=xcol, pch=pch, lty=1, cex=cex)
        if (out.type=="all") {
          for (i in 1:(n.pass - 1)) {
            aux.col <- bfcol[n.pass + 1 - i]
            points(1:n, baseflow[, i], type="o", col=aux.col, pch=pch, lty=1,
                   cex=cex)
          }

          legend.text <- c("Streamflow", paste("Baseflow pass", n.pass:1))
          legend.cols <- c(xcol, bfcol)
        } else {
            legend.cols <- c(xcol, bfcol[1])
          } # ELSE end
          points(1:n, baseflow[, n.pass], type="o", col=bfcol[1], pch=pch,
                 lty=1, cex=cex)
      } # ELSE end

    legend("topright", legend = legend.text, bty="n",
           lty=c(1,1), pch=c(pch, pch), col=legend.cols,
           #lty = 1:2, xjust = 1, yjust = 1,
           title = "")
  } # IF end

  return(out)

} # 'baseflow' END

#setwd("/dataMZB/2018-UFRO/work/Investigacion/Colaboracion_de_estudiantes/Cristobal_Soto/data/ts")
#library(zoo)
#x <- read.csv(file="Caudales_Curacautin_en_Rari_Ruca.csv")
#dates <- as.Date(x[,1], format="%d-%m-%Y")
#q <- x[1:3597, 2]
#dates <- dates[1:3597]
#q.na.filled <- na.approx(q)
#bf <- baseflow(q.na.filled)
#bf <- zoo(bf, dates)

# File sname2ts.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# sname2ts: Station name -> time series                                        #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 13-Jan-2009                                                         #
# Updates: 18-Oct-2012                                                         #
#          29-May-2013 ; 29-May-2013                                           #
#          23-Aug-2022                                                         #
################################################################################
# This function takes a data.frame whose columns contains the time series
# (without missing dates) of several gauging stations, it takes the name
# of one gauging station and extracts a time
# series with daily, monthly or annual time step
# 'x'        : data.frame containing the complete (without missing dates)
#              times series of all the stations.
#              It can also contain 1 column with the dates of the measurements,
#              or they can be provided in a separated way
# 'sname'    : string representing the name of the station, which have to correspond
#              with one column name in 'x'
# 'tstep.out': character that defines the time step of the desired output time series
#              it must be one of { "daily", "monthly", "annual" }
# 'dates'    : "numeric", "factor", "Date" indicating how to obtain the
#              dates for correponding to the 'sname' station
#              If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#              If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#              If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt' : format in which the dates are stored in 'dates'.
#              ONLY required when class(dates)=="factor" or "numeric"
# 'var.type' : character representing the type of variable being plotted
#              Used for determining the function used for computing the
#              Monthly and Annual values when 'FUN' is missing
#              Valid values are:
#              -) "Precipitation" => FUN = sum
#              -) "Temperature"   => FUN = mean
#              -) "Flow"          => FUN = mean
# 'FUN'      : ONLY required when 'var.type' is missing
#              Function that have to be applied for transforming from daily to monthly or annual time step
#              For precipitation FUN MUST be "sum"
#              For temperature and flow time series, FUN MUST be "mean"#
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the monthly and annual values  are computed considering only those values different from NA
#              FALSE: if there is AT LEAST one NA within a year, the monthly and annual values are NA
sname2ts <- function(x, sname, dates=1, date.fmt="%Y-%m-%d", var.type,
                     tstep.out="daily", FUN, na.rm=TRUE, from, to) {

  # Checking that the user provied a valid argument for 'x'
  if ( is.na( match( class(x), c("data.frame") ) ) )
      stop("Invalid argument: 'x' must be of class 'data.frame'")

  # Checking the user provides 'sname'
  if (missing(sname)) { stop("Missing argument: 'sname' must be provided")
  } else
    # Checking the the station provided for the user exist within 'x'
    if ( !(sname %in% colnames(x) ) )
      stop(paste("Invalid argument: ' The station '", sname, "' is not a column name in 'x'", sep="") )

  # If monthly or annual values are required, 'FUN' or 'var.type' must be provided
  if (tstep.out != "daily") {
    # Checking that the user provied a valid argument for 'var.type'
    if (missing(FUN)) {
       # If the user did not provide a title for the plots, this is created automatically
       if (missing(var.type)) {
         stop("Missing argument: 'var.type' OR 'FUN' must be provided")
       } else # If 'var.type' is provided
        # Checking that the user provied a valid argument for 'var.type'
        if (is.na(match(var.type, c("Precipitation", "Temperature", "Flow") ) ) ) {
              stop("Invalid argument: 'var.type' must be in c('Precipitation', 'Temperature', 'Flow')")
          } else {
                 if (var.type=="Precipitation") { FUN= sum  }
                 else if (var.type=="Temperature") { FUN= mean  }
                 else if (var.type=="Flow") { FUN= mean  }
                 } #ELSE end
    }
  } # IF end

  # Checking that the user provied a valid argument for 'tstep.out'
  if (is.na(match( tstep.out, c("daily", "monthly", "annual") ) ) )
      stop("Invalid argument: 'tstep.out' must be in c('daily', 'monthly', 'annual'")

  # Checking that the user provided a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  if ( inherits(dates, "numeric") ) dates <- as.Date(as.character(x[, dates]), format= date.fmt) # zoo::as.Date

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt) # zoo::as.Date

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # column index of the station identified by 'sname' within 'x'
  col.index <- which( colnames(x) == sname )

  # If the station name exists within 'x'
  if ( length(col.index) > 0 ) {

  # Selecting the time series within 'x' corresponding to the 'sname' station
  x <- x[ ,col.index]

  # Transform the vector of time series ('x') and the vector with dates ('dates')
  # into a zoo variable, using the format psecified by 'date.fmt'
  x.daily   <- vector2zoo(x, dates, date.fmt="%Y-%m-%d")


  ## In case 'fom' and 'to' are provided  ##

  # Checking the validity of the 'from' argument
  if (missing(from)) { 
     from     <- dates[1]
     from.pos <- 1
  } else {
      from <- as.Date(from, format=date.fmt) # zoo::as.Date
      if ( length( which(dates == from) ) > 0 ) {
        from.pos <- which( dates == from )
       } else stop("Invalid argument: 'from' is not in 'dates' ")
    } # ELSE end

  # Checking the validity of the 'to' argument
   if (missing(to)) { 
     to.pos <- length(dates)
     to     <- dates[to.pos]     
  } else {
      to <- as.Date(to, format=date.fmt) # zoo::as.Date
      if ( length( which(dates == to) ) > 0 ) {
        to.pos <- which( dates == to )
      } else stop("Invalid argument: 'to' is not in 'dates' ")
    } # ELSE end

  # Checking that 'to' is larger than 'from'
  if (to.pos < from.pos) stop("Invalid argument: 'to' have to be located in a row below the row corresponding to 'from'")

  # Extracting a subset of the values
  x.daily <- window(x.daily, start=from, end=to)

  # Output
  if (tstep.out == "daily") { return (x.daily)

    } else if (tstep.out =="monthly") {

      # Transformation from daily to monthly
      x.monthly <- daily2monthly(x.daily, FUN, na.rm )

      return (x.monthly)

      } else if (tstep.out =="annual") {

        # Transformation from daily to annual
        x.annual  <- daily2annual(x.daily, FUN, na.rm )

        return (x.annual)

        } # IF/ELSE/ELSE END

  } # IF end

} # 'sname2ts' END

# File subdaily2weekly.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2023-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#          subdaily2weekly                                                    #
################################################################################
# This function transform a (sub)DAILY regular time series into a MONTHLY one

# 'x'   : daily values that will be converted into monthly ones.
#         class(x) must be zoo/xts
# 'FUN' : Function that have to be applied for transforming from daily into 
#         monthly time step
#         For precipitation FUN MUST be "sum"
#         For temperature and flow time series, FUN MUST be "mean"
# 'na.rm': Logical. Should missing values be removed?
#          TRUE : the monthly and annual values  are computed considering only those values different from NA
#          FALSE: if there is AT LEAST one NA within a year, the monthly and annual values are NA

subdaily2weekly <-function(x, ...) UseMethod("subdaily2weekly")

################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Oct-2023                                                         #
# Updates:                                                                     # 
################################################################################
subdaily2weekly.default <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                    start.fmt= "%H:%M:%S", tz, ...) {

  # Checking that 'x' is a zoo object
  if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !")

  # Automatic detection of 'tz'
  #if (missing(tz)) tz <- ""
  if (missing(tz)) tz <- format(time(x), "%Z")[1]

  subdaily2weekly.zoo(x=x, FUN=FUN, na.rm=na.rm, start=start, start.fmt=start.fmt, tz=tz, ...)

} # 'subdaily2weekly.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Oct-2023                                                         #
# Updates: 04-Nov-2023                                                         # 
#          05-Feb-2025                                                         #
################################################################################
subdaily2weekly.zoo <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                start.fmt= "%H:%M:%S", tz, ...) {

  # testing the existence of 'na.rm' argument
  #args <- list(...)
  #exist <- "na.rm" %in% names(args)
  #exist

#  # Checking that the user provied a valid class for 'x'   
#  if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !")
#
#  # Automatic detection of 'tz'
#  if (missing(tz))
#    tz <- format(time(x), "%Z")[1]
#
#  d <- subdaily2daily.zoo(x=x, FUN=fn, na.rm=na.rm, start=start, start.fmt=start.fmt, tz=tz, ...)
#
#  daily2monthly.zoo(x=d, FUN=fn, na.rm=na.rm, ...)

    # testing the existence of 'na.rm' argument
    #args <- list(...)
    #exist <- "na.rm" %in% names(args)
    #exist

    # Checking that the user provied a valid class for 'x'   
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !!")

    # Checking the user provide a valid value for 'FUN'
    if (missing(FUN))
      stop("Missing argument: 'FUN' must contain a valid function for aggregating the sub-daily values")

    # Checking that 'na.rm.max' is in [0, 1]
    if ( (na.rm.max < 0) | (na.rm.max > 1) )
      stop("Invalid argument: 'na.rm.max' must be in [0, 1] !")  

    # Automatic detection of 'tz'
    #if (missing(tz)) tz <- ""
    if (missing(tz)) tz <- format(time(x), "%Z")[1]

    # Analysis of days different from 00:00 to 23:59 hrs
    if ( start != "00:00:00" ) {
      # Storing the original time
      time.old <- time(x)

      # Converting the new starting time provided by the user into a POSIXct object
      start <- as.POSIXct(start, format=start.fmt, tz=tz)

      # normal staring time for a day
      nstart <- as.POSIXct("00:00:00", format="%H:%M:%S", tz=tz)

      # time difference between the desired starting time 'strat' and the "normal"
      # starting time 'nstart', [s]
      delta <- difftime(start, nstart, units="secs")

      # Computing teh time difference between 'start' and the "normal" starting time, [s]
      #time.new <- as.POSIXct(time.old, tz=tz) - delta
      time.new <- time.old - delta

      # Changing the time in 'x' in 'delta' seconds
      time(x)  <- time.new
    } # IF end

     
    # Making sure that the time serie is complete before aggregation
    # This is useful when the first element of 'x' is not given at the time defined by 'start'.
    # For example, if the first element of 'x' starts at 08:00:00 hrs, but 'start=00:00:00', 
    # what happens with all the values from 00:00:00 to 07:59:59 hrs?
    # The following lines of code makes sure that the missing elements in a day are actually 
    # considered as missing

    st <- paste(format(start(x), "%Y-%m-%d"), "00:00:00", tz)
    et <- paste(format(end(x), "%Y-%m-%d"), "23:59:59", tz)
    x  <- izoo2rzoo(x, from=st, to=et, tz=tz)

    # Weekly index for 'x'
    dates <- time(x)
    weeks <- format(dates, "%Y-%W")

    # Computing the Weekly time series 
    tmp <- aggregate(x, by=weeks, FUN=FUN, na.rm= na.rm, ...)

    # Getting the weekly time attribute of the aggregated output object
    weeks.unique <- time(tmp)

    # Removing weekly values in the output object for weeks with 
    # more than 'na.rm.max' percentage of NAs in a given week
    if ( na.rm ) {

      # Computing the percentage of missing values in each week
      na.pctg <- cmv(x, tscale="weekly")

      # identifying weeks with a percentage of missing values higher than 'na.rm.max'
      na.pctg.index <- which( na.pctg > na.rm.max)

      # Setting as NA all the weeks with a percentage of missing values higher than 'na.rm.max'
      tmp[na.pctg.index] <- NA 
    } # IF end

    # Replacing the NaNs by 'NA.
    # mean(NA:NA, na.rm=TRUE) == NaN
    nan.index <- which(is.nan(tmp))
    if ( length(nan.index) > 0 ) tmp[nan.index] <- NA
  
    # Replacing all the Inf and -Inf by NA's
    # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
    inf.index <- which(is.infinite(tmp))
    if ( length(inf.index) > 0 ) tmp[inf.index] <- NA   

    # Removing subdaily time attibute, but not the dates
    if (NCOL(tmp) == 1) {
      tmp <- zoo(as.numeric(tmp), weeks.unique ) 
    } else tmp <- zoo(coredata(tmp), weeks.unique )    

    return(tmp)
} # 'subdaily2weekly.zoo' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Oct-2023                                                         #
# Updates:                                                                     # 
################################################################################
subdaily2weekly.data.frame <- function(x, FUN, na.rm=TRUE, na.rm.max=0, 
                                       start="00:00:00", start.fmt= "%H:%M:%S", tz, 
                                       dates=1, date.fmt="%Y-%m-%d %H:%M:%S",
				                               out.fmt="zoo",
				                               verbose=TRUE,...) {

   # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN))
      stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values !!")

  # Checking that the user provied a valid argument for 'out.fmt'
  if (is.na(match( out.fmt, c("numeric", "zoo") ) ) )
      stop("Invalid argument: 'out.fmt' must be in c('numeric', 'zoo')")

  # Checking that the user provied a valid argument for 'dates'
  if (missing(dates)) {
      stop("Missing argument: 'dates' must be provided")
  } else
     # Checking that the user provied a valid argument for 'dates'
     if ( !( inherits(dates, "numeric") | inherits(dates, "factor") | inherits(dates, "POSIXt")) )
         stop("Invalid argument: 'class(dates)' must be in c('numeric', 'factor', 'POSIXct', 'POSIXt') !")

  # Automatic detection of 'tz'
  if (missing(tz)) tz <- ""

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( TRUE && ( inherits(dates, "numeric") ) ) {
    tmp   <- dates
    dates <- as.POSIXct(x[, dates], format= date.fmt, tz=tz) 
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( TRUE && ( inherits(dates, "factor") ) ) dates <- as.POSIXct(dates, format= date.fmt) 

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( (TRUE && ( inherits(dates, "POSIXt") ) ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
     
  # Transforming 'x' into a zoo object
  x <- zoo::zoo(x, dates)
  

  ##############################################################################
  
  z <- subdaily2weekly.zoo(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, 
                           start=start, start.fmt=start.fmt, tz=tz, ...)
    
  if (out.fmt == "numeric") {
     snames      <- colnames(z)
     dates.lab   <- as.character(time(z))
     z           <- coredata(z)
     colnames(z) <- snames
     rownames(z) <- dates.lab        
  } # IF end

  return( z )

} # 'subdaily2weekly.data.frame' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Oct-2023                                                         #
# Updates:                                                                     # 
################################################################################
subdaily2weekly.matrix <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                   start.fmt= "%H:%M:%S", tz, 
                                   dates=1, date.fmt="%Y-%m-%d %H:%M:%S",
				                           out.fmt="zoo",
				                           verbose=TRUE,...) {

  # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN))
      stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values !!")

  # Checking that the user provied a valid argument for 'out.fmt'
  if (is.na(match( out.fmt, c("numeric", "zoo") ) ) )
      stop("Invalid argument: 'out.fmt' must be in c('numeric', 'zoo')")

  # Checking that the user provied a valid argument for 'dates'
  if (missing(dates)) {
      stop("Missing argument: 'dates' must be provided")
  } else
     # Checking that the user provied a valid argument for 'dates'
     if ( !( inherits(dates, "numeric") | inherits(dates, "factor") | inherits(dates, "POSIXt")) )
         stop("Invalid argument: 'class(dates)' must be in c('numeric', 'factor', 'POSIXct', 'POSIXt') !")

  # Automatic detection of 'tz'
  if (missing(tz)) tz <- ""

   x <- as.data.frame(x)
   #NextMethod("daily2annual")  # I don't know why is redirecting to 'daily2monthly.default' instead of 'daily2monthly.data.frame'....
   subdaily2weekly.data.frame(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, 
                              start=start, start.fmt=start.fmt, tz=tz,
                              dates=dates, date.fmt=date.fmt,
			                        out.fmt=out.fmt,
                              verbose=verbose,...)

} # 'subdaily2weekly.matrix' end

# File infillxy.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2010-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                                'infillxy'                                    #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Fills in ALL the 'NA' values in 'x' with the corrensponding values in 'sim'. #
# Started: 16-Dic-2008                                                         #
# Updates: 04-Sep-2009                                                         #
################################################################################
#
# 'x'   : 'data.frame' or 'matrix' in which some (observed) values are 'NA'
# 'sim' : 'data.frame' or 'matrix', with the same dimension of 'obs',
#          that contains the values that will be used for filling in
#          the 'NA' values in 'obs'
# Result: a 'data.frame' or 'matrix', with the same dimension of 'obs',
#         without 'NA' values.

infillxy <-function(x,...) UseMethod("infillxy")

infillxy.default <- function(x, sim,...) {

  if (length(x) != length(sim))
    stop("'x' and 'sim' does not have the same dimension !!")

  # vector with the index of all the elements in 'x' that are 'NA'
  na.index <- which( is.na(x) )

  # Replacing the 'NA' values in 'filled' by the correponding values in 'sim'
  x[na.index] <- sim[na.index]

  return(x)

} # 'infillxy.default' END


infillxy.matrix <- function(x, sim, ...) {

    if ( !identical(dim(x), dim(sim) ) )
      stop("'x' and 'sim' does not have the same dimension !!")

    if (is.na(match(class(sim), c("matrix"))))
        stop("Invalid argument: 'sim' must be of class 'matrix'")

    # Creating a copy of the original observed values
	z <- x

	z[,1:ncol(z)] <- sapply(1:ncol(z), function(j,y) {

		# Putting the monthly values in the output data.frame
		# The first column of 'x' corresponds to the Year
		z[,j] <- infillxy.default(x= y[,j], sim=sim[, j])

	}, y = x) # sapply END

 return(z)

} # 'infillxy.matrix' END


infillxy.data.frame <- function(x, sim, ...) {

    x   <- as.matrix(x)
    sim <- as.matrix(sim)

    NextMethod("infillxy.matrix")

} # 'infillxy.data.frame' END

# File vector2zoo.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# vector2zoo: Transforms a numeric vector 'x' with a corresponding             #
#             'Date' vector (any format) into a 'zoo' object                   #
################################################################################
# Author    : Mauricio Zambrano-Bigiarini                                      #
################################################################################
# Started   : 17-Dec-2008                                                      #
# Updates   : 01-Oct-2009                                                      #
#             06-Oct-2010                                                      #
#             05-May-2011                                                      #
#             15-Oct-2012 ; 16-Oct-2012                                        #     
#             29-May-2013                                                      #               
################################################################################
#  Transform a numeric vector and its corresponding dates into a 'zoo' object

# 'x':      : numeric vector
# 'dates'   : vector with a complete series of dates, in the same order of 'ts'
# 'date.fmt': format in which the dates are stored in 'dates'
vector2zoo <- function(x, dates, date.fmt="%Y-%m-%d") {

  if (is.na(match(class(dates)[1], c("Date", "POSIXct", "POSIXlt", "character", "factor"))))
      stop("Invalid argument: 'class(dates)' must be in c('Date', 'POSIXct', 'POSIXlt', 'character', 'factor')")
      
  if (length(x) != length(dates)) 
     stop("Invalid argument: length(x) != length(dates) (", length(x), "!=", length(dates), ")")

  # If 'dates' is a factor or character, it have to be converted into 'Date' class, 
  # using the date format  specified by 'date.fmt'
  if ( class(dates)[1] %in% c("factor", "character") ) {
     ifelse ( grepl("%H", date.fmt, fixed=TRUE) | grepl("%M", date.fmt, fixed=TRUE) |
            grepl("%S", date.fmt, fixed=TRUE) | grepl("%I", date.fmt, fixed=TRUE) |
            grepl("%p", date.fmt, fixed=TRUE) | grepl("%X", date.fmt, fixed=TRUE),
            subdaily.date.fmt <- TRUE, subdaily.date.fmt <- FALSE )
     ifelse(subdaily.date.fmt, dates <- as.POSIXct(dates, format= date.fmt), 
                               dates <- as.Date(dates, format= date.fmt) )  
  } # IF end 

  return( zoo(x, dates) )

}  # 'vector2zoo' END

# File plot_p_q.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
#                                 http://www.rforge.net/hydroTSM/ 
# Copyright 2023-2026 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                              'plot_pq'                                       #  
#       Plot precipitation and streamflow time series in the same figure       #
################################################################################       
# Purpose: Given a time series of precipitation and streamflow, this function  #
#          plots the two time series in the same figure, streamflows as a      #
#          normal time series and precipitation as bars comming from the upper #
#          part of the plotting window                                         #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 09-Jun-2018                                                         #
# Updates: 26-Nov-2023 ; 22-Dec-2023                                           #
#          12-Jan-2024 ; 13-Jan-2024 ; 17-Jan-2024                             #
#          26-Apr-2026                                                         #
################################################################################

# 'p'          : zoo object with precipitation time series, with any time frequency 
# 'q'          : zoo object with streamflow time series, with any time frequency 
# 'from'       : Character indicating the starting date for creating the regularly 
#                spaced zoo object. The default value corresponds to the date of 
#                the first element of \code{x} \cr
#                It has to be in the format indicated by \code{date.fmt}. 
# 'to'         : Character indicating the ending date for creating the regularly 
#                spaced zoo object. The default value corresponds to the date of 
#                the last element of \code{x} \cr
#                It has to be in the format indicated by \code{date.fmt}.
# 'date.fmt'   : Character indicating the format in which the dates are stored in 
#                \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. 
#                See \sQuote{Details} section in \code{\link[base]{strptime}}.
#                By default, \code{date.fmt} is missing, and it is automatically 
#                set to \kbd{\%Y-\%m-\%d} when \code{time(x)} is \code{Date} 
#                object, and set to \kbd{\%Y-\%m-\%d \%H:\%M:\%S} when \code{x} 
#                is a sub-daily zoo object.
# 'tz'         : Character, with the specification of the time zone used for 
#                \code{from}, \code{to}. System-specific (see time zones), 
#                but \code{""} is the current time zone, and \code{"GMT"} is 
#                UTC (Universal Time, Coordinated). 
#                See \code{\link[base]{Sys.timezone}} and 
#                \code{\link[base]{as.POSIXct}}. \cr
#                If \code{tz} is missing (the default), it is automatically set 
#                to the time zone used in \code{time(x)}. \cr
#                This argument can be used when working with sub-daily zoo objects
#                to force using  time zones other than the local time zone for 
#                \code{from} and \code{to}. It should be used with caution, 
#                being well aware of the time zone of the data. See examples.
# 'na.fill'    : Character indicating how to fill any NA present in 'x'.
#                Valid values are:
#                -) "remove" => NAs are not plotted
#                -) "linear" => NAs are removed by linear interpolation, using 
#                               \code{\link[zoo]{na.approx}}
#                -) "spline" => NAs are removed by spline interpolation, using 
#                               \code{\link[zoo]{na.spline}}
# 'xlab'       : a title for the \code{x} axis: see \code{\link[graphics]{title}}
# 'ylab'       : a two-element title for the \code{y} axis. 
#                See \code{\link[graphics]{title}}.
#                The first element is used for the right \code{y} axis 
#                (i.e., for precipitation). The second element is used for the 
#                left \code{y} axis (i.e., for streamflows).
# 'main'       : The main title (on top) of the figure.
# 'leg.title'  : a character string or length-one expression giving a title to be 
#                placed at the top of the legend. \code{\link[graphics]{legend}}.
# 'leg.text'   : a two-element character to appear in the legend placed at the 
#                bottom of the figure. 
#                The first element is used for precipitation and the second 
#                element is used for streamflows.
# 'cols'       : character, representing the colors to be used for ploting the 
#                precipitation and streamflow time series. 
#                The first element is used for precipitation and the second 
#                element is used for streamflows.
# 'q.pch'      : numeric, representing the symbols used for ploting the streamflow 
#                time series.
# 'q.cex'      : a numerical vector giving the amount by which plotting characters 
#                and symbols should be scaled relative to the default. 
#                This works as a multiple of par("cex"). 
#                See \code{\link[graphics]{plot.default}}
# '...'        : further arguments passed to or from other methods. Not used yet.

plot_pq <- function(p, ...) UseMethod("plot_pq")

plot_pq.zoo <- function(p, 
                        q, 
                        ptype=c("original", "monthly"),
                        
                        na.fill=c("remove", "linear", "spline"), 
                        
                        from=start(p), 
                        to=end(p),

                        date.fmt=NULL, 
                        tz=NULL,

                        main=ifelse(ptype=="original", "Precipitation and Streamflows", "Monthly Precipitation and Streamflows"),
                        xlab=ifelse(ptype=="original", "Time", "Month"), 
                        ylab=c("P, [mm]", "Q, [m3/s]"), 
                        p.col=ifelse(ptype=="original", "blue", "lightblue"),
                        q.col=ifelse(ptype=="original", "black", "blue"),

                        leg.title="",
                        leg.text=c("P", "Q"),
                        
                        q.pch=16,
                        q.cex=0.3,
                            
                        start.month=1,

                        plot.p.probs=TRUE,
                        p.probs=c(0.25, 0.75),
                        p.alpha=0.8,

                        plot.q.probs=TRUE,
                        q.probs=c(0.25, 0.75),
                        q.probs.col="lightskyblue1",
                        q.probs.alpha=0.8,
                            
                        labels=TRUE,
                        labels.cex=0.8,
                        labels.p.dy=NULL,
                        labels.q.dx=c(rep(-0.2,6), rep(0.2,6)),
                        labels.q.dy=rep(median(q, na.rm=TRUE)*1.3, 12),
                        
                        ...
                        ) {

  # Checking 'p'
  if (missing(p)) {
    stop("Missing argument: 'p' must be provided !")
  } else 
      # Checking that 'p' is a zoo object
      if ( !is.zoo(p) ) stop("Invalid argument: 'class(p)' must be 'zoo' !")

  # Checking 'q'
  if (missing(q)) {
    stop("Missing argument: 'q' must be provided !")
  } else 
      # Checking that 'q' is a zoo object
      if ( !is.zoo(q) ) stop("Invalid argument: 'class(q)' must be 'zoo' !")

  # Checking that 'p' and 'q' have the same time index
  if ( !isTRUE( all.equal( time(p), time(q) ) ) )
    stop("Invalid argument(s): 'p' and 'q' must have the same time index !")

  # Checking 'ptype'
  ptype <- match.arg(ptype)

  # Checking 'na.fill'
  na.fill <- match.arg(na.fill)

  # sampling frequency of 'x'           
  p.freq <- sfreq(p)
  
  # Checking if 'x is a sub-daily zoo object  
  if (p.freq %in% c("minute","hourly") ) {
    subdaily.ts <- TRUE
  } else subdaily.ts <- FALSE

  # Checking that 'q' and 'p' have the same dates
  dates.q  <- time(q)
  if (!missing(p)) {
    dates.p <- time(p)
    if ( !isTRUE( all.equal(dates.q, dates.p) ) )
      stop("Invalid arguments: 'dates(q)' must be equal to 'dates(p)' !")
  } # IF end
  
  ####################################################################################
  # Lines 147-231 are taken from izoo2rzoo.R to check 'from' and 'to'
  ####################################################################################
  # Automatic detection of 'date.fmt'
  if ( is.null(date.fmt) ) {
    if ( subdaily.ts ) {
      date.fmt <- "%Y-%m-%d %H:%M:%S"
    } else date.fmt <- "%Y-%m-%d"
  } # IF end

  # Automatic detection of 'tz'
  missingTZ <- FALSE
  if (is.null(tz)) {
    missingTZ <- TRUE
    tz        <- ""
  } # IF end
      
  #ifelse ( grepl("%H", date.fmt, fixed=TRUE) | grepl("%M", date.fmt, fixed=TRUE) |
  #         grepl("%S", date.fmt, fixed=TRUE) | grepl("%I", date.fmt, fixed=TRUE) |
  #         grepl("%p", date.fmt, fixed=TRUE) | grepl("%X", date.fmt, fixed=TRUE), 
  #         subdaily.date.fmt <- TRUE, subdaily.date.fmt <- FALSE )
  subdaily.date.fmt <-
    grepl("%H", date.fmt, fixed=TRUE) |
    grepl("%M", date.fmt, fixed=TRUE) |
    grepl("%S", date.fmt, fixed=TRUE) |
    grepl("%I", date.fmt, fixed=TRUE) |
    grepl("%p", date.fmt, fixed=TRUE) |
    grepl("%X", date.fmt, fixed=TRUE)

  # If the index of 'p' is character, it is converted into a Date object
  if ( class(time(p))[1] %in% c("factor", "character") )
    #ifelse(subdaily.date.fmt, time(x) <- as.POSIXct(time(p), format=date.fmt, tz=tz),
    ifelse(subdaily.date.fmt, time(p) <- as.POSIXct(time(p), format=date.fmt),
                              time(p) <- as.Date(time(p), format=date.fmt) )

  # If 'from' was given as Date, but 'x' is sub-daily
  if (!missing(from)) {
    if (from > to) stop("Invalid argument: 'from > to' !")

    if (from > end(p)) stop("Invalid argument: 'from > end(p)' !")

    if ( subdaily.date.fmt & !(grepl(":", from, fixed=TRUE) ) )
      from <- paste(from, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      from <- as.POSIXct(from, tz=tz)
  } # IF end

  # If 'to' was given as Date, but 'x' is sub-daily
  if (!missing(to)) {
    if (to < from ) stop("Invalid argument: 'to < from' !")

    if (to < start(p) ) stop("Invalid argument: 'to < start(p)' !")

    if ( subdaily.date.fmt & !(grepl(":", to, fixed=TRUE) ) )
      to <- paste(to, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      to <- as.POSIXct(to, tz=tz)
  } # IF end
        
  # checking that date.fmt and the sampling frequency of 'x' are compatible 
  if ( subdaily.ts ) {
    if (!subdaily.date.fmt) 
      stop("Invalid argument: 'date.fmt' (", date.fmt, 
           ") is not compatible with a sub-daily time series !!")
  } else {
           if (subdaily.date.fmt) {
             #time(x) <- as.POSIXct(time(x), tz=tz)
             time(p) <- as.POSIXct(time(p))
             warning("'date.fmt' (", date.fmt, ") is sub-daily, while 'p' is a '", 
                     p.freq, "' ts => 'time(p)=as.POSIXct(time(p), tz)'")
           } # IF end    
          } # ELSE end

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(from, format=date.fmt, tz=tz))
    #dt <-  try(as.POSIXct(from, format=date.fmt))
  } else dt <- try(as.Date(from, format=date.fmt))
  if("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'from' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      from <- as.POSIXct(from, format=date.fmt, tz=tz)
      #from <- as.POSIXct(from, format=date.fmt)
    } else from <- as.Date(from, format=date.fmt)

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(to, format=date.fmt, tz=tz))
  } else dt <- try(as.Date(to, format=date.fmt))
  if("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'to' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      to <- as.POSIXct(to, format=date.fmt, tz=tz)
    } else to <- as.Date(to, format=date.fmt)
  ####################################################################################

  # Selecting only those data within the time period between 'from' and 'to'
  p <- window(p, start=from, end=to)
  q <- window(q, start=from, end=to)

  if (is.null(labels.p.dy)) 
    labels.p.dy <- -median( daily2monthly(p, FUN=sum, na.rm=TRUE), na.rm=TRUE) * 1.1

  # If required, filling in any missing value(s)
  if (na.fill != "remove") {
    p.na.index <- which(is.na(p))
    q.na.index <- which(is.na(q))

    if (length(p.na.index) > 0) {
      if (na.fill == "linear") {
        p <- zoo::na.approx(p)
      } else p <- zoo::na.spline(p)
    } # IF end
    if (length(q.na.index) > 0) {
      if (na.fill == "linear") {
        q <- zoo::na.approx(q)
      } else q <- zoo::na.spline(q)
    } # IF end
  } # IF end

  oldpars <- par(no.readonly = TRUE)

  layout(1)
  par(mfrow = c(1, 1))
  par(new = FALSE)

  on.exit({
    layout(1)
    par(oldpars)
  }, add = TRUE)

  if (ptype=="original") {
     .plot_pq_ts.zoo(p, q, 
                     #ptype=ptype, na.fill=na.fill, from=from, end=end,
                     #date.fmt=date.fmt, tz=tz,
                     main=main, xlab=xlab, ylab=ylab, cols=c(p.col, q.col),
                     leg.title=leg.title, leg.text=leg.text,
                     q.pch=q.pch, q.cex=q.cex) 
  } else .plot_pq_monthly.zoo(p, q, 
                              #ptype=ptype, na.fill=na.fill, from=from, end=end,
                              #date.fmt=date.fmt, tz=tz,
                              main=main, xlab=xlab, ylab=ylab, cols=c(p.col, q.col),

                              #leg.title=leg.title, leg.text=leg.text,
                              #q.pch=q.pch, q.cex=q.cex,

                              start.month=start.month, plot.q.probs=plot.q.probs,
                              q.probs=q.probs, q.probs.col=q.probs.col,
                              q.probs.alpha=q.probs.alpha,
                         
                              plot.p.probs=plot.p.probs,
                              p.probs=p.probs, p.alpha=p.alpha,
                            
                              labels=labels, labels.cex=labels.cex,
                              labels.q.dx=labels.q.dx,
                              labels.q.dy=labels.q.dy,
                              labels.p.dy=labels.p.dy
                              )
} # 'plot_pq.zoo' END




.plot_pq_ts.zoo <- function(p, 
                            q, 

                            #na.fill=c("remove", "linear", "spline"), 
                            
                            #from=start(x), 
                            #to=end(x),
                            
                            #date.fmt, 
                            #tz,
                            
                            xlab="Time", 
                            ylab=c("P", "Q"), 
                            main="Precipitation and Streamflows",
                            leg.title="",
                            leg.text=c("P", "Q"),
                            cols=c("blue", "black"),
                            q.pch=16,
                            q.cex=0.3,
                            ...
                            ) {

  # saving graphical parameters
  oldpars <- par(no.readonly=TRUE)
  on.exit(par(oldpars))

  #par(mar=0.1 + c(5, 4, 4, 8), xpd=TRUE ) # bottom, left, top and right. Default: 0.1+ c(5, 4, 4, 2)
  par(mar=0.1 + c(7, 4, 4, 4), xpd=TRUE ) # bottom, left, top and right. Default: 0.1+ c(5, 4, 4, 2)

  # Plotting Q as time series
  ylim    <- range(q, na.rm=TRUE)
  ylim[2] <- ylim[2]*1.5
  plot(q, type="n", xaxt="n", xlab=xlab, ylab=ylab[2], ylim=ylim, main=main)
  drawTimeAxis(q)
  points(q, type="o", col=cols[2], pch=q.pch, cex=q.cex, lty=1)
  grid()

  # Plotting P as barplot from top to bottom
  par(new=TRUE)
  ylim    <- rev(range(p, na.rm=TRUE))
  ylim[1] <- ylim[1]*3
  barplot(zoo::coredata(p), ylim = ylim, xaxt="n", yaxt = "n", col=cols[1], border=NA)
  axis(4, at = pretty(p), col=cols[1], col.axis=cols[1])
  mtext(side = 4, line = 2, text="P", col=cols[1], srt=45)
  #at <- pretty(p)
  #axis(4, at=at, label=FALSE, col=cols[1])
  #text(y=at-2.5, x=par("usr")[2]+30, labels = at, srt = -90, pos = 4, xpd = TRUE, col=cols[1], offset=0.5)

  # legend("topright", inset=c(-0.07, 0), legend = leg.text, bty="n",
  #        lty=1, pch=pchs, col=cols,
  #        #lty = 1:2, xjust = 1, yjust = 1,
  #        title=leg.title)
  p.pch     <- 15
  leg.text  <- c(leg.text[1], " ", leg.text[2])
  cols      <- c(cols[1], "white", cols[2])
  pchs      <- c(p.pch, 3, q.pch)

  legend("bottom", horiz=TRUE, , bty="n", inset=c(0, -0.45), 
         x.intersp=0.1, y.intersp=3, 
         lty=1, pch=pchs, col=cols, cex=1,
         title=leg.title, legend= leg.text)
  par(new = FALSE)
} # '.plot_pq_ts.zoo' END



###################################################################################
# .plot_pq_monthly.zoo: Function for drawing a figure with mean monthly           #
#                       precipitation (on the top panel) and mean monthly         #
#                       streamflows (on the bottom pannel)                        #
###################################################################################
# Originally this function was called 'monthlycurve', but afterwards it was       # 
# merged with the 'plot_pq' function                                              #
###################################################################################
# Author : Mauricio Zambrano-Bigiarini                                            #
###################################################################################
# Started: 26-Jul-2022                                                            #
# Updates: 22-Sep-2022 ; 11-Oct-2022 ; 25-Oct-2022                                #
#          22-Dec-2023 ; 27-Dec-2023                                              #
#          12-Jan-2024 ; 17-Jan-2024                                              # 
#          26-Apr-2026                                                            #         
###################################################################################
# 'q'        : object of type 'zoo' with monthly, daily or subdaily streamflow data.
#              If q is a monthly zoo object, it must have 12 elments and it should be 
#              named with the names of the months (levels(time(q))), otherwise, 
#              automatic names will be asigned from Jan to Dec for each one of the 
#              12 monthly values
# 'date.fmt' : format in which the dates.q are stored in 'from' and 'to'.
# 'na.rm'    : Logical. Should missing values in 'q' be removed when using FUN?. 
#              It is also used when the optional argument 'p' is submonthly (e.g., daily, hourly), to decide whether missing values in the optional argument 'p' should be removed before aggregated into monthly scale
#              TRUE : the monthly values  are computed considering only those values in 'q' (and 'p') different from NA
#              FALSE: if 'q' (and 'p') has AT LEAST one NA within a month, the corresponding monthly values are NA

.plot_pq_monthly.zoo <- function(p,
                                 q, 
                            
                                 #na.rm=TRUE, 
                                 #na.fill=c("remove", "linear", "spline"), 
                            
                                 #from, 
                                 #to, 
                            
                                 #date.fmt, 
                                 #tz,
                            
                                 main="Monthly Precipitation and Streamflows", 
                                 xlab="Month",
                                 ylab=c("P, [mm]", "Q, [m3/s]"),
                                 cols=c("lightblue", "blue"),
                            
                                 start.month=1,

                                 plot.q.probs=TRUE,
                                 q.probs=c(0.25, 0.75),
                                 q.probs.col="lightskyblue1",
                                 q.probs.alpha=0.8,
                         
                                 plot.p.probs=TRUE,
                                 p.probs=c(0.25, 0.75),
                                 p.alpha=0.8,
                            
                                 labels=TRUE,
                                 labels.cex=0.8,
                                 labels.q.dx=c(rep(-0.2,6), rep(0.2,6)),
                                 labels.q.dy=rep(-median(q, na.rm=TRUE)/10, 12),
                                 labels.p.dy=-median(daily2monthly(p, FUN=sum, na.rm=TRUE), na.rm=TRUE)/10
                                 ) {

  .plotbands <- function(x, lband, uband, col="", border=NA) {
    t <- c(x, rev(x))
    bands <- c(as.numeric(lband), rev(as.numeric(uband)))
    polygon(t, bands, col=col, border=border)
  } # .plotbands END

  .shift <- function(x, imonth) {
    L <- length(x)
    if (imonth>L) stop("[ Invalid value: 'imonth' can not be larger than ", L, " !]")
    delta <- imonth-1
    index.old <- 1:L
    index.new <- index.old-delta
    neg <- which(index.new <=0)
    index.new[neg] <- index.new[neg]+L
    if ( is.zoo(x) ) {
      x.raw    <- zoo::coredata(x)
      x.labels <- as.character(time(x))
      out        <- x.raw[match(index.old, index.new)] 
      names(out) <- x.labels[match(index.old, index.new)] 
    } else out <- x[match(index.old, index.new)] 
    return(out)
  } # .shift END
   

  ## --- capture graphics state ---
  old_par <- par(no.readonly = TRUE)

  ## reset layout explicitly
  layout(matrix(1))

  on.exit({
    layout(matrix(1))
    par(old_par)
  }, add = FALSE)

  ###########################################
  ## In case 'q' is not average monthly values
  if ( (sfreq(q) != "monthly") ) {
    q.m    <- daily2monthly(q, FUN=mean, na.rm=TRUE)
  } else q.m <- q

  if ( (sfreq(q) != "monthly") | ( (sfreq(q) == "monthly") & ( length(q) > 12) ) ) {
    q.m.med     <- monthlyfunction(q.m, FUN=quantile, probs=0.5, na.rm=TRUE)
    month.names <- levels(time(q.m.med))
  } else {
      q.m.med     <- q
      month.names <- levels(time(q))
      if (is.null(month.names))
        month.names <- month.abb
    } # ELSE end  

  q.m.q1 <- monthlyfunction(q.m, FUN=quantile, probs=q.probs[1], na.rm=TRUE)
  q.m.q2 <- monthlyfunction(q.m, FUN=quantile, probs=q.probs[2], na.rm=TRUE)

  if (start.month != 1) q.m.med     <- .shift(x=q.m.med    , imonth=start.month)
  if (start.month != 1) q.m.q1      <- .shift(x=q.m.q1     , imonth=start.month)
  if (start.month != 1) q.m.q2      <- .shift(x=q.m.q2     , imonth=start.month)
  if (start.month != 1) month.names <- .shift(x=month.names, imonth=start.month)


  ###########################################
  ## In case 'q' is not average monthly values
  if ( (sfreq(q) != "monthly") ) {
    p.m <- daily2monthly(p, FUN=sum, na.rm=TRUE)
  } else p.m <- p

  if ( (sfreq(p) != "monthly") | ( (sfreq(p) == "monthly") & ( length(p) > 12) ) ) {
    p.m.med <- monthlyfunction(p.m, FUN=quantile, probs=0.5, na.rm=TRUE)
  } else p.m.med <- p

  p.m.q1 <- monthlyfunction(p.m, FUN=quantile, probs=p.probs[1], na.rm=TRUE)
  p.m.q2 <- monthlyfunction(p.m, FUN=quantile, probs=p.probs[2], na.rm=TRUE)

  if (start.month != 1) p.m.med <- .shift(x=p.m.med, imonth=start.month)
  if (start.month != 1) p.m.q1  <- .shift(x=p.m.q1 , imonth=start.month)
  if (start.month != 1) p.m.q2  <- .shift(x=p.m.q2 , imonth=start.month)


  ##############################################################################
  # Definining the plotting area (1 column, 2 rows), where the lower row has 
  # a height 3 times larger than the upper window, AND 
  ##############################################################################
  par(mar=c(3, 4.1, 3, 1.5), xpd=TRUE) # default  c(5.1, 4.1, 4.1, 2.1)
  layout(matrix(c(1,2), 2, 1, byrow = TRUE), widths=5, heights=c(1,3))  
 
  ######################################################
  # 1st Figure: Monthly Precipitation in the upper panel
  ######################################################

  ylim <- range(pretty(p.m.q1), pretty(p.m.q2))
  x <- barplot(p.m.med, ylim=rev(ylim), xlab="", ylab=ylab[1], axes=TRUE, col=cols[1], names.arg=month.names, main=main)
  #axis(side=1, at=lx, labels=month.names, line=0.02, outer=TRUE, pos=1)
  if (labels) text(x, labels.p.dy, cex=labels.cex, adj=0.5, labels= round(p.m.med,1), col="black")

  # Adding error bars
  if (plot.p.probs) 
    graphics::arrows(x0 = x, y0 = p.m.q2, y1 = p.m.q1, angle=90, code=3, length=0.1)

  #######################################
  # 2nd Figure: Drawing the monthly curve
  #######################################
  if (!missing(p)) {
    par(mar=c(3, 4.1, 0.5, 1.5)) # default  c(5.1, 4.1, 4.1, 2.1)
    main <- ""
  } else par(mar=c(3, 4.1, 4.1, 1.5))
  lubands.col <- grDevices::adjustcolor(q.probs.col, alpha.f=q.probs.alpha)
  xlim <- c(0.5, 12.5)
  ylim <- range(pretty(q.m.q1), pretty(q.m.q2))

  # Monthly values as lines
  lx   <- 1:12 
  plot(lx, q.m.med, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab[2], type="n", axes=TRUE, xaxt="n", main=main)
  #plot(lx, q.m.med, xlim=xlim, ylim=ylim, col= cols[2], type="o", lwd=3, pch=15, cex=1.4, axes=TRUE, xaxt="n", xlab=xlab, ylab=ylab[2])

  if (plot.q.probs) 
    .plotbands(x=lx, lband=q.m.q1, uband=q.m.q2, col=lubands.col, border=NA)
  
  grid()
  lines(lx, q.m.med, xlim=xlim, ylim=ylim, col= cols[2], type = "o", lwd=3, pch=15, cex=1.4)
  axis(side=1, at=lx, labels=month.names)
  if (labels) text(lx+labels.q.dx, q.m.med+labels.q.dy, cex=labels.cex, adj=0.5, labels= round(q.m.med,1), col="black" )

  # to avoid returning plotting objects that can interfere with device recording.
  invisible(NULL)
} # 'plot_pq_monthly.zoo' END

# File dm2seasonal.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2020 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# dm2seasonal: Generic function for computing seasonal values for every        #
#              year of a daily/monthly zoo object                              #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 15-May-2009 ;                                                       #
# Updates: 31-Aug-2009 ; 19-Jun-2011 ; 08-Aug-2011                             #
################################################################################

# 'x   '    : variable of type 'zoo' or 'data.frame'
# 'season'  : character, indicating the weather season to be used for selecting the data
#             Valid values are:
#             -) "DJF": December, January, February
#             -) "MAM": March, April, May
#             -) "JJA": June, July, August
#             -) "SON": September, October, November
# 'FUN'      : Function that will be applied to ALL the values of 'x' belonging to the given weather season
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the seasonal values  are computed considering only those values different from NA
#              FALSE: if there is AT LEAST one NA within a season, the corresponding values are NA
# 'out.fmt'  : character indicating the format for the output time series. Possible values are:
#              -) "%Y"      : only the year will be used for the time. Default option. (e.g., "1961" "1962"...)
#              -) "%Y-%m-%d": a complete date format will be used for the time. Default option. (e.g., "1961" "1962"...)

dm2seasonal <-function(x, ...) UseMethod("dm2seasonal")


dm2seasonal.default <- function(x, season, FUN, na.rm=TRUE, out.fmt="%Y", ...) {

  # Checking that 'x' is a zoo object
  if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

  dm2seasonal.zoo(x=x, season=season, FUN=FUN, na.rm=na.rm, out.fmt=out.fmt, ...)

} # 'dm2seasonal.default' END




################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 08-Aug-2011                                                         #
# Updates: 08-Aug-2011                                                         #
#          12-Oct-2022                                                         #
################################################################################
dm2seasonal.zoo <- function(x, season, FUN, na.rm=TRUE, out.fmt="%Y", ...) {

  # Checking that the user provied a valid argument for 'season'
  if ( missing(season) ) {
      stop("Missing argument: 'season' must be provided")
  } else { # If 'season' is provided
    
      seasons.default         <- c("DJF",  "MAM", "JJA",  "SON")
      seasons.FrenchPolynesia <- c("DJFM", "AM",  "JJAS", "ON")
          
      # Checking that the user provied a valid class for 'season'   
      valid.seasons <- valid.seasons <- union(seasons.default, seasons.FrenchPolynesia)
             
      if (length(which(!is.na(match(season, valid.seasons )))) <= 0)  
         stop("Invalid argument: 'season' must be in 'c(", paste(valid.seasons, collapse=", "), ")'") 
            
      # Finding out if 'season' belongs to 'seasons.default' or to ' seasons.FrenchPolynesia'.
      if ( season %in% seasons.default ) {
        season.type <- "default"
      } else if ( season %in% seasons.FrenchPolynesia ) {
          season.type <- "FrenchPolynesia"
        } # ELSE end
            
    } # ELSE end

  # Checking that the user provied a valid argument for 'FUN'
  if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")

  # Checking the user provide a valid value for 'x'
  if (sfreq(x) %in% c("quarterly", "annual"))
    stop("Invalid argument: 'x' is not a sub-daily, daily, weekly or monthly ts. 'x' is a ", sfreq(x), " ts" )
      
  # Checking 'out.fmt'
  if ( is.na(match(out.fmt, c("%Y", "%Y-%m-%d") ) ) )
    stop("Invalid argument: 'out.fmt' must be in c('%Y', '%Y-%m-%d')" )

  # Getting the Monthly values beloonging ONLY to the desired weather season
  s <- extract(x= x, trgt=season)

  # Moving forward all the December values, in order that
  # December of 1991 be used together with Jan/92 and Feb/92,
  # instead of with Jan/91 and Feb/91
  if ( (season == "DJF") | (season == "DJFM") ) {
  
	  syears            <- as.numeric(format( time(s), "%Y" ))
	  dec.index         <- which(format(time(s), "%m") == 12)
	  dec.years         <- syears[dec.index]
	  dec.years         <- dec.years + 1
	  syears[dec.index] <- dec.years

	  s.a <- aggregate( s, by= syears, FUN=FUN, na.rm= na.rm)

  	# Removing the last value of december, because it is outside of the analysis period
	  if ( (is.matrix(s.a)) | (is.data.frame(s.a)) ) {
	    s.a <- s.a[1:(nrow(s.a)-1), ]
	  } else s.a <- s.a[1:(length(s.a)-1)]
			
  } else  s.a <- aggregate( s, by= format( time(s), "%Y" ), FUN=FUN, na.rm= na.rm )	

  # Replacing the NaNs by 'NA.
  # mean(NA:NA, na.rm=TRUE) == NaN
  nan.index <- which(is.nan(s.a))
  if ( length(nan.index) > 0 ) s.a[nan.index] <- NA 

  # Replacing all the Inf and -Inf by NA's
  # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
  inf.index <- which(is.infinite(s.a))
  if ( length(inf.index) > 0 ) s.a[inf.index] <- NA 
  
  # If the user wants a complete data format for the output ts:
  if (out.fmt == "%Y-%m-%d")
    time(s.a) <- as.Date(paste( time(s.a), "-01-01", sep=""))

  return(s.a)

} # 'dm2seasonal.zoo' END



################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 15-May-2009                                                         #
# Updates: 08-Aug-2011                                                         #
#          29-May-2013 ; 03-Jun-2013                                           #
#          10-March-2020                                                       #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': character indicating the format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with as many columns as stations
#                              are included in 'x'
#             -) "db"        : a data.frame, with 4 colums will be produced.
#                              The first column will store the ID of the station,
#                              The second column will store the Year,
#                              The third column will store the season
#                              The fouth column will contain the seasonal value, corresponding to the year specified in the second column
dm2seasonal.data.frame <- function(x, season, FUN, na.rm=TRUE,
                                   dates=1, date.fmt="%Y-%m-%d",
				   out.type="data.frame", 
				   out.fmt="%Y", ... ) {

  # Checking that the user provied a valid argument for 'out.type'
  if (is.na(match( out.type, c("data.frame", "db") ) ) )
      stop("Invalid argument: 'out.type' must be in c('data.frame', 'db'")

  # Checking that the user provied a valid argument for 'season'
  if ( missing(season) ) {
      stop("Missing argument: 'season' must be provided")
  } else { # If 'season' is provided
    
      seasons.default         <- c("DJF",  "MAM", "JJA",  "SON")
      seasons.FrenchPolynesia <- c("DJFM", "AM",  "JJAS", "ON")
          
      # Checking that the user provied a valid class for 'season'   
      valid.seasons <- valid.seasons <- union(seasons.default, seasons.FrenchPolynesia)
             
      if (length(which(!is.na(match(season, valid.seasons )))) <= 0)  
         stop("Invalid argument: 'season' must be in 'c(", paste(valid.seasons, collapse=", "), ")'" ) 
            
      # Finding out if 'season' belongs to 'seasons.default' or to ' seasons.FrenchPolynesia'.
      if ( season %in% seasons.default ) {
        season.type <- "default"
      } else if ( season %in% seasons.FrenchPolynesia ) {
          season.type <- "FrenchPolynesia"
        } # ELSE end
            
    } # ELSE end

  # Checking that the user provied a valid argument for 'FUN'
  if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")
  
  # Checking 'out.fmt'
  if ( is.na(match(out.fmt, c("%Y", "%Y-%m-%d") ) ) )
    stop("Invalid argument: 'out.fmt' must be in c('%Y', '%Y-%m-%d')" )

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( is(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt)
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( is(dates, "factor") ) dates <- as.Date(dates, format= date.fmt)

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( is(dates, "Date") & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # Transforming 'x' into zoo
  x <- zoo(x, dates)  
     
  ##############################################################################
  if (out.type == "data.frame") {
  
    dm2seasonal.zoo(x=x, season=season, FUN=FUN, na.rm=na.rm, out.fmt=out.fmt, ...)
    
  } else if (out.type == "db") {

        # Amount of stations in 'x'
        nstations <- ncol(x)

        # ID of all the stations in 'x'
        snames <- colnames(x)

        # Computing the Starting and Ending Year of the analysis
        Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
        Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))

        # Amount of Years belonging to the desired season
        nyears <- Ending.Year - Starting.Year + 1

        message("[Starting computations...]")

        # Creating a vector with the names of the field that will be used for storing the results
        field.names <- c("StationID", "Year", "Season", "Value" )

        # Creating the data.frame that will store the computed averages for each subcatchment
        z <- as.data.frame(matrix(data = NA, nrow = nyears*nstations, ncol = 4,
                            byrow = TRUE, dimnames = NULL) )
        colnames(z) <- field.names

        y = x

        for (j in 1:nstations) {

            message("[ Station: ", format(snames[j], width=10, justify="left"),
                    " : ",format(j, width=3, justify="left"), "/",
                    nstations, " => ",
                    format(round(100*j/nstations,2), width=6, justify="left"),
                    "% ]" )

            # Transforming the column of 'x' into a zoo object,
	    # using the dates provided by the user
            tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)

	    # Computing the seasonal values
            s.a <- dm2seasonal.default(x= tmp, season=season, FUN=FUN, na.rm=na.rm, out.fmt=out.fmt)

            # Putting the annual seasonal values in the output data.frame
            # The first column of 'x' corresponds to the Year
            row.ini <- (j-1)*nyears + 1
            row.fin <- j*nyears

            z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'nyears' times
            z[row.ini:row.fin, 2] <- Starting.Year:Ending.Year
            z[row.ini:row.fin, 3] <- season    # it is automatically repeted 'nyears' times
            z[row.ini:row.fin, 4] <- s.a

        } # FOR end
        
        return( z )

    } # IF end

 } #'dm2seasonal.data.frame' END
 

################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 2009                                                                #
# Updates: 2011                                                                #
#          29-May-2013                                                         #
################################################################################
dm2seasonal.matrix  <- function(x, season, FUN, na.rm=TRUE,
                                dates=1, date.fmt="%Y-%m-%d",
				out.type="data.frame", 
				out.fmt="%Y", ... ) {

   x <- as.data.frame(x)
   #NextMethod("daily2annual")
   dm2seasonal.data.frame(x=x, season=season, FUN=FUN, na.rm=na.rm,
                          dates=dates, date.fmt=date.fmt,
			  out.type=out.type, 
			  out.fmt=out.fmt, ... )

} # 'dm2seasonal.matrix  ' END

# File fdc.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM/
# Copyright 2007-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# fdc: Flow Duration Curve, computation and plot                               #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: June 04, 2009                                                       #
# Updates: 25-Feb-2011 ; 04-Nov-2011                                           #
#          02-May-2012; 16-Oct-2012                                            #
#          05-Aug-2013                                                         #
#          15-Jan-2014                                                         # 
#          04-Feb-2015                                                         #
#          27-Dec-2025                                                         #
################################################################################

# Plot the flow Duration Curve in the original time units of 'x' and
# also gives the probability of exceedence of each element

# plot   : logical. Indicates if the flow duration curve should be plotted or not
# thr.shw: logical, indicating if the stremflow values corresponding to the user-defined thresholds 'lQ.thr' and 'hQ.thr'  have to be plotted
# new    : logical, indicates if a new graphics device has to be started
# log    : character, indicates which axis has to be plotted with a logarithmic scale. By default is 'y'

fdc <-function(x, ...) UseMethod("fdc")

fdc.default <- function (x,
                         lQ.thr=0.7,
                         hQ.thr=0.2,
                         plot=TRUE,
                         log="y",
                         main="Flow Duration Curve",
                         xlab="% Time flow equalled or exceeded",
                         ylab="Q, [m3/s]",
                         ylim=NULL,
                         yat=c(0.01, 0.1, 1), 
                         xat=c(0.01, 0.025, 0.05),
                         #yaxp=c(range(x),2),
                         col="black",
                         pch=1,
                         lwd=1,
                         lty=1,
                         cex=0.4,
                         cex.axis=1.2,
                         cex.lab=1.2,
                         leg.txt=NULL,
                         leg.cex=1,
                         leg.pos="topright",
                         thr.pos="bottomleft",
                         verbose= TRUE,
                         thr.shw=TRUE,
                         new=TRUE,
                         ...) {

     # Returns the position in the vector 'x' where the scalar 'Q' is located
     Qposition <- function(x, Q) {
       Q.dist  <- abs(x - Q)
       Q.index <- which.min( Q.dist )
       return(Q.index)
     } # end

     # If 'x' is of class 'ts' or 'zoo'
     #if ( !is.na( match( class(x), c("ts", "zoo") ) ) )
     x <- as.numeric(x)

     # Storing the original values
     x.old <- x

     # 1) Sort 'x' in drecreasing order. This is just for avoiding misleading
     #lines when using 'type="o"' for plotting
     x <- sort(x)

     # Detecting zero values
     x.zero.index <- which(x==0)
     nzeros <- length(x.zero.index)

     # Index with the position of the original values
     ind <- match(x.old, x)

     # 2) Compute the length of 'x'
     n <- length(x)

     # 3) Creation of the output vector
     dc <- rep(NA, n)

     # 4) Exceedence Probability
     dc[1:n] <- sapply(1:n, function(j,y) {
                  dc[j] <- length( which(y >= y[j]) )
                }, y = x)
 
     # Computing the probabilitites
     dc <- dc/n

     # Another way
     # Fn <- ecdf(x)
     # dc <- 1 - Fn(x) + 1/n

     if (plot) {

          dc.plot <- dc
  
          if (log == "y") {
            if (nzeros > 0) {
              x       <- x[-x.zero.index]
              dc.plot <- dc.plot[-x.zero.index]
              if (verbose) message("[Note: all 'x' equal to zero (", nzeros, ") will not be plotted ]")
            } # IF end
          } # IF end

          if ( is.null(ylim) ) ylim <- range(x, na.rm=TRUE)
             
          if ( ((log=="y") | (log=="xy") | (log=="yx")) & min(ylim)==0 ) {
             tmp <- x
             tmp[which(tmp==0)] <- NA
             ylim[1] <- min(tmp, na.rm=TRUE)
          } # IF end
          
          # If a new plot has to be created
          if (new) {
               plot(dc.plot, x,  xaxt = "n", yaxt = "n", type="o", col=col, pch=pch, lwd=lwd, lty=lty,
                    cex=cex, cex.axis=cex.axis, cex.lab=cex.lab, main=main, xlab=xlab, ylab=ylab, ylim=ylim, log=log, ...)
          } else lines(dc.plot, x,  xaxt = "n", type="o", col=col, pch=pch, lwd=lwd, lty=lty, cex=cex)

          # Y axis: Drawing the ticks and labels
          ylabels <- pretty(ylim)
          if ( (log=="y") | (log=="xy") | (log=="yx") ) {            
            ylabels <- union( yat, ylabels )            
          } # IF end
          axis( side = 2, at =ylabels, cex.axis=cex.axis, labels = ylabels)
          
          # X axis: Drawing the ticks and labels
          xpos    <- seq(0.0, 1, by=0.05)
          xlabels <- seq(0.0, 1, by=0.1)
          if ( (log=="x") | (log=="xy") | (log=="yx") ) {            
            xpos    <- union( xat, xpos ) 
            xlabels <- union( xat, xlabels )            
          } # IF end
          axis(side = 1, at = xpos, cex.axis=cex.axis, labels = FALSE)
          axis(side = 1, at = xlabels, cex.axis=cex.axis, labels = paste(100*xlabels,"%", sep="") )               

          # If the user provided a value for 'lQ.thr', a vertical line is drawn
          if ( !is.na(lQ.thr) ) abline(v=lQ.thr, col="grey", lty=3, lwd=2)

          # If the user provided a value for 'hQ.thr', a vertical line is drawn
          if ( !is.na(hQ.thr) ) abline(v=hQ.thr, col="grey", lty=3, lwd=2)

          # Drawing a legend. bty="n" => no border
          if ( !is.null(leg.txt) )
           legend(x=leg.pos, legend=leg.txt, cex=leg.cex, col=col, pch=pch, lwd=lwd, lty=lty, bty="n") # cex=cex*1.5,

          if (thr.shw) {
              # Finding the flow values corresponding to the 'lQ.thr' and 'hQ.thr' pbb of excedence
              x.lQ <- x[Qposition(dc.plot, lQ.thr)]
              x.hQ <- x[Qposition(dc.plot, hQ.thr)]

              legend(x=thr.pos, c(paste("Qhigh.thr=", round(x.hQ, 2), sep=""),
                                  paste("Qlow.thr=", round(x.lQ, 2), sep="") ),
                     cex=0.8, bty="n") #bty="n" => no box around the legend
          } # IF end
     } # IF end

     # Restoring the original positions
     dc <- dc[ind]

     return(dc)

} # 'fdc.default' END


################################################################################
# fdc.matrix: Computation and/or Plot of Multiple Flow Duration Curves,        #
#             mainly for comparison                                            #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 04-Jun-2009                                                         #
# Updates: 16-Sep-2011 ; 03-Nov-2011                                           #
#          02-May-2012                                                         #
#          05-Aug-2013                                                         #
#          27-Dec-2025                                                         #
################################################################################

fdc.matrix <- function (x,
                        lQ.thr=0.7,
                        hQ.thr=0.2,
                        plot=TRUE,
                        log="y",
                        main= "Flow Duration Curve",
                        xlab="% Time flow equalled or exceeded",
                        ylab="Q, [m3/s]",
                        ylim=NULL,
                        yat=c(0.01, 0.1, 1), 
                        xat=c(0.01, 0.025, 0.05),
                        col=palette("default")[1:ncol(x)],
                        pch=1:ncol(x),
                        lwd=rep(1, ncol(x)),
                        lty=1:ncol(x),
                        cex=0.4,
                        cex.axis=1.2,
                        cex.lab=1.2,
                        leg.txt=NULL,
                        leg.cex=1,
                        leg.pos="topright",
                        thr.pos="bottomleft",
                        verbose=TRUE,
                        thr.shw=TRUE,
                        new=TRUE,
                        ...) {

  if (is.null(ylim)) ylim <- range(x, na.rm=TRUE)
  
  if ( ((log=="y") | (log=="xy") | (log=="yx")) & min(ylim)==0 ) {
     tmp <- unlist(x)
     tmp[which(tmp==0)] <- NA
     ylim[1] <- min(tmp, na.rm=TRUE)
  } # IF end
  
  # Computing the FDC for each column of 'x'
  out <- apply(x, FUN=fdc.default, MARGIN=2, plot=FALSE, log="")
  
  # Plotting the FDC for each column of 'x'
  if (plot) {

    if (thr.shw==TRUE) {
      message("[Note: 'thr.shw' was set to FALSE to avoid confusing legends...]")
      thr.shw <- FALSE
    } # IF end
    
    n <- ncol(x)
    j <- 1 # starting column for the analysis

    if (verbose) message("[Column: ", format(j, width=10, justify="left"),
                          " : ", format(j, width=3, justify="left"), "/",
                          n, " => ",
                          format(round(100*j/n,2), width=6, justify="left"),
                          "% ]" )

    # Computing and plotting the Flow Duration Curve for the first column
    fdc(x=x[,1], plot=plot, log=log, col=col[1], pch=pch[1], lwd=lwd[1], lty=lty[1],
        cex=cex, cex.axis=cex.axis, cex.lab=cex.lab, main=main, 
        xlab= xlab, ylab=ylab, ylim=ylim, yat=yat, 
        verbose=verbose, thr.shw=FALSE, new=TRUE, ...)

    # Plotting the Flow Duration Curves for all the columns but the first one
    sapply(2:n, function(j) {

          if (verbose) message("[Column: ", format(j, width=10, justify="left"),
                                " : ", format(j, width=3, justify="left"), "/",
                                n, " => ",
                                format(round(100*j/n,2), width=6, justify="left"),
                                "% ]")

         # Computing and plotting the Flow duration Curve for the other columns
         tmp <- sort(x[,j])
              yval <- fdc(x=tmp, plot=FALSE, log="", verbose=verbose)
              points(yval, tmp, cex=cex, col=col[j], pch=pch[j], lwd=lwd[j], lty=lty[j], type="o")
            
          } )

    # Drawing a legend. bty="n" => no border
    if ( is.null(leg.txt) ) {
      if (!is.null(colnames(x))) {
        leg.txt <- colnames(x)
      } else leg.txt <- paste("Q", 1:ncol(x), sep="")  
    }
    legend(x=leg.pos, legend=leg.txt, cex=leg.cex, col=col, pch=pch, lwd=lwd, lty=lty, bty="n") # cex=cex*2.2,
  
  } # IF end

  return(out)
  
} # 'fdc.matrix' END


################################################################################
# fdc.data.frame: Computation and/or Plot of Multiple Flow Duration Curves,    #
#                 mainly for comparison                                        #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 04-Jun-2009                                                         #
# Updates: 03-Nov-2011                                                         #
#          02-May-2012                                                         #
#          05-Aug-2013                                                         #
#          27-Dec-2025                                                         #
################################################################################
fdc.data.frame <- function(x,
                           lQ.thr=0.7,
                           hQ.thr=0.2,
                           plot=TRUE,
                           log="y",
                           main= "Flow Duration Curve",
                           xlab="% Time flow equalled or exceeded",
                           ylab="Q, [m3/s]",
                           ylim,
                           yat=c(0.01, 0.1, 1), 
                           xat=c(0.01, 0.025, 0.05), 
                           col=palette("default")[1:ncol(x)],
                           pch=1:ncol(x),
                           lwd=rep(1, ncol(x)),
                           lty=1:ncol(x),
                           cex=0.4,
                           cex.axis=1.2,
                           cex.lab=1.2,
                           leg.txt=NULL,
                           leg.cex=1,
                           leg.pos="topright",
                           thr.pos="bottomleft",
                           verbose=TRUE,
                           thr.shw=TRUE,
                           new=TRUE,
                           ...) {

   x <- as.matrix(x)
   
   if (missing(ylim)) ylim <- range(x, na.rm=TRUE)

   fdc.matrix(x,
              lQ.thr=lQ.thr,
              hQ.thr=hQ.thr,
              plot=plot,
              log=log,
              main=main,
              xlab=xlab,
              ylab=ylab,
              ylim=ylim,
              yat=yat,   
              xat=xat,
              col=col,
              pch=pch,
              lwd=lwd,
              lty=lty,
              cex=cex,
              cex.axis=cex.axis,
              cex.lab=cex.axis,
              verbose=verbose,
              leg.txt=leg.txt,
              leg.cex=leg.cex,
              leg.pos=leg.pos,
              thr.pos=thr.pos,
              thr.shw=thr.shw,
              new=new,               
              ...)

} # 'fdc.data.frame' END


################################################################################
# fdc.zoo: Computation and/or Plot of Multiple Flow Duration Curves,           #
#             mainly for comparison                                            #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 03-Nov-2011                                                         #
# Updates: 02-May-2012                                                         #
#          05-Aug-2013                                                         #
#          27-Dec-2025                                                         #
################################################################################
fdc.zoo <- function (x,
                     lQ.thr=0.7,
                     hQ.thr=0.2,
                     plot=TRUE,
                     log="y",
                     main= "Flow Duration Curve",
                     xlab="% Time flow equalled or exceeded",
                     ylab="Q, [m3/s]",
                     ylim=NULL,
                     yat=c(0.01, 0.1, 1), 
                     xat=c(0.01, 0.025, 0.05),
                     col=palette("default")[1:NCOL(x)],
                     pch=1:NCOL(x),
                     lwd=rep(1, NCOL(x)),
                     lty=1:NCOL(x),
                     cex=0.4,
                     cex.axis=1.2,
                     cex.lab=1.2,
                     leg.txt=NULL,
                     leg.cex=1,
                     leg.pos="topright",
                     thr.pos="bottomleft",
                     verbose=TRUE,
                     thr.shw=TRUE,
                     new=TRUE,
                     ...) {

  x <- coredata(x)
  if ( is.matrix(x) | is.data.frame(x) ) {  
    fdc.matrix(x,
               lQ.thr=lQ.thr,
               hQ.thr=hQ.thr,
               plot=plot,
               log=log,
               main=main,
               xlab=xlab,
               ylab=ylab,
               ylim=ylim,
               yat=yat,   
               xat=xat,
               col=col,
               pch=pch,
               lwd=lwd,
               lty=lty,
               cex=cex,
               cex.axis=cex.axis,
               cex.lab=cex.axis,
               verbose=verbose,
               leg.txt= leg.txt,
               leg.cex=leg.cex,
               leg.pos=leg.pos,
               thr.pos=thr.pos,
               thr.shw=thr.shw,
               new=new,               
               ...)
  } else fdc.default(x,
                     lQ.thr=lQ.thr,
                     hQ.thr=hQ.thr,
                     plot=plot,
                     log=log,
                     main=main,
                     xlab=xlab,
                     ylab=ylab,
                     ylim=ylim,
                     yat=yat,   
                     xat=xat,
                     col=col,
                     pch=pch,
                     lwd=lwd,
                     lty=lty,
                     cex=cex,
                     cex.axis=cex.axis,
                     cex.lab=cex.axis,
                     verbose=verbose,
                     leg.txt=leg.txt,
                     leg.cex=leg.cex,                     
                     leg.pos=leg.pos,
                     thr.pos=thr.pos,
                     thr.shw=thr.shw,
                     new=new,               
                     ...)
  
  
} # 'fdc.zoo' END

# File diy.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# 'diy' : Given a numeric value of a year, it generates:                       #
#         1) a vector with all the days (dates) within the year, OR            #
#	  2) the amount of days in the year                                    #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 2008                                                                #
# Updates:                                                                     #
################################################################################
# year    : numeric, the year for which the sequence of days will be generated
# out.type: Character indicating the type of result that is given by this function.
#           Valid values are:
#		    -) type= "seq"  => a vectorial sequence with all the days within the given year
#		    -) type= "nmbr" => the number of days in the vectorial sequence with all the days within the given year
diy <- function(year, out.type="seq") {

   # Checking 'out.type'
   if (is.na(match(out.type, c("seq", "nmbr"))))
        stop("Invalid argument: 'out.type' must be of class 'seq' or 'nmbr'")

   # Generating a Daily-regular time series of Dates,
   # just for being counted as the maximum amount of possible daily data
   vec <- seq( from=as.Date( paste(year,"-01-01", sep="") ), to=as.Date( paste(year,"-12-31", sep="") ), by= "days" )

   if (out.type=="seq") return(vec)
   else if (out.type=="nmbr") return ( length(vec) )


} # 'diy' END

# File subdaily2annual.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#          subdaily2annual                                                     #
################################################################################
# Generic function for transforming a DAILY (sub-daily, weekly, monthly or quarterly) 
# regular time series into an ANNUAL one

# 'x'      : zoo/xts object which values will be converted into annual ones
# 'FUN'    : Function that have to be applied for aggregating into Annual time step
#            For Precipitation FUN MUST be 'sum'
#            For Temperature and Flow time series, FUN MUST be 'mean'
# 'na.rm'  : TRUE : the annual mean  value is computed considering only those values different from NA
#            FALSE: if there is AT LEAST one NA within a year, the monthly mean value is NA
# 'out.fmt': character indicating the format for the output time series. Possible values are:
#            -) "%Y"      : only the year will be used for the time. Default option. (e.g., "1961" "1962"...)
#            -) "%Y-%m-%d": a complete date format will be used for the time. Default option. (e.g., "1961" "1962"...)

subdaily2annual <-function(x, ...) UseMethod("daily2annual")

# File annualfunction.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# annualfunction: Generic function for computing monthly totals/mean values    #
#                 for a zoo object or data.frame                               #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
#  May 15th, 2009; Sep 01st 2009 ;                                             # 
#  06-Apr-2013                                                                 #
################################################################################
# 'x   '  :  daily, monthly or annual 'zoo' or 'data.frame' object
# 'FUN'   :  Function that will be applied to ALL the values in 'x' belonging to each weather season of the year
#             (e.g., Fun can be some of c('mean', 'max', 'min', 'sd'))
# 'na.rm' : Logical. Should missing values be removed?
#              TRUE : the annual values are computed considering only those values different from NA
#              FALSE: if there is AT LEAST one NA within a year, the annual values are NA
annualfunction <- function(x, FUN, na.rm=TRUE,...) UseMethod("annualfunction")

annualfunction.default <- function(x, FUN, na.rm=TRUE,...) {

     # Checking that 'x' is a zoo object
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

     annualfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)

} # 'annualfunction.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 09-Aug-2011                                                         #
# Updates: 09-Aug-2011                                                         #
#          03-Abr-2013                                                         #
#          29-Nov-2015                                                         #
################################################################################
annualfunction.zoo <- function(x, FUN, na.rm=TRUE,...) {

     # If the user did not provide a title for the plots, this is created automatically
     if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")

     # Annual index for 'x'
     dates  <- time(x)
     y      <- as.numeric(format( dates, "%Y" ))
     years  <- factor( y, levels=unique(y) )

     # 'FUN' is first applied to all the values of 'x' belonging to the same year
     totals <- aggregate( x, by= years, FUN=FUN, na.rm= na.rm, ... )
     
     #  'FUN' is applied to all the previously computed annual values to get the final result.
     if ( (is.matrix(x)) | (is.data.frame(x)) ) {
       #totals <- apply(totals, MARGIN=2, FUN=FUN, an.rm=na.rm) # up to version 0.3-6
       totals <- apply(totals, MARGIN=2, FUN=FUN, na.rm=na.rm, ...) # since version 0.3-7
     } else totals <- aggregate(totals, by = rep("value", length(totals)), FUN = FUN, na.rm = na.rm, ...)
     
     # Replacing the NaNs by 'NA.
     # NaN's are obtained when using the FUN=mean with complete NA values
     nan.index          <- which(is.nan(totals))
     if ( length(nan.index) > 0 )  totals[ nan.index] <- NA
     
     # Replacing all the Inf and -Inf by NA's
     # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
     inf.index <- which(is.infinite(totals))
     if ( length(inf.index) > 0 ) totals[inf.index] <- NA

     # Giving meaningful names to the output
     if ( (is.matrix(x)) | (is.data.frame(x)) ) {
       totals <- t(totals) # For having the years as column names
     } # IF end

     return(totals)

} # 'annualfunction.zoo' end


############################################################################
# annualfunction: Generic function for computing monthly totals/mean values#
#                 for a zoo object or data.frame                           #
############################################################################
# Started: 15-May-2009                                                     #
# Updates: 01-Sep-2009 ; 09-Aug-2011                                       #
#          29-May-2013 ; 03-Jun-2013                                       #
#          22-Aug-2022                                                     #
############################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the 
#             dates correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in 
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class, 
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the 
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with as many columns as stations 
#                              are included in 'x', and an additional column indicating the Year
#             -) "db"        : a data.frame, with 3 colums will be produced.
#                              The first column will store the Year, 
#                              The second column will store the ID of the station,
#                              The third column will contain the seasonal 
#                                value corresponding to that year and that station.
# 'verbose' : logical; if TRUE, progress messages are printed 
annualfunction.data.frame <- function(x, FUN, na.rm=TRUE,
                                      dates=1, date.fmt="%Y-%m-%d",
                                      verbose=TRUE,...) {
	  
  # Checking that the user provied a valid argument for 'FUN'
  if (missing(FUN))
    stop("Missing argument: 'FUN' must be provided")

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
     stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")
  
  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates,  "numeric" ) ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt)
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt)
  
  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  x <- zoo(x, dates)
  
  ##############################################################################
  annualfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)

  
} #'annualfunction.data.frame' END


############################################################################
# annualfunction: Generic function for computing monthly totals/mean values#
#                 for a zoo object or data.frame                           #
############################################################################
# Started: 2009-May-15                                                     #
# Updates: 2009-Sep-01st ; 2011-Aug-09                                     #
#          29-May-2013                                                     #
############################################################################
annualfunction.matrix <- function(x, FUN, na.rm=TRUE,
                                  dates=1, date.fmt="%Y-%m-%d",
                                  verbose=TRUE,...) {
 
 x <- as.data.frame(x)
 #NextMethod("daily2annual")
 annualfunction.data.frame(x=x, FUN=FUN, na.rm=na.rm,
                           dates=dates, date.fmt=date.fmt,
                           verbose=verbose,...)
                                                                 
} # 'annualfunction.matrix' END

# File si.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
#                                 https://www.rforge.net/hydroTSM/
# Copyright 2023-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                      'si' : seasonality index                                #  
################################################################################       
# Purpose: Given a zoo object with daily/monthly precipitation, it computes    #
#          the seasonality index, as:                                          #
#               si = (1/R) *sum(i=1, i=12, abs(xi - R/12) )                    #
#          where:                                                              #
#            -) xi: mean monthly precipitation for month i                     #
#            -) R: mean annual precipitation                                   #    
################################################################################
# Reference:                                                                   #
# Walsh, R., & Lawler, D. (1981). Rainfall seasonality: Description, spatial   #
# patterns and change through time (British Isles, Africa). Weather, 36(7),    #
# 201-208. doi:10.1002/j.1477-8696.1981.tb05400.x                              #
################################################################################
# This index can theoretically vary from 0 (when all months have the same      #
# rainfall) to 1.83 (when all the rainfall ocurrs in a single month)           #
# A qualitative classification of degrees of seasonality is the following:     #
# -----------------------------------------------------------------------------#
#  si values   |                    Rainfall regime                            #
# -----------------------------------------------------------------------------#
#     <= 0.19  | Very equable                                                  #
# 0.20 - 0.39  | Equable but with a definite wetter season                     #
# 0.40 - 0.59  | Rather seasonal with a short drier season                     #
# 0.60 - 0.79  | Seasonal                                                      #
# 0.80 - 0.99  | Markedly seasonal with a long drier season                    #
# 1.00 - 1.19  | Most rain in 3 months or less                                 #
#     >= 1.20  | Extreme, almost all rain in 1-2 months                        #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
# Started: 28-Nov-2023                                                         #
# Updates: 15-Jan-2024                                                         #
################################################################################


si <- function(x, na.rm=TRUE, from=start(x), to=end(x), 
               date.fmt="%Y-%m-%d", start.month=1) {

  .shift <- function(x, imonth) {
    L <- length(x)
    if (imonth>L) stop("[ Invalid value: 'imonth' can not be larger than ", L, " !]")
    delta <- imonth-1
    index.old <- 1:L
    index.new <- index.old-delta
    neg <- which(index.new <=0)
    index.new[neg] <- index.new[neg]+L
    if (inherits(x, "zoo")) {
      x.raw    <- zoo::coredata(x)
      x.labels <- as.character(time(x))
      out        <- x.raw[match(index.old, index.new)] 
      names(out) <- x.labels[match(index.old, index.new)] 
    } else out <- x[match(index.old, index.new)] 
    return(out)
  } # .shift END


  if (missing(x)) {
    stop("Missing argument: 'x' must be provided !")
  } else {
      # Checking that 'x' is a zoo object
      if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !)")

      # Checking that 'x' is a subdaily or monthly object
      if ( sfreq(x) %in% c("quarterly", "annual")) 
        stop("Invalid argument: 'sfreq(x)' must be in c('minute', 'hourly', 'daily', 'weekly', 'monthly' !)")
    } # ELSE end

  ###########################################
  ## In case 'from' and 'to' are provided  ##
  dates.pcp  <- time(x)
     
  # Checking the validity of the 'from' argument
  if (!missing(from)) { 
     from <- as.Date(from, format=date.fmt)

     if (from < dates.pcp[1])
       stop("Invalid argument: 'from' is lower than the first date in 'x' !")

     x <- window(x, start=from)
   } # ELSE end

  # Checking the validity of the 'to' argument
  if (!missing(to)) { 
     to <- as.Date(to, format=date.fmt)

     if (to > dates.pcp[length(x)])
       stop("Invalid argument: 'to' is greater than the last date in 'x' !")

     if (to > dates.pcp[length(x)])

     x <- window(x, end=to)
   } # ELSE end


  # Detecting if 'x' is already mean monthly values (i.e., 12 values)
  pcp.is.mean.monthly <- FALSE

  if ( ( sfreq(x) == "monthly" ) & (length(x) == 12) ) {
    pcp.is.mean.monthly <- TRUE

    months   <- format(dates.pcp, "%b")
    x        <- as.numeric(x)
    names(x) <- months

    if (start.month != 1)
      x <- .shift(x=x, imonth=start.month)

    x.m.avg <- x 
  } # IF end

  ###########################################
  ## In case 'x' was not given as average monthly values
  if ( !pcp.is.mean.monthly ) {

    nyears <- yip(from=from, to=to, date.fmt="%Y-%m-%d", out.type="nmbr")

    # Computing mean monthly values of 'x'
    if ( (sfreq(x) != "monthly") | ( (sfreq(x) == "monthly") & ( length(x) > 12) ) )
      x.m.avg <- monthlyfunction(x, FUN=sum, na.rm=na.rm) / nyears

    # Shifting the monthly values when 'start.month != 1'
    if (start.month != 1)
      x.m.avg <- .shift(x=x.m.avg, imonth=start.month)
  } # IF end

  ###########################################
  # Computation of the mean annual rainfall
  R  <- sum(x.m.avg)  

  ###########################################
  # Computation of the seasonality index
  si <- (1/R) * sum( abs(x.m.avg - R/12) )

  return(si)

} # 'si' END

# File daily2annual.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#          daily2annual                                                        #
################################################################################
# Generic function for transforming a DAILY (sub-daily, weekly, monthly, quarterly) 
# regular time series into an ANNUAL one

# 'x'      : zoo/xts object which values will be converted into annual ones
# 'FUN'    : Function that have to be applied for aggregating into Annual time step
#            For Precipitation FUN MUST be 'sum'
#            For Temperature and Flow time series, FUN MUST be 'mean'
# 'na.rm'  : TRUE : the annual mean  value is computed considering only those values different from NA
#            FALSE: if there is AT LEAST one NA within a year, the monthly mean value is NA
# 'out.fmt': character indicating the format for the output time series. Possible values are:
#            -) "%Y"      : only the year will be used for the time. Default option. (e.g., "1961" "1962"...)
#            -) "%Y-%m-%d": a complete date format will be used for the time. Default option. (e.g., "1961" "1962"...)

daily2annual <-function(x, ...) UseMethod("daily2annual")

################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: XX-XXX-2008                                                         #
# Updates: 09-Aug-2011                                                         #
#          08-Apr-2013                                                         #
#          30-Jul-2023                                                         #
#          02-may-2025 (EGU 2025)                                              #
################################################################################
daily2annual.default <- function(x, FUN, na.rm=TRUE, na.rm.max=0, 
                                 out.fmt=c("%Y", "%Y-%m-%d"), 
                                 start.month=1, ...) {

     # Checking that 'x' is a zoo object
     if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !")

     daily2annual.zoo(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max, 
                      out.fmt=out.fmt, start.month=start.month, ...)
     
} # 'daily2annual.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 09-Aug-2011                                                         #
# Updates: 09-Aug-2011                                                         #
#          04-Jun-2012                                                         #
#          08-Apr-2013                                                         #
#          21-Jul-2015                                                         #
#          21-May-2022 ; 25-May-2022 ; 23-Dic-2022 ; 27-Dec-2022               #
#          20-Jun-2023 ; 30-Jul-2023                                           #
#          05-Feb-2025 (EGU 2025)                                              #
################################################################################
daily2annual.zoo <- function(x, FUN, na.rm=TRUE, na.rm.max=0, 
                             out.fmt=c("%Y", "%Y-%m-%d"), 
                             start.month=1, ...) {


  get.dates <- function(x, years, fun, fn.name) {

    is.subdaily <- ( (inherits(time(x), "POSIXct")) | (inherits(time(x), "POSIXlt")) )
    is.POISXct  <- inherits(time(x), "POSIXct")

    years.unique <- as.numeric(unique(years))
    nyears       <- length(years.unique)

    if ( (fn.name=="max") | (fn.name=="min")) {

      if (fn.name=="max") {
        datetimes.pos <- aggregate(x, by=years, FUN=which.max)
      } else datetimes.pos <- aggregate(x, by=years, FUN=which.min)

      datetimes.pos <- as.numeric(datetimes.pos)

      # Getting the datetime where the min/max value occurs for each year in 'x'.
      # this is a list object
      xMaxMin <- sapply(1:nyears, function(i, x, datetimes.pos) { 
                          all.dates.inyear <- x[ years == years.unique[i] ]
                          all.dates.inyear[ as.numeric( datetimes.pos[i] ) ]
                   }, x=x, datetimes.pos=datetimes.pos, simplify=FALSE)   

      # unblisting (and preserving datetime attrribute)
      xMaxMin <- do.call("c", xMaxMin)

      datetimes <- time(xMaxMin)

    } else if (is.subdaily) {
        datetimes <- paste0(years.unique, "-01-01 00:00:00")
        if (is.POISXct) {
          datetimes <- as.POSIXct(datetimes)
        } else datetimes <- as.POSIXlt(datetimes)
      } else datetimes <- as.Date(paste0(years.unique, "-01-01"))

    return( datetimes )
    
  } # 'get.dates' END


  # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN)) 
    stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values")
           
   # Checking the user provide a valid value for 'x'
  if (sfreq(x) %in% c("annual"))
    stop("Invalid argument: 'x' is already an annual ts !!" ) 

  # Checking 'out.fmt'
  out.fmt <- match.arg(out.fmt)

  # Checking that 'na.rm.max' is in [0, 1]
  if ( (na.rm.max < 0) | (na.rm.max > 1) )
    stop("Invalid argument: 'na.rm.max' must be in [0, 1] !")

  # Checking that 'na.rm.max' is in [1, 12]
  if ( (start.month < 1) | (start.month > 12) | 
       ( ( trunc(start.month) -  start.month ) != 0 ) )
    stop("Invalid argument: 'start.month' must be an integer in [1, 12] !")
	   
  # Getting the time index for each element in 'x'
  ltimes <- time(x)

  # Getting the year corresponding to each element in 'x'
  years  <- format( ltimes, "%Y")

  # Shifting backwards the year each element in 'x', 
  # only when start.month != 1
  if ( start.month != 1 )
    years <- shiftyears(ltime=ltimes, lstart.month=start.month)

  na.index    <- is.na(x)
  x.nonNA     <- x[!na.index]
  years.nonNA <- years[!na.index]
  # Computing Annual time series
  tmp <- aggregate(x.nonNA, by=years.nonNA, FUN, na.rm=na.rm, ...)


  # Removing annual values in the output object for months with 
  # more than 'na.rm.max' percentage of NAs in a given year
  if ( na.rm ) {

    # Computing the percentage of missing values in each year
    na.pctg <- cmv(x.nonNA, tscale="annual", start.month=start.month)

    # identifying years with a percentage of missing values higher than 'na.rm.max'
    na.pctg.index <- which( na.pctg > na.rm.max)

    # Setting as NA all the years with a percentage of missing values higher than 'na.rm.max'
    tmp[na.pctg.index] <- NA 
  } # IF end


  # Replacing the NaNs by 'NA.
  # mean(NA:NA, na.rm=TRUE) == NaN
  nan.index <- which(is.nan(tmp))
  if ( length(nan.index) > 0 ) tmp[nan.index] <- NA
          
  # Replacing all the Inf and -Inf by NA's
  # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
  inf.index <- which(is.infinite(tmp))
  if ( length(inf.index) > 0 ) tmp[inf.index] <- NA

  # getting the name of the function as character
  fn.name <- substitute(FUN)

  na.index    <- is.na(x)
  x.nonNA     <- x[!na.index]
  years.nonNA <- years[!na.index]

  # date format for the output annual series:
  if ( (fn.name=="max") | (fn.name=="min")) {
    if (NCOL(tmp) == 1) {
      ltimes <- get.dates(x.nonNA, fun=FUN, years=years.nonNA, fn.name=fn.name)
      out    <- zoo::zoo(tmp, ltimes)
    } else { # NCOL(tmp) > 1
        out   <- vector("list", NCOL(tmp))
        for (i in 1:NCOL(tmp)) {
          ltimes   <-  get.dates(x.nonNA, fun=FUN, years=years.nonNA, fn.name=fn.name)
          out[[i]] <-  zoo(tmp[,i], ltimes)
        } # FOR end
      } # ELSE end
  } else out <- tmp

  # Checking the format of the time attribute of the output object
  if ( out.fmt == "%Y-%m-%d" )
    time(out) <- as.Date( paste0( time(out), "-01-01") )

  return(out)

} # 'daily2annual.zoo' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: XX-XXX-2008                                                         #
# Updates: 09-Aug-2011                                                         #
#          04-Jun-2012                                                         #
#          29-May-2013                                                         #      
#          22-Aug-2022                                                         #
#          20-Jun-2023 ; 30-Jul-2023                                           #
#          02-May-2025 (EGU 2025) ; 07-Aug-205                                 #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': character indicating the format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with as many columns as stations
#                              are included in 'x', and an additional column indicating the Year
#             -) "db"        : a data.frame, with 3 colums will be produced.
#                              The first column will store the Year,
#                              The second column will store the ID of the station,
#                              The third column will contain the seasonal
#                                value corresponding to that year and that station.
# 'verbose' : logical; if TRUE, progress messages are printed
daily2annual.data.frame <- function(x, FUN, na.rm=TRUE, na.rm.max=0, 
                                    out.fmt=c("%Y", "%Y-%m-%d"), 
                                    start.month=1, 
                                    dates=1, date.fmt="%Y-%m-%d",
                                    out.type="data.frame",
                                    verbose=TRUE, ...) {
                                    
  # Checking that the user provied a valid argument for 'out.type'
  if (is.na(match( out.type, c("data.frame", "db") ) ) )
      stop("Invalid argument: 'out.type' must be in c('data.frame', 'db'")
  
  # Checking 'out.fmt'
  out.fmt <- match.arg(out.fmt)

  # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN))
    stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values")

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt) # zoo::as.Date
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt) # zoo::as.Date

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
     
  # Transforming 'x' into a zoo object
  x <- zoo(x, dates)
  
  ##############################################################################
  #if (out.type == "data.frame") {
  
  #  z <- daily2annual.zoo(x=x, FUN=FUN, ..., na.rm=na.rm, 
  #                        na.rm.max=na.rm.max, out.fmt=out.fmt, 
  #                        start.month=start.month)
    
  #} else if (out.type == "db") { 

       if (verbose) message("[Starting computations...]")
       
       # Amount of stations in 'x'
       nstations <- ncol(x)

       # ID of all the stations in 'x'
       snames <- colnames(x)

       # Computing the Starting and Ending Year of the analysis
       Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
       Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))

       # Amount of Years belonging to the desired period
       #nyears <- Ending.Year - Starting.Year + 1

       # Computing the amount of years with data within 'x'
       ndays    <- length(dates) # number of days in the period
       tmp      <- vector2zoo(rep(0, ndays), dates)
       tmp      <- daily2annual.zoo(x= tmp, FUN=FUN, ..., na.rm=na.rm, 
                                    na.rm.max=na.rm.max, out.fmt="%Y-%m-%d", 
                                    start.month=start.month)
       nyears   <- length(tmp) #number of years in the period

       # Generating a string vector with the years effectively within 'x'
       if (out.fmt != "%Y") {
          chryears <- time(tmp)
       } else chryears <- format(time(tmp), "%Y")
 
       # Creating a vector with the names of the field that will be used for storing the results
       field.names <- c("StationID", "Year", "Value" )

       # Creating the data.frame that will store the computed averages for each subcatchment
       z <- as.data.frame(matrix(data = NA, nrow = nyears*nstations, ncol = 3,
                           byrow = TRUE, dimnames = NULL) )
       colnames(z) <- field.names

       for (j in 1:nstations) {

           if (verbose) message( "Station: ", format(snames[j], width=10, justify="left"),
                                 " : ",format(j, width=3, justify="left"), "/",
                                 nstations, " => ",
                                 format(round(100*j/nstations,2), width=6, justify="left"),
                                 "%" )

          # Computing the annual values
          a <- daily2annual.zoo(x= x[,j], FUN=FUN, ..., na.rm=na.rm, 
                                na.rm.max=na.rm.max, out.fmt="%Y-%m-%d", 
                                start.month=start.month)

          # Putting the annual/monthly values in the output data.frame
          # The first column of 'x' corresponds to the Year
          row.ini <- (j-1)*nyears + 1
          row.fin <-  j*nyears

          z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'nmonths' times
          z[row.ini:row.fin, 2] <- format(as.Date(time(a)), "%Y") # zoo::as.Date
          z[row.ini:row.fin, 3] <- a

      } # FOR end

  #   } # ELSE end

  return( z )

 } #'daily2annual.data.frame' END


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: XX-XXX-2008                                                         #
# Updates: 09-Aug-2011                                                         #
#          29-May-2013                                                         # 
#          30-Jul-2023                                                         #  
################################################################################
daily2annual.matrix  <- function(x, FUN, na.rm=TRUE, na.rm.max=0, 
                                 out.fmt=c("%Y", "%Y-%m-%d"), 
                                 start.month=1,
                                 dates=1, date.fmt="%Y-%m-%d",
                                 out.type="data.frame",
                                 verbose=TRUE,...) {

   x <- as.data.frame(x)
   #NextMethod("daily2annual")
   daily2annual.data.frame(x=x, FUN=FUN, na.rm=na.rm, na.rm.max=na.rm.max,
                           out.fmt=out.fmt, start.month=start.month,
                           dates=dates, date.fmt=date.fmt,
                           out.type=out.type,
                           verbose=verbose, ...)

} # 'daily2annual.matrix  ' END

# File shiftyears.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2023-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# shiftyears: Shifting years backwards for correct computation of annual       # 
#             values when the hydrological year does not start in january      #
################################################################################
# This function shift backwards a vector of dates/times (Date/POSIXct/POSIXlt),#
# by 'lstart.month' (number representing the starting month to be used in the  #
# computation of annual values, by default 1:Jan)                              #
################################################################################
# This is an internal function, just developed to be used by some function that#
# needs to compute annual values with a year starting in a month different     #
# from January.                                                                #
################################################################################
# Output: numeric vector representing the year corresponing to each element of 
#         'ltime', considering 'lstart.month' as the starting month for annual
#         computations
shiftyears <- function(ltime,       # vector with the date/times of each element 
                                    # of a zoo object
                       lstart.month # numeric in [2,..,12], 2:Feb, 12:Dec, 
                                    # It represents the starting month to be 
                                    # used in the computation of annual values.
                       ) {
  syears.bak        <- as.numeric(format( ltime, "%Y" ))
  syears            <- syears.bak
  smonths           <- as.numeric(format( ltime, "%m"))
  months2moveback   <- 1:(lstart.month-1)
  N                 <- length(months2moveback)
  for (i in 1:N) {
    m.index          <- which(smonths == months2moveback[i])
    syears[m.index]  <- syears[m.index] - 1
  } # FOR end

  return(syears)
} # 'shiftyears' END

# File weeklyfunction.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2023-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# weeklyfunction: Generic function for applying any R function to              #
#                  ALL the values in 'x' belonging to a given week             #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 12-Oct-2023                                                         #
# Updates: 31-Oct-2025                                                         #
################################################################################
# 'x   '    : variable of type 'zoo' or 'data.frame', with daily or monthly frequency
# 'FUN'      : Function that will be applied to ALL the values in 'x' belonging to each one of the 12 months of the year
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the monthly values  are computed considering only those values in 'x' different from NA
#              FALSE: if there is AT LEAST one NA within a month, the FUN and monthly values are NA
weeklyfunction <- function(x, ...) UseMethod("weeklyfunction")

weeklyfunction.default <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                   start.fmt= "%H:%M:%S", tz, ...) {

    # Checking that 'x' is a zoo object
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !!")

    weeklyfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, start=start, 
                       start.fmt=start.fmt, tz=tz, ...)

} # 'weeklyfunction.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 12-Oct-2023                                                         #
# Updates:                                                                     #
################################################################################
weeklyfunction.zoo <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                               start.fmt= "%H:%M:%S", tz, ...) {

    # Checking that the user provied a valid argument for 'FUN'
    if (missing(FUN)) stop("Missing argument: 'FUN' must be provided !")
     
    # Checking the user provide a valid value for 'x'
    if (sfreq(x) %in% c("weekly", "monthly", "quarterly", "annual"))
	    stop("Invalid argument: 'x' is not a sub-daily or daily ts. 'x' is a ", sfreq(x), " ts" )

    # Automatic detection of 'tz'
    #if (missing(tz)) tz <- ""
    if (missing(tz)) tz <- format(time(x), "%Z")[1]

    # Analysis of days different from 00:00 to 23:59 hrs
    if ( start != "00:00:00" ) {
      # Storing the original time
      time.old <- time(x)

      # Converting the new starting time provided by the user into a POSIXct object
      start <- as.POSIXct(start, format=start.fmt, tz=tz)

      # normal staring time for a day
      nstart <- as.POSIXct("00:00:00", format="%H:%M:%S", tz=tz)

      # time difference between the desired starting time 'strat' and the "normal"
      # starting time 'nstart', [s]
      delta <- difftime(start, nstart, units="secs")

      # Computing teh time difference between 'start' and the "normal" starting time, [s]
      #time.new <- as.POSIXct(time.old, tz=tz) - delta
      time.new <- time.old - delta

      # Changing the time in 'x' in 'delta' seconds
      time(x)  <- time.new
    } # IF end

     
    # Making sure that the time serie is complete before aggregation
    # This is useful when the first element of 'x' is not given at the time defined by 'start'.
    # For example, if the first element of 'x' starts at 08:00:00 hrs, but 'start=00:00:00', 
    # what happens with all the values from 00:00:00 to 07:59:59 hrs?
    # The following lines of code makes sure that the missing elements in a day are actually 
    # considered as missing

    if (sfreq(x) %in% c("min", "hourly") ) {
      st <- paste(format(start(x), "%Y-%m-%d"), "00:00:00", tz)
      et <- paste(format(end(x), "%Y-%m-%d"), "23:59:59", tz)
    } else {
        st <- start(x)
        et <- end(x)
      } # ELSE end
    x  <- izoo2rzoo(x, from=st, to=et, tz=tz)

    # Computing the Weekly time series 
    totals <- aggregate(x, by= function(tt) format(tt, "%W"), FUN=FUN, na.rm= na.rm, ...)

     

    # Replacing the NaNs by 'NA.
    # NaN's are obtained when using the FUN=mean with complete NA values
    nan.index          <- which(is.nan(totals))
    if ( length(nan.index) > 0 )  totals[ nan.index] <- NA
     
    # Replacing all the Inf and -Inf by NA's
    # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
    inf.index <- which(is.infinite(totals))
    if ( length(inf.index) > 0 ) totals[inf.index] <- NA

    # Giving meaningful names to the output
    if ( (is.matrix(x)) | (is.data.frame(x)) ) {
      totals <- t(totals) # For having the months' names as column names
      colnames(totals) <- levels(months)
    } #IF end

    return(totals)

} # 'weeklyfunction.zoo' end
 
 

################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2011                                                         #
# Updates: 08-Aug-2011                                                         #
#          29-May-2013 ; 03-Jun-2013                                           #
#          23-Aug-2022                                                         #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with 12 columns representing the months,
#                              and as many rows as stations are included in 'x'
#             -) "db"        : a data.frame, with 4 colums will be produced.
#                              The first column stores the ID of the station
#                              The second column stores the Year,
#                              The third column stores the ID of the station,
#                              The fourth column contains the monthly value corresponding to the year specified in the second column
# 'verbose'      : logical; if TRUE, progress messages are printed
weeklyfunction.data.frame <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                      start.fmt= "%H:%M:%S", tz, 
                                      dates=1, date.fmt="%Y-%m-%d",
                                      out.type="data.frame",
                                      verbose=TRUE,...) {

  # Checking that the user provied a valid argument for 'out.type'
  if (is.na(match( out.type, c("data.frame", "db") ) ) )
    stop("Invalid argument: 'out.type' must be in c('data.frame', 'db'")

  # Checking that the user provied a valid argument for 'FUN'
  if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
     stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt)
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt)
  
  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # Transforming 'x' into a zoo object
  x <- zoo(x, dates)
  
  ##############################################################################
  if (out.type == "data.frame") {
  
    weeklyfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)
    
  } else if (out.type == "db") {
  
        # Amount of stations in 'x'
        nstations <- ncol(x)
        
        # ID of all the stations in 'x'
        snames <- colnames(x)
      
        if (is.null(snames)) snames <- paste("V", 1:nstations, sep="")
  
        # Computing the Starting and Ending Year of the analysis
        Starting.Year <- as.numeric(format(start(x), "%Y"))
        Ending.Year   <- as.numeric(format(end(x), "%Y"))

        # Amount of Years belonging to the desired period
        nyears <- Ending.Year - Starting.Year + 1
        
        # Computing the numeric index of the resulting months
        month.index <- unique(as.numeric(format( time(x), "%m" )))
     
        # Computing the amount of weeks with data within the desired period
        ndays   <- length(dates) # number of days in the period
        tmp     <- vector2zoo(rep(0,ndays), dates)
        tmp     <- weeklyfunction.default(x= tmp, FUN=FUN, na.rm=na.rm)
        nweeks  <- length(tmp)

        # Creating a vector with the names of the field that will be used for storing the results
        field.names <- c("StationID", "Year", "Week", "Value" )

        # Creating the data.frame that will store the computed averages for each subcatchment
        z <- as.data.frame(matrix(data = NA, nrow = nweeks*nstations, ncol = 4,
                           byrow = TRUE, dimnames = NULL) )
        colnames(z) <- field.names

        
        for (j in 1:nstations) {

          if (verbose) message( "[ Station: ", format(snames[j], width=10, justify="left"),
                                " : ", format(j, width=3, justify="left"), "/",
                                nstations, " => ",
                                format(round(100*j/nstations,2), width=6, justify="left"),
                                "% ]" )

	        # Computing the annual values
	        tmp <- weeklyfunction.default(x= x[,j], FUN=FUN, na.rm=na.rm, ...)

	        # Putting the annual/monthly values in the output data.frame
          # The first column of 'x' corresponds to the Year
          row.ini <- (j-1)*nweeks + 1
          row.fin <-  j*nweeks

          z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'nweeks' times
          z[row.ini:row.fin, 2] <- rep(Starting.Year:Ending.Year, each=nweeks)
          z[row.ini:row.fin, 3] <- time(tmp)
          z[row.ini:row.fin, 4] <- tmp

        } # FOR end
        
        colnames(z) <- field.names
        
        return( z )
      
    } # ELSE end

 } #'weeklyfunction.data.frame' END
 
 
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 12-Oct-2023                                                         #
# Updates:                                                                     #
################################################################################
weeklyfunction.matrix <- function(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                  start.fmt= "%H:%M:%S", tz, 
                                  dates=1, date.fmt="%Y-%m-%d",
                                  out.type="data.frame",
                                  verbose=TRUE,...) {

 x <- as.data.frame(x)
 #NextMethod("weeklyfunction")
 weeklyfunction.data.frame(x=x, FUN=FUN, na.rm=na.rm,
                            dates=dates, date.fmt=date.fmt,
                            out.type=out.type,
                            verbose=verbose,...)
                                                                    
 } # 'weeklyfunction.matrix' END  

# File seasonalfunction.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2022 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                            seasonalfunction                                  #
################################################################################
# Generic function for applying any R function to summarize the seasonal values#
# of a time series                                                             #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Sep-2009                                                         #
################################################################################
# 'x   '    : variable of type 'zoo' or 'data.frame'
# 'FUN'      : Function that will be applied to ALL the values in 'x' belonging to each one of the 4 weather seasons
#              (e.g., Fun can be some of c('mean', 'max', 'min', 'sd'))
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the monthly values  are computed considering only those values in 'x' different from NA
#              FALSE: if there is AT LEAST one NA within a month, the FUN and monthly values are NA
seasonalfunction <- function(x, ...) UseMethod("seasonalfunction")


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Sep-2009                                                         #
# Updates: 08-Aug-2011                                                         #
#          21-May-2013                                                         #
################################################################################
seasonalfunction.default <- function(x, FUN, na.rm=TRUE, type="default",...) {

     # Checking that 'x' is a zoo object
     if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

     seasonalfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, type=type, ...)

} # 'seasonalfunction.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 08-Aug-2011                                                         #
# Updates: 08-Aug-2011                                                         #
#          03-Abr-2013                                                         #
#          29-Nov-2015                                                         #
#          22-Oct-2022                                                         #
################################################################################
seasonalfunction.zoo <- function(x, FUN, na.rm=TRUE, type="default", ...) {

     # Checking that the user provied a valid argument for 'FUN'
     if (missing(FUN))  stop("Missing argument: 'FUN' must be provided")
     
     # Checking the user provide a valid value for 'x'
     if (is.na(match(sfreq(x), c("daily", "monthly"))))
	     stop(paste("Invalid argument: 'x' is not a daily or monthly ts, it is a ", sfreq(x), " ts", sep="") )
	
     # Checking that the user provied a valid value for 'type'   
     valid.types <- c("default", "FrenchPolynesia")    
     if (length(which(!is.na(match(type, valid.types )))) <= 0)  
       stop("Invalid argument: 'type' must be in c('default', 'FrenchPolynesia')")

     # Time index of 'x'
     if (type=="default") { 
       seasons.lab <- c("DJF",  "MAM", "JJA", "SON")
     } else if (type=="FrenchPolynesia") { 
         seasons.lab <- c("DJFM", "AM",  "JJAS", "ON")
       } # ELSE end       
     dates   <- time(x)
     seasons <- factor( time2season( dates, type=type ), levels=seasons.lab )
     
     # zoo::aggregate
     s <- aggregate(x, by= seasons, FUN=FUN, na.rm= na.rm, ...)

     # Replacing the NaNs by 'NA.
     # NaN's are obtained when using the FUN=mean with complete NA values
     nan.index          <- which(is.nan(s))
     if ( length(nan.index) > 0 )  s[ nan.index] <- NA
     
     # Replacing all the Inf and -Inf by NA's
     # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
     inf.index <- which(is.infinite(s))
     if ( length(inf.index) > 0 ) s[inf.index] <- NA
     
     # Giving meaningful names to the output
     if ( (is.matrix(x)) | (is.data.frame(x)) ) {
       # Getting the name of the actual seasons in 's'
       cnames <- time(s)

       # Transformation needed in order to change the default names of the result
       s <- coredata(s)
     
       s <- t(s) # For having the season' names as column names

       # Giving the name of the seasons
       colnames(s) <- cnames
     } # IF end

     return(s)

} # 'seasonalfunction.zoo' end



################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Sep-2009                                                         #
# Updates: 08-Aug-2011                                                         #
#          03-Jun-2013                                                         #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.type': string that define the desired type of output. Possible values are
#             -) "data.frame": a data.frame, with as many columns as stations
#                              are included in 'x', and an additional column indicating the Year
#             -) "db"        : a data.frame, with 3 colums will be produced.
#                              The first column will store the Year,
#                              The second column will store the ID of the station,
#                              The third column will contain the seasonal
#                                value corresponding to that year and that station.
# 'verbose'      : logical; if TRUE, progress messages are printed
seasonalfunction.data.frame <- function(x, FUN, na.rm=TRUE, type="default",
                                        dates=1, date.fmt="%Y-%m-%d",
                                        out.type="data.frame",
                                        verbose=TRUE,...) {

  # Checking that the user provied a valid argument for 'out.type'
  if (is.na(match( out.type, c("data.frame", "db") ) ) )
    stop("Invalid argument: 'out.type' must be in c('data.frame', 'db'")

  # Checking that the user provied a valid argument for 'FUN'
  if (missing(FUN))
    stop("Missing argument: 'FUN' must be provided")
       
  # Checking that the user provied a valid value for 'type'   
  valid.types <- c("default", "FrenchPolynesia")    
  if (length(which(!is.na(match(type, valid.types )))) <= 0)  
     stop("Invalid argument: 'type' must be in c('default', 'FrenchPolynesia')")
  
  # Defining 'seasons.lab'
  if (type=="default") { 
          seasons.lab <- c("DJF",  "MAM", "JJA", "SON")
  } else if (type=="FrenchPolynesia") { 
       seasons.lab <- c("DJFM", "AM",  "JJAS", "ON")
    } # ELSE end 

  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")
  
  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt)
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt)
  
  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
  
  # Transforming 'x' into zoo
  x <- zoo(x, dates)
     
  ##############################################################################
  if (out.type == "data.frame") {
  
    z      <- seasonalfunction.zoo(x=x, FUN=FUN, na.rm=na.rm, type=type, ...)
    
    snames      <- colnames(x)
    rownames(z) <- snames
    colnames(z) <- seasons.lab
    
  } else if (out.type == "db") {

        if (verbose) message("[Starting the computations...]")
        
        # Amount of stations in 'x'
        nstations <- ncol(x)

        # ID of all the stations in 'x'
        snames <- colnames(x)

        # Computing the Starting and Ending Year of the analysis
        Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
        Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))

        # Amount of Years belonging to the desired period
        nyears <- Ending.Year - Starting.Year + 1

        # Amount of months belonging to the desired period
        nmonths <- 12*nyears 

        # Creating a vector with the names of the field that will be used for storing the results
        field.names <- c("StationID", "Season", "Value" )

        # Creating the data.frame that will store the computed averages for each subcatchment
        z <- as.data.frame(matrix(data = NA, nrow = 4*nstations, ncol = 3,
                           byrow = TRUE, dimnames = NULL) )

        for (j in 1:nstations) {

            if (verbose) message( paste("[ Station: ", format(snames[j], width=10, justify="left"),
                                      " : ", format(j, width=3, justify="left"), "/",
                                      nstations, " => ",
                                      format(round(100*j/nstations,2), width=6, justify="left"),
                                      "% ]", sep="") )

            # Computing the annual values
	    tmp <- seasonalfunction.default(x= tmp, FUN=FUN, na.rm=na.rm, type=type)

	    # Putting the annual/monthly values in the output data.frame
            # The first column of 'x' corresponds to the Year
            row.ini <- (j-1)*4 + 1
            row.fin <-  j*4

            z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 4 times
            z[row.ini:row.fin, 2] <- seasons.lab
            z[row.ini:row.fin, 3] <- tmp

        } # FOR end

        colnames(z) <- field.names

    } # IF end

  return( z )

 } #'seasonalfunction.data.frame' END
 
 
 
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 11-Sep-2009                                                         #
# Updates: 08-Aug-2011                                                         #
################################################################################
 seasonalfunction.matrix <- function(x, FUN, na.rm=TRUE, type="default",
                                     dates=1, date.fmt="%Y-%m-%d",
                                     out.type="data.frame",
                                     verbose=TRUE,...) {
                                     
   x <- as.data.frame(x)
   #NextMethod("daily2annual")
   seasonalfunction.data.frame(x=x, FUN=FUN, na.rm=na.rm, type=type,
                               dates=dates, date.fmt=date.fmt,
                               out.type=out.type,
                               verbose=verbose,...)

} # 'seasonalfunction.matrix  ' END

# File shiftyears.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2025-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# .check_from_and_to_and_subset:                                               #
################################################################################
# This function checks the validity of the 'from' and 'to' arguments before    # 
# being used to subset a zoo object                                            #
################################################################################
# This is an internal function, just developed to be used by some functions    #
# that needs to perform this checking                                          #
################################################################################
# Output: zoo object obtined from window(x, start=from, end=to), with 'from' 
#         and 'to' checked in relation to 'x' and 'date.fmt'

check_from_and_to_and_subset <- function(x, from= start(x), to= end(x), 
                   date.fmt=NULL, tz=NULL) {

  # Date/Time of 'x'
  tx <- time(x)

  # sampling frequency of 'x'           
  x.freq <- sfreq(x)

  # Cheking if 'x is a sub-daily zoo object  
  if (x.freq %in% c("minute","hourly") ) {
    subdaily.ts <- TRUE
  } else subdaily.ts <- FALSE

  # Automatic detection of 'date.fmt'
  if ( is.null(date.fmt) ) {
    if ( subdaily.ts ) {
      date.fmt <- "%Y-%m-%d %H:%M:%S"
    } else date.fmt <- "%Y-%m-%d"
  } # IF end

  # Automatic detection of 'tz'
  missingTZ <- FALSE
  if (is.null(tz)) {
    missingTZ <- TRUE
    tz        <- attr(tx, "tzone")
  } else {
      if ( subdaily.ts ) {
        # For the Date/Time of 'x' to be in the time zone specified by 'tz'
        tx.new  <- timechange::time_force_tz(tx, tz=tz)
        time(x) <- tx.new
      } # IF end
    } # ELSE end
      
  ifelse ( grepl("%H", date.fmt, fixed=TRUE) | grepl("%M", date.fmt, fixed=TRUE) |
           grepl("%S", date.fmt, fixed=TRUE) | grepl("%I", date.fmt, fixed=TRUE) |
           grepl("%p", date.fmt, fixed=TRUE) | grepl("%X", date.fmt, fixed=TRUE), 
           subdaily.date.fmt <- TRUE, subdaily.date.fmt <- FALSE )

  # If the index of 'x' is character, it is converted into a Date object
  if ( is.factor(x) | is.character(x) )
    ifelse(subdaily.date.fmt, time(x) <- as.POSIXct(time(x), format=date.fmt),
                              time(x) <- as.Date(time(x), format=date.fmt) )
      
  
  # If 'from' was given as Date, but 'x' is sub-daily
  if (!missing(from)) {

    if ( from > to ) stop("Invalid argument: 'from > to' !")

    if ( from < start(x) ) stop("Invalid argument: 'from < start(x)' !")

    if ( from > end(x) ) stop("Invalid argument: 'from > end(x)' !")

    if ( subdaily.date.fmt & !(grepl(":", from, fixed=TRUE) ) )
      from <- paste(from, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      from <- as.POSIXct(from, tz=tz)
  } # IF end

  # If 'to' was given as Date, but 'x' is sub-daily
  if (!missing(to)) {
    if ( to < from ) stop("Invalid argument: 'to < from' !")

    if ( to > end(x) ) stop("Invalid argument: 'to > end(x)' !")

    if ( to < start(x) ) stop("Invalid argument: 'to < start(x)' !")

    if ( subdaily.date.fmt & !(grepl(":", to, fixed=TRUE) ) )
      to <- paste(to, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      to <- as.POSIXct(to, tz=tz)
  } # IF end
        
  # checking that date.fmt and the sampling frequency of 'x' are compatible 
  if ( subdaily.ts ) {
    if (!subdaily.date.fmt) 
      stop("Invalid argument: 'date.fmt' (", date.fmt, 
           ") is not compatible with a sub-daily time series !!")
  } else {
           if (subdaily.date.fmt) {
             #time(x) <- as.POSIXct(time(x), tz=tz)
             time(x) <- as.POSIXct(time(x))
             warning("'date.fmt' (", date.fmt, ") is sub-daily, while 'x' is a '", 
                     x.freq, "' ts => 'time(x)=as.POSIXct(time(x), tz)'")
           } # IF end    
          } # ELSE end

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(from, format=date.fmt, tz=tz))
  } else dt <- try(as.Date(from, format=date.fmt))

  if ("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'from' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      from <- as.POSIXct(from, format=date.fmt, tz=tz)
    } else from <- as.Date(from, format=date.fmt)

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(to, format=date.fmt, tz=tz))
  } else dt <- try(as.Date(to, format=date.fmt))

  if ("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'to' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      to <- as.POSIXct(to, format=date.fmt, tz=tz)
    } else to <- as.Date(to, format=date.fmt)

  return ( window(x, start=from, end=to) )

} # END 'check_from_and_to_and_subset'

# File hydroplot.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 http://www.rforge.net/hydroTSM/ ; 
#                                 https://cran.r-project.org/package=hydroTSM
# Copyright 2008-2026 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

hydroplot <-function(x, ...) UseMethod("hydroplot")



################################################################################
# .hydroplotts Daily, Monthly and Annual Time Series                           #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
# Started: 2008                                                                #
# Updates: 17-Apr-2011 ; 10-Aug-2011                                           #
#          15-Jan-2014                                                         #
#          06-Aug-2017                                                         #
################################################################################
# It requires the function'drawTimeAxis' that is stored in the 'lib_Plot.R' library
# 'x'		 : daily time series of type 'zoo'
# 'x.monthly : monthly time series of type 'zoo'
# 'x.annual' : annual time series of type 'zoo'
# 'win.len1' : number of days for being used in the computation of the first moving average
# 'win.len2' : number of days for being used in the computation of the second moving average
# 'var.type' : string representing the type of variable being plotted (e.g., "Precipitation", "Temperature" or "Flow").
#              ONLY used for labelling the y axis and the title of the plot (in case it is missing)
# 'var.unit' : string representing the measurement unit of the variable being plotted ("mm" for precipitation, "C" for temperature, and "m3/s" for flow).
#              ONLY used for labelling the y axis
# 'main'     : string representing the main title of the plot
# 'pfreq'    : string indicating how many plots are desired by the user.
#              Valid values are:
#              -) 'dma': Daily, Monthly and Annual values are plotted
#              -) 'ma' : Monthly and Annual values are plotted
#              -) 'dm' : Daily and Monthly values are plotted
# 'tick.tstep': string indicating the time step that have to be used for
#               putting the ticks ont he time axis.
#               Possible values are: 'days', 'months', 'years'
# 'lab.tstep' : string indicating the time step that have to be used for
#               putting the labels ont he time axis.

.hydroplotts <- function(x, x.monthly, x.annual, win.len1=0, win.len2=0,
			 pfreq="dma", tick.tstep= "auto", lab.tstep= "auto", lab.fmt=NULL,
                         var.type, var.unit="units", main=NULL, xlab="Time", ylab=NULL, 
                         cex.main=1.3, cex.lab=1.3, cex.axis=1.3, col="blue", 
                         lwd=1, lty=1, ...) {

      # Checking that the user provied a valid class for 'x'   
      valid.class <- c("xts", "zoo")    
      if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
         stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")

      # Checking that 'x.monthly' is a zoo or xts object
      if ( length(x.monthly) > 1 ) {
        if (is.na(match(class(x.monthly), c("zoo", "xts"))))
            stop("Invalid argument: 'x.monthly' must be in c('zoo', 'xts')")
      } # IF end

      # Checking that 'x.annual' is a zoo or xts object
      if ( length(x.annual) > 1 ) {
        if (is.na(match(class(x.annual), c("zoo", "xts"))))
            stop("Invalid argument: 'x.annual' must be be in c('zoo', 'xts')")
      } # IF end

      # Checking that the user provied a valid argument for 'pfreq'
      if (is.na(match(pfreq, c("o", "dma", "ma", "dm"))))
          stop("Invalid argument: 'pfreq' must be in c('o', 'dma', 'ma', 'dm')")
          
      # Valid tseps for ''tick.tstep' and 'lab.tstep' 
      valid.tstep <- c("auto", "years", "quarters", "months", "weeks", "days", 
                       "hours", "minutes", "seconds")

      # Checking that the user provied a valid argument for 'tick.tstep'
      if (is.na(match(tick.tstep, valid.tstep ) ) )
         stop("Invalid argument: 'tick.tstep' must be in c('auto', 'years', 'quarters',
              'months', 'weeks', 'days', 'hours', 'minutes', 'seconds')")

      # Checking that the user provied a valid argument for 'lab.tstep'
      if (is.na(match(lab.tstep, valid.tstep ) ) )
         stop("Invalid argument: 'lab.tstep' must be in c('auto', 'years', 'quarters',
              'months', 'weeks', 'days', 'hours', 'minutes', 'seconds')")

      # Requiring the Zoo Library (Zoo's ordered observations)
      #require(xts)

      # Booleans indicating if the moving averages for the dayly and monthly
      # time series can be computed and ploted. By default they  are FALSE,
      # and only if the lenght(x) is large enough they are changed into TRUE
      d.ma1 <- FALSE
      d.ma2 <- FALSE
      m.ma1 <- FALSE
      m.ma2 <- FALSE

      # Generating a Moving Average of the Daily time series, with a window width 'win.len1'
      if (win.len1 > 0 ) { # If 'win.len1==0', the moving average is not computed
          win.len <- win.len1
          if (length(x) >= win.len) {
            d.ma1 <- TRUE
            daily.ma1 <- ma.zoo(x, win.len) 
            #if (!is.xts(daily.ma1)) daily.ma1 <- as.xts(daily.ma1)
          }            
      } # IF end

      # Generating a Moving Average of the Daily time series, with a window width 'win.len2'
      if (win.len2 > 0 ) {  # If 'win.len2==0', the moving average is not computed
          win.len <- win.len2
          if (length(x) >= win.len) {
            d.ma2 <- TRUE
            daily.ma2 <- ma.zoo(x, win.len)
            #if (!is.xts(daily.ma2)) daily.ma2 <- as.xts(daily.ma2)
          }            
      } # IF end

      # Generating a Moving Average of the Monthly time series, with a window width 'win.len1'
      if (win.len1 > 0 ) { # If 'win.len1==0', the moving average is not computed
        win.len <- round(win.len1/365,1)*12
        if (length(x.monthly) >= win.len) {
          m.ma1 <- TRUE
          monthly.ma1 <- ma.zoo( x.monthly, win.len )  }
      } # IF end

      # Generating a Moving Average of the Monthly time series, with a window width 'win.len2'
      if (win.len2 > 0 ) {  # If 'win.len2==0', the moving average is not computed
        win.len <- round(win.len2/365,1)*12
        if (length(x.monthly) >= win.len) {
          m.ma2 <- TRUE
          monthly.ma2 <- ma.zoo( x.monthly, win.len ) }        
      } # IF end
      
      # If 'x' is not 'xts' it is transformed into one
      #if ( !(is.xts(x)) ) x <- as.xts(x)

      # Plotting only the original zoo or xts object, without moving averages and legends
      if ( pfreq == "o") {

          # Plotting the Daily Time Series
          zoo::plot.zoo(x, xaxt = "n", yaxt = "n", type="o", 
                   main=main, xlab=xlab, ylab=ylab, 
                   cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                   lty=lty, lwd=lwd, ...)
          axis(2, cex.lab=1.3, cex.axis=1.3)
          # Draws monthly ticks in the X axis, but labels only in years
          drawTimeAxis(x, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                    cex.lab=cex.lab, cex.axis=cex.axis, ...)

          # manually adding a grid
          grid(nx=NA, ny=NULL)
          abline(v=time(x)[xts::axTicksByTime(x)], col = "lightgray", lty = "dotted")
      } # IF end


      # Plotting the Daily, if needed
      if ( pfreq %in% c("dma", "dm") ) {
          # Generating the title of the Daily Time Series plot
          title <- paste("Daily time series", main, sep= " ")
          # Plotting the Daily Time Series
          # xaxt = "n": is for avoiding drawing the x axis
          zoo::plot.zoo(x, xaxt = "n", yaxt = "n", type="o", 
                   main=title, xlab=xlab, ylab=paste(ylab," [", var.unit,"/day]", sep=""), 
                   cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                   lty=lty, lwd=lwd, ...)
          axis(2, cex.lab=cex.lab, cex.axis=cex.axis)
          # Draws monthly ticks in the X axis, but labels only in years
          drawTimeAxis(x, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                    cex.lab=cex.lab, cex.axis=cex.axis, ...)

          # manually adding a grid
          grid(nx=NA, ny=NULL)
          abline(v=time(x)[xts::axTicksByTime(x)], col = "lightgray", lty = "dotted")

          if (d.ma1) {
            # Plotting the 1st Moving Average of the Daily time series. If win.len1=365*1 => "Annual Moving Average"
            lines(daily.ma1, type="o", lty=2, lwd=1, col="green", cex = .5) }
          if (d.ma2) {
            # Plotting the 2nd Moving Average of the Daily time series. If win.len2=365*3 => "Moving Average of 3 Years"
            lines(daily.ma2, type="o", lty=3, lwd=1, col="red", cex = .5) }
          # Drawing a legend. y.intersp=0.5, is for the vertical spacin in the legend
          leg.text <- "Daily series"
          leg.lwd  <- lwd
          leg.lty  <- lty
          leg.col  <- col
          if (d.ma1) {
            leg.text <- c(leg.text, paste("MA(", round(win.len1/365,2), " years)", sep="") )
            leg.lwd  <- c(leg.lwd, 1)
            leg.lty  <- c(leg.lty, 2)
            leg.col  <- c(leg.col, "green")
          } # IF end
          if (d.ma2) {
            leg.text <- c(leg.text, paste("MA(", round(win.len2/365,2), " years)", sep="") )
            leg.lwd  <- c(leg.lwd, 1)
            leg.lty  <- c(leg.lty, 3)
            leg.col  <- c(leg.col, "red")
          } # IF end
          legend("topleft", leg.text, bty="n", cex =0.9, col= leg.col, lwd= leg.lwd, lty=leg.lty ) #bty="n" => no box around the legend
      } # IF end


      # Plotting the Monthly, if needed
      if ( pfreq %in% c("dma", "dm", "ma") ) {
        # Generating the title of the Monthly Time Series plot
        title <- paste("Monthly time series", main, sep= " ")
        # Plotting the Monthly time series
        zoo::plot.zoo(x.monthly, xaxt = "n", yaxt = "n", type="o", 
                 main=title, xlab=xlab, ylab=paste(ylab," [", var.unit,"/month]", sep=""),
                 cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                 lty=lty, lwd=lwd, ... )
        axis(2, cex.lab=cex.lab, cex.axis=cex.axis)


        # Draws monthly ticks in the X axis, but labels only in years
        drawTimeAxis(x.monthly, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                  cex.lab=cex.lab, cex.axis=cex.axis, ...)

        # manually adding a grid
        grid(nx=NA, ny=NULL)
        abline(v=time(x)[xts::axTicksByTime(x.monthly)], col = "lightgray", lty = "dotted")

        if (m.ma1) {
        # Plotting the 1st Moving Average of the Daily time series. If win.len1=365*1 => "Annual Moving Average"
        lines(monthly.ma1, type="o", lty=2, lwd=1, col="green", cex = .5) }
        if (m.ma2) {
        # Plotting the 2nd Moving Average of the Daily time series. If win.len2=365*3 => "Moving Average of 3 Years"
        lines(monthly.ma2, type="o", lty=3, lwd=1, col="red", cex = .5) }
        # Drawing a legend        
        leg.text <- "Monthly series"
        leg.lwd  <- lwd
        leg.lty  <- lty
        leg.col  <- col
        if (m.ma1) {
          leg.text <- c(leg.text, paste("MA(", round(win.len1/365,1), " years)", sep="") )
          leg.lwd  <- c(leg.lwd, 1)
          leg.lty  <- c(leg.lty, 2)
          leg.col  <- c(leg.col, "green")
        } # IF end
        if (m.ma2) {
          leg.text <- c(leg.text, paste("MA(", round(win.len2/365,1), " years)", sep="") )
          leg.lwd  <- c(leg.lwd, 1)
          leg.lty  <- c(leg.lty, 3)
          leg.col  <- c(leg.col, "red")
        } # IF end
        legend("topleft", leg.text, bty="n", cex =0.9, col= leg.col, lwd= leg.lwd, lty=leg.lty ) #bty="n" => no box around the legend
      } # IF end

      # Plotting the Annual, if needed
      if ( pfreq %in% c("dma", "ma") ) {
          # Generating the title of the Annual Time Series plot
           title <- paste("Annual time series", main, sep= " ")
          # Plotting the Annual time series
          zoo::plot.zoo(x.annual, xaxt = "n", yaxt = "n", type="o", 
                   main=title, xlab="Time", ylab=paste(ylab," [", var.unit,"/year]", sep=""),
                   cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                   lty=lty, lwd=lwd, ...)
          axis(2, cex.lab=cex.lab, cex.axis=cex.axis)
          # Draws monthly ticks in the X axis, but labels only in years
          drawTimeAxis(x.annual, tick.tstep="years", lab.tstep="years", lab.fmt="%Y",
                    cex.lab=cex.lab, cex.axis=cex.axis, ...)

          # manually adding a grid
          grid(nx=NA, ny=NULL)
          abline(v=time(x)[xts::axTicksByTime(x.annual)], col = "lightgray", lty = "dotted")
      } # IF end

} # '.hydroplotts' end



################################################################################
# BoxPlot of Daily, Monthly and Annual Time Serires                            #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 2008                                                                #
# Updates: 17-Apr-2011                                                         #
#          29-May-2013                                                         #
################################################################################
# 'x'		 : daily time series of type 'zoo'
# 'x.monthly : monthly time series of type 'zoo'
# 'x.annual' : annual time series of type 'zoo'
# 'var.type' : string representing the type of variable being plotted
#              (e.g., "Precipitation", "Temperature" or "Flow").
#              ONLY used for labelling the y axis and the title of the plot (in case it is missing)
# 'var.unit' : string representing the measurement unit of the variable
#              being plotted ("mm" for precipitation, "C" for temperature, and "m3/s" for flow).
#              ONLY used for labelling the y axis
# 'main'     : string representing the main title of the plot
# 'pfreq'    : string indicating how many plots are desired by the user.
#              Valid values are:
#              -) 'dma': Daily, Monthly and Annual values are plotted
#              -) 'dm' : Daily and Monthly values are plotted
#              -) 'ma' : Monthly and Annual values are plotted
#
.hydroplotboxplot <- function(x, x.monthly, x.annual, pfreq="dma",
                              var.type, var.unit="units", 
                              main=NULL, xlab=NULL, ylab=NULL,
                              cex.main=1.3, cex.lab=1.3, cex.axis=1.3, 
                              col="lightblue", 
                              ...
			      ) {

  # Checking that 'x' is a zoo or xts object
  if (is.na(match(class(x), c("zoo", "xts"))))
     stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

  # Checking that 'x.monthly' is a zoo object
  if (is.na(match(class(x.monthly), c("zoo", "xts"))))
     stop("Invalid argument: 'class(x.monthly)' must be in c('zoo', 'xts')")

  # Checking that 'x.annual' is a zoo object
  if (is.na(match(class(x.annual), c("zoo", "xts"))))
     stop("Invalid argument: 'class(x.annual)' must be in c('zoo', 'xts')")

  # Checking that the user provied a valid argument for 'pfreq'
  if (is.na(match(pfreq, c("dma", "ma", "dm"))))
      stop("Invalid argument: 'pfreq' must be in c('dma', 'ma', 'dm')")

 # Checking if the Daily Boxplot have to be plotted
 if ( pfreq %in% c("dma", "dm") ) {
   # Generating a factor based on the year in which each daily date falls
   cyear <- format(time(x), "%Y")
   years <- factor(cyear, levels=unique(cyear), ordered=TRUE)
   # Generating the title of the Daily plot
   title <- paste("Daily Boxplot", main, sep= " ")
   # Drawing boxplot of Daily values against Year
   boxplot( coredata(x)~years, main=title, ylab=paste(ylab," [", var.unit, "/day]", sep=""), 
            cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, ...)
 } # IF end

 # Checking if the Monthly Boxplot have to be plotted
 if ( pfreq %in% c("dma", "dm", "ma") ) {
   # Generating a factor based on the month in which each monthly date falls
   cmonth <- format(time(x.monthly), "%b")
   months <- factor(cmonth, levels=unique(cmonth), ordered=TRUE)
   # Generating the title of the Monthly plot
   title <- paste("Monthly Boxplot", main, sep= " ")
   # Drawing boxplot of Monthly values against Year
   boxplot( coredata(x.monthly)~months, main=title, ylab=paste(ylab," [", var.unit,"/month]", sep=""), 
            cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, ...)
 } # IF end

 # Checking if the Annual Boxplot have to be plotted
 if ( pfreq %in% c("dma", "ma") ) {
   # Generating the title of the Annual plot
   title <- paste("Annual Boxplot", main, sep= " ")
   # Drawing boxplot of Annual values against Year
   boxplot( coredata(x.annual), main=title, ylab=paste(ylab," [", var.unit, "/year]", sep=""), 
            cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, ...)
 } # IF end

} #'.hydroplotboxplot' end


################################################################################
# Histogram of Daily, Monthly and Annual Time Serires                          #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 2008                                                                #
# Updates: 17-Apr-2011                                                         #
#          29-May-2013                                                         #
################################################################################
# 'x'		 : daily time series of type 'zoo'
# 'x.monthly : monthly time series of type 'zoo'
# 'x.annual' : annual time series of type 'zoo'
# 'var.type' : string representing the type of variable being plotted (e.g., "Precipitation", "Temperature" or "Flow").
#              ONLY used for labelling the y axis and the title of the plot (in case it is missing)
# 'var.unit' : string representing the measurement unit of the variable being plotted ("mm" for precipitation, "C" for temperature, and "m3/s" for flow).
#              ONLY used for labelling the x axis
# 'main'     : string representing the main title of the plot
# 'pfreq'    : string indicating how many plots are desired by the user.
#              Valid values are:
#              -) 'dma': Daily, Monthly and Annual values are plotted
#              -) 'ma' : Monthly and Annual values are plotted
#              -) 'dm' : Daily and Monthly values are plotted
.hydroplothist <- function(x, x.monthly, x.annual, pfreq="dma",
                           var.type, var.unit="units", 
                           main=NULL, xlab=NULL, ylab=NULL, 
                           cex.main=1.3, cex.lab=1.3, cex.axis=1.3, col="lightblue", 
                           ...
			   ) {

      # Checking that 'x' is a zoo or xts object
      if (is.na(match(class(x), c("zoo", "xts"))))
        stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

      # Checking that 'x.monthly' is a zoo object
      if (is.na(match(class(x.monthly), c("zoo", "xts"))))
        stop("Invalid argument: 'class(x.monthly)' must be in c('zoo', 'xts')")

      # Checking that 'x.annual' is a zoo object
      if (is.na(match(class(x.annual), c("zoo", "xts"))))
        stop("Invalid argument: 'class(x.annual)' must be in c('zoo', 'xts')")

      # Checking that the user provied a valid argument for 'pfreq'
      if (is.na(match(pfreq, c("dma", "ma", "dm"))))
          stop("Invalid argument: 'pfreq' must be in c('dma', 'ma', 'dm')")

     # Checking if the Daily ts have to be plotted
     if ( pfreq %in% c("dma", "dm") ) {
       # Generating the title of the Daily plot
       title <- paste("Daily Histogram", main, sep= " ")
       # Drawing an histogram of Daily Precipitation
       hist(x, br=100, freq=FALSE, main=title, xlab=paste(ylab," [", var.unit, "/day]", sep=""), 
            ylab="Pbb", cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, ...)
     } # IF end

     # Checking if the Monthly ts have to be plotted
     if ( pfreq %in% c("dma", "dm", "ma") ) {
       # Generating the title of the Monthly plot
       title <- paste("Monthly Histogram", main, sep= " ")
       # Drawing an histogram of Monthly Precipitation
       hist(x.monthly, br=10, freq=FALSE, main=title, xlab=paste(ylab," [", var.unit, "/month]", sep=""), 
            ylab="Pbb", cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, ...)
     } # IF end

     # Checking if the Annual ts have to be plotted
     if ( pfreq %in% c("dma", "ma") ) {
       # Generating the title of the Annual plot
       title <- paste("Annual Histogram", main, sep= " ")
       # Drawing an histogram of Annual Precipitation
       hist(x.annual, br=5, freq=FALSE, main=title, xlab=paste(ylab," [", var.unit, "/year]", sep=""), 
            ylab="Pbb", cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, ...)
     } # IF end

} # '.hydroplothist' end


#########################################################################
# hydroplotseasonal: Seasonal plots of hydrological time series         #
#########################################################################
# Author : Mauricio Zambrano-Bigiarini                                  # 
# Started: 19-Jun-2011                                                  #
# Updates: 10-Aug-2011                                                  #
#          06-Aug-2017                                                  #
#          26-Apr-2026                                                  #
#########################################################################
.hydroplotseasonal <- function(x, FUN, na.rm=TRUE,
		               tick.tstep= "auto", lab.tstep= "auto", lab.fmt=NULL,
                               var.unit="units", main=NULL, xlab="Time", ylab=NULL, 
                               cex.main=1.3, cex.lab=1.3, cex.axis=1.3, col="blue", 
                               lwd=1, lty=1, stype="default", 
                               season.names=c("Winter", "Spring", "Summer", "Autumn"),
                               h=NULL, ...) {
      
      # checking the class of 'x'      
      if (!is.zoo(x))
         stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")
          
      # Valid tseps for ''tick.tstep' and 'lab.tstep' 
      valid.tstep <- c("auto", "years", "quarters", "months", "weeks", "days", 
                       "hours", "minutes", "seconds")

      # Checking that the user provied a valid argument for 'tick.tstep'
      if (is.na(match(tick.tstep, valid.tstep ) ) )
         stop("Invalid argument: 'tick.tstep' must be in c('auto', 'years', 'quarters',
              'months', 'weeks', 'days', 'hours', 'minutes', 'seconds')")

      # Checking that the user provied a valid argument for 'lab.tstep'
      if (is.na(match(lab.tstep, valid.tstep ) ) )
         stop("Invalid argument: 'lab.tstep' must be in c('auto', 'years', 'quarters',
              'months', 'weeks', 'days', 'hours', 'minutes', 'seconds')")
              
      # Checking that the user provied a valid value for 'stype'   
      valid.types <- c("default", "FrenchPolynesia")    
      if (length(which(!is.na(match(stype, valid.types )))) <= 0)  
        stop("Invalid argument: 'stype' must be in c('default', 'FrenchPolynesia')")

      # Checking that the user provied a valid argument for 'season.names'
      if ( length(season.names) != 4 )
         stop("Invalid argument: 'season.names' must have 4 elements !")

      # Checking the length of x
      if ( sfreq(x) == "daily" ) {
        if (length(x) < 365 )
          stop("Invalid argument: daily time series need -at least- 365 values !")
      } else if ( sfreq(x) == "monthly" ) {
          if (length(x) < 12 )
            stop("Invalid argument: monthly time series need -at least- 12 values !")
        } else if ( sfreq(x) == "annual" ) 
                 stop("Invalid argument: seasonal plots can not be drawn for annual time series !")    
      
      # Labels for the seasons
      if (stype=="default") { 
        seasons.lab <- c("DJF",  "MAM", "JJA", "SON")
      } else if (stype=="FrenchPolynesia") { 
          seasons.lab <- c("DJFM", "AM",  "JJAS", "ON")
        } # ELSE end 
       
      # Computing the seasonal values
      winter <- dm2seasonal(x, season=seasons.lab[1], FUN=FUN, out.fmt="%Y-%m-%d")
      spring <- dm2seasonal(x, season=seasons.lab[2], FUN=FUN, out.fmt="%Y-%m-%d")
      summer <- dm2seasonal(x, season=seasons.lab[3], FUN=FUN, out.fmt="%Y-%m-%d")
      autumm <- dm2seasonal(x, season=seasons.lab[4], FUN=FUN, out.fmt="%Y-%m-%d")

      # Transforming the seasonal values into xts objects
      #winter <- as.xts(winter)
      #spring <- as.xts(spring)
      #summer <- as.xts(summer)
      #autumm <- as.xts(autumm)


      #################################
      # Plotting seasonal time series #
      #################################
      def.par <- par(no.readonly = TRUE) # save default, for resetting... 
      # To ensure both graphical parameters (par) and layout are restored deterministically.
      on.exit({ 
        par(def.par)
        layout(1)
      })
      
      layout( matrix( c(1,1,1,1,1,1,1,1,1,5,5,2,2,2,2,2,2,2,2,2,6,6,3,3,3,3,3,3,3,3,3,7,7,4,4,4,4,4,4,4,4,4,8,8), ncol=11, byrow=TRUE) ) 
      
        if (length(h)==1) h <- rep(h,4)
        
        # winter
        zoo::plot.zoo(winter, xaxt = "n", yaxt = "n", type="o", 
                 main=paste(season.names[1], " (", seasons.lab[1], ")", sep=""), xlab=xlab, ylab=ylab, 
                 cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                 lty=lty, lwd=lwd, ...)
        axis(2, cex.lab=1.3, cex.axis=1.3)
        drawTimeAxis(winter, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                     cex.lab=cex.lab, cex.axis=cex.axis, ...)
        abline(h=h[1], col="red", lty=2)
        # manually adding a grid
        grid(nx=NA, ny=NULL)
        abline(v=time(x)[xts::axTicksByTime(winter)], col = "lightgray", lty = "dotted")
                
        # spring
        zoo::plot.zoo(spring, xaxt = "n", yaxt = "n", type="o", 
                 main=paste(season.names[2], " (", seasons.lab[2], ")", sep=""), xlab=xlab, ylab=ylab, 
                 cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                 lty=lty, lwd=lwd, ...)
        axis(2, cex.lab=1.3, cex.axis=1.3)
        drawTimeAxis(spring, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                     cex.lab=cex.lab, cex.axis=cex.axis, ...)
        abline(h=h[2], col="red", lty=2)
        # manually adding a grid
        grid(nx=NA, ny=NULL)
        abline(v=time(x)[xts::axTicksByTime(spring)], col = "lightgray", lty = "dotted")
                
        # summer
        zoo::plot.zoo(summer, xaxt = "n", yaxt = "n", type="o", 
                 main=paste(season.names[3], " (", seasons.lab[3], ")", sep=""), xlab=xlab, ylab=ylab, 
                 cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                 lty=lty, lwd=lwd, ...)
        axis(2, cex.lab=1.3, cex.axis=1.3)
        drawTimeAxis(summer, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                     cex.lab=cex.lab, cex.axis=cex.axis, ...)
        abline(h=h[3], col="red", lty=2)
        # manually adding a grid
        grid(nx=NA, ny=NULL)
        abline(v=time(x)[xts::axTicksByTime(summer)], col = "lightgray", lty = "dotted")
      
        # autumm
        zoo::plot.zoo(autumm, xaxt = "n", yaxt = "n", type="o", 
                 main=paste(season.names[4], " (", seasons.lab[4], ")", sep=""), xlab=xlab, ylab=ylab, 
                 cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                 lty=lty, lwd=lwd, ...)
        axis(2, cex.lab=1.3, cex.axis=1.3)
        drawTimeAxis(autumm, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt,
                     cex.lab=cex.lab, cex.axis=cex.axis, ...)
        abline(h=h[4], col="red", lty=2)
        # manually adding a grid
        grid(nx=NA, ny=NULL)
        abline(v=time(x)[xts::axTicksByTime(autumm)], col = "lightgray", lty = "dotted")
      
      #################################
      # Plotting seasonal boxplots    #
      #################################
        boxplot(coredata(winter), col= "lightblue", ylab = ylab, 
                main = paste(season.names[1], " (", seasons.lab[1], ")", sep=""),
                pars=list(cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis))
        abline(h=h[1], col="red", lty=2) 
        boxplot(coredata(spring), col= "lightblue", ylab = ylab, 
                main = paste(season.names[2], " (", seasons.lab[2], ")", sep=""),
                pars=list(cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis))
        abline(h=h[2], col="red", lty=2) 
        boxplot(coredata(summer), col= "lightblue", ylab = ylab, 
                main = paste(season.names[3], " (", seasons.lab[3], ")", sep=""),
                pars=list(cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis))
        abline(h=h[3], col="red", lty=2) 
        boxplot(coredata(autumm), col= "lightblue", ylab = ylab, 
                main = paste(season.names[4], " (", seasons.lab[4], ")", sep=""),
                pars=list(cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis))   
        abline(h=h[4], col="red", lty=2)  
                                
} # .hydroplotseasonal END


################################################################################
# hydroplot.zoo: Daily, Monthly and Annual plots of hydrological time series   #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 2008                                                                #
# Updates: 19-Apr-2011 ; 19-Jun-2011  ; 10-Aug-2011                            #
#          04-Jun-2012                                                         #
#          04-Apr-2013 ; 29-May-2013                                           #
#          07-Nov-2020                                                         #
#          31-Oct-2025                                                         #
################################################################################
hydroplot.default <- function(x, 
                              FUN, na.rm=TRUE,
                              ptype="ts+boxplot+hist",
                              pfreq="dma",                      
                              var.type,                      
                              var.unit="units",
                              main=NULL, xlab="Time", ylab,
                              win.len1=0,
                              win.len2=0,                      
                              tick.tstep="auto",
                              lab.tstep="auto",
                              lab.fmt=NULL,
                              cex=0.3,
                              cex.main=1.3,
                              cex.lab=1.3,
                              cex.axis=1.3,
                              col=c("blue", "lightblue", "lightblue"),
                              from=start(x), 
                              to=end(x),
                              dates=1, date.fmt = "%Y-%m-%d", tz=NULL, 
                              stype="default",
                              season.names=c("Winter", "Spring", "Summer", "Autumn"),
                              h=NULL, ...) {

    # Checking that 'x' is a zoo object
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be in c('zoo', 'xts')")

    hydroplot.zoo(x, FUN=FUN, na.rm=na.rm, ptype=ptype, pfreq=pfreq,                      
              var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,
              win.len1=win.len1, win.len2=win.len2, tick.tstep=tick.tstep, 
              lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex=cex, cex.main=cex.main, cex.lab=cex.lab,
              cex.axis=cex.axis, col=col, from=from, to=to, stype=stype, season.names=season.names, h=h, ...)

} # 'hydroplot.default' end

################################################################################
# hydroplot.zoo: Daily, Monthly and Annual plots of hydrological time series   #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 2008                                                                #
# Updates: 19-Apr-2011 ; 19-Jun-2011  ; 10-Aug-2011                            #
#          04-Jun-2012                                                         #
#          04-Apr-2013 ; 29-May-2013                                           #
#          07-Nov-2020                                                         #
#          04-Sep-2024                                                         #
#          31-Oct-2025                                                         #
################################################################################
# 9 plots:
# 1: Line plot with Daily time series, with 2 moving averages, specified by 'win.len1' and 'win.len2'
# 2: Line plot with Monthly time series, with 2 moving averages, specified by 'win.len1' and 'win.len2'
# 3: Line plot with Annual time series
# 4: Boxplot with daily time series
# 5: Boxplot with monthly time series
# 6: Boxplot with annual time series
# 7: Histogram of the daily time series
# 8: Histogram of the monthly time series
# 9: Histogram of the annual time series
hydroplot.zoo <- function(x, 
                          FUN, na.rm=TRUE,
                          ptype="ts+boxplot+hist",
                          pfreq="dma",                      
                          var.type,                      
                          var.unit="units",
                          main=NULL, xlab="Time", ylab,
                          win.len1=0,
                          win.len2=0,                      
                          tick.tstep="auto",
                          lab.tstep="auto",
                          lab.fmt=NULL,
                          cex=0.3,
                          cex.main=1.3,
                          cex.lab=1.3,
                          cex.axis=1.3,
                          col=c("blue", "lightblue", "lightblue"),
                          from=start(x), 
                          to=end(x),
                          dates=1, date.fmt = "%Y-%m-%d", tz=NULL, 
                          stype="default",
                          season.names=c("Winter", "Spring", "Summer", "Autumn"),
                          h=NULL, ...) {

     # Checking that the user provied a valid class for 'x'   
     if (!is.zoo(x)) 
        stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")
            
     # 'xname' value
     xname <- deparse(substitute(x))

     # 'ylab' value
     if ( missing(ylab) ) { 
       ylab <- xname
     } else if ( is.null(ylab) ) ylab <- xname

     # Checking that the user provied a valid argument for 'ptype'
     if (is.na(match(ptype, c("ts", "ts+boxplot", "ts+hist", "ts+boxplot+hist"))))
            stop("Invalid argument: 'ptype' must be in c('ts', 'ts+boxplot', 'ts+hist', 'ts+boxplot+hist')")

     # Checking that the user provied a valid argument for 'pfreq'
     if ( sfreq(x) == "daily" ) {
       if (is.na(match(pfreq, c("o", "dma", "dm", "ma", "seasonal"))))
          stop("Invalid argument: 'pfreq' must be in c('o', 'dma', 'ma', 'dm', 'seasonal')")
     } else if ( sfreq(x) == "monthly" ) {
         if (is.na(match(pfreq, c("o", "ma", "seasonal")))) 
           stop("Invalid argument: 'pfreq' must be in c('o', 'ma', 'seasonal')")
       } # ELSE end

     if ( (pfreq == "o") & (ptype != "ts") ) {
          message(paste("[Note: pfreq='o' => ptype has been changed to 'ts']" , sep="") )
          ptype <- "ts"
     } # IF end

     # Checking that the user provied a valid argument for 'var.type'
     if (missing(FUN) & (pfreq != "o") ) {
        # If the user did not provide a title for the plots, this is created automatically
        if (missing(var.type)) {
          stop("Missing argument: 'var.type' OR 'FUN' must be provided")
        } else # If 'var.type' is provided
             # Checking that the user provied a valid argument for 'var.type'
             if (is.na(match(var.type, c("Precipitation", "Temperature", "Flow") ) ) ) {
                   stop("Invalid argument: 'var.type' must be in c('Precipitation', 'Temperature', 'Flow')")
             } else {
                      if (var.type=="Precipitation") {
                          FUN <- sum
                          if (missing(var.unit)) { var.unit <- "mm"   }
                      } else if (var.type=="Temperature") {
                               FUN <- mean
                               if (missing(var.unit)) { var.unit <- "dC" }
                             } else if (var.type=="Flow") {
                                    FUN <- mean
                                    if (missing(var.unit)) { var.unit= "m3/s" }
                               }
                      } #ELSE end
     } # IF end
     
     ##########################################   
     ## In case 'from' and 'to' are provided  ##
     dates  <- time(x)
     ndates <- length(dates)
     
     # Checking the validity of the 'from' and 'to' arguments and 
     if ( sfreq(x) %in% c("minute", "hourly") )
       date.fmt <- "%Y-%m-%d %H:%M:%S"
     x <- check_from_and_to_and_subset(x, from=from, to=to, date.fmt=date.fmt, tz)


     #################

     # Assigning a dummy value to FUN, which is not used when pfreq="o"
     if (pfreq == "o") FUN <- mean

     def.par <- par(no.readonly = TRUE) # save default, for resetting...
     on.exit(par(def.par))

     # IF the user wants SEASONAL plots
     if (pfreq == "seasonal") {
       
       # Checking that the user provied a valid value for 'stype'   
       valid.types <- c("default", "FrenchPolynesia")    
       if (length(which(!is.na(match(stype, valid.types )))) <= 0)  
         stop("Invalid argument: 'stype' must be in c('default', 'FrenchPolynesia')")

       # Checking that the user provied a valid argument for 'season.names'
       if ( length(season.names) != 4 )
         stop("Invalid argument: 'season.names' must have 4 elements !")
       
       if (!missing(ptype)) {
         if ( ptype != "ts+boxplot") {
           message("[Note: 'pfreq=seasonal' => 'ptype' has been changed to 'ts+boxplot']")
           ptype <- "ts+boxplot"
         } # IF end
       } # IF end
       
        if ( lab.tstep != "auto" ) {
          if ( lab.tstep != "years" ) {
             message("[Note: 'pfreq=seasonal' => 'lab.tstep' has been changed to 'years']")
             lab.tstep <- "years"
          } # IF end 
        } else lab.tstep <- "years"
        
        if ( !is.null(lab.fmt) ) {       
          if ( lab.fmt != "%Y" ) {
             message("[Note: 'pfreq=seasonal' => 'lab.fmt' has been changed to '%Y']")
             lab.fmt <- "%Y"
          } # IF end
        } else lab.fmt <- "%Y"
       
       .hydroplotseasonal(x=x, FUN=FUN, na.rm=na.rm, tick.tstep= tick.tstep, 
                          lab.tstep= lab.tstep, lab.fmt=lab.fmt, var.unit=var.unit, 
                          main=main, xlab=xlab, ylab=ylab, cex.main=cex.main, 
                          cex.lab=cex.lab, cex.axis=cex.axis, col=col, 
                          stype=stype, season.names=season.names, h=h, ...)
                               
     } else {
     
     if (pfreq != "o") {

       # If 'x' is too short for plotting annual values, 'pfreq' is automatically changed
       if ( ( (sfreq(x) == "daily") & ( length(x) <= 366 ) ) |
            ( (sfreq(x) == "monthly") & ( length(x) <= 12 ) ) ) {
           if ( pfreq %in% c("dma", "ma") ) {
             if (pfreq == "dma") pfreq <- "dm"
             if (pfreq == "ma") pfreq <- "m"
             message(paste("[Warning: your ts is too short for plotting annual time series => 'pfreq'= ", pfreq, "]", sep="") )
           }
       } # IF end

       # Computing the monthly time series
       if ( sfreq(x) == "daily" ) {
         x.monthly <- daily2monthly(x, FUN=FUN, na.rm=na.rm)
       } else if ( sfreq(x) == "monthly" ) {
           x.monthly <- x
         } else x.monthly <- NA

       # Computing the annual time series
       if ( !is.na( match( sfreq(x), c("daily", "monthly") ) ) ) {
         x.annual <- daily2annual(x, FUN=FUN, na.rm=na.rm, out.fmt="%Y-%m-%d")
       } else if ( sfreq(x) == "annual" ) {
           x.annual <- x
         } else x.annual <- NA

     } else {
         x.monthly <- NA
         x.annual  <- NA
       } # ELSE end   
     

     if (ptype=="ts") {

       # Setting up the screen with 3 rows and 3 columns
       if (pfreq == "o") { 
          par(mfcol=c(1,1))
       } else if (pfreq == "dma") { 
            par(mfcol=c(3,1))
         } else if (pfreq %in% c("dm", "ma")) { 
            par(mfcol=c(2,1))
           } # ELSE end

       # if 'x' and 'y' are subdaily ts, 'tick.tstep', 'lab.tstep' and 'lab.fmt' are adopted
       if (sfreq(x) %in% c("minute", "hourly") ) {
         nmonths <- hydroTSM::mip(from=start(x), to=end(x), out.type="nmbr")

         if (nmonths <= 40) { # 40 months or less
           if (tick.tstep=="auto") tick.tstep <- "days"
           if (lab.tstep=="auto") lab.tstep <- "months"
           if ( is.null(lab.fmt) ) lab.fmt <- "%Y-%b"
         } else {
             if (tick.tstep=="auto") tick.tstep <- "months"
             if (lab.tstep=="auto") lab.tstep <- "years"
             if ( is.null(lab.fmt) ) lab.fmt <- "%Y"
           } # ELSE end     

       } # IF end

       # Drawing the daily, monthly and annual time series of the variable against time
       .hydroplotts(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                    win.len1=win.len1, win.len2=win.len2, var.type=var.type, 
                    var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,  
                    tick.tstep= tick.tstep, lab.tstep= lab.tstep, lab.fmt, 
                    cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[1], ...)
     } # IF end

     else if ( (ptype=="ts+boxplot") & (pfreq != "seasonal") ) {
       # Setting up the screen with 3 rows and 3 columns
       if (pfreq == "dma") { par(mfcol=c(3,2))
       } else if (pfreq %in% c("dm", "ma")) { par(mfcol=c(2,2))
         } # ELSE end
       # Drawing the daily, monthly and annual time series of the variable against time
       .hydroplotts(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                    win.len1=win.len1, win.len2=win.len2, var.type=var.type, 
                    var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,  
                    tick.tstep= tick.tstep, lab.tstep= lab.tstep, lab.fmt, 
                    cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[1], ...)
       # Drawing a boxplot of the daily, monthly and annual time series of the variable
       .hydroplotboxplot(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                         var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab, 
                         cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[2], ...)
     } # ELSE end

     else if (ptype=="ts+hist") {
       # Setting up the screen with 3 rows and 3 columns
       if (pfreq == "dma") { par(mfcol=c(3,2))
       } else if (pfreq %in% c("dm", "ma")) { par(mfcol=c(2,2))
         } # ELSE end
       # Drawing the daily, monthly and annual time series of the variable against time
       .hydroplotts(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                    win.len1=win.len1, win.len2=win.len2, var.type=var.type, 
                    var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,  
                    tick.tstep= tick.tstep, lab.tstep= lab.tstep, lab.fmt, 
                    cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[1], ...)
       # Drawing an histogram of the daily, monthly and annual time series of the variable
       .hydroplothist(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                      var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab, 
                      cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[3], ...)
     } # ELSE end

     else if (ptype=="ts+boxplot+hist") {
       # Setting up the screen with 3 rows and 3 columns
       if (pfreq == "dma") { par(mfcol=c(3,3))
       } else if (pfreq %in% c("dm", "ma")) { par(mfcol=c(2,3))
         } # ELSE end
       # Drawing the daily, monthly and annual time series of the variable against time
       .hydroplotts(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                    win.len1=win.len1, win.len2=win.len2, var.type=var.type, 
                    var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,  
                    tick.tstep= tick.tstep, lab.tstep= lab.tstep, lab.fmt, 
                    cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[1], ...)
       # Drawing a boxplot of the daily, monthly and annual time series of the variable
       .hydroplotboxplot(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                         var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab, 
                         cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[2], ...)
       # Drawing an histogram of the daily, monthly and annual time series of the variable
       .hydroplothist(x=x, x.monthly=x.monthly, x.annual=x.annual, pfreq=pfreq,
                      var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab, 
                      cex=cex, cex.main=cex.main, cex.lab=cex.lab, cex.axis=cex.axis, col=col[3], ...)
     } # ELSE end
     
   } # ELSE end (if (pfreq == "seasonal")

 } # 'hydroplot.zoo end



################################################################################
# hydroplot.data.frame: Daily, Monthly and Annual plots of hydrological ts     #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         # 
################################################################################
# Started: 2008                                                                #
# Updates: 19-Apr-2011 ; 19-Jun-2011  ; 10-Aug-2011                            #
#          04-Jun-2012                                                         #
#          04-Apr-2013 ; 29-May-2013                                           #
#          07-Nov-2020                                                         #
#          23-Aug-2022                                                         #
#          31-Oct-2025                                                         #
################################################################################
hydroplot.data.frame <- function(x, 
                                 FUN, na.rm=TRUE,
                                 ptype="ts+boxplot+hist",
                                 pfreq="dma",                      
                                 var.type,                      
                                 var.unit="units",
                                 main=NULL, xlab="Time", ylab,
                                 win.len1=0,
                                 win.len2=0,                      
                                 tick.tstep="auto",
                                 lab.tstep="auto",
                                 lab.fmt=NULL,
                                 cex=0.3,
                                 cex.main=1.3,
                                 cex.lab=1.3,
                                 cex.axis=1.3,
                                 col=c("blue", "lightblue", "lightblue"),
                                 from=start(x), 
                                 to=end(x),
                                 dates=1, date.fmt = "%Y-%m-%d", tz=NULL, 
                                 stype="default",
                                 season.names=c("Winter", "Spring", "Summer", "Autumn"),
                                 h=NULL, ...) {

  # Checking the user provides the dates
  if ( !any( class(dates) %in% c("numeric", "factor", "character", "Date" ,"POSIXct", "POSIXlt", "POSIXt") ) )
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'character', 'Date', 'POSIXct', 'POSIXlt', 'POSIXt'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  if ( inherits(dates, "numeric") ) {
    temp  <- x[, -dates]
    dates <- as.Date(as.character(x[, dates]), format= date.fmt) # zoo::as.Date
    x     <- temp
  } # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( ( inherits(dates, "factor") ) | ( inherits(dates, "character") ) ) 
    dates <- as.Date(dates, format= date.fmt) # zoo::as.Date

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'sname' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
    stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")  

  # converting from data.frame to zoo
  x <- zoo::zoo(x, dates) 

  hydroplot.zoo(x, FUN=FUN, na.rm=na.rm, ptype=ptype, pfreq=pfreq,                      
              var.type=var.type, var.unit=var.unit, main=main, xlab=xlab, ylab=ylab,
              win.len1=win.len1, win.len2=win.len2, tick.tstep=tick.tstep, 
              lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex=cex, cex.main=cex.main, cex.lab=cex.lab,
              cex.axis=cex.axis, col=col, from=from, to=to, stype=stype, season.names=season.names, h=h, ...)

} # 'hydroplot.data.frame' END

# File subhourly2nminutes.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2022-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                          subhourly2nminutes                                  #
################################################################################
# This function transform a SUB-HOURLY time series into a n-minutes one

# 'x'   : sub-hourly values that will be agreggated into n-minutes ones.
#         class(x) must be 'zoo'
# 'FUN' : Function that have to be applied for transforming from sub-hourly into 
#         n-minutes time step. E.g., for precipitation FUN MUST be "sum"
#         For temperature and flow time series, FUN MUST be "mean"
# 'na.rm': Logical. Should missing values be removed?
#          TRUE : the monthly and annual values  are computed considering only those values different from NA
#          FALSE: if there is AT LEAST one NA within a year, the monthly and annual values are NA

subhourly2nminutes <-function(x, ...) UseMethod("subhourly2nminutes")


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 15-Oct-2022                                                         #
# Updates: 29-Ago-2023                                                         #
################################################################################
subhourly2nminutes.default <- function(x, nminutes, FUN, na.rm=TRUE, 
                                       from=start(x), to=end(x), ...) {

  # Checking that 'x' is a zoo object
  if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo'")

  subhourly2nminutes.zoo(x=x, nminutes=nminutes, FUN=FUN, na.rm=na.rm, ...)

} # 'subhourly2nminutes.default' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 15-Oct-2022                                                         #
# Updates: 25-Oct-2022                                                         #
#          29-Ago-2023 ; 11-Oct-2023                                           #
#          23-Jul-2024                                                         #
################################################################################
subhourly2nminutes.zoo <- function(x, nminutes, FUN, na.rm=TRUE, 
                                   from=start(x), to=end(x), tz, ...) {

    # testing the existence of 'na.rm' argument
    #args <- list(...)
    #exist <- "na.rm" %in% names(args)
    #exist

    # Checking that the user provided a valid class for 'x'   
    if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !!")

    # Checking that the user provided a sub-hourly 'x' object
    if ( !(sfreq(x) == "minute" ) ) stop("Invalid argument: 'x' is not a sub-hourly ts !!")

    # Checking the user provide a valid value for 'FUN'
    if (missing(FUN))
      stop("Missing argument: 'FUN' must contain a valid function for aggregating the sub-daily values")

    # Getting the amount of minutes between each 'x' value, which might be irregularly spaced
    x.nmin       <- diff(time(x))
    x.nmin.units <- units(x.nmin)
    x.nmin       <- min(x.nmin)
    if (x.nmin.units == "secs") {
      x.nmin <- as.numeric(x.nmin/60)
    } else x.nmin <- as.numeric(x.nmin)

    # Checking that the amount of minutes used for aggregation ('nminutes') is provided and larger than 'x.nmin'
    if (missing(nminutes)) {
     stop("Missing argument: 'nminutes' must be provided !")
    } else if ( nminutes <= x.nmin) 
             stop("Invalid argument: 'nminutes' must be larger than the time frequency of 'x' !!")

    lstart <- start(x)
    if ( !missing(from) ) {
      if (from < lstart) {
        x <- izoo2rzoo(x, from=from)
      } else x <- window(x, start=from)
    } # IF end

    lend <- end(x)
    if ( !missing(to) ) {
      if (to > lend) {
        x <- izoo2rzoo(x, to=to)
      } else x <- window(x, end=to)
    } # IF end

    # Automatic detection of 'tz'
    #ltz <- format(time(x), "%Z")[1]
    #ltz <- ""
    tx <- time(x)
    missingTZ <- FALSE
    if (missing(tz)) {
      missingTZ <- TRUE
      tz        <- attr(tx, "tzone")
    } else {
        # For the Date/Time of 'x' to be in the time zone specified by 'tz'
        tx.new  <- timechange::time_force_tz(tx, tz=tz)
        time(x) <- tx.new
      } # ELSE end

    lstart <- start(x)
    lend   <- end(x)
    temp   <- seq( lstart, lend, by=paste0(nminutes, " min") )

    # temporal aggregation from 'x.nmin' minutes to 'nminutes' minutes
    h <- aggregate(x, by= function(tt) cut(tt, breaks=temp), FUN=FUN, na.rm= na.rm, ...)

    # I don't know why, but the time attribute of the resulting object 'h' is 'factor' (not POSIXct), 
    # and the '00:00:00' is removed from time corresponding to the full hour 
    # (e.g., '2021-01-01 00:00:00' becomes '2021-01-01')

    # Transforming 'time(x)' from factor into a POSIXct object
    lt        <- as.character(time(h))
    index     <- which(substr(lt, 12, 19) == "")
    lt[index] <- paste(lt[index], "00:00:00")
    h         <- zoo(coredata(h), as.POSIXct(lt, format="%Y-%m-%d %H:%M", tz=tz ))

    # Replacing the NaNs by 'NA.
    # mean(NA:NA, na.rm=TRUE) == NaN
    nan.index <- which(is.nan(h))
    if ( length(nan.index) > 0 ) h[nan.index] <- NA
  
    # Replacing all the Inf and -Inf by NA's
    # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
    inf.index <- which(is.infinite(h))
    if ( length(inf.index) > 0 ) h[inf.index] <- NA      

    return(h)

} # 'subhourly2nminutes.zoo' end


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 15-Oct-2022                                                         #
# Updates: 29-Ago-2023                                                         #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'out.fmt' : character, for selecting if the result will be 'numeric' or 'zoo'. Valid values are: c('numeric', 'zoo')
# 'verbose'      : logical; if TRUE, progress messages are printed
subhourly2nminutes.data.frame <- function(x, nminutes, FUN, na.rm=TRUE, 
                                          from=start(x), to=end(x), 
                                          dates=1, date.fmt="%Y-%m-%d %H:%M:%S",
				                                  out.fmt="zoo", verbose=TRUE,...) {

  # Checking that the user provide a valid value for 'FUN'
  if (missing(FUN))
      stop("Missing argument value: 'FUN' must contain a valid function for aggregating the values !!")

  # Checking that the user provided a valid argument for 'out.fmt'
  if (is.na(match( out.fmt, c("numeric", "zoo") ) ) )
      stop("Invalid argument: 'out.fmt' must be in c('numeric', 'zoo')")

  # Checking that the user provided a valid argument for 'dates'
  if (missing(dates)) {
      stop("Missing argument: 'dates' must be provided")
  } else
     # Checking that the user provided a valid argument for 'dates'
     if (FALSE && (class(dates) %in% c("numeric", "factor", "POSIXct", "POSIXt")) )
         stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'POSIXct', 'POSIXt'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easing the further computations
  if ( TRUE && ( inherits(dates, "numeric") ) ) {
    tmp   <- dates
    dates <- as.POSIXct(x[, dates], format= date.fmt) 
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( TRUE && ( inherits(dates, "factor") ) ) dates <- as.POSIXct(dates, format= date.fmt) 

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( (TRUE && (class(dates) %in% c("POSIXct", "POSIXt")) ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
     
  # Transforming 'x' into a zoo object
  x <- zoo::zoo(x, dates)
  
  ##############################################################################
  
  z <- subhourly2nminutes.zoo(x=x, nminutes=nminutes, FUN=FUN, na.rm=na.rm, 
                              from=from, to=to, ...)
    
  if (out.fmt == "numeric") {
     snames      <- colnames(z)
     dates.lab   <- as.character(time(z))
     z           <- coredata(z)
     colnames(z) <- snames
     rownames(z) <- dates.lab        
  } # IF end

  return( z )

 } #'subhourly2nminutes.data.frame' END


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 15-Oct-2022                                                         #
# Updates: 29-Ago-2023                                                         #
################################################################################
subhourly2nminutes.matrix  <- function(x, nminutes, FUN, na.rm=TRUE, 
                                       from=start(x), to=end(x),
                                       dates=1, date.fmt="%Y-%m-%d %H:%M:%S",
				                               out.fmt="zoo",
                                       verbose=TRUE,...) {

   x <- as.data.frame(x)
   #NextMethod("daily2annual")  # I don't know why is redirecting to 'daily2monthly.default' instead of 'daily2monthly.data.frame'....
   subhourly2nminutes.data.frame(x=x, nminutes=nminutes, FUN=FUN, na.rm=na.rm, 
                                 from=from, to=to, dates=dates, date.fmt=date.fmt,
			                           out.fmt=out.fmt, verbose=verbose,...)

} # 'subhourly2nminutes.matrix  ' END

# File izoo2rzoo.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################ 
#         Irregular Zoo -> Regular Zoo                                         #
################################################################################ 
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################  

# This function takes a time series of (very likely) irregular (with
# missing dates) daily time series and then transforms it into a variable
# regulary spaced, filling the voids with some value (by default: NA)

# x         : time series of type zoo (very likely with some missing days)
# date.fmt  : character indicating the format in which the dates are stored in 
#             \code{from} and \code{to}, e.g. "%Y-%m-%d"
#             ONLY required when class(dates)=="factor" or "numeric"
# from      : starting date for the merged output
# to	      : ending date for the merged output
# tstep     : time step in which are stored the values of 'x'
# tz        : specification of the time zone used for 'time(x)', 'from', 'to'
#             System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} (the default value) is UTC (Universal Time, Coordinated). 
#             See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
#             This argument can be used when working with subdaily zoo objects to force using UTC instead of the local time zone
# na.action: character, indicating whether to keep 'NA' values (default) or interpolate them with linear or spline interpolation

izoo2rzoo <-function(x, ...) UseMethod("izoo2rzoo")


################################################################################ 
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################ 
# Started: XX-XX-2009                                                          #
# Updates: 23-Aug-2011                                                         #
#          09-Oct-2022                                                         #
#          03-Aug-2023                                                         #
################################################################################ 
izoo2rzoo.default <- function(x, from= start(x), to= end(x), 
                              date.fmt, tstep, tz, 
                              na.action=c("keep", "linear", "spline"), ... ) {

  # Checking that the user provied a valid class for 'x'    
  if ( !is.zoo(x) )  
     stop("Invalid argument: 'class(x)' must be 'zoo' !")

  izoo2rzoo.zoo(x=x, from=from, to=to, date.fmt=date.fmt, tstep=tstep, tz=tz, 
                na.action=na.action, ... )

} # 'izoo2rzoo.default' END


################################################################################ 
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################ 
# Started: XX-XX-2009                                                          #
# Updates: 23-Aug-2011                                                         #
#          07-May-2012                                                         #
#          16-Oct-2012                                                         #
#          29-May-2013                                                         #
#          17-Jun-2022 ; 20-Jun-2022 ; 08-Oct-2022 ; 09-Oct-2022 ; 11-Oct-2022 #
#          12-Oct-2022                                                         #
#          25-May-2023 ; 03-Aug-2023 ; 04-Nov-2023 ; 16-Nov-2023 ; 17-Nov-2023 #
#          25-Nov-2023                                                         #
#          28-Apr-2024                                                         #
#          29-Oct-2025                                                         #
################################################################################ 

izoo2rzoo.zoo <- function(x, from= start(x), to= end(x), 
                          date.fmt, tstep, tz, 
                          na.action=c("keep", "linear", "spline"), ... ) {

  if (!is.zoo(x)) stop("Invalid argument: 'x' must be of class 'zoo' !")

  # Checking 'na.action' argument
  na.action <- match.arg(na.action)

  # sampling frequency of 'x'           
  x.freq <- sfreq(x)

  # Date/Time of 'x'
  tx <- time(x)

  # Cheking if 'x is a sub-daily zoo object  
  if (x.freq %in% c("minute","hourly") ) {
    subdaily.ts <- TRUE
  } else subdaily.ts <- FALSE

  # Defining the 'tstep' value
  if ( missing(tstep) )
    switch(x.freq,
           minute = {tstep <- "1 min"},
           hourly = {tstep <- "1 hour"},
           daily = {tstep <- "1 day"},
           weeekly = {tstep <- "1 week"},
           monthky = {tstep <- "1 month"},
           quarterly = {tstep <- "1 quarter"},
           annual = {tstep <- "1 year"}
           )

  # Automatic detection of 'date.fmt'
  if ( missing(date.fmt) ) {
    if ( subdaily.ts ) {
      date.fmt <- "%Y-%m-%d %H:%M:%S"
    } else date.fmt <- "%Y-%m-%d"
  } # IF end

  # Automatic detection of 'tz'
  missingTZ <- FALSE
  if (missing(tz)) {
    missingTZ <- TRUE
    tz        <- attr(tx, "tzone")
  } else {
      if ( subdaily.ts ) {
        # For the Date/Time of 'x' to be in the time zone specified by 'tz'
        tx.new  <- timechange::time_force_tz(tx, tz=tz)
        time(x) <- tx.new
      } # IF end
    } # ELSE end
      
  ifelse ( grepl("%H", date.fmt, fixed=TRUE) | grepl("%M", date.fmt, fixed=TRUE) |
           grepl("%S", date.fmt, fixed=TRUE) | grepl("%I", date.fmt, fixed=TRUE) |
           grepl("%p", date.fmt, fixed=TRUE) | grepl("%X", date.fmt, fixed=TRUE), 
           subdaily.date.fmt <- TRUE, subdaily.date.fmt <- FALSE )

  # If the index of 'x' is character, it is converted into a Date object
  if ( class(time(x))[1] %in% c("factor", "character") )
    #ifelse(subdaily.date.fmt, time(x) <- as.POSIXct(time(x), format=date.fmt, tz=tz),
    ifelse(subdaily.date.fmt, time(x) <- as.POSIXct(time(x), format=date.fmt),
                              time(x) <- as.Date(time(x), format=date.fmt) )

  # If 'from' was given as Date, but 'x' is sub-daily
  if (!missing(from)) {

    if (from > to) stop("Invalid argument: 'from > to' !")

    if (from > end(x)) stop("Invalid argument: 'from > end(x)' !")

    if ( subdaily.date.fmt & !(grepl(":", from, fixed=TRUE) ) )
      from <- paste(from, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      from <- as.POSIXct(from, tz=tz)
  } # IF end

  # If 'to' was given as Date, but 'x' is sub-daily
  if (!missing(to)) {
    if (to < from ) stop("Invalid argument: 'to < from' !")

    if (to < start(x) ) stop("Invalid argument: 'to < start(x)' !")

    if ( subdaily.date.fmt & !(grepl(":", to, fixed=TRUE) ) )
      to <- paste(to, "00:00:00")

    if ( subdaily.date.fmt & missingTZ )
      to <- as.POSIXct(to, tz=tz)
  } # IF end
        
  # checking that date.fmt and the sampling frequency of 'x' are compatible 
  if ( subdaily.ts ) {
    if (!subdaily.date.fmt) 
      stop("Invalid argument: 'date.fmt' (", date.fmt, 
           ") is not compatible with a sub-daily time series !!")
  } else {
           if (subdaily.date.fmt) {
             #time(x) <- as.POSIXct(time(x), tz=tz)
             time(x) <- as.POSIXct(time(x))
             warning("'date.fmt' (", date.fmt, ") is sub-daily, while 'x' is a '", 
                     x.freq, "' ts => 'time(x)=as.POSIXct(time(x), tz)'")
           } # IF end    
          } # ELSE end

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(from, format=date.fmt, tz=tz))
    #dt <-  try(as.POSIXct(from, format=date.fmt))
  } else dt <- try(as.Date(from, format=date.fmt))

  if ("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'from' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      from <- as.POSIXct(from, format=date.fmt, tz=tz)
      #from <- as.POSIXct(from, format=date.fmt)
    } else from <- as.Date(from, format=date.fmt)

  if (subdaily.ts) {
    dt <-  try(as.POSIXct(to, format=date.fmt, tz=tz))
  } else dt <- try(as.Date(to, format=date.fmt))

  if ("try-error" %in% class(dt) || is.na(dt)) {
    stop("Invalid argument: format of 'to' is not compatible with 'date.fmt' !")
  } else if (subdaily.ts) {
      to <- as.POSIXct(to, format=date.fmt, tz=tz)
    } else to <- as.Date(to, format=date.fmt)

  # Creating a regular time series with NA's in all dates in [from, to]
  dates.reg <- seq(from=from, to=to, by= tstep)
  #na.zoo    <- zoo(rep(NA, length(dates.reg)), dates.reg)
  x.reg     <- merge(x, zoo(, dates.reg))

  # If the user wants to fill in NA values
  if (na.action == "linear") {
    x.reg <- zoo::na.approx(x.reg, na.rm=FALSE)
  } else if (na.action == "spline") 
      x.reg <- zoo::na.spline(x.reg, na.rm=FALSE)

  # Selecting only those data within the time period between 'from' and 'to'
  x.sel <- window(x.reg, start=from, end=to)

  ## Creating a regular time series with NA's in dates.reg in which 'x' has no data
  #x.merged <- merge(na.zoo, x.sel)
  
  ## Removing the fictitious column corresponding to 'na.zoo'
  #x.merged <- x.merged[,-1]
  
  # Giving the same column names than the original 'x'
  if ( is.matrix(x) | is.data.frame(x) )
    colnames(x.sel) <- colnames(x)

  # Returning only the column containing the Regular ts with NA's in the empty records
  return( x.sel )

} # 'izoo2rzoo.zoo' end

# File drawxaxis.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later
################################################################################
# 'drawTimeAxis': It draws an X axies with daily, monthly, or annual time marks#
################################################################################
# From version 0.3-0 it changed its name from 'drawxaxis' to 'drawTimeAxis', in#
# order to have a more intuitive name. The old 'drawxaxis' function is         #
# deprecated, but still be kept for compatibility reasons.                     #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 2008                                                                #
# Updates: March 2009, Nov 2010, April 2011                                    #
#          29-May-2013;                                                        #
#          04-Jul-2016                                                         #
################################################################################
drawTimeAxis <- function(x, tick.tstep="auto", lab.tstep="auto", lab.fmt=NULL, 
                         cex.axis=1, mgp=c(3, 2, 0), ... ) {

 # Valid tseps for ''tick.tstep' and 'lab.tstep' 
 valid.tstep <- c("auto", "years", "quarters", "months", "weeks", "days", 
                  "hours", "minutes", "seconds")

 # Checking that the user provied a valid argument for 'tick.tstep'
 if (is.na(match(tick.tstep, valid.tstep ) ) )
     stop("Invalid argument: 'tick.tstep' must be in c('auto', 'years', 'quarters',
          'months', 'weeks', 'days', 'hours', 'minutes', 'seconds')")

 # Checking that the user provied a valid argument for 'lab.tstep'
 if (is.na(match(lab.tstep, valid.tstep ) ) )
     stop("Invalid argument: 'lab.tstep' must be in c('auto', 'years', 'quarters',
          'months', 'weeks', 'days', 'hours', 'minutes', 'seconds')")

 # 'lab.fmt': If the user didn't provide a value, default values are used
 #if ( missing(lab.fmt) | is.null(lab.fmt) ) {
 if ( is.null(lab.fmt) ) {
   if (lab.tstep == "auto") {
     lab.fmt <- TRUE
   } else {
       if (lab.tstep %in% c("years", "quarters", "months") ) {
         lab.fmt <- "%b-%Y"
       } else if (lab.tstep %in% c("weeks", "days") ) {
          lab.fmt <- "%Y-%m-%d"
         } else if (lab.tstep %in% c("hours", "minutes") ) {
            lab.fmt <- "%b %d %H:%M"
           } else lab.fmt <- "%d %H:%M:%S"
     } # ELSE end
 } # IF end

# # Margin line values (in mex units) for the axis title, axis labels and axis line
# if (lab.tstep == "auto") {
#   mgp = c(3, 2, 0)
# } else mgp = c(3, 1, 0)

 # Ticks: Draws the ticks for the time series in the x axis, without labels
 if (tick.tstep == "auto") {
   ticks <- x
 } else ticks <- suppressWarnings(to.period(x, period=tick.tstep, indexAt='startof')) #xts::to.period
 ticks.coords <- xy.coords(.index(ticks), ticks[, 1])
 axis(side =1, at = ticks.coords$x, labels = FALSE, col = "#BBBBBB", ...)

 # Labels: Computes the string for the labels of the X axis
 if (lab.tstep == "auto") {
   labs.dates  <- suppressWarnings(axTicksByTime(x, ticks.on=lab.tstep, gt=1)) # xts::axTicksByTime ; xts::
   labs.coords <- xy.coords(.index(x), x[, 1]) 
   labs.lab    <- names(labs.dates)
 } else {
   new.dates   <- suppressWarnings(to.period(x, period=lab.tstep, indexAt='startof'))  #xts::to.period
   labs.dates  <- match(index(new.dates), time(x))
   labs.coords <- xy.coords(.index(x), x[, 1])
   labs.lab    <- format(time(new.dates), lab.fmt)
   } # ELSE end
 
 # Drawing the X axis
 axis(side =1, at = labs.coords$x[labs.dates], labels = labs.lab, cex.axis = cex.axis, tcl = -0.7, mgp= mgp, ...)
 if (lab.tstep != "auto") box()

} # 'drawTimeAxis' END


################################################################################
# 'drawxaxis': It draws an X axies with daily, monthly, or annual time marks   #
################################################################################
# Started on 2008                                                              #
# Updates: March 2009, Nov 2010, April 2011                                    #
#          08-May-2017                                                         #
################################################################################
drawxaxis <- function(x, tick.tstep="auto", lab.tstep="auto", lab.fmt=NULL, cex.axis=1, mgp=c(3, 2, 0), ... ) {

  drawTimeAxis(x, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex.axis=cex.axis, mgp=mgp, ... ) 
}

# File dwi.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################ 
# Generic Days With Information function                                       #
################################################################################ 
dwi <-function(x, ...) UseMethod("dwi")


################################################################################ 
#            Zoo Days with Information                                         #
################################################################################ 
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################ 
# Started: XX-XXX-2009                                                         #
# Updates: 22-Aug-2011                                                         #
#          29-May-2013                                                         #
################################################################################ 
# This function generates a table indicating the number of days
# with information (<>NA's) within a zoo object,
# aggregated by: Year, Month or Month by Year

# x        : variable of type 'zoo'
# out.unit : aggregation time for the computation of the amount of days with info.
#	     Valid values are:
#            -) "months": monthly;
#            -) "years" : annual;
#            -) "mpy"   : month per year
# from     : Character indicating the starting date for the values stored in all the files that
#            will be read. It HAs to be in the format indicated by 'date.fmt'
# to       : Character indicating the starting date for the values stored in all the files that
#            will be read. It HAs to be in the format indicated by 'date.fmt'
# date.fmt : Character indicating the date format in which you provide 'from' and 'to', e.g. "%d-%m-%Y"
# 'tstep'  : since hydroTSM 0.3-0 it is not required any more, because it is not used any longer.
#            It is kept for backwards compatibility purposes only.

dwi.default <- function(x, out.unit="years", from = start(x), to = end(x), 
                        date.fmt="%Y-%m-%d", tstep="days", ...) {

    # Checking that the user provied a valid class for 'x'   
     valid.class <- c("xts", "zoo")    
     if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
        stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")

     dwi.zoo(x=x, out.unit=out.unit, from=from, to=to, date.fmt=date.fmt, tstep=tstep, ...)

 } # 'dwi.default' end
 

 
 
################################################################################ 
#    dwi.zoo:  Zoo Days with Information                                       #
################################################################################ 
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################ 
# Started: 22-Aug-2009                                                         #
# Updates: 22-Aug-2011                                                         #
#          07-May-2012                                                         #
#          29-May-2013                                                         #
################################################################################ 
dwi.zoo <- function(x, out.unit="years", from= start(x), to= end(x), 
                    date.fmt="%Y-%m-%d", tstep="days", ...) {
                    
  # Checking the validity of the 'out.unit' argument
  if ( is.matrix(x) | is.data.frame(x) ) {  
    if ( is.na( match(out.unit, c("years", "months") ) ) )
       stop("Invalid argument value: 'out.unit' must be in c('years', 'months')" )
  } else {
    if ( is.na( match(out.unit, c("years", "months", "mpy") ) ) )
       stop("Invalid argument value: 'out.unit' must be in c('years', 'months', 'mpy')" )
    } # ELSE end
         
  # Checking 'from' and 'to'
  if (to < from) stop("Invalid argument: 'from > to')" )    
    
  from <- as.Date(from, format=date.fmt) # zoo::as.Date
  to   <- as.Date(to, format=date.fmt)   # zoo::as.Date

  # Selecting only those data that are within the time period between 'from' and 'to'
  x.sel <- window(x, start=from, end=to )
    
  .dwi <- function(x) { 
    nona.index  <- which(!is.na(x))
    return(length( nona.index ))    
  } # '.dwi' END
  
  .dwi2 <- function(trgt, x) { 
    tmp <- extract(x, trgt=trgt)
    nona.index  <- which(!is.na(tmp))
    return(length( nona.index ))    
  } # '.dwi' END
  
  dates  <- time(x)

  if (out.unit == "months")   {         
     # Monthly index for 'x'     
     m      <- as.numeric(format( dates, "%m" ))
     months <- factor( month.abb[m], levels=unique(month.abb[m]) )
     
     # 'as.numeric' is necessary for being able to change the names to the output
     a <- aggregate(x, by= months, FUN=.dwi)
     
     # changing the row names of the matrix/data.frame
     if ( is.data.frame(a) | is.matrix(a) )
       rownames(a) <- unique(months)
           
  } else if (out.unit == "years") {
         # Annual index for 'x'
         y      <- as.numeric(format( dates, "%Y" ))
         years  <- factor( y, levels=unique(y) )

         # 'FUN' is first applied to all the values of 'x' belonging to the same year
         a <- aggregate( x, by= years, FUN=.dwi)    
         
         # changing the row names of the matrix/data.frame
         if ( is.data.frame(a) | is.matrix(a) )
           rownames(a) <- unique(years)
           
     } else if (out.unit == "mpy") {
     
         # Computing the Starting and Ending Year of the analysis
         Starting.Year <- as.numeric(format(from, "%Y"))
         Ending.Year   <- as.numeric(format(to, "%Y"))

         # Amount of Years belonging to the desired period
         nyears <- Ending.Year - Starting.Year + 1
         
         # Dummy variable to know how many different months are present in 'x'. 
         # Only needed when x is shorter than a year
         m          <- as.numeric(format( dates, "%m" ))
         months     <- factor( month.abb[m], levels=unique(month.abb[m]) )
         tmp        <- aggregate(x, by= months, FUN=.dwi)
         nmonths    <- length(tmp)
         monthnames <- index(tmp)

         a <- matrix(data=NA, nrow=nyears, ncol=nmonths)

         #a <- sapply(Starting.Year:Ending.Year, function(i,y) {
         for (i in Starting.Year:Ending.Year) {

             tmp                       <- extract(x.sel, trgt= i)
                                         
             #a[i-Starting.Year+1,1:12] <-  sapply(1:12, FUN=.dwi2, x=tmp)
             dates  <- time(tmp)
             m      <- as.numeric(format( dates, "%m" ))
             months <- factor( month.abb[m], levels=unique(month.abb[m]) )
     
             # 'as.numeric' is necessary for being able to change the names to the output
             a[i-Starting.Year+1,1:nmonths] <-  aggregate(tmp, by= months, FUN=.dwi)
         
         } # FOR end

         #Change the names of the columns of the matrix
         #colnames(a) <- month.abb
         colnames(a) <- monthnames
         #Change the names of the rows of the matrix
         rownames(a) <- as.character(Starting.Year:Ending.Year)

    }  # ELSE IF end
    
    return(a)

 } # 'dwi.zoo' end


################################################################################ 
# dwi.data.frame: days with info in each station stored in a data frame        #
################################################################################ 
# Started: 21-Mar-2009                                                         #
# Updates: 29-May-2013 ; 06-Aug-2013                                           #
#          23-Aug-2022                                                         #
################################################################################ 
# This function generates a table indicating the number of days
# with information (<>NA's) within a data.frame

# 'x'         : variable of type 'data.frame'
# out.unit    : aggregation time for the computation of the amount of days with info.
#	        Valid values are:
#               -) "months": monthly;
#               -) "years" : annual;
# from        : starting date for detection of days with inormation
# to          : date format that will be used in the output variable
# date.fmt    : date format of "from" and "to". For CHE files, the format must be "%d-%m-%Y"
# 'dates'     : "numeric", "factor", "Date" indicating how to obtain the
#               dates for correponding to the 'sname' station
#               If 'dates' is a number, it indicates the index of the column in
#                 'x' that stores the dates
#               If 'dates' is a factor, it have to be converted into 'Date' class,
#                 using the date format  specified by 'date.fmt'
#               If 'dates' is already of Date class, the following line verifies that
#                 the number of days in 'dates' be equal to the number of element in the
#                 time series corresponding to the 'st.name' station
# 'verbose'  : logical; if TRUE, progress messages are printed
# 'tstep'  : since hydroTSM 0.3-0 it is not required any more, because it is not used any longer.
#            It is kept for backwards compatibility purposes only.

dwi.data.frame <- function(x, out.unit="years", from, to,
                           date.fmt="%Y-%m-%d", tstep="days", dates=1, verbose=TRUE,...) {

  # Checking the validity of the 'out.unit' argument
  if ( is.na( match(out.unit, c("years", "months") ) ) ) {
         stop("Invalid argument value: For data.frames, 'out.unit' must be in c('years', 'months')" ) }

  
  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
      stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # Verification that the number of days in 'dates' be equal to
  # the number of elements in 'x'
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
       stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt) # zoo::as.Date
    x     <- x[-tmp]
  }  else
      # If 'dates' is a factor, it have to be converted into 'Date' class,
      # using the date format  specified by 'date.fmt'
      if ( inherits(dates, "factor") ) {
	    dates <- as.Date(dates, format= date.fmt) # zoo::as.Date
	  } else
	    # If 'dates' is already of Date class, the following line verifies that
            # the number of days in 'dates' be equal to the number of element in the
            # time series corresponding to the 'st.name' station
            if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
              stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # Checking the validity of the 'from' argument
  if (missing(from)) { from <- dates[1]
  } else if ( is.na( match(class(from), c("Date", "character") ) ) ) {
            stop("Invalid argument value: 'class(from)' must be in c('Date', 'character')" ) }

  # Checking the validity of the 'to' argument
  if (missing(to)) { to <- dates[length(dates)]
  } else if ( is.na( match(class(to), c("Date", "character") ) ) ) {
            stop("Invalid argument value: 'class(to)' must be in c('Date', 'character')" ) }
            
  # Transforming 'x' into a zoo object
  x <- zoo(x, dates)  
  
  ##############################################################################
  dwi.zoo(x=x, out.unit=out.unit, from=from, to=to, date.fmt=date.fmt, tstep=tstep, ...)

} # 'dwi.data.frame' END


################################################################################ 
dwi.matrix <- function(x, out.unit="years", from, to, 
                       date.fmt="%Y-%m-%d", tstep="days", dates=1, verbose=TRUE,...) {

 x <- as.data.frame(x)
 #NextMethod("daily2annual")
 dwi.data.frame(x=x, out.unit=out.unit, from=from, to=to, date.fmt=date.fmt, 
                tstep=tstep, dates=dates, verbose=verbose,...)

} # 'dwi.matrix' END

# File matrixplot.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2026 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

####################################################################
# matrixplot: Plots a color matrix representing the amount of days #
#          with information in a set of gauging stations           #
####################################################################
# Author : Mauricio Zambrano-Bigiarini                             #
# Started: 21-May-2009;                                            #
# Updates: 22-Sep-2009, 24-Sep-2010 ; 23-Aug-2011                  #
#          02-Feb-2015                                             #
#          10-Mar-2020                                             #
#          03-Sep-2024                                             #
#          26-Apr-2026                                             #
####################################################################
# Adapted (and thank you very much) from:
# http://www2.warwick.ac.uk/fac/sci/moac/currentstudents/peter_cock/r/matrix_contour/

# 'x'         : variable of type 'matrix', with the amount of days with information in each station
#               -) The rows represent the gauging stations
#               -) The columns represetn the years, and they stores the amount of
#                  days with information in each station
# 'ColorRamp' :  Character or function defining a personalized color ramp for ploting the maps. \cr
#                Valid character values are in c("Days", "Precipitation", "Temperature",
#                "PCPAnomaly", "PCPAnomaly2" "TEMPAnomaly", "TEMPAnomaly2", "TEMPAnomaly3").
# 'ncolors'   :  Number of color intervals that will be used for differentiating
#                from 0 to 366 days with information
# 'main'      :  Main title for the plot

matrixplot <- function(x, 
                       ColorRamp="Days", 
                       ncolors=10, 
                       main="", 
                       cuts,                                # Numeric, indicating the values used to divide the range of 'x' in the legend of colours. If not provided, it is automatically selected as a function of 'lenght(col)'
                       cuts.dec=2,                          # Number of decimal places used to present the numbers that divide the range of 'x' in the legend of colours
                       cuts.labels,                         # Character indicating the label to be used in the ccolour legend for each one of the values defined by 'cuts'. If not provided, as.character(cuts)' is used
                       cuts.style=c("equal", "pretty", "fixed", "sd", "quantile", "kmeans", "bclust", "fisher"), # discarded becsue takes too much time or not alway provide the required number of classes: "dpih", "headtails", "hclust",  "jenks",
                       legend.cex=0.8,                      # character expansion factor *relative* to current \code{par("cex")} used for the legend text.
                       legend.title="",                     # text to be displayed above the legned of colours (e.g., showing the measurement units of the raster being displayed)
                       legend.title.cex=1.5,                # expansion factor(s) for the legend title
                       legend.fontsize=15,                  # The size of text (in points) used in the legend title                    
                       ...) {
     
  # If 'x' is a zoo, it trys to coherce into a matrix
  if (is.zoo(x)) x <- coredata(x) # zoo::coredata
  
  # Checking that the user provied a valid class for 'x'   
  valid.class <- c("matrix", "data.frame")    
  if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
     stop("Invalid argument: 'class(x)' must be in c('matrix', 'data.frame')")

  # If 'x' is a data.frame, it trys to coherce into a matrix
  if (is(x, "data.frame")) x <- as.matrix(x)

  # checking 'cuts.style'
  cuts.style <- match.arg(cuts.style)




  # Generate the traspose of the matrix, in order to get the years in the 'x' axis
  # and the stations in the 'y' axis
  #x <- t(x)

  Days.cols          <- colorRampPalette(c("red3", "orange", "darkgoldenrod1", "yellow", "darkolivegreen2", "green"))

  #Precipitation.cols <- colorRampPalette(c("aquamarine", "blue", "darkblue"))
  Precipitation.cols <- colorRampPalette(c("red3", "orange", "yellow", "darkolivegreen3", "lightskyblue", "royalblue3"))
  Temperature.cols   <- colorRampPalette(c("yellow4", "yellow", "orange", "red3", "darkred"))

  PCPAnomaly.cols    <- colorRampPalette(c("sienna4", "peachpuff", "royalblue", "blue"))
  PCPAnomaly2.cols   <- colorRampPalette(c("darkred", "red3", "orange", "yellow", "lightskyblue", "royalblue3", "darkblue"))

  TEMPAnomaly.cols   <- colorRampPalette(c("lightyellow", "yellow", "red", "darkred"))
  TEMPAnomaly2.cols  <- colorRampPalette(c("yellow4", "yellow", "orange", "red3", "darkred"))
  TEMPAnomaly3.cols  <- colorRampPalette(c("darkblue", "royalblue3", "lightskyblue", "yellow", "orange", "red3", "darkred"))


  # Another way for temperature colors, using the reverse order (from white to red):
  # Temperature.cols <- rev(heat.colors(100))

  # # Generating palettes of colors
  if (!(is(ColorRamp, "function") ) ) {
     # Checking that the user provided a valid argument for 'ColorRamp'
    if (is.na(match(ColorRamp, c("Days", "Precipitation", "Temperature", "PCPAnomaly", "PCPAnomaly2", "TEMPAnomaly", "TEMPAnomaly2", "TEMPAnomaly3") ) ) ) {
      stop("Invalid argument: 'ColorRamp' must be in c('Days', 'Precipitation', 'Temperature', 'PCPAnomaly', 'PCPAnomaly2', 'TEMPAnomaly', 'TEMPAnomaly2', 'TEMPAnomaly3')")
    } else {
      # Assgning the color ramp, when 'ColorRamp' was given as a character
      if (ColorRamp == "Days") {
      ColorRamp <- Days.cols
      } else if (ColorRamp == "Precipitation") {
        ColorRamp <- Precipitation.cols
        } else if (ColorRamp == "Temperature") {
            ColorRamp <- Temperature.cols
          } else if (ColorRamp == "PCPAnomaly") {
               ColorRamp <- PCPAnomaly.cols
            } else if (ColorRamp == "PCPAnomaly2") {
               ColorRamp <- PCPAnomaly2.cols
              }  else if (ColorRamp == "TEMPAnomaly") {
                  ColorRamp <- TEMPAnomaly.cols
                } else if (ColorRamp == "TEMPAnomaly2") {
                    ColorRamp <- TEMPAnomaly2.cols
                  } else if (ColorRamp == "TEMPAnomaly3") {
                      ColorRamp <- TEMPAnomaly3.cols
                    }# ELSE end
      } # ELSE end
  } # IF end

  # Creating the colors to be used
  col <- ColorRamp(ncolors)

 
  # Defining the cuts to be used inhe plot
  if (missing(cuts)) {
    ncuts <- length(col) + 1
    temp <-  x[!is.nan(x)]

    lcuts <- classInt::classIntervals(temp, n=length(col), dataPrecision=cuts.dec, style = cuts.style )[["brks"]]
    lcuts <- round(lcuts, cuts.dec)
    ncuts <- length(lcuts)
    col   <- col[1:(ncuts-1)]
  } else {
      if ( (length(col)+1) != length(cuts) ) {
        stop("Invalid argument: 'length(col)+1 != length(cuts)' ") 
      } else lcuts <- cuts
    } # ELSE end


  if ( missing(cuts.labels) ) {
    cuts.labels <- as.character(lcuts)
  } else {
      if (length(lcuts) != length(cuts.labels) )
        stop("Invalid argument: 'length(cuts) != length(cuts.labels)' ") 
    } # ELSE end


  #par(fig=c(0,0.8,0,0.8), new=TRUE)
  #require(lattice) # for levelplot()
  #y <- levelplot(x, scales=list(tck=0, x=list(rot=90)),
  #               col.regions=ColorRamp(ncolors),
  #               #at= seq(0, 366, length.out= ncolors),
  #               main= main,
  #               xlab=NULL, ylab=NULL,...)

  # Limit number of legend labels to 15
  max.labels  <- min(15, length(lcuts))
  label.idx   <- round( seq(1, length(lcuts), length.out = max.labels) )
  cuts.labels <- cuts.labels[label.idx]
  legend.at   <- as.numeric(factor(lcuts))[label.idx]
  

  y <- levelplot(x, 
                 scales=list(tck=0, x=list(rot=90)),
                 colorkey = list(title = paste0(legend.title, " \n"),
                                 title.gpar=list(fontsize=legend.fontsize, font=1), #font=2 -> bold font
                                 space = "right",
                                 at=as.numeric(factor(lcuts)),                                                  # equally spaced color bins in the legend
                                 labels=list(labels=cuts.labels, at=legend.at, cex=legend.cex), # equally spaced color bins in the legend
                                 col=col                                     
                                 ), 
                 main=main,
                 xlab=NULL, ylab=NULL,...,
                 at=lcuts, col.regions=col
                 ) 

  #par(fig=c(0,0.8,0.55,1), new=TRUE)
  #up <- colMeans(x)
  #up.g <- barplot(up)

  #par(fig=c(0.65,1,0,0.8),new=TRUE)
  #right <- rowMeans(x)
  #r.g <- barplot(right)

  #print(y, split = c(1, 1, 10, 10))
  #print(up.g, split = c(1, 1, 10, 10), newpage = FALSE)

  return(y)
} # 'matrixplot' END

# File subdaily2seasonal.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# subdaily2seasonal: Generic function for computing seasonal values for every  #
#                    year of a sub-daily xts object                            #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 05-Abr-2013                                                         #
# Updates: 08-Apr-2013                                                         #
################################################################################

# 'x   '    : variable of type 'xts' or 'data.frame'
# 'season'  : character, indicating the weather season to be used for selecting the data
#             Valid values are:
#             -) "DJF": December, January, February
#             -) "MAM": March, April, May
#             -) "JJA": June, July, August
#             -) "SON": September, October, November
# 'FUN'      : Function that will be applied to ALL the values of 'x' belonging to the given weather season
# 'na.rm'    : Logical. Should missing values be removed?
#              TRUE : the seasonal values  are computed considering only those values different from NA
#              FALSE: if there is AT LEAST one NA within a season, the corresponding values are NA
# 'out.fmt'  : character indicating the format for the output time series. Possible values are:
#              -) "%Y"      : only the year will be used for the time. Default option. (e.g., "1961" "1962"...)
#              -) "%Y-%m-%d": a complete date format will be used for the time. Default option. (e.g., "1961" "1962"...)

subdaily2seasonal <-function(x, ...) UseMethod("dm2seasonal")

# File monthlycurve.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2022-2023 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

###################################################################################
# .plot_pq_mm.zoo: Function for drawing a figure with mean monthly precipitation  #
#                  and mean monthly streamflows (on the bottom)                   #
###################################################################################
# Originally this function was called 'monthlycurve', but afterwards it was       # 
# merged with the 'plot_pq' function                                              #
###################################################################################
# Author : Mauricio Zambrano-Bigiarini                                            #
###################################################################################
# Started: 26-Jul-2022                                                            #
# Updates: 22-Sep-2022 ; 11-Oct-2022 ; 25-Oct-2022                                #
#          22-Dec-2023 ; 27-Dec-2023                                              #          
###################################################################################
# 'q'        : object of type 'zoo' with monthly, daily or subdaily streamflow data.
#              If q is a monthly zoo object, it must have 12 elments and it should be 
#              named with the names of the months (levels(time(q))), otherwise, 
#              automatic names will be asigned from Jan to Dec for each one of the 
#              12 monthly values
# 'date.fmt' : format in which the dates.q are stored in 'from' and 'to'.
# 'na.rm'    : Logical. Should missing values in 'q' be removed when using FUN?. 
#              It is also used when the optional argument 'p' is submonthly (e.g., daily, hourly), to decide whether missing values in the optional argument 'p' should be removed before aggregated into monthly scale
#              TRUE : the monthly values  are computed considering only those values in 'q' (and 'p') different from NA
#              FALSE: if 'q' (and 'p') has AT LEAST one NA within a month, the corresponding monthly values are NA

.plot_pq_mm.zoo <- function(p,
                            q, 
                            
                            #na.rm=TRUE, 
                            na.fill=c("remove", "linear", "spline"), 
                            
                            from, 
                            to, 
                            
                            date.fmt, 
                            tz,
                            
                            main="Monthly Precipitation and Streamflows", 
                            xlab="Month",
                            ylab=c("P, [mm]", "Q, [m3/s]"),
                            cols=c("lightskyblue1", "blue"),
                            
                            start.month=1,

                            plot.q.probs=TRUE,
                            q.probs=c(0.25, 0.75),
                            q.probs.col="lightskyblue1",
                            q.probs.alpha=0.8,
                         
                            plot.p.probs=TRUE,
                            p.probs=c(0.25, 0.75),
                            p.alpha=0.8,
                            
                            labels=TRUE,
                            labels.cex=0.8,
                            labels.q.dx=c(rep(-0.2,6), rep(0.2,6)),
                            labels.q.dy=rep(-median(q, na.rm=TRUE)/10, 12),
                            labels.p.dy=-median(daily2monthly(p, FUN=sum, na.rm=TRUE), na.rm=TRUE)/10
                            ) {

  .plotbands <- function(x, lband, uband, col="", border=NA) {
    t <- c(x, rev(x))
    bands <- c(as.numeric(lband), rev(as.numeric(uband)))
    polygon(t, bands, col=col, border=border)
  } # .plotbands END

  .shift <- function(x, imonth) {
    L <- length(x)
    if (imonth>L) stop("[ Invalid value: 'imonth' can not be larger than ", L, " !]")
    delta <- imonth-1
    index.old <- 1:L
    index.new <- index.old-delta
    neg <- which(index.new <=0)
    index.new[neg] <- index.new[neg]+L
    if ( is.zoo(x) ) {
      x.raw    <- zoo::coredata(x)
      x.labels <- as.character(time(x))
      out        <- x.raw[match(index.old, index.new)] 
      names(out) <- x.labels[match(index.old, index.new)] 
    } else out <- x[match(index.old, index.new)] 
    return(out)
  } # .shift END
   

  ###########################################
  ## In case 'from' and 'to' are provided  ##
    
  dates.q  <- time(q)

  # Checking the validity of the 'from' argument
  if (!missing(from)) { 
     from <- as.Date(from, format=date.fmt)

     q <- window(q, start=from)

     if (!missing(p)) p <- window(p, start=from)
   } # ELSE end

  # Checking the validity of the 'to' argument
  if (!missing(to)) { 
     to <- as.Date(to, format=date.fmt)

     q <- window(q, end=to)

     if (!missing(p)) p <- window(p, end=to)
   } # ELSE end


  # Checking that 'q' and 'p' have the same dates
  dates.q  <- time(q)
  if (!missing(p)) {
    dates.p <- time(p)
    if (!all.equal(dates.q, dates.p))
      stop("Invalid arguments: 'dates(q)' must be equal to 'dates(p)' !!")
  } # IF end

  ###########################################
  ## In case 'q' is not average monthly values
  from <- time(q)[1]
  to   <- time(q)[length(q)]

  if ( (sfreq(q) != "monthly") ) {
    q.m    <- daily2monthly(q, FUN=mean, na.rm=TRUE)
  } else q.m <- q

  if ( (sfreq(q) != "monthly") | ( (sfreq(q) == "monthly") & ( length(q) > 12) ) ) {
    q.m.med     <- monthlyfunction(q.m, FUN=quantile, probs=0.5, na.rm=TRUE)
    month.names <- levels(time(q.m.med))
  } else {
      q.m.med     <- q
      month.names <- levels(time(q))
      if (is.null(month.names))
        month.names <- month.abb
    } # ELSE end  

  q.m.q1 <- monthlyfunction(q.m, FUN=quantile, probs=q.probs[1], na.rm=TRUE)
  q.m.q2 <- monthlyfunction(q.m, FUN=quantile, probs=q.probs[2], na.rm=TRUE)

  if (start.month != 1) q.m.med     <- .shift(x=q.m.med    , imonth=start.month)
  if (start.month != 1) q.m.q1      <- .shift(x=q.m.q1     , imonth=start.month)
  if (start.month != 1) q.m.q2      <- .shift(x=q.m.q2     , imonth=start.month)
  if (start.month != 1) month.names <- .shift(x=month.names, imonth=start.month)

  if (!missing(p)) {

    if ( (sfreq(q) != "monthly") ) {
      p.m <- daily2monthly(p, FUN=sum, na.rm=TRUE)
    } else p.m <- p

    if ( (sfreq(p) != "monthly") | ( (sfreq(p) == "monthly") & ( length(p) > 12) ) ) {
      p.m.med <- monthlyfunction(p.m, FUN=quantile, probs=0.5, na.rm=TRUE)
    } else p.m.med <- p

    p.m.q1 <- monthlyfunction(p.m, FUN=quantile, probs=p.probs[1], na.rm=TRUE)
    p.m.q2 <- monthlyfunction(p.m, FUN=quantile, probs=p.probs[2], na.rm=TRUE)

    if (start.month != 1) p.m.med <- .shift(x=p.m.med, imonth=start.month)
    if (start.month != 1) p.m.q1  <- .shift(x=p.m.q1 , imonth=start.month)
    if (start.month != 1) p.m.q2  <- .shift(x=p.m.q2 , imonth=start.month)
  } # IF end

  
  # the next line is required just in case a previous plot modified the graphical 'layout'
  par(mfrow=c(1,1)) 

  ##############################################################################
  # Definining the plotting area (1 column, 2 rows), where the lower row has 
  # a height 3 times larger than the upper window, AND 
  # Plotting the Monthly Precipitation in the upper panel
  ##############################################################################
  if ( !missing(p) ) {
    par(mar=c(3, 4.1, 3, 1.5), xpd=TRUE) # default  c(5.1, 4.1, 4.1, 2.1)
    layout(matrix(c(1,2), 2, 1, byrow = TRUE), widths=5, heights=c(1,3))  
 
    ylim <- range(pretty(p.m.q1), pretty(p.m.q2))
    x <- barplot(p.m.med, ylim=rev(ylim), xlab="", ylab=ylab[1], axes=TRUE, col=cols[1], names.arg=month.names, main=main)
    #axis(side=1, at=lx, labels=month.names, line=0.02, outer=TRUE, pos=1)
    if (labels) text(x, labels.p.dy, cex=labels.cex, adj=0.5, labels= round(p.m.med,1), col="black")

    # Adding error bars
    if (plot.p.probs) 
      graphics::arrows(x0 = x, y0 = p.m.q2, y1 = p.m.q1, angle=90, code=3, length=0.1)
  } # IF end


  #######################################
  # 2nd Figure: Drawing the monthly curve
  #######################################
  if (!missing(p)) {
    par(mar=c(3, 4.1, 0.5, 1.5)) # default  c(5.1, 4.1, 4.1, 2.1)
    main <- ""
  } else par(mar=c(3, 4.1, 4.1, 1.5))
  lubands.col <- grDevices::adjustcolor(q.probs.col, alpha.f=q.probs.alpha)
  xlim <- c(0.5, 12.5)
  ylim <- range(pretty(q.m.q1), pretty(q.m.q2))

  # Monthly values as lines
  lx   <- 1:12 
  plot(lx, q.m.med, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab[2], type="n", axes=TRUE, xaxt="n", main=main)
  #plot(lx, q.m.med, xlim=xlim, ylim=ylim, col= cols[2], type="o", lwd=3, pch=15, cex=1.4, axes=TRUE, xaxt="n", xlab=xlab, ylab=ylab[2])

  if (plot.q.probs) 
    .plotbands(x=lx, lband=q.m.q1, uband=q.m.q2, col=lubands.col, border=NA)
  
  grid()
  lines(lx, q.m.med, xlim=xlim, ylim=ylim, col= cols[2], type = "o", lwd=3, pch=15, cex=1.4)
  axis(side=1, at=lx, labels=month.names)
  if (labels) text(lx+labels.q.dx, q.m.med+labels.q.dy, cex=labels.cex, adj=0.5, labels= round(q.m.med,1), col="black" )
 
} # 'monthlycurve' END

# File smry.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2008-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                                 'smry'                                       # 
################################################################################
# Min, 1stQ, Mean, Median, 3rdQ, Max, IQR, sd, cv, skewness, kurtosis, amount  #
# of elements and amount of NA's                                               #
#       For numerical variables.                                               #
#       Skewness and Kurtosis are computed with the e1071 package              #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 14-Jun-2008;                                                        #
# Updates: 11-Sep-2009 ; 30-Aug-2011  ; 14-Sep-2011 ; 15-Sep-2011              #
#          29-May-2013 ; 21-Jul-2015                                           #
#          23-Aug-2022                                                         #
################################################################################
smry <-function(x, ...) UseMethod("smry")


smry.default <- function(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)  {

    if ( class(x) %in% c("zoo", "xts") ) {
        x <- coredata(x) # zoo::coredata
    } # IF end

    xname <- deparse(substitute(x))
    
    # Creating the resulting object
    z <- as.data.frame(matrix(NA, ncol=1, nrow=13))  

    n <- length(x)

    nas <- is.na(x)
    nna <- sum(nas)

    if (na.rm) {
          if (nna > 0) x  <- x[!nas]
    } # IF end
        
    s    <- summary(x, ..., digits=digits)

    if ( class(x) %in% c("numeric", "integer") ) {
        
        # min, q1, median, mean, q3, max
        z[1:6, 1] <- s

        z[7,1] <- IQR(x, na.rm = na.rm)	                         # Interquantile Range IQR = Q(0.75) - Q(0.25)
        z[8,1] <- sd(x, na.rm = na.rm)	                         # Standard Deviation
        z[9,1] <- sd(x, na.rm = na.rm) / abs(mean(x, na.rm = na.rm)) # Coefficient of variation ( coef. of variation = sd / |mean| )

        #require(e1071) # for the following 2 functions
        z[10,1] <- e1071::skewness(x, na.rm = na.rm)  # Skewness (using  e1071 package)
        z[11,1] <- e1071::kurtosis(x, na.rm = na.rm)  # Kurtosis (using  e1071 package)

        z <- round( z, digits)
            
        z[12,1] <- nna # Amount of NA's
        z[13,1] <- n   # Number of elements


    } else if ( inherits(x, "Date") ) { 
              z <- smry.Date( x, na.rm=na.rm, digits=digits, ... ) 
           } else z[1:13, 1] <- NA

    row.names(z) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.",
                      "Max.", "IQR", "sd", "cv", "Skewness", "Kurtosis",
                      "NA's", "n")

    names(z) <- xname
    return(z)

} # 'smry.default' end


smry.Date <- function(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)  {

        # Creating the resulting object
        z <- as.data.frame(matrix(NA, ncol=1, nrow=13))

        n <- length(x)

        nas <- is.na(x)
        nna <- sum(nas)

        if (na.rm) {
          if (nna > 0) {
            x  <- x[!nas] }
        } # IF end

        s <- summary(x, ..., digits=digits)

        # min, q1, median, mean, q3, max            
        z[1:6, 1] <- as.character(s)

        z[12, 1] <- nna # Amount of NA's
        z[13, 1] <- n   # Number of elements

 
    row.names(z) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.",
                      "Max.", "IQR", "sd", "cv", "Skewness", "Kurtosis",
                      "NA's", "n")

    names(z) <- deparse(substitute(x))
    return(z)

} # 'smry.Date' end


################################################################################
# Author: Mauricio Zambrano-Bigiarini                                          #
################################################################################
# Started: 14-Jun-2008;                                                        #
# Updates: 11-Sep-2009 ; 30-Aug-2011  ; 14-Sep-2011                            #
################################################################################
smry.data.frame <- function(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)  {

    # Creating a copy of the original observed values
    z <- as.data.frame( matrix(NA, nrow=13, ncol=ncol(x)) )

    z[,1:ncol(z)] <- sapply(1:ncol(x), function(j,y) {

       z[,j] <- smry.default(x= y[,j], na.rm=na.rm, digits=digits, ...)

    }, y = x) # sapply END


    rownames(z) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.",
                     "Max.", "IQR", "sd", "cv", "Skewness", "Kurtosis",
                     "NA's", "n")
    colnames(z) <- colnames(x)

    return(z)

} # 'smry.data.frame' end



################################################################################
# Author: Mauricio Zambrano-Bigiarini                                          #
################################################################################
# Started: 14-Jun-2008;                                                        #
# Updates: 11-Sep-2009                                                         #
################################################################################
smry.matrix <- function(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)  {

    x <- as.data.frame(x)
    smry.data.frame(x, na.rm=na.rm, digits=digits,...)
    #NextMethod("smry", x, na.rm=TRUE, digits=digits,...)

} # 'smry.data.frame' end


################################################################################
# Author: Mauricio Zambrano-Bigiarini                                          #
################################################################################
# Started: 30-Aug-2011                                                         #
# Updates: 29-May-2013                                                         #
################################################################################
smry.zoo <- function(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)  {

    dates <- time(x)  
    z     <- coredata(x) # zoo::coredata
    
    # Giving meaningful names to the output
    if ( (is.matrix(x)) | (is.data.frame(x)) ) {
        zname <- colnames(z)
        z     <- apply(z, MARGIN=2, FUN=smry.default, na.rm=na.rm, digits=digits, ...)      
    } else {
        zname <- deparse(substitute(x))
        z     <- smry.default(z, na.rm=na.rm, digits=digits,...) 
      } # ELSE end    
    
    z  <- data.frame(Index=c(summary(dates), rep(NA,7)), Data=z)
    
    # Giving meaningful names to columns and rows in 'z'
    colnames(z) <- c("Index", zname) 
    rownames(z) <- c("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.",
                     "Max.", "IQR", "sd", "cv", "Skewness", "Kurtosis",
                     "NA's", "n")
    return(z)

} # 'smry.zoo' end

# File dwdays.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2010-2017 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
# dwdays:  average amount of dry/wet days per each month                       #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 24-Jan-2010                                                         #
# Updates: 29-May-2013                                                         #
#          23-Aug-2022                                                         #
################################################################################
# Given a daily time series of precipitation, this function computes the average amount
# of dry/wet days (pcp > thr or pcp < thr for wet and dry days, respectively) on each month

# 'x'  : zoo. Daily time series of precipitation.
# 'thr': numeric. Value of daily precipitation used as threshold for classifying a day as dry/wet or not.
#        Days with a precipitation value larger or equal to 'thr' are classified as 'wet days',
#        whereas precipitation values lower or equal to 'thr' are classified as 'dry days'
# 'type': character, indicating if the daily values have to be calssified as dry or wet days. It works liked to the values specified in 'thr'. Valid values aer c('wet', dry')
dwdays <-function(x, ...) UseMethod("dwdays")

dwdays.default <- function(x, thr=0, type="wet", na.rm=TRUE, ... ) {

  # Checking the user provide a valid value for 'x'
  if ( !inherits(x, "zoo") )
    stop("Invalid argument: 'x' must be of class 'zoo'")

  # Checking the user provide a valid value for 'x'
  if (is.na(match(sfreq(x), c("daily")))) {
    stop(paste("Invalid argument: 'x' is not a daily ts, it is a ", sfreq(x), " ts", sep="") ) }

  # Checking the user provide a valid value for 'type'
  if ( is.na(match(type, c("dry", "wet"))) )
    stop("Invalid argument: 'type' must be in c('dry', 'wet'")

  # getting the dates of 'x'
  dates <- time(x)

  # Computing the Starting and Ending Year of the analysis
  Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
  Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))

  # Total amount of Years of 'x'
  nyears <- Ending.Year - Starting.Year + 1

  # Array with the average monthly amount of wet days
  wdays <- rep(NA, 12)

  for (m in 1:12) {
    #Extracts all the days of 'x' belonging to the month 'm'
    pcp.m <- extract(x, m)

    if (type=="wet") {
      wdays[m] <- length(which(pcp.m > thr )) / nyears
    } else if (type=="dry") {
        wdays[m] <- length(which(pcp.m < thr )) / nyears
      } # ELSE end
  } # FOR m end

  names(wdays) <- month.abb

  return(wdays)

} # 'dwdays.default' end



################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 24-Jan-2010                                                         #
# Updates: 29-May-2013                                                         #
#          23-Aug-2022                                                         #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'verbose' : logical; if TRUE, progress messages are printed
dwdays.data.frame <- function(x, thr=0, type="wet", na.rm=TRUE,
                              dates=1,
                              date.fmt="%Y-%m-%d",
							  verbose=TRUE,...) {


  # Checking that the user provied a valid argument for 'dates'
  if (missing(dates)) {
      stop("Missing argument: 'dates' must be provided")
  } else
     # Checking that the user provied a valid argument for 'dates'
     if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
         stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates,  "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt)
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt)

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")

  # Amount of stations in 'x'
  nstations <- ncol(x)

  # ID of all the stations in 'x'
  snames <- colnames(x)

  # Computing the Starting and Ending Year of the analysis
  Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
  Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))

  # Amount of Years belonging to the desired period
  nyears <- Ending.Year - Starting.Year + 1

  if (verbose) message("[Starting the computations...]")

  # Creating the data.frame that will store the computed averages for each station
  z <- as.data.frame(matrix(data = NA, ncol = 12, nrow = nstations,
                     byrow = TRUE, dimnames = NULL) )

  colnames(z) <- month.abb
  rownames(z) <- snames

  y = x

  for (j in 1:nstations) {

      if (verbose) message( paste("Station: ", format(snames[j], width=10, justify="left"),
                                " : ",format(j, width=3, justify="left"), "/",
                                nstations, " => ",
                                format(round(100*j/nstations,2), width=6, justify="left"),
                                "%", sep="") )

      # Transforming the column of 'x' into a zoo object,
      # using the dates provided by the user
      tmp <- vector2zoo(x=y[,j], dates=dates, date.fmt=date.fmt)

      # Computing the monthly values
      z[j, ] <- as.numeric(dwdays.default(x=tmp, thr=thr, type=type, na.rm=na.rm, ...))


  } # FOR end


  return( z )

 } #'dwdays.data.frame' END


################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 24-Jan-2010                                                         #
# Updates: 29-May-2013                                                         #
################################################################################
dwdays.matrix  <- function(x, thr=0, type="wet", na.rm=TRUE,
                           dates=1,
                           date.fmt="%Y-%m-%d",
			   verbose=TRUE,...) {

   x <- as.data.frame(x)

   NextMethod("dwdays", x, thr=thr, type=type,  na.rm=na.rm,
              dates=dates,
              date.fmt=date.fmt,
              verbose=verbose,...)

} # 'dwdays.matrix  ' END

# File cmv.R
# Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
#                                 https://CRAN.R-project.org/package=hydroTSM
# Copyright 2023-2025 Mauricio Zambrano-Bigiarini
# Distributed under GPL 2 or later

################################################################################
#                         cmv: Counting Missing Values                         #
################################################################################
# This function counts the percentage/amount of missing data in a zoo object, 
# using a user-defined temporal scale

# 'x'        :  zoo object to be analised
# 'tscale'   :  character with the temporal scale to be used for analysing the mssing data.
#               Valid values for 'tscale' are: 
#             -) "hourly"    :  the percentage/amount of missing values will be given for each 
#                               hour and ,therefore, the expected time frequency of 'x' must 
#                               be sub-hourly.
#             -) "daily"     :  the percentage/amount of missing values will be given for each 
#                               day and, therefore, the expected time frequency of 'x' must 
#                               be sub-daily (i.e., hourly or sub-hourly).
#             -) "weekly"    :  the percentage/amount of missing values will be given for each 
#                               week (starting on Monday) and, therefore, the expected time 
#                               frequency of 'x' must be sub-weekly (i.e., daily, (sub)hourly).
#             -) "monthly"   :  the percentage/amount of missing values will be given for each 
#                               month and, therefore, the expected time frequency of 'x' must 
#                               be sub-monthly (i.e., daily, hourly or sub-hourly).
#             -) "quarterly" :  the percentage/amount of missing values will be given for each 
#                               quarter and, therefore, the expected time frequency of 'x' must 
#                               be sub-quarterly (i.e., monthly, daily, hourly or sub-hourly).
#             -) "seasonal"  :  the percentage/amount of missing values will be given for each 
#                               weather season (see ?time2season) and, therefore, the expected 
#                               time frequency of 'x' must be sub-seasonal (i.e., monthly, daily, 
#                               hourly or sub-hourly).
#             -) "annual"    :  the percentage/amount of missing values will be given for each 
#                               year and, therefore, the expected time frequency of 'x' must 
#                               be sub-annual (i.e., seasonal, monthly, daily, hourly or sub-hourly).
#             -) 'out.type'  :  character indicating how should be returned the missing values 
#                               for each temporal scale. Valid values for 'out'type' are: 
#             -) "percentage":  the missing values are returned as an real value, representing
#                               the percentage of missing values in each temporal scale. 
#             -) "amount"    :  the missing values are returned as an integer value, representing
#                               the absolute amount of missing values in each temporal scale. 
#             -) 'dec'       :  integer indicating the amount of decimal places included in the output. 
#                               It is only used when "out.type=='percentage'"
#             -) 'start'     :  character, indicating the starting time used for aggregating sub-daily time 
#                               series into daily ones. 
#                               It MUST be provided in the format specified by \code{start.fmt}. \cr
#                               This value is used to define the time when a new day begins (e.g., for some 
#                               rain gauge stations). \cr
#                               -) All the values of \code{x} with a time attribute before \code{start} are
#                                  considered as belonging to the day before the one indicated in the time 
#                                  attribute of those values. \cr
#                               -) All the values of \code{x} with a time attribute equal to \code{start} 
#                                  are considered to be equal 
#                                  to \code{"00:00:00"} in the output zoo object. \cr
#                               -) All the values of \code{x} with a time attribute after \code{start} are 
#                                  considered as belonging to the same day as the one indicated in the time 
#                                  attribute of those values. \cr
#                                  It is useful when the daily values start at a time different from
#                                  \code{"00:00:00"}. Use with caution. See examples.
#             -) 'start.fmt' :  character indicating the format in which the time is provided in \code{start}.
#                               By default \code{date.fmt=\%H:\%M:\%S}. See \code{format} in 
#                               \code{\link[base]{as.POSIXct}}.
#             -) 'tz'        :  character, with the specification of the time zone used in both 
#                               \code{x} and \code{start}. 
#                               System-specific (see time zones), but \code{""} is the current time zone,
#                               and \code{"GMT"} is UTC (Universal Time, Coordinated). 
#                               See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
#                               If \code{tz} is missing (the default), it is automatically set to the
#                               time zone used in \code{time(x)}. \cr
#                               This argument can be used to force using the local time zone or any other 
#                               time zone instead of UTC as time zone.
#             -) 'start.month': numeric in [1,..,12], representing the starting month (1:Jan, ..., 12:Dec) 
#                               to be used in the computation of annual values. By default 'start.month=1'.


cmv <-function(x, ...) UseMethod("cmv")


################################################################################
#                              cmv.default                                     #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2023 (Buenos Aires)                                          #
# Updates: 28-Jul-2023 ; 03-Aug-2023 ; 27-Nov-2023                             #   
#          03-May-2025 (EGU 2025)                                              #
################################################################################
cmv.default <- function(x, 
	                      tscale=c("hourly", "daily", "weekly", "monthly", "quarterly", "seasonal", "annual"), 
	                      out.type=c("percentage", "amount"),
	                      dec=3, 
                        start="00:00:00", 
                        start.fmt= "%H:%M:%S", 
                        tz,
                        start.month=1,
                        ...) {

  # Checking that 'x' is a zoo object
  if ( !is.zoo(x) ) stop("Invalid argument: 'class(x)' must be 'zoo' !")

  # checking 'tscale'
  tscale <- match.arg(tscale)

  # checking 'out.type'
  out.type <- match.arg(out.type)

  cmv.zoo(x, tscale=tscale, out.type=out.type, dec=dec,
          start=start, start.fmt=start.fmt, ...)
     
} # 'cmv.default' end



################################################################################
#                              cmv.zoo                                         #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2023 (Buenos Aires)                                          #
# Updates: 28-Jul-2023 ; 03-Aug-2023 ; 27-Nov-2023                             #
#          03-May-2025 (EGU 2025) ; 26-Jul-2025                                #
################################################################################
cmv.zoo <- function(x, 
	                tscale=c("hourly", "daily", "weekly", "monthly", "quarterly", "seasonal", "annual"), 
	                out.type=c("percentage", "amount"),
	                dec=3,
                    start="00:00:00", 
                    start.fmt= "%H:%M:%S", 
                    tz,
                    start.month=1,
                    ...) {

  # checking 'tscale'
  tscale <- match.arg(tscale)

  # checking 'out.type'
  out.type <- match.arg(out.type)

  # Checking 'dec'
  if ( trunc(abs(dec) -  abs(dec) ) != 0 )
    stop("Invalid argument: 'dec' must be integer !")

  # Checking that 'na.rm.max' is in [1, 12]
  if ( (start.month < 1) | (start.month > 12) | 
       ( trunc(abs(start.month) -  abs(start.month) ) != 0 ) )
    stop("Invalid argument: 'start.month' must be an integer in [1, 12] !")

  # Automatic detection of 'tz'
  #if (missing(tz)) tz <- ""
  if (missing(tz)) tz <- format(time(x), "%Z")[1]

  # Getting the original time index for each element in 'x'
  time.old  <- time(x)

  # Analysis of days different from 00:00 to 23:59 hrs
  if ( start != "00:00:00" ) {
    # Converting the new starting time provided by the user into a POSIXct object
    start <- as.POSIXct(start, format=start.fmt, tz=tz)

    # normal staring time for a day
    nstart <- as.POSIXct("00:00:00", format="%H:%M:%S", tz=tz)

    # time difference between the desired starting time 'strat' and the "normal"
    # starting time 'nstart', [s]
    delta <- difftime(start, nstart, units="secs")

    # Computing teh time difference between 'start' and the "normal" starting time, [s]
    #time.new <- as.POSIXct(time.old, tz=tz) - delta
    time.new <- time.old - delta

    # Changing the time in 'x' in 'delta' seconds
    time(x)  <- time.new
  } # IF end

  # Getting the year corresponding to each element in 'x'
  years  <- format( time.old, "%Y")

  # Shifting backwards the year each element in 'x', 
  # only when start.month != 1
  if ( start.month != 1 )
    years <- shiftyears(ltime=time.old, lstart.month=start.month)

  # Checking that the time frequency of 'x' is compatible with 'tscale'
  if ( tscale == "hourly") {
  	if ( !(sfreq(x) %in% c("minute") ) )
  	  stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
  } else if ( tscale == "daily") {
  	  if ( !(sfreq(x) %in% c("minute", "hourly") ) )
  	    stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
    } else if ( tscale == "weekly") {
  	    if ( !(sfreq(x) %in% c("minute", "hourly", "daily") ) )
  	      stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
      } else if ( tscale == "monthly") {
  	      if ( !(sfreq(x) %in% c("minute", "hourly", "daily", "weekly") ) )
  	        stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
        } else if ( tscale == "quarterly") {
  	        if ( !(sfreq(x) %in% c("minute", "hourly", "daily", "weekly", "monthly") ) )
  	          stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
          } else if ( tscale == "seasonal") {
  	          if ( !(sfreq(x) %in% c("minute", "hourly", "daily", "weekly", "monthly") ) )
  	            stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
            } else if ( tscale == "annual") {
  	            if ( !(sfreq(x) %in% c("minute", "hourly", "daily", "weekly", "monthly", "quarterly", "seasonal") ) )
  	              stop("Invalid argument: 'tscale' and the time frequency of 'x' are not compatible !") 
              } # else END

  
  # Computing the total amount of data at the desired temporal scale
  fun <- length
  ndata <- switch( tscale,  
    "hourly"    = aggregate(x, by= function(tt) format(tt, "%Y-%m-%d %H"), FUN=fun),  
    "daily"     = aggregate(x, by= function(tt) format(tt, "%Y-%m-%d")   , FUN=fun),  
    "weekly"    = aggregate(x, by= function(tt) format(tt, "%Y-%W")      , FUN=fun),  # week starting on Monday 
    "monthly"   = aggregate(x, by= function(tt) format(tt, "%Y-%m")      , FUN=fun),  
    "quarterly" = aggregate(x, by= function(tt) zoo::format.yearqtr(tt)  , FUN=fun),  
    "seasonal"  = aggregate(x, by= function(tt) paste0(format(tt, "%Y"), "-", time2season(tt)), FUN=fun),  
    #"annual"    = aggregate(x, by= function(tt) format(tt, "%Y")         , FUN=fun)
    "annual"    = aggregate(x, by= years                                 , FUN=fun)
  ) # 'md' END

  # Function for obtaining the amount of missing values in 'x'
  smv <- function(x) {
    na.index <- is.na(x)
    return( sum(na.index) )
  } # 'smv' END

  # Computing the amount of missing values at the desired temporal scale
  fun <- smv
  nNA<- switch( tscale,  
    "hourly"    = aggregate(x, by= function(tt) format(tt, "%Y-%m-%d %H"), FUN=fun),  
    "daily"     = aggregate(x, by= function(tt) format(tt, "%Y-%m-%d")   , FUN=fun),  
    "weekly"    = aggregate(x, by= function(tt) format(tt, "%Y-%W")      , FUN=fun),  # week starting on Monday 
    "monthly"   = aggregate(x, by= function(tt) format(tt, "%Y-%m")      , FUN=fun),  
    "quarterly" = aggregate(x, by= function(tt) zoo::format.yearqtr(tt)  , FUN=fun),  
    "seasonal"  = aggregate(x, by= function(tt) paste0(format(tt, "%Y"), "-", time2season(tt)), FUN=fun),  
    #"annual"    = aggregate(x, by= function(tt) format(tt, "%Y")         , FUN=fun)
    "annual"    = aggregate(x, by= years                                 , FUN=fun)
  ) # 'md' END

  if (out.type=="percentage") {
  	out <- nNA/ndata
  	out <- round(out, dec)
  } else out <- nNA  # out.type=="amount"

  return(out)

} # 'cmv.zoo' END



################################################################################
#                              cmv.data.frame                                  #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2023 (Buenos Aires)                                          #
# Updates: 28-Jul-2023 ; 03-Aug-2023 ; 27-Nov-2023                             #
#          03-May-2025 (EGU 2025)                                              #
################################################################################
# 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
#             dates for correponding to the 'sname' station
#             If 'dates' is a number, it indicates the index of the column in
#                'x' that stores the dates
#             If 'dates' is a factor, it have to be converted into 'Date' class,
#                using the date format  specified by 'date.fmt'
#             If 'dates' is already of Date class, the following line verifies that
#                the number of days in 'dates' be equal to the number of element in the
#                time series corresponding to the 'st.name' station
# 'date.fmt': character indicating the format in which the dates are stored in 'dates'.
#             ONLY required when class(dates)=="factor" or "numeric"
# 'verbose' : logical; if TRUE, progress messages are printed
cmv.data.frame <- function(x, tscale=c("hourly", "daily", "weekly", "monthly", "quarterly", "seasonal", "annual"), 
	                         out.type=c("percentage", "amount"),
	                         dec=3,
                           start="00:00:00", 
                           start.fmt= "%H:%M:%S", 
                           tz,
                           start.month=1,
                           dates=1, 
                           date.fmt="%Y-%m-%d", ...) {
                                    
  # Checking that the user provied a valid argument for 'dates'
  if (is.na(match(class(dates), c("numeric", "factor", "Date"))))
    stop("Invalid argument: 'dates' must be of class 'numeric', 'factor', 'Date'")

  # If 'dates' is a number, it indicates the index of the column of 'x' that stores the dates
  # The column with dates is then substracted form 'x' for easening the further computations
  if ( inherits(dates, "numeric") ) {
    tmp   <- dates
    dates <- as.Date(x[, dates], format= date.fmt) # zoo::as.Date
    x     <- x[-tmp]
  }  # IF end

  # If 'dates' is a factor, it have to be converted into 'Date' class,
  # using the date format  specified by 'date.fmt'
  if ( inherits(dates, "factor") ) dates <- as.Date(dates, format= date.fmt) # zoo::as.Date

  # If 'dates' is already of Date class, the following line verifies that
  # the number of days in 'dates' be equal to the number of element in the
  # time series corresponding to the 'st.name' station
  if ( ( inherits(dates, "Date") ) & (length(dates) != nrow(x) ) )
     stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
     
  # Transforming 'x' into a zoo object
  tryCatch(
          #this is the chunk of code we want to run
          { x <- zoo(x, dates)
          #when it throws an error, the following block catches the error
          }, error = function(msg){return(NA)}
          )
  
  cmv.zoo(x=x, tscale=tscale, out.type=out.type, dec=dec, 
          start=start, start.fmt=start.fmt, tz=tz,
          start.month=start.month)

 } #'cmv.data.frame' END


################################################################################
#                                cmv.matrix                                    #
################################################################################
# Author : Mauricio Zambrano-Bigiarini                                         #
################################################################################
# Started: 25-Jul-2023 (Buenos Aires)                                          #
# Updates: 28-Jul-2023 ; 03-Aug-2023 ; 27-Nov-2023                             #
#          03-May-2025 (EGU 2025)                                              #
################################################################################
cmv.matrix  <- function(x, tscale=c("hourly", "daily", "weekly", "monthly", "quarterly", "seasonal", "annual"), 
	                      out.type=c("percentage", "amount"),
	                      dec=3,
                        start="00:00:00", 
                        start.fmt= "%H:%M:%S", 
                        tz,
                        start.month=1,
                        dates=1, 
                        date.fmt="%Y-%m-%d", ...) {

   x <- as.data.frame(x)
   cmv.data.frame(x=x, tscale=tscale, out.type=out.type, dec=dec,
                  start=start, start.fmt=start.fmt, tz=tz,
                  start.month=start.month,
                  dates=dates, date.fmt=date.fmt)

} # 'cmv.matrix  ' END

---
title: Introductory analysis of daily streamflows with hydroTSM
author:
- Mauricio Zambrano-Bigiarini^[mauricio.zambrano@ufrontera.cl]
date: "version 0.3, 27-Dec-2025"
output:
  pdf_document: 
    number_sections: true
    fig_caption: true
  html_document:
    df_print: paged
    number_sections: yes
vignette: |
  %\VignetteIndexEntry{Introductory analysis of daily streamflows with hydroTSM} 
  %\VignetteKeyword{hydrology}
  %\VignetteKeyword{hydrological modelling}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown} 
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Citation

If you use *[hydroTSM](https://cran.r-project.org/package=hydroTSM)*, please cite it as Zambrano-Bigiarini (2024):

Zambrano-Bigiarini, Mauricio (2024). hydroTSM: Time Series Management and Analysis for Hydrological Modelling. R package version 0.7-0. URL:https://cran.r-project.org/package=hydroTSM. doi:10.5281/zenodo.839565.


# Installation

Installing the latest stable version (from [CRAN](https://cran.r-project.org/package=hydroTSM)):
```{r installation1, eval=FALSE}
install.packages("hydroTSM")
```


\noindent Alternatively, you can also try the under-development version (from  [Github](https://github.com/hzambran/hydroTSM)):
```{r installation2, eval=FALSE}
if (!require(devtools)) install.packages("devtools")
library(devtools)
install_github("hzambran/hydroTSM")
```


# Setting up the environment


Loading the *hydroTSM* package, which contains data and functions used in this analysis:

```{r LoadingPkg}
library(hydroTSM)
```

Loading daily streamflow data at the station Cauquenes en el Arrayan, Maule Region, Chile, from 01/Jan/1979 to 31/Dec/2020.

```{r LoadingData}
data(Cauquenes7336001)
```

Selecting only a 30-years time slice for the analysis

```{r Window1}
x <- window(Cauquenes7336001, start="1981-01-01", end="2010-12-31")
```

Dates of the daily values of 'x':

```{r Dates}
dates <- time(x)
```

Amount of years in 'x' (needed for computations):

```{r yip}
( nyears <- yip(from=start(x), to=end(x), out.type="nmbr" ) )
```


The `Cauquenes7336001` dataset stores 5 variables (in this order): P, [mm], Tmx, [degC], Tmn, [deg C], PET, [mm], Qobs, [mm], Qobs, [m3/s].
For the rest of the analysis, only streamflows (Q, [mm]) and precipitations (P, [mm]) will be selected:

```{r SelectingPyQ}
P <- x[, 1]
Q <- x[, 5]
```


# Basic exploratory data analysis (EDA)


1)  Summary statistics of streamflows:

```{r smry}
smry(Q)
```

2)  Amount of days with information (not NA) per year:

```{r dwi1}
dwi(Q)
```

3)  Amount of days with information (not NA) per month per year:

```{r dwi2}
dwi(Q, out.unit="mpy")
```

4) Since v0.7-0, hydroTSM allows the computation of the amount/percentage of days with missing data in different temporal scales (e.g., hourly, weekly, seasonal). By default, the `cmv` function returns the percentage of missing values in the desired temporal scale using decimal values:

```{r cmv1, R.options=list(max.print=20)}
( pmd <- cmv(Q, tscale="monthly") )
```

Identifying months with more than 10 percent of missing data:

```{r cmv2}
index <- which(pmd >= 0.1)
time(pmd[index])
```

5) Computation of monthly values only when the percentage of NAs in each month is lower than a user-defined percentage (10% in this example).

```{r Custom_daily2monthly, R.options=list(max.print=20)}     
## Daily to monthly, only for months with less than 10% of missing values
(m2 <- daily2monthly(Q, FUN=mean, na.rm=TRUE, na.rm.max=0.1))
```



6)  Basic exploratory figures:

Using the *hydroplot* function, which (by default) plots 9 different graphs: 3 ts plots, 3 boxplots and 3 histograms summarizing 'x'. For this example, only daily and monthly plots are produced, and only data starting on 01-Jan-1987 are plotted.

```{r hydroplot, fig.width=10, fig.height=8}
hydroplot(Q, var.type="Flow", main="at Cauquenes en el Arrayan", 
          pfreq = "dm", from="2000-01-01")
```

Plotting P and Q for the full time period of both time series:

```{r plot_pq1, fig.width=10, fig.height=8}
plot_pq(p=P, q=Q)
```
     
Plotting precipitation and streamflows only for a specific time period, from April to December 2000:
```{r plot_pq2, fig.width=10, fig.height=8}
plot_pq(p=P, q=Q, from="2000-04-01", to="2000-12-31")
```

Plotting monthly values of precipitation and streamflows for the full time period of both time series:
```{r plot_pq3, fig.width=10, fig.height=8}
plot_pq(p=P, q=Q, ptype="monthly")
```

Plotting monthly values of precipitation and streamflows for the full time period of both time series, but using a hydrologic year starting on April:

```{r plot_pq4, fig.width=10, fig.height=8}
plot_pq(p=P, q=Q, ptype="monthly", start.month=4)
```
     


Selecting only a six-year time period for streamflows and then plotting a calendar heatmap (six years maximum) to visually identify dry, normal and wet days:

```{r calendarHeatmap, fig.width=8.6, fig.height=8.7}
q <- window(Q, start="2005-01-01", end="2010-12-31")
calendarHeatmap(q)
```

This figure allows to easily identify periods with missing data (e.g., Apr/2008 and Aug/2009).
For each month, this figure is read from top to bottom. For example, January 1st 2007 was Monday, January 31th 2007 was Wednesday and October 1st 2010 was Friday.

# Flow duration curve (FDC)


Flow duration curve of the 30-year daily streamflow data using logarithmic scale for the `y` axis (i.e., to put focus on the low flows):

```{r FDC2}
fdc2 <- fdc(Q)
```

Please note that `log="y"` was not provided as an argument to `fdc` because it is the default value used in the function.


Flow duration curve of the 30-year daily streamflow data using logarithmic scale for the `x` axis (i.e., to put focus on the high flows):

```{r FDC3}
fdc3 <- fdc(Q, log="x")
```

Traditional flow duration curve of the 30-year daily streamflow data:

```{r FDC1}
fdc1 <- fdc(Q, log="", thr.pos="topright")
```

# Baseflow

Since v0.7-0, hydroTSM allows the computation of baseflow using the filter proposed by Arnold and Allen (1999), which is based on earlier work by Lyne and Hollick (1979).

This first exmaple illustrates the basic usage of the `baseflow` function for computing and plotting the baseflow for the full time period of a given time series of streamflows:

```{r baseflow1, eval=FALSE}
baseflow(Q) 
```
The previous code did not run because the streamflow time series has some missing values. 
You might fill in the missing values using the technique that you like the most and then call this function again.
For this example, we will use one of the two built-in techniques already incorporated in the `baseflow` function the missing data, i.e., `na.fill="spline`:

```{r baseflow2, eval=TRUE, R.options=list(max.print=20)}
baseflow(Q, na.fill="spline") 
```

Now, we will compute and plot the daily baseflow (i.e., the value obtained after the thir pass of the filter) for the full time period:

```{r baseflow3, eval=TRUE, fig.width=11, fig.height=6, R.options=list(max.print=20)}
baseflow(Q, na.fill="spline", plot=TRUE)
```


You might also want to compute and plot the daily baseflow for a specific time period. For this example, from April to December 2000:

```{r baseflow4, eval=TRUE, R.options=list(max.print=20)}
baseflow(Q, na.fill="spline", from="2000-04-01", to="2000-12-31")
```


You might want to compute and plot the three daily baseflows (one for each pass of the filter), for a specific time period (April to December 2000):

```{r baseflow5, eval=TRUE, fig.width=11, fig.height=6, R.options=list(max.print=20)}
baseflow(Q, na.fill="spline", from="2000-04-01", to="2000-12-31", 
         out.type="all", plot=TRUE)
```


         

# Software details

This tutorial was built under: 

```{r echo=FALSE}
sessionInfo()$platform
sessionInfo()$R.version$version.string
paste("hydroTSM"  , sessionInfo()$otherPkgs$hydroTSM$Version)
```

# Version history

-) v0.30: 27-Dec-2025

-) v0.20: 21-Jan-2024

-) v0.01: 2009


# Appendix

In order to make easier the use of \texttt{hydroTSM} for users not familiar with R, in this section a minimal set of information is provided to guide the user in the [R](https://www.r-project.org/) world.


## Editors, GUI

*  **Multi-platform**: [Sublime Text](https://www.sublimetext.com/) (https://www.sublimetext.com/ ; [RStudio](https://posit.co/) (https://posit.co/)

*  **GNU/Linux only**: [ESS](https://ess.r-project.org/) (https://ess.r-project.org/)

*  **Windows only**  : [NppToR](https://sourceforge.net/projects/npptor/) (https://sourceforge.net/projects/npptor/)


## Importing data

*  `?read.table`, `?write.table`: allow the user to read/write a file (in $~$table format) and create a data frame from it. Related functions are `?read.csv`, `?write.csv`, `?read.csv2`, `?write.csv2`.

*  `?zoo::read.zoo`, `?zoo::write.zoo`: functions for reading and writing time series from/to text files, respectively.

*  [**R Data Import/Export**](https://CRAN.R-project.org/doc/manuals/r-release/R-data.html): https://CRAN.R-project.org/doc/manuals/r-release/R-data.html

*  [**foreign** R package](https://CRAN.R-project.org/package=foreign): read data stored in several R-external formats (dBase, Minitab, S, SAS, SPSS, Stata, Systat, Weka, ...)

*  [**readxl** R package](https://CRAN.R-project.org/package=readxl): Import MS Excel files into R. 

*  [**some examples**](https://www.datacamp.com/doc/r/importingdata): https://www.datacamp.com/doc/r/importingdata



## Useful Websites

*  [**Quick R**](https://www.datacamp.com/doc/r/category/r-documentation): https://www.datacamp.com/doc/r/category/r-documentation

*  [**Time series in R**](https://CRAN.R-project.org/view=TimeSeries): https://CRAN.R-project.org/view=TimeSeries

*  [**Quick reference for the `zoo` package**](https://CRAN.R-project.org/package=zoo/vignettes/zoo-quickref.pdf): https://CRAN.R-project.org/package=zoo/vignettes/zoo-quickref.pdf


## F.A.Q.

# How to print more than one `matrixplot` in a single Figure?
Because `matrixplot` is based on lattice graphs, normal plotting commands included in base R does not work. 
Therefore, for plotting ore than 1 matrixplot in a single figure, you need to save the individual plots in an R object and then print them as you want.

In the following sequential lines of code, you can see two examples that show you how to plot two matrixplots in a single Figure:


```{r FAQmatrixplot1, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
library(hydroTSM)
data(SanMartinoPPts)
x <- window(SanMartinoPPts, end=as.Date("1960-12-31"))
m <- daily2monthly(x, FUN=sum, na.rm=TRUE)
M <- matrix(m, ncol=12, byrow=TRUE)
colnames(M) <- month.abb
rownames(M) <- unique(format(time(m), "%Y"))
p <- matrixplot(M, ColorRamp="Precipitation", main="Monthly precipitation,")

print(p, position=c(0, .6, 1, 1), more=TRUE)
print(p, position=c(0, 0, 1, .4))
```

The second and easier way allows you to obtain the same previous figure (not shown here), but you are required to install the `gridExtra` package:

```{r FAQmatrixplot2, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9), eval=TRUE}
if (!require(gridExtra)) install.packages("gridExtra")
require(gridExtra) # also loads grid
require(lattice)

grid.arrange(p, p, nrow=2)
```

---
title: Introductory analysis of daily precipitation with hydroTSM
author:
- Mauricio Zambrano-Bigiarini^[mauricio.zambrano@ufrontera.cl]
date: "version 1.0, 27-Dec-2025"
output:
  pdf_document:
    number_sections: yes
  html_document:
    df_print: paged
    number_sections: yes
vignette: |
  %\VignetteIndexEntry{Introductory analysis of daily precipitation with hydroTSM} 
  %\VignetteKeyword{hydrology}
  %\VignetteKeyword{hydrological modelling}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown} 
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Citation

If you use *[hydroTSM](https://CRAN.R-project.org/package=hydroTSM)*, please cite it as Zambrano-Bigiarini (2025):

Zambrano-Bigiarini, Mauricio (2025). hydroTSM: Time Series Management and Analysis for Hydrological Modelling. R package version 0.8-0. URL:https://CRAN.R-project.org/package=hydroTSM. doi:10.32614/CRAN.package.hydroTSM.


# Installation

Installing the latest stable version (from [CRAN](https://CRAN.R-project.org/package=hydroTSM)):
```{r installation1, eval=FALSE}
install.packages("hydroTSM")
```


\noindent Alternatively, you can also try the under-development version (from  [Github](https://github.com/hzambran/hydroTSM)):
```{r installation2, eval=FALSE}
if (!require(devtools)) install.packages("devtools")
library(devtools)
install_github("hzambran/hydroTSM")
```


# Setting up the environment


Loading the *hydroTSM* package, which contains data and functions used in this analysis:

```{r LoadingPkg}
library(hydroTSM)
```

Loading daily precipitation data at the station San Martino di Castrozza, Trento Province, Italy, from 01/Jan/1921 to 31/Dec/1990.

```{r LoadingData}
data(SanMartinoPPts)
```

Selecting only a 6-years time slice for the analysis

```{r Window1}
x <- window(SanMartinoPPts, start="1985-01-01")
```

Dates of the daily values of 'x'
```{r Dates}
dates <- time(x)
```

Amount of years in 'x' (needed for computations)
```{r yip}
( nyears <- yip(from=start(x), to=end(x), out.type="nmbr" ) )
```


# Basic exploratory data analysis (EDA)


1)  Summary statistics
```{r smry}
smry(x)
```

2)  Amount of days with information (not NA) per year
```{r dwi1}
dwi(x)
```

3)  Amount of days with information (not NA) per month per year
```{r dwi2}
dwi(x, out.unit="mpy")
```


4) Computation of monthly values only when the percentage of NAs in each month is lower than a user-defined percentage (10% in this example).

```{r Custom_daily2monthly}     
# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
y <- SanMartinoPPts
     
# Subsetting 'y' to its first three months (Jan/1921 - Mar/1921)
y <- window(y, end="1921-03-31")
     
## Transforming into NA the 10% of values in 'y'
set.seed(10) # for reproducible results
n           <- length(y)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
y[na.index] <- NA
     
## Daily to monthly, only for months with less than 10% of missing values
(m2 <- daily2monthly(y, FUN=sum, na.rm=TRUE, na.rm.max=0.1))
     
# Verifying that the second and third month of 'x' had 10% or more of missing values
cmv(y, tscale="month")
```



4)  Basic exploratory figures:

Using the *hydroplot* function, which (by default) plots 9 different graphs: 3 ts plots, 3 boxplots and 3 histograms summarizing 'x'. For this example, only daily and monthly plots are produced, and only data starting on 01-Jan-1987 are plotted.

```{r hydroplot, fig.width=10, fig.height=8}
hydroplot(x, var.type="Precipitation", main="at San Martino", 
          pfreq = "dm", from="1987-01-01")
```

Global view of daily precipitation values a calendar heatmap (six years maximum), useful for visually identifying dry, normal and wet days:

```{r calendarHeatmap, fig.width=8.6, fig.height=8.7}
calendarHeatmap(x)
```

For each month, the previous figure is read from top to bottom. For example, January 1st 1987 was Thursday, January 31th 1987 was Saturday and November 1st 1990 was Thursday.


Selecting only a three-month time slice for the analysis:

```{r Window3}
yy <- window(SanMartinoPPts, start="1990-10-01")
```

Plotting the selected time series:

```{r hydroplot3}
hydroplot(yy,  ptype="ts", pfreq="o", var.unit="mm")
```


# Annual analysis


Annual values of precipitation

```{r daily2annual}
daily2annual(x, FUN=sum, na.rm=TRUE)
```

Average annual precipitation

Obvious way:

```{r daily2annual2}
mean( daily2annual(x, FUN=sum, na.rm=TRUE) )
```

Another way (more useful for streamflows, where `FUN=mean`):

The function *annualfunction* applies `FUN` twice over `x`: 

( i) firstly, over all the elements of `x` belonging to the same year, in order to obtain the corresponding annual values, and
(ii) secondly, over all the annual values of `x` previously obtained, in order to obtain a single annual value.

```{r annualfunction}
annualfunction(x, FUN=sum, na.rm=TRUE) / nyears
```



# Monthly analysis

1) Plotting the monthly precipitation values for each year, useful for identifying dry/wet months.


```{r matrixplot}
# Daily zoo to monthly zoo
m <- daily2monthly(x, FUN=sum, na.rm=TRUE)
     
# Creating a matrix with monthly values per year in each column
M <- matrix(m, ncol=12, byrow=TRUE)
colnames(M) <- month.abb
rownames(M) <- unique(format(time(m), "%Y"))
     
# Plotting the monthly precipitation values
require(lattice)
print(matrixplot(M, ColorRamp="Precipitation", 
           main="Monthly precipitation at San Martino st., [mm/month]"))
```

2) Median of the monthly values at station 'x'. Not needed, just for looking at these values in the boxplot.

```{r monthlyfunction}
monthlyfunction(m, FUN=median, na.rm=TRUE)
```

3) Vector with the three-letter abbreviations for the month names

```{r cmonth}
cmonth <- format(time(m), "%b")
```

4) Creating ordered monthly factors

```{r months}
months <- factor(cmonth, levels=unique(cmonth), ordered=TRUE)
```

5) Boxplot of the monthly values

```{r boxplotMonthly}
boxplot( coredata(m) ~ months, col="lightblue", main="Monthly Precipitation", 
         ylab="Precipitation, [mm]", xlab="Month")
```



# Seasonal analysis

Average seasonal values of precipitation

```{r seasonalfunction}
seasonalfunction(x, FUN=sum, na.rm=TRUE) / nyears
```

Extracting the seasonal values for each year
```{r dm2seasonal}
( DJF <- dm2seasonal(x, season="DJF", FUN=sum) )
( MAM <- dm2seasonal(m, season="MAM", FUN=sum) )
( JJA <- dm2seasonal(m, season="JJA", FUN=sum) )
( SON <- dm2seasonal(m, season="SON", FUN=sum) )
```


Plotting the time evolution of the seasonal precipitation values

```{r hydroplot2, fig.width=12, fig.height=10}
hydroplot(x, pfreq="seasonal", FUN=sum, stype="default")
```





# Some extreme indices

Common steps for the analysis of this section:


Loading daily precipitation data at the station San Martino di Castrozza, Trento Province, Italy, with data from 01/Jan/1921 to 31/Dec/1990.
```{r LoadingData2}
data(SanMartinoPPts)
```
	
Selecting only a 6-year time slice for the analysis
```{r Window4}
x <- window(SanMartinoPPts, start="1985-10-01")
```

Plotting the selected time series
```{r hydroplot4}
hydroplot(x,  ptype="ts", pfreq="o", var.unit="mm")
```


## Seasonality index

Computing the seasonality index defined by Walsh and Lawler (1981) to classify the precipitation regime of `x`:

```{r SeasonalityIndex}
si(x)
```

According to the seasonality index defined by Walsh and Lawler (1981), a value of 0.35 corresponds to a precipitation regime that can be classified as "Equable but with a definite wetter season" (see more details with `?si`).

## Heavy precipitation days (R10mm)

Counting and plotting the number of days in the period where precipitation is > 10 [mm]:

```{r R10mm}
( R10mm <- length( x[x>10] ) )
```





## Very wet days (R95p)

Identifying the wet days (daily precipitation >= 1 mm):
```{r wet_index}
wet.index <- which(x >= 1)
```
	
	
Computing the 95th percentile of precipitation on wet days (*PRwn95*):

```{r PRwn95}
( PRwn95 <- quantile(x[wet.index], probs=0.95, na.rm=TRUE) )
```

**Note 1**: this computation was carried out for the three-year time period 1988-1990, not the 30-year period 1961-1990 commonly used. 

**Note 2**: missing values are removed from the computation.


Identifying the very wet days (daily precipitation >= *PRwn95*):

```{r very_wet_index}
(very.wet.index <- which(x >= PRwn95))
```


Computing the total precipitation on the very wet days:

```{r R95p}
( R95p <- sum(x[very.wet.index]) )
```

**Note 3**: this computation was carried out for the three-year time period 1988-1990, not the 30-year period 1961-1990 commonly used 



## 5-day total precipitation

Computing the 5-day total (accumulated) precipitation:

```{r x_5max}
x.5max <- rollapply(data=x, width=5, FUN=sum, fill=NA, partial= TRUE, 
                    align="center")

hydroplot(x.5max,  ptype="ts+boxplot", pfreq="o", var.unit="mm")
```

Maximum annual value of 5-day total precipitation:

```{r (x_5max_annual}
(x.5max.annual <- daily2annual(x.5max, FUN=max, na.rm=TRUE))
```

**Note 1**: for this computation, a moving window centred in the current day is used. If the user wants the 5-day total precipitation accumulated in the 4 days before the current day + the precipitation in the current day, the user have to modify the moving window.\newline

**Note 2**: For the first two and last two values, the width of the window is adapted to ignore values not within the time series


# Climograph

Since v0.5-0, `hydroTSM` includes a function to plot a climograph, considering not only precipitation but air temperature data as well.

```{r climograph1, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
# Loading daily ts of precipitation, maximum and minimum temperature
data(MaquehueTemuco)

# extracting individual ts of precipitation, maximum and minimum temperature
pcp <- MaquehueTemuco[, 1]
tmx <- MaquehueTemuco[, 2]
tmn <- MaquehueTemuco[, 3]
```

Plotting a full climograph:
```{r climograph2, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)
```

\newpage
Plotting a climograph with uncertainty bands around mean values, but with no labels for tmx and tmn:
```{r climograph3, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, tmx.labels=FALSE, tmn.labels=FALSE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)
```

\newpage
Plotting a climograph with uncertainty bands around mean values, but with no labels for tmx, tmn and pcp:
```{r climograph4, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, 
                pcp.labels=FALSE, tmean.labels=FALSE, tmx.labels=FALSE, tmn.labels=FALSE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)
```

\newpage
To better represent the hydrological year in Chile (South America), the following figure will plot a full climograph starting in April (`start.month=4`) instead of January (`start.month=1`):
```{r climograph5, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, 
                start.month=4, temp.labels.dx=c(rep(-0.2,4), rep(0.2,6),rep(-0.2,2)),
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)
```

# Software details
This tutorial was built under: 

```{r echo=FALSE}
sessionInfo()$platform
sessionInfo()$R.version$version.string 
paste("hydroTSM", sessionInfo()$otherPkgs$hydroTSM$Version)
```


# Version history

* v1.0: 27-Dec-2025
* v0.9: 21-Jan-2024
* v0.8: Nov 2023
* v0.7: Mar 2020
* v0.6: Aug 2017
* v0.5: May 2013
* v0.4: Aug 2011
* v0.3: Apr 2011
* v0.2: Oct 2010
* v0.1: 30-May-2013

# Appendix

In order to make easier the use of \texttt{hydroTSM} for users not familiar with R, in this section a minimal set of information is provided to guide the user in the [R](https://www.r-project.org/) world.


## Editors, GUI

*  **Multi-platform**: [Sublime Text](https://www.sublimetext.com/) (https://www.sublimetext.com/) ; [RStudio](https://posit.co/) (https://posit.co/)

*  **GNU/Linux only**: [ESS](https://ess.r-project.org/) (https://ess.r-project.org/)

*  **Windows only**  : [NppToR](https://sourceforge.net/projects/npptor/) (https://sourceforge.net/projects/npptor/)


## Importing data

*  `?read.table`, `?write.table`: allow the user to read/write a file (in $~$table format) and create a data frame from it. Related functions are `?read.csv`, `?write.csv`, `?read.csv2`, `?write.csv2`.

*  `?zoo::read.zoo`, `?zoo::write.zoo`: functions for reading and writing time series from/to text files, respectively.

*  [**R Data Import/Export**](https://cran.r-project.org/doc/manuals/r-release/R-data.html): https://cran.r-project.org/doc/manuals/r-release/R-data.html

*  [**foreign** R package](https://cran.r-project.org/package=foreign): read data stored in several R-external formats (dBase, Minitab, S, SAS, SPSS, Stata, Systat, Weka, ...)

*  [**readxl** R package](https://cran.r-project.org/package=readxl): Import MS Excel files into R. 

*  [**some examples**](https://www.datacamp.com/doc/r/importingdata): https://www.datacamp.com/doc/r/importingdata



## Useful Websites

*  [**Quick R**](https://www.datacamp.com/doc/r/category/r-documentation): https://www.datacamp.com/doc/r/category/r-documentation

*  [**Time series in R**](https://cran.r-project.org/view=TimeSeries): https://cran.r-project.org/view=TimeSeries

*  [**Quick reference for the `zoo` package**](https://cran.r-project.org/package=zoo/vignettes/zoo-quickref.pdf): https://cran.r-project.org/package=zoo/vignettes/zoo-quickref.pdf


## F.A.Q.

# How to print more than one `matrixplot` in a single Figure?
Because `matrixplot` is based on lattice graphs, normal plotting commands included in base R does not work. 
Therefore, for plotting ore than 1 matrixplot in a single figure, you need to save the individual plots in an R object and then print them as you want.

In the following sequential lines of code, you can see two examples that show you how to plot two matrixplots in a single Figure:


```{r FAQmatrixplot1, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9)}
library(hydroTSM)
data(SanMartinoPPts)
x <- window(SanMartinoPPts, end=as.Date("1960-12-31"))
m <- daily2monthly(x, FUN=sum, na.rm=TRUE)
M <- matrix(m, ncol=12, byrow=TRUE)
colnames(M) <- month.abb
rownames(M) <- unique(format(time(m), "%Y"))
p <- matrixplot(M, ColorRamp="Precipitation", main="Monthly precipitation,")

print(p, position=c(0, .6, 1, 1), more=TRUE)
print(p, position=c(0, 0, 1, .4))
```

The second and easier way allows you to obtain the same previous figure (not shown here), but you are required to install the `gridExtra` package:

```{r FAQmatrixplot2, fig.width = 8, fig.height = 7, , dpi=100, fig.align = "center", dev.args=list(pointsize = 9), eval=TRUE}
if (!require(gridExtra)) install.packages("gridExtra")
require(gridExtra) # also loads grid
require(lattice)

grid.arrange(p, p, nrow=2)
```

%% File smry.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{smry}
\Rdversion{1.1}
\alias{smry}
\alias{smry.default}
\alias{smry.zoo}
\alias{smry.Date}
\alias{smry.matrix}
\alias{smry.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Summary
}
\description{
Extended summary function for numeric objects, with 13 summary statistics. 
}
\usage{
smry(x, ...)

\method{smry}{default}(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)

\method{smry}{zoo}(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)

\method{smry}{Date}(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)

\method{smry}{matrix}(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)

\method{smry}{data.frame}(x, na.rm=TRUE, digits = max(3, getOption("digits")-3), ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
a numeric object, vector, matrix or data.frame, for which a summary is desired.
}
  \item{na.rm}{
a logical value indicating whether 'NA' values should be stripped before the computation proceeds.
}
  \item{digits}{
numeric, with the amount of decimal places to be included in the result
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
Computed summary statistics are: \cr
\item{Min}{Minimum}
\item{1stQ}{First quartile (lower-hinge)}
\item{Mean}{Mean value}
\item{Median}{Median}
\item{3rdQ}{Third quartile ( upper-hinge}
\item{Max}{Maximum of the input values.}
\item{IQR}{Interquartile Range. \code{IQR(x) = quantile(x,3/4) - quantile(x,1/4)}}
\item{sd}{Standard deviation. It uses 'n-1' as denominator.}
\item{cv}{Coefficient of variation ( \code{cv= sd / |mean|} )}
\item{skewness}{Skewness (using \pkg{e1071} package)   }
\item{kurtosis}{Kurtosis (using \pkg{e1071} package)   }
\item{n}{Total number of elements}
\item{NA's}{Amount of missing values}
}
%%\references{
%% ~put references to the literature/web site here ~
%%}

\author{
Mauricio Zambrano-Bigiarini \email{mzb.devel@gmail}
}
\note{
Skewness and Kurtosis are computed with the e1071 package
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{summary}}, \code{\link{fivenum}}, \code{\link{IQR}}, \code{\link{sd}}, \link[e1071]{skewness}, \link[e1071]{kurtosis} 
}
\examples{
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

## Summary of monthly precipitation values for the first 7 stations in 'EbroPPtsMonthly'
smry(EbroPPtsMonthly[,2:8])
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File extract.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2015 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{extract}
\Rdversion{1.1}
\alias{extract}
\alias{extract.default}
\alias{extract.zoo}
\alias{extractzoo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Extract from Zoo
}
\description{
Extracts from a zoo object all the values belonging to a given month, year or weather season.
}
\usage{
extract(x, ...)

\method{extract}{default}(x, trgt, ...)
     
\method{extract}{zoo}(x, trgt, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo object
}
  \item{trgt}{
numeric or character indicating the elements to extract from \code{x}. Valid values are: \cr
1) integer(s) from 1 to 12: \code{trgt} is considered as month(s) (1=JAN, 2=FEB,...., 12=DEC), and all the values in \code{x} belonging to the month(s) specified by \code{trgt} will be extracted. \cr
2) integer(s) > 12: \code{trgt} is considered as year(s), and all the values in \code{x} belonging to the year(s) specified by \code{trgt} will be extracted \cr
3) character: \code{trgt} is considered as a weather season, and all the values in \code{x} belonging to the season specified by \code{trgt} will be extracted. Valid values are: \cr
   -) \kbd{DJF} : December, January, February   \cr                   
   -) \kbd{MAM} : March, April, May             \cr                     
   -) \kbd{JJA} : June, July, August            \cr                      
   -) \kbd{SON} : September, October, November  \cr
   -) \kbd{DJFM}: December, January, February, March  \cr                   
   -) \kbd{AM}  : April, May                    \cr  
   -) \kbd{JJAS}: June, July, August, September \cr            
   -) \kbd{ON}  : October, November
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
a zoo object with the extracted values.
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{time2season}}, \code{\link{seasonalfunction}}, \code{\link{daily2annual}}, \code{\link{daily2monthly}}
}
\examples{
### Loading temperature data ##
data(SanMartinoPPts)
x <- SanMartinoPPts

## Extracting all the values belonging to February (FEB=2)
extract(x, trgt=2)

## Extracting all the values belonging to February (FEB=2) and April (APR=4)
extract(x, trgt=c(2,4))

## Extracting all the values belonging to the year 1970
extract(x, trgt=1970)

## Extracting all the values belonging to the years 1970 and 1972
extract(x, trgt=c(1970,1972))

## Extracting all the values belonging to the autumn
extract(x, trgt="SON")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File dm2seasonal.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{dm2seasonal}
\Rdversion{1.1}
\alias{dm2seasonal}
\alias{dm2seasonal.default}
\alias{dm2seasonal.zoo}
\alias{dm2seasonal.data.frame}
\alias{dm2seasonal.matrix}
\alias{subdaily2seasonal}
\alias{subdaily2seasonal.default}
\alias{subdaily2seasonal.zoo}
\alias{subdaily2seasonal.data.frame}
\alias{subdaily2seasonal.matrix}

\title{
(sub)Daily/Monthly -> Seasonal Values
}
\description{
Generic function for computing a seasonal value for every year of a sub-daily/daily/weekly/monthly time series
}
\usage{
dm2seasonal(x, ...)
subdaily2seasonal(x, ...)

\method{dm2seasonal}{default}(x, season, FUN, na.rm = TRUE, out.fmt="\%Y", ...)

\method{dm2seasonal}{zoo}(x, season, FUN, na.rm = TRUE, out.fmt="\%Y", ...)

\method{dm2seasonal}{data.frame}(x, season, FUN, na.rm = TRUE, dates=1, date.fmt = "\%Y-\%m-\%d", 
            out.type = "data.frame", out.fmt="\%Y", ...)
        
\method{dm2seasonal}{matrix}(x, season, FUN, na.rm = TRUE, dates=1, date.fmt = "\%Y-\%m-\%d", 
            out.type = "data.frame", out.fmt="\%Y", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, xts, data.frame or matrix object, with sub-daily, daily, weekly or monthly time series. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{season}{
character, indicating the weather season to be used for selecting the data. Valid values are: \cr
-) \kbd{DJF} : December, January, February   \cr
-) \kbd{MAM} : March, April, May             \cr
-) \kbd{JJA} : June, July, August            \cr
-) \kbd{SON} : September, October, November  \cr
-) \kbd{DJFM}: December, January, February, March  \cr                   
-) \kbd{AM}  : April, May                    \cr   
-) \kbd{JJAS}: June, July, August, September \cr            
-) \kbd{ON}  : October, November
}
  \item{FUN}{
Function that will be applied to ALL the values of \code{x} belonging to the given weather season (e.g., \code{FUN} can be some of \code{mean}, \code{max}, \code{min}, \code{sd}). \cr
\bold{The FUN value for the winter season (DJF or DJFM) is computed considering the consecutive months of December, January and February/March}. See 'Note' section.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the seasonal values  are computed considering only those values different from NA (\bold{very important when \code{FUN=sum}}) \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a weather season, the corresponding seasonal values are NA
}
  \item{out.fmt}{
Character indicating the date format for the output time series. See \code{format} in \code{\link[base]{as.Date}}. Possible values are: \cr
-) \kbd{\%Y}        : only the year will be used for the time. Default option. (e.g., "1961" "1962"...) \cr
-) \kbd{\%Y-\%m-\%d}: a complete date format will be used for the time. (e.g., "1961-01-01" "1962-01-01"...)
}
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates. \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, by using the date format specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days on it be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
Character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
Character that defines the desired type of output. Valid values are: \cr
-) \kbd{data.frame}: a data.frame, with as many columns as stations are included in \code{x}, the year corresponding to each seasonal value are used as row names. \cr
-) \kbd{db}        : a data.frame, with 4 columns will be produced. \cr
                 The first column (StationID) stores the ID of the station 
                 The second column (Year) stores the year,  \cr
                 The third column (Season) stores the season,\cr
                 The fourth column (Value) contains the seasonal value corresponding to the values specified in the previous three columns
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
A numeric vector with the seasonal values for all the years in which \code{x} is defined.
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
\code{FUN} is applied to all the values of \code{x} belonging to the selected season, so the results of this function depends on the frequency sampling of \code{x} and the type of function given by \code{FUN} \cr
}
\section{Warning}{
For any year, the \code{FUN} value for the winter season (DJF), is computed considering only January and February, and the value of December is used for computing the winter value of the next year.
}
\seealso{
, \code{\link{hydroplot}}, \code{\link{seasonalfunction}}, \code{\link{time2season}}, \code{\link{extract}}, \code{\link{daily2monthly}}, \code{\link{daily2annual}}, \code{\link{monthly2annual}}
}
\examples{
############
## Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

## Winter (DJF) values of precipitation for each year of 'x'
dm2seasonal(x, FUN=sum, season="DJF")

############
## Loading the HOURLY discharge data for the Karamea at Gorge streamgauge station
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

## Mean winter (DJF) values of streamflow for each year of 'x'
dm2seasonal(x, FUN=mean, season="DJF")
subdaily2seasonal(x, FUN=mean, season="DJF") # same as above


\dontshow{
############
############
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)
x <- EbroPPtsMonthly

## Winter (DJF) values of precipitation for the first 3 stations 
## in 'EbroPPtsMonthly' (its first column stores the dates)
dm2seasonal(x[,1:4], FUN=sum, season="DJF", dates=1)

## The same previous example, but using a zoo object
dates <- as.Date(x[,1]) # dates of the zoo object
z     <- zoo(x[ ,2:ncol(x)], dates) # zoo creation
dm2seasonal(z, FUN=sum, season="DJF")
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File sfreq.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{sfreq}
\Rdversion{1.1}
\alias{sfreq}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Sampling Frequency 
}
\description{
This function identifies the sampling frequency of a zoo object. It is wrapper to the \code{\link[xts]{periodicity}} function of the \pkg{xts} package.
}
\usage{
sfreq(x, min.year = 1800)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
variable of type zoo, xts or ts, with AT LEAST 2 elements, AND with a (sub)hourly, hourly, daily, weekly, monthly, quarterly, or annual (yearly) sampling frequency.
}
  \item{min.year}{
integer used for a correct identification of the sampling frequency when \code{x} is an annual (yearly) time series.
}
}
\details{
See further details in the \code{\link[xts]{periodicity}} function of the \pkg{xts} package.
}
\value{
Character. Possible values are: \cr
-) \kbd{minute}    : indicating that the sampling frequency in \code{x} is sub-hourly \cr
-) \kbd{hourly}    : indicating that the sampling frequency in \code{x} is hourly     \cr
-) \kbd{daily}     : indicating that the sampling frequency in \code{x} is daily      \cr
-) \kbd{weekly}    : indicating that the sampling frequency in \code{x} is weekly     \cr
-) \kbd{monthly}   : indicating that the sampling frequency in \code{x} is monthly    \cr
-) \kbd{quarterly} : indicating that the sampling frequency in \code{x} is quarterly  \cr
-) \kbd{annual}    : indicating that the sampling frequency in \code{x} is annual 
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{frequency}}, \link[xts]{periodicity}
}
\examples{
## Ex1: sub-hourly data
## Creating a dummy 15-min zoo object, with 1 as the only value in each time period
dt  <- seq( from=as.POSIXct("2021-06-30 00:15"), to=as.POSIXct("2021-06-30 23:45"), by="15 min" )
ndt <- length(dt)
shr <- zoo( rep(1, ndt), dt)
sfreq(shr)


## Ex2: hourly data 
## Loading the time series of HOURLY streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
hr <- KarameaAtGorgeQts
sfreq(hr)


## Ex3: Daily data
## Loading daily streamflows at the station Oca en Ona (Ebro River basin, Spain)
data(OcaEnOnaQts)
d <- OcaEnOnaQts
sfreq(d)


## Ex4: Monthly data
m <- daily2monthly(d, FUN=mean, na.rm=TRUE)
sfreq(m)


## Ex5: Annual data
a <- daily2annual(d, FUN=mean, na.rm=TRUE, out.fmt="\%Y-\%m-\%d")
sfreq(a)


}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File si.Rd
%% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%%                                 http://www.rforge.net/hydroTSM/ ; 
%% Copyright 2008-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{si}
\Rdversion{1.1}
\alias{si}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Seasonality Index
}
\description{
Function to compute the seasonality index defined by Walsh and Lawler (1981) to classify the precipitation regime.
}
\usage{
  si(x, na.rm=TRUE, from=start(x), to=end(x), date.fmt="\%Y-\%m-\%d", start.month=1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo object with daily or subdaily precipitation data.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the monthly values  are computed considering only those values different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the resulting average monthly value is NA .
}
  \item{from}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the starting date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{to}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the ending date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{date.fmt}{
Character indicating the format in which the dates are stored in \var{dates}, \var{from} and \var{to}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{start.month}{
[OPTIONAL]. Only used when the (hydrological) year of interest is different from the calendar year.

numeric in [1:12] indicating the starting month of the (hydrological) year. Numeric values in [1, 12] represents months in [January, December]. By default \code{start.month=1}.
}
}
\details{
The seasonality index is computed as following:\cr

               si = (1/R) *sum(i=1, i=12, abs(xi - R/12) ) \cr            
where: \cr                                                   
 -) xi: mean monthly precipitation for month i \cr
 -) R: mean annual precipitation               \cr             

This index can theoretically vary from 0 (when all months have the same rainfall) to 1.83 (when all the rainfall ocurrs in a single month).
A qualitative classification of degrees of seasonality is the following: \cr
-------------------------------------------------------- \cr
 si values   |                    Rainfall regime        \cr                    
-------------------------------------------------------- \cr
     <= 0.19 | Very equable                              \cr                    
0.20 - 0.39  | Equable but with a definite wetter season \cr                    
0.40 - 0.59  | Rather seasonal with a short drier season \cr                    
0.60 - 0.79  | Seasonal                                  \cr                    
0.80 - 0.99  | Markedly seasonal with a long drier season\cr                    
1.00 - 1.19  | Most rain in 3 months or less             \cr                                  
    >= 1.20  | Extreme, almost all rain in 1-2 months                        
}
\value{
numeric with the seasonality index
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
\references{
\cite{Walsh, R. and Lawler, D. (1981). Rainfall seasonality: Description, spatial patterns and change through time (British Isles, Africa). Weather, 36(7), 201-208. doi:10.1002/j.1477-8696.1981.tb05400.x}.
}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{subdaily2daily}}
}
\examples{

############################
## Ex 1: Seasonality index for a rain gauge with equable precipitation , 
##       but with a definite wetter season 

## Loading daily precipitation data at the station San Martino di Castrozza, 
## Trento Province, Italy, from 01/Jan/1921 to 31/Dec/1990.
data(SanMartinoPPts)
x <- SanMartinoPPts

## Amount of years in 'x' (needed for computations)
( nyears <- yip(from=start(x), to=end(x), out.type="nmbr" ) )


## Boxplot of monthly values, to look at the seasonal cycle

## Daily to Monthly
m <- daily2monthly(x, FUN=sum, na.rm=TRUE)

## Mean monthly values at the station
monthlyfunction(m, FUN=sum, na.rm=TRUE) / nyears

## Vector with the three-letter abbreviations of the month names
cmonth <- format(time(m), "\%b")

## Creating ordered monthly factors
months <- factor(cmonth, levels=unique(cmonth), ordered=TRUE)

## Boxplot of the monthly values of precipitation
boxplot( coredata(m) ~ months, col="lightblue", 
         main="Monthly precipitation, [mm]", ylab="P, [mm]")

# computing seasonality index
( si(x) )

############################
## Ex 2: Seasonality index for a rain gauge with markedly seasonal regime 
##       with a long dry season

## Loading daily precipitation data at the station Cauquenes en El Arrayan, 
## Maule Region, Chile, from 01/Jan/1979 to 31/Dec/2020.
data(Cauquenes7336001)
x <- Cauquenes7336001[, 1] # P is the first column

## Boxplot of monthly values, to look at the seasonal cycle

## Daily to Monthly
m <- daily2monthly(x, FUN=sum, na.rm=TRUE)

## Mean monthly values at the station
monthlyfunction(m, FUN=sum, na.rm=TRUE) / nyears

## Vector with the three-letter abbreviations of the month names
cmonth <- format(time(m), "\%b")

## Creating ordered monthly factors
months <- factor(cmonth, levels=unique(cmonth), ordered=TRUE)

## Boxplot of the monthly values of precipitation
boxplot( coredata(m) ~ months, col="lightblue", 
         main="Monthly precipitation, [mm]", ylab="P, [mm]")

# computing seasonality index
( si(x) )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File seasonalfunction.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{seasonalfunction}
\Rdversion{1.1}
\alias{seasonalfunction}
\alias{seasonalfunction.default}
\alias{seasonalfunction.zoo}
\alias{seasonalfunction.data.frame}
\alias{seasonalfunction.matrix}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Seasonal Function
}
\description{
Generic function for applying any R function to a zoo object, in order to obtain 4 representative seasonal values.
}
\usage{
seasonalfunction(x, ...)

\method{seasonalfunction}{default}(x, FUN, na.rm = TRUE, type="default", ...)

\method{seasonalfunction}{zoo}(x, FUN, na.rm = TRUE, type="default", ...)

\method{seasonalfunction}{data.frame}(x, FUN, na.rm = TRUE, type="default",
                          dates=1, date.fmt = "\%Y-\%m-\%d", 
                          out.type = "data.frame", verbose = TRUE, ...)
                          
\method{seasonalfunction}{matrix}(x, FUN, na.rm = TRUE, type="default",
                          dates=1, date.fmt = "\%Y-\%m-\%d", 
                          out.type = "data.frame", verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with daily or monthly time series. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that will be applied to ALL the values in \code{x} belonging to each one of the 4 weather seasons (e.g., \code{FUN} can be some of \code{mean}, \code{max}, \code{min}, \code{sd}). \cr
}
  \item{na.rm}{
Logical. Should missing values be removed before the computations? \cr
-) \kbd{TRUE} : the monthly values  are computed considering only those values in \code{x} different from NA  (\bold{very important when \code{FUN=sum}}) \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the FUN and monthly values are NA    
}
  \item{type}{
character, indicating which weather seasons will be used for computing the output. Possible values are: \cr
-) \kbd{default}          =>  "winter"= Dec, Jan, Feb;      "spring"= Mar, Apr, May;  "summer"=Jun, Jul, Aug;       "autumn"= Sep, Oct, Nov  \cr
-) \kbd{FrenchPolynesia}  =>  "winter"= Dec, Jan, Feb, Mar; "spring"= Apr, May;       "summer"=Jun, Jul, Aug, Sep;  "autumn"= Oct, Nov 
}
  \item{dates}{
numeric, factor, Date indicating how to obtain the dates. \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, by using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of element in \code{x}
}
  \item{date.fmt}{
Character indicating the format in which the dates are stored in \code{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
Character defining the desired type of output. Valid values are: \cr
-) \kbd{data.frame}: a data.frame, with 4 columns representing the weather seasons, and as many rows as stations are included in \code{x} \cr
-) \kbd{db}        : a data.frame, with 4 colums will be produced. Useful for a posterior boxplot \cr
                 The first column (StationID) will store the ID of the station,  \cr
                 The second column (Year) will store the year,              \cr
                 The third column (Season) will store the season,  \cr
                 The fourth column (Value) will contain the seasonal value corresponding to that year and that station.
}
  \item{verbose}{
Logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
\code{FUN} is applied to all the values of \code{x} belonging to each one of the four weather seasons, so the results of this function depends on the frequency sampling of \code{x} and the type of function given by \code{FUN} \cr
}
\section{Warning}{
The \var{FUN} value for the winter season (DJF) is computed considering the consecutive months of December, January and February. Therefore, if \code{x} starts in January and ends in December of any year, the winter value of the first year is computed considering only the January and February value of that year, whereas the December value of the first year is used to compute the winter value of the next year.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{dm2seasonal}}, \code{\link{time2season}}, \code{\link{monthlyfunction}}, \code{\link{annualfunction}}, \code{\link{extract}} 
}
\examples{
## Loading the SanMartino precipitation data
data(SanMartinoPPts)
x <- SanMartinoPPts

# Amount of years
nyears <- yip(from=start(x), to=end(x), out.type="nmbr")

## Mean annual precipitation.
# It is necessary to divide by the amount of years to obtain the mean annual value, 
# otherwise it will give the total precipitation for all the 70 years
seasonalfunction(x, FUN=sum, na.rm=TRUE) / nyears

#####################
### verification ####
# Mean winter (DJF) value
sum( extractzoo(x, trgt="DJF") ) / nyears

# Mean spring (MAM) value
sum( extractzoo(x, trgt="MAM") ) / nyears

# Mean summer (JJA) value
sum( extractzoo(x, trgt="JJA") ) / nyears

# Mean autumn (SON) value
sum( extractzoo(x, trgt="SON") ) / nyears

\dontshow{
############
############
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)
x <- EbroPPtsMonthly

## Winter (DJF) mean values of precipitation for the first 3 stations 
## in 'EbroPPtsMonthly' (its first column stores the dates)
seasonalfunction(x[,1:4], FUN=mean, dates=1)

## The same previous example, but using a zoo object
dates <- as.Date(x[,1]) # dates of the zoo object
z     <- zoo(x[ ,2:ncol(x)], dates) # zoo creation
seasonalfunction(z, FUN=mean)
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File zoo2RHtest.Rd
%% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%%                                 http://www.rforge.net/hydroTSM/
%% Copyright 2008-2020 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{zoo2RHtest}
\Rdversion{1.1}
\alias{zoo2RHtest}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Zoo -> RHTest 
}
\description{
It creates the input file to the \var{'RHtest_dlyPrcp.r'} script,  used for testing the homogeneity of climatological time series (available at: \url{https://etccdi.pacificclimate.org/software.shtml})
}
\usage{
zoo2RHtest(x, fname="pcp.txt", tstep.out="daily", dec=".", na="-999.0")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
time series that will be written. class(x) must be a zoo object
}
  \item{fname}{
Character, with the filename of the precipitation time series 
}
  \item{tstep.out}{
Character indicating the time step that have to be used for writing \code{x} into the output file
}
  \item{dec}{
the string to use for decimal points in numeric or complex columns: must be a single character.
}
  \item{na}{
character to be used for representing the missing values in the data
}
}
%%\details{
%%
%%}
%%\value{
%%
%%}
\references{
%\url{http://ccma.seos.uvic.ca} % not available anynmore, reported by Gang Dong on March 12th, 2013
\url{https://etccdi.pacificclimate.org/software.shtml}
\url{https://etccdi.pacificclimate.org/RHtest/RHtestsV4_UserManual_10Dec2014.pdf}
%\url{http://css.escwa.org.lb/sdpd/1802/d2-1.pdf} % not available anynmore, rdetected on March 10th, 2020

% \url{http://www.cmc.org.ve/mediawiki/index.php?title=Preparando_los_datos}
%\url{http://cmc.org.ve/descargas/Cursos/CRRH/}
}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%%}
\examples{
## Loading the SanMartino precipitation data
data(SanMartinoPPts)
x <- SanMartinoPPts

#Getting the monthly ts
pcp.m <- daily2monthly(x, FUN=sum, na.rm=FALSE)

\dontrun{
# From zoo to the input format required by 'FindU.dlyPrcp' function
zoo2RHtest(x=pcp.m, fname="pcp-monthly.txt", tstep.out="monthly", na="-999.0")

# Homogeneity analysis
FindU.dlyPrcp(InSeries="pcp-monthly.txt", output="pcp-monthly", MissingValueCode="-999.0", 
GUI=FALSE, pthr=0, Mq=10, p.lev=0.95, Iadj=10000)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File monthlyfunction.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{monthlyfunction}
\Rdversion{1.1}
\alias{monthlyfunction}
\alias{monthlyfunction.zoo}
\alias{monthlyfunction.default}
\alias{monthlyfunction.matrix}
\alias{monthlyfunction.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Monthly Function
}
\description{
Generic function for obtaining 12 monthly values of a zoo object, by applying any R function to ALL the values in the object belonging to each one of the 12 calendar months (Jan...Dec).
}
\usage{
monthlyfunction(x, ...)

\method{monthlyfunction}{default}(x, FUN, na.rm = TRUE, ...)

\method{monthlyfunction}{zoo}(x, FUN, na.rm=TRUE,...)

\method{monthlyfunction}{data.frame}(x, FUN, na.rm = TRUE, dates=1, 
        date.fmt = "\%Y-\%m-\%d", out.type = "data.frame", verbose = TRUE, ...)
             
\method{monthlyfunction}{matrix}(x, FUN, na.rm = TRUE, dates=1, 
        date.fmt = "\%Y-\%m-\%d", out.type = "data.frame", verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, xts, data.frame or matrix object, with daily or monthly time series. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that will be applied to ALL the values in \code{x} belonging to each one of the 12 months of the year (e.g., FUN can be some of \code{mean}, \code{sum}, \code{max}, \code{min}, \code{sd}).
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the monthly values and FUN are computed considering only those values in \code{x} different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the corresponding monthly value will be NA 
}
  \item{dates}{
It is only used when \code{x} is not a zoo object. \cr
numeric, factor, Date indicating how to obtain the dates. \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into 'Date' class, using the date format specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
It is only used when \code{x} is not a zoo object. \cr
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
It is only used when \code{x} is a matrix or data.frame. \cr
Character defining the desired type of output. Valid values are: \cr
-) \kbd{data.frame}: a data.frame, with 12 columns representing the months, and as many rows as gauging stations are included in \code{x} \cr
-) \kbd{db}        : a data.frame, with 4 columns will be produced. Useful for a posterior boxplot \cr
                 The first column ('StationID') will store the ID of the station,  \cr
                 The second column ('Year') will store the year, \cr
                 The third column ('Month') will store month,  \cr
                 The fourth column ('Value') will contain the monthly value corresponding to the three previous columns.
}
  \item{verbose}{
Logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
When \code{x} is a zoo object, a numeric vector with 12 elements representing the computed monthly value for each month. \cr
When \code{x} is a data.frame which columns represent measurements at different gauging stations, the resulting object is a data.frame with 12 columns and as many rows as gauging stations are in \code{x}, each row storing the computed 12 monthly value for each gauging station.
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
Due to the fact that \code{FUN} is applied over all the elements in \code{x} belonging to a given calendar month, its result will depend on the sampling frequency of \code{x} and the type of function provided by \code{FUN} (\bold{special attention have to be put when \code{FUN=sum}})
}

\seealso{
\code{\link{annualfunction}}, \code{\link{seasonalfunction}}, \code{\link{dm2seasonal}}, \code{\link{daily2monthly}}, \code{\link{daily2annual}}
}
\examples{
## Loading daily streamflows (3 years) at the station 
## Oca en Ona (Ebro River basin, Spain)
data(OcaEnOnaQts)
x <- OcaEnOnaQts

## Mean monthly streamflows at station 'x'
monthlyfunction(x, FUN=mean, na.rm=TRUE)


############################
## Boxplot of monthly values

## Daily to Monthly
m <- daily2monthly(x, FUN=mean, na.rm=TRUE)

## Median of the monthly values at the station
monthlyfunction(m, FUN=median, na.rm=TRUE)

## Vector with the three-letter abbreviations of the month names
cmonth <- format(time(m), "\%b")

## Creating ordered monthly factors
months <- factor(cmonth, levels=unique(cmonth), ordered=TRUE)

## Boxplot of the monthly values
boxplot( coredata(m) ~ months, col="lightblue", main="Monthly streamflows, [m3/s]")


##############################
##############################
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)
x <- EbroPPtsMonthly

## Dates of 'x'
dates <- as.Date(x[,1])

## Monthly precipitation of all the stations in 'x'
\dontrun{

## Sum of the monthly values in each station of 'x'
z <- zoo( x[, 2:ncol(x)], dates)

# Amount of years in 'x' (needed for computing the average)
nyears <- yip(from=start(z), to=end(z), out.type="nmbr" )

m <- monthlyfunction(z, FUN=sum)


## Another way of computing the sum of the monthly values in each station of 'x'
## This way is usefult for posteriori boxplots
m2 <- monthlyfunction(x, FUN=sum, dates=1, out.type="db")

## Average monthly precipitation in each station of 'x'
m2$Value <- m2$Value / nyears 

## Creating monthly factors
m2$Month <- factor(m2$Month, levels=month.abb)

## boxplot of the monthly values in all stations
boxplot(Value ~ Month, m2, col="lightyellow", main="Monthly Precipitation, [mm/month]")
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File fdc.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2015 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{fdc}
\Rdversion{1.1}
\alias{fdc.default}
\alias{fdc.matrix}
\alias{fdc.data.frame}
\alias{fdc.zoo}
\alias{fdc}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Flow Duration Curve
}
\description{
Computes and plots the Flow Duration Curve (\acronym{FDC}) corresponding to a given time series of streamflow discharges. 
}
\usage{
fdc(x, ...)

\method{fdc}{default}(x,lQ.thr=0.7,hQ.thr=0.2, plot=TRUE, log="y", 
    main="Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
    ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), col="black", 
    pch=1, lwd=1, lty=1, cex=0.4, cex.axis=1.2, cex.lab=1.2, leg.txt=NULL, leg.cex=1, 
    leg.pos="topright", thr.pos="bottomleft", 
    verbose= TRUE, thr.shw=TRUE, new=TRUE, ...)

\method{fdc}{matrix}(x, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
    main= "Flow Duration Curve",  xlab="\% Time flow equalled or exceeded", 
    ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
    col=palette("default")[1:ncol(x)], pch=1:ncol(x), lwd=rep(1, ncol(x)), 
    lty=1:ncol(x), cex=0.4, cex.axis=1.2, cex.lab=1.2, leg.txt=NULL, 
    leg.cex=1, leg.pos="topright", thr.pos="bottomleft", 
    verbose=TRUE, thr.shw=TRUE, new=TRUE, ...)

\method{fdc}{data.frame}(x, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
     main= "Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
     ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
     col=palette("default")[1:ncol(x)], pch=1:ncol(x), lwd=rep(1, ncol(x)), 
     lty=1:ncol(x), cex=0.4, cex.axis=1.2, cex.lab=1.2, leg.txt=NULL, 
     leg.cex=1, leg.pos="topright", thr.pos="bottomleft", 
     verbose=TRUE, thr.shw=TRUE, new=TRUE, ...)
     
\method{fdc}{zoo}(x, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
     main= "Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
     ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
     col=palette("default")[1:NCOL(x)], pch=1:NCOL(x), lwd=rep(1, NCOL(x)), 
     lty=1:NCOL(x), cex=0.4, cex.axis=1.2, cex.lab=1.2, leg.txt=NULL, 
     leg.cex=1, leg.pos="topright", thr.pos="bottomleft", 
     verbose=TRUE, thr.shw=TRUE, new=TRUE, ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
numeric, zoo, data.frame or matrix object with the observed streamflows for which the flow duration curve have to be computed. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter). When \code{x} is a matrix or data.frame, the flow duration curve is computed for each column.
}
\item{lQ.thr}{
numeric, low-flow separation threshold. If this value is different from NA, a vertical line is drawn in this value, and all the values to the right of it should be deemed as low flows. Default value is 0.7.
}
\item{hQ.thr}{
numeric, high-flow separation threshold. If this value is different from NA, a vertical line is drawn in this value, and all the values to the left of it should br deemed as high flows. Default value is 0.2.
}
  \item{plot}{
logical. Indicates if the flow duration curve should be plotted or not. Default value is TRUE.
}
  \item{log}{
character, indicates which axis has to be plotted with a logarithmic scale. Default value is \kbd{y} 
}
  \item{main}{
See \code{\link[graphics]{plot}}. An overall title for the plot: see \code{\link[graphics]{title}}.
}
  \item{xlab}{
A title for the x axis. See \code{\link[graphics]{plot}}. 
}
  \item{ylab}{
A title for the y axis. See \code{\link[graphics]{plot}}. 
}
  \item{ylim}{
The y limits of the plot. See \code{\link[graphics]{plot.default}}. 
}
  \item{yat}{
Only used when \code{log="y"}. \cr
numeric, with points at which tick-marks will try to be drawn in the Y axis, in addition to the defaults computed by R. See the \code{at} argument in \code{\link[graphics]{Axis}}. 
}
  \item{xat}{
Only used when \code{log="x"}. \cr
numeric, with points at which tick-marks will try to be drawn in the x axis, in addition to the defaults computed by R. See the \code{at} argument in \code{\link[graphics]{Axis}}. 
}
  \item{col}{
The colors to be used for lines and points. Multiple colors can be specified so that each point can be given its own color. If there are fewer colors than points they are recycled in the standard fashion. Lines will all be plotted in the first colour specified. See \code{\link[graphics]{plot.default}}. 
}
  \item{pch}{
A vector of plotting characters or symbols: see \code{\link[graphics]{points}}. See \code{\link[graphics]{plot.default}}. 
}
  \item{lwd}{
The line width, see \code{\link[graphics]{par}}. See \code{\link[graphics]{plot.default}}. 
}
  \item{lty}{
The line type, see \code{\link[graphics]{par}}. See \code{\link[graphics]{plot.default}}. 
}
  \item{cex}{
See \code{\link[graphics]{plot.default}}. A numerical vector giving the amount by which plotting characters and symbols should be scaled relative to the default.  \cr
This works as a multiple of \env{par("cex")}. 'NULL' and 'NA' are equivalent to '1.0'.  Note that this does not affect annotation
}
  \item{cex.axis}{
magnification of axis annotation relative to 'cex'.
}
  \item{cex.lab}{
Magnification to be used for x and y labels relative to the current setting of 'cex'. See '?par'.
}
  \item{leg.txt}{
vector with the names that have to be used for each column of \code{x}.
}
  \item{leg.cex}{
numeric, indicating the character expansion factor for the legend, *relative* to current \code{par("cex")}. Default value = 1
}
  \item{leg.pos}{
keyword to be used to position the legend. One of the list \sQuote{"bottomright", "bottom", "bottomleft", "left", "topleft", "top", "topright", "right", "center"}. This places the legend on the inside of the plot frame at the given location. See \code{\link[graphics]{legend}}.
}
  \item{thr.pos}{
keyword to be used to position the streamflow values corresponding to the user-defined thresholds \code{lQ.thr} and \code{hQ.thr}. It is only used when \code{thr.shw=TRUE}. \cr
  One of the list \sQuote{"bottomright", "bottom", "bottomleft", "left", "topleft", "top", "topright", "right", "center"}. This places the the streamflow values on the inside of the plot frame at the given location. See \code{\link[graphics]{legend}}.
}
  \item{verbose}{
logical; if TRUE, progress messages are printed (when \code{x} is a matrix or data.frame). 
}
  \item{thr.shw}{
logical, indicating if the streamflow values corresponding to the user-defined thresholds \code{lQ.thr} and \code{hQ.thr} have to be shown in the plot.
}
  \item{new}{
logical, if TRUE (default), a new plotting window is created. 
}
  \item{\dots}{
further arguments passed to or from other methods (to the plotting functions)
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
numeric, matrix or data.frame whose columns contains the \% of time each one of the streamflow magnitudes given as input was equalled or exceeded. The resulting values have to be multiplied by 100 to get a percentage. \cr

When \code{plot} is TRUE (default), the resulting flow duration curve is plotted in a new window.
}
\references{
\cite{Vogel, R., and N. M. Fennessey (1994), Flow duration curves I: A new interpretation and confidence intervals, ASCE, Journal of Water Resources Planning and Management, 120(4)}. \cr

\cite{Vogel, R., and N. Fennessey (1995), Flow duration curves II: A review of applications in water resources planning, Water Resources Bulletin, 31(6), 1029-1039, doi:10.1111/j.1752-1688.1995.tb03419.x}. \cr

\cite{Yilmaz, K. K., H. V. Gupta, and T. Wagener  (2008), A process-based diagnostic approach to model evaluation: Application to the NWS distributed hydrologic model, Water Resour. Res., 44, W09417, doi:10.1029/2007WR006716}.

}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{fdcu}}
}
\examples{
## Loading daily streamflows at the station Oca en Ona (Ebro River basin, Spain) ##
data(OcaEnOnaQts)

## Daily Flow Duration Curve
fdc(OcaEnOnaQts)

###################
# Getting the streamflow values corresponding to 5 and 95\% of time equalled or 
# exceeded (and also the first streamflow value in 'x' just for verification)
x  <- OcaEnOnaQts

# First streamflow value (x1=42.1 m3/s)
x1 <- x[1]

# Daily FDC for 'x'
y <- fdc(x)

# value of the FDC for x1 (y1=0.002739726)
y1 <- y[1]

# Performing cubic (or Hermite) spline interpolation of 'x' and 'y'
f <- splinefun(y,x)

# Getting the (known) streamflow value for 'y1'
f(y1) # 42.1 m3/s, equal to the known 'x1'

# Streamflow values corresponding to 5 and 95\% of time equalled or exceeded
f(c(.05, .95))

###################
## Getting 
data(OcaEnOnaQts)

## Daily Flow Duration Curve
fdc(OcaEnOnaQts)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File shiftyears.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2023-2025 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{shiftyears}
\Rdversion{1.1}
\alias{shiftyears}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Shiftyears
}
\description{
It shifts years backwards for correct computation of annual values when the hydrological year does not start in january. Originally created only for internal usage in this pacakge. 
}
\usage{
shiftyears(ltime, lstart.month)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{ltime}{
vector with the date/times of each element of a zoo object.
}
  \item{lstart.month}{
numeric in [1,..,12], representing the starting month to be used in the computation of annual values. By default \code{lstart.month=1} (Jan).}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
Integer vector with the shifted years corresponding to each elemen of \code{x}.
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{climograph}}, \code{\link{cmv}}, \code{\link{daily2annual}}
}
\examples{
# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting 'x' to its first three months (Jan/1921 - Mar/1921)
x <- window(x, end="1921-12-31")

# Starting in January, it means no shift
shiftyears(ltime=time(x), lstart.month=1)

# Starting in April:
shiftyears(ltime=time(x), lstart.month=4)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File ma.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{ma}
\Rdversion{1.1}
\alias{ma.zoo}
\alias{ma.default}
\alias{ma}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Moving Average
}
\description{
Generic function for computing a moving (sliding) average of ts.
}
\usage{
ma(x, ...)

\method{ma}{default}(x, win.len, FUN = mean, ...)

\method{ma}{zoo}(x, win.len, FUN = mean, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
ts or zoo object.
}
  \item{win.len}{
number of terms that will be considered in the mean. It have to be odd
}
  \item{FUN}{
Function that have to be applied for computing the moving average. Usually, \code{FUN} MUST be \code{mean}
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
a vector with the moving average termns. The length of the resulting vector is the same of \code{x}, but the first and last \code{(win.len-1)/2} elements will be NA.

%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%%}
\examples{
## Loading daily streamflows at the station Oca en Ona (Ebro River basin, Spain) ##
data(OcaEnOnaQts)
x <- OcaEnOnaQts

## Daily to Monthly ts
m <- daily2monthly(x, FUN=mean, na.rm=FALSE)

# Plotting the monthly values
plot(m, xlab="Time")

## Plotting the annual moving average in station 'x'
lines(ma(m, win.len=12), col="blue")

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File OcaEnOnaQts.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{OcaEnOnaQts}
\docType{data}
\alias{OcaEnOnaQts}
\title{
Oca in "Ona" (Q0931), time series of daily streamflows.
}
\description{
Time series with daily streamflows of the Oca River (subcatchment of the Ebro River basin, Spain) measured at the gauging station "Ona" (Q093), for the period 01/Jan/1961 to 31/Dic/1963
}
\usage{
data(OcaEnOnaQts)
}

%%\details{
%%
%%}
\format{
zoo object.
}
\source{
Downloaded from the web site of the Confederacion Hidrografica del Ebro (CHE) \url{http://www.chebro.es/} (original link http://oph.chebro.es/documentacion/CaudalEA/CaudalEA.htm, last accessed [March 2010]). \cr

These data are intended to be used for research purposes only, being distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
}

%%\references{
%% ~put references to the literature/web site here ~
%%}
\keyword{datasets}

%% File calendarHeatmap.Rd
%% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%%                                 https://www.rforge.net/hydroRTS/ ; 
%% Copyright 2008-2022 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{calendarHeatmap}
\Rdversion{1.1}
\alias{calendarHeatmap}
\alias{calendarHeatmap.zoo}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calendar heatmap
}
\description{
Function to plot a heatmap of a daily zoo object with a calendar shape
}
\usage{
calendarHeatmap(x, ...)

\method{calendarHeatmap}{zoo}(x, from, to, date.fmt="\%Y-\%m-\%d",
                main="Calendar Heat Map",
                col=colorRampPalette(c("red", "orange", "yellow", "white", 
                    "lightblue2", "deepskyblue", "blue3"),  space = "Lab")(8),  
                cuts, cuts.dec=0, cuts.labels, 
                cuts.style=c("fisher", "equal", "pretty", "fixed", "sd", 
                             "quantile", "kmeans", "bclust", "mzb"), 
                legend.title="", legend.fontsize=15,              
                do.png=FALSE, png.fname="mypng.png", png.width=1500, 
                png.height=900, png.pointsize=12, png.res=90,
                do.pdf=FALSE, pdf.fname="mypdf.pdf", pdf.width=11,
                pdf.height=8.5, pdf.pointsize=12, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
daily zoo object to be plotted. Its maximum amount of daily data should be less than 6 years or otherwise it will not be plotted.
}
  \item{from}{
Character indicating the starting date for subsetting \code{x}. It has to be in the format indicated by \code{date.fmt}. 
}
  \item{to}{
Character indicating the ending date for subsetting \code{x}. It has to be in the format indicated by \code{date.fmt}. 
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. See \sQuote{Details} section in \code{\link[base]{strptime}}.
}
\item{main}{
character, Main chart title.
}
\item{col}{
A color palette, i.e. a vector of n contiguous colors generated by functions like rainbow, heat.colors, topo.colors, bpy.colors or one of your own making, perhaps using \code{\link[grDevices]{colorRampPalette}}. If none is provided, a color ramp with 8 colours is created using \code{\link[grDevices]{colorRampPalette}}. 
}
\item{cuts}{
Numeric, indicating the values used to divide the range of \code{x} in the legend of colours. If not provided, it is automatically selected as a function of \code{lenght(col)}.
}
\item{cuts.dec}{
Number of decimal places used to present the numbers in the legend of colours.
}
\item{cuts.labels}{
Character indicating the label to be used in the colour legend for each one of the values defined by \code{cuts}. If not provided, \code{as.character(cuts)} is used.
}
\item{cuts.style}{
discarded because takes too much time or not alway provide the required number of classes: "dpih", "headtails", "hclust",  "jenks".
}
\item{legend.title}{
text to be displayed above the legend of colours.
}
\item{legend.fontsize}{
size of text (in points) used in the legend of colours.
}
\item{do.png}{
Do you want to write the figure as a .png file? logical, set TRUE to save the image.
}
\item{png.fname}{
character, indicating the name of the file (possibly with a meaninful file extension) that will be used to write the output file.
}
\item{png.width}{
numeric, the width of the device.
}
\item{png.height}{
numeric, the height of the device.
}
\item{png.pointsize}{
integer, the default pointsize of plotted text, interpreted as big points (1/72 inch) at \code{res} ppi.
}
\item{png.res}{
integer, the nominal resolution in ppi which will be recorded in the bitmap file, if a positive integer. Also used for units other than the default, and to convert points to pixels.
}
\item{do.pdf}{
Do you want to write the figure as a .pdf file? logical, set TRUE to save the image.
}
\item{pdf.fname}{
character, indicating the name of the file (possibly with a significant extension) that will be used to write the output file.
}
\item{pdf.width}{
numeric, the width of the device.
}
\item{pdf.height}{
numeric, the height of the device.
}
\item{pdf.pointsize}{
integer, the default point size to be used. Strictly speaking, in bp, that is 1/72 of an inch, but approximately in points. Defaults to 12.
}
  \item{\dots}{
further arguments passed to functions or from other methods. Not used yet.
}
}
\details{
The original function \code{calendarHeat} was developed by Paul Bleicher, as an R version of a graphic from http://stat-computing.org/dataexpo/2009/posters/wicklin-allison.pdf (not available any longer).
The original function was made available online in 2009, but then it was removed. Now it is available at \url{https://github.com/tavisrudd/r_users_group_1/blob/master/calendarHeat.R}.
The original function has "Copyright 2009 Humedica", but it was distributed under the GPL-2 licence, as well as this new version of the function.

This slighly modified verison of the function is also distributed under the GPL-2 licence, and teh main changes with respect to the original function are: \cr

1) uses a zoo object instead of a numeric and character vector, for values and dates, respectively. \cr
2) it allows a customisation of the mian title of the output figure. \cr
3) it allows a customisation of the color palette. \cr
4) it uses a categorical legend instead of a continuos one. \cr
5) it is named \code{calendarHeatmap} instead of \code{calendarHeat}.


}
\value{
The output of this function is a lattice figure with the calendar heatmap
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail.com}
}
\note{
The maximum amount of years to be plotted is six (6).
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[grDevices]{colorRampPalette}}, \code{\link[lattice]{levelplot}}
}
\examples{
###########
# EXAMPLE 1: basic plotting of a calendar heatmap
###########

# Loading daily streamflow data for Karamet at Gorges.
data(KarameaAtGorgeQts)

x <- KarameaAtGorgeQts
x <- subdaily2daily(x, FUN=mean)

# Temporal subsetting for a amaximum of 6 years
x <- window(x, start="1980-01-01", end="1985-12-31")

# Calendar Heatmap, 8 colours and cuts defined using Fisher method 
calendarHeatmap(x)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}
%%keyword{manip}

% File hydroplot.Rd
% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%                                 http://www.rforge.net/hydroTSM/ ; 
%                                 https://cran.r-project.org/package=hydroTSM
% Copyright 2008-2026 Mauricio Zambrano-Bigiarini
% Distributed under GPL 2 or later

\name{hydroplot}
\Rdversion{1.1}
\alias{sname2plot}
\alias{sname2plot.default}
\alias{sname2plot.zoo}
\alias{sname2plot.data.frame}
\alias{hydroplot}
\alias{hydroplot.default}
\alias{hydroplot.zoo}
\alias{hydroplot.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Hydrological time series plotting and extraction.
}
\description{
\code{hydroplot}: When \code{x} is a zoo object it plots (a maximum of) 9 graphs (lines plot, boxplots and/or histograms) of the daily, monthly, annual and/or seasonal time series. \cr

\code{sname2plot}: When \code{x} is a data frame whose columns contain the time series of several gauging stations, it takes the name of one gauging station and plots the graphs described above.
}
\usage{
hydroplot(x, ...)
sname2plot(x, ...)

\method{hydroplot}{default}(x, FUN, na.rm=TRUE, ptype="ts+boxplot+hist", pfreq="dma",                      
          var.type, var.unit="units", main=NULL, xlab="Time", ylab,
          win.len1=0, win.len2=0, tick.tstep="auto", lab.tstep="auto", 
          lab.fmt=NULL, cex=0.3, cex.main=1.3, cex.lab=1.3, cex.axis=1.3, 
          col=c("blue", "lightblue", "lightblue"), 
          from=start(x), to=end(x), dates=1, date.fmt= "\%Y-\%m-\%d", tz=NULL,
          stype="default", season.names=c("Winter", "Spring", "Summer", "Autumn"), 
          h=NULL, ...)

\method{hydroplot}{zoo}(x, FUN, na.rm=TRUE, ptype="ts+boxplot+hist", pfreq="dma",                      
          var.type, var.unit="units", main=NULL, xlab="Time", ylab,
          win.len1=0, win.len2=0, tick.tstep="auto", lab.tstep="auto", 
          lab.fmt=NULL, cex=0.3, cex.main=1.3, cex.lab=1.3, cex.axis=1.3, 
          col=c("blue", "lightblue", "lightblue"), 
          from=start(x), to=end(x), dates=1, date.fmt= "\%Y-\%m-\%d", tz=NULL, 
          stype="default", season.names=c("Winter", "Spring", "Summer", "Autumn"), 
          h=NULL, ...)

\method{hydroplot}{data.frame}(x, FUN, na.rm=TRUE, ptype="ts+boxplot+hist", pfreq="dma",                      
          var.type, var.unit="units", main=NULL, xlab="Time", ylab,
          win.len1=0, win.len2=0, tick.tstep="auto", lab.tstep="auto", 
          lab.fmt=NULL, cex=0.3, cex.main=1.3, cex.lab=1.3, cex.axis=1.3, 
          col=c("blue", "lightblue", "lightblue"), 
          from=start(x), to=end(x), dates=1, date.fmt= "\%Y-\%m-\%d", tz=NULL, 
          stype="default", season.names=c("Winter", "Spring", "Summer", "Autumn"), 
          h=NULL, ...)

\method{sname2plot}{default}(x, sname, FUN, na.rm=TRUE, ptype="ts+boxplot+hist", 
           pfreq="dma", var.type, var.unit="units", main=NULL, 
           xlab="Time", ylab=NULL, win.len1=0, win.len2=0, 
           tick.tstep="auto", lab.tstep="auto", lab.fmt=NULL, 
           cex=0.3, cex.main=1.3, cex.lab=1.3, cex.axis=1.3,
           col=c("blue", "lightblue", "lightblue"), 
           dates=1, date.fmt = "\%Y-\%m-\%d", from=NULL, to=NULL, stype="default", 
           season.names=c("Winter", "Spring", "Summer", "Autumn"), 
           h=NULL, ...)

\method{sname2plot}{zoo}(x, sname, FUN, na.rm=TRUE, ptype="ts+boxplot+hist", 
           pfreq="dma", var.type, var.unit="units", main=NULL, 
           xlab="Time", ylab=NULL, win.len1=0, win.len2=0, 
           tick.tstep="auto", lab.tstep="auto", lab.fmt=NULL, 
           cex=0.3, cex.main=1.3, cex.lab=1.3, cex.axis=1.3,
           col=c("blue", "lightblue", "lightblue"), 
           dates=1, date.fmt = "\%Y-\%m-\%d", from=NULL, to=NULL, stype="default", 
           season.names=c("Winter", "Spring", "Summer", "Autumn"), 
           h=NULL, ...)

\method{sname2plot}{data.frame}(x, sname, FUN, na.rm=TRUE, ptype="ts+boxplot+hist", 
           pfreq="dma", var.type, var.unit="units", main=NULL, 
           xlab="Time", ylab=NULL, win.len1=0, win.len2=0, 
           tick.tstep="auto", lab.tstep="auto", lab.fmt=NULL, 
           cex=0.3, cex.main=1.3, cex.lab=1.3, cex.axis=1.3,
           col=c("blue", "lightblue", "lightblue"), 
           dates=1, date.fmt = "\%Y-\%m-\%d", from=NULL, to=NULL, stype="default", 
           season.names=c("Winter", "Spring", "Summer", "Autumn"), 
           h=NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, xts or data.frame object, with columns storing the time series of one or more gauging stations.
}
  \item{sname}{
ONLY required when \code{x} is a data frame. \cr
Character representing the name of a station, which have to correspond to one column name in \code{x}
}
  \item{FUN}{
ONLY required when \code{var.type} is missing AND \code{pfreq != "o"}. \cr
Function that have to be applied for transforming from daily to monthly or annual time step (e.g., For precipitation \code{FUN=sum} and for temperature and flow ts, \code{FUN=mean}) 
}
  \item{na.rm}{
Logical. Should missing values be removed before the computations?
}
  \item{ptype}{
Character indicating the type of plot that will be plotted. Valid values are: \cr
-) \kbd{ts}              => only time series                  \cr
-) \kbd{ts+boxplot}      => only time series + boxplot        \cr       
-) \kbd{ts+hist}         => only time series + histogram      \cr
-) \kbd{ts+boxplot+hist} => time series + boxplot + histogram 
}
  \item{pfreq}{
Character indicating how many plots are desired by the user. Valid values are: \cr                    
-) \kbd{dma}     : Daily, Monthly and Annual values are plotted \cr
-) \kbd{dm}      : Daily and Monthly values are plotted  \cr
-) \kbd{ma}      : Monthly and Annual values are plotted \cr
-) \kbd{o}       : Only the original zoo object is plotted, and \code{ptype} is changed to \kbd{ts} \cr
-) \kbd{seasonal}: Line and bloxplots of seasonal time series (see \code{stype} and \code{season.names}). When \code{pfreq} is \kbd{seasonal}, \code{ptype} is set to  \kbd{ts+boxplot}
}
  \item{var.type}{
ONLY required when \code{FUN} is missing. \cr
character representing the type of variable being plotted. Used for determining the function used for computing the monthly and annual values when \code{FUN} is missing. Valid values are: \cr
-) \kbd{Precipitation} => \code{FUN=sum}  \cr
-) \kbd{Temperature}   => \code{FUN=mean} \cr
-) \kbd{Flow}          => \code{FUN=mean} \cr
}
  \item{var.unit}{
Character representing the measurement unit of the variable being plotted. ONLY used for labelling the axes (e.g., "mm" for precipitation, "C" for temperature, and "m3/s" for flow.)
}
  \item{main}{
Character representing the main title of the plot. If the user do not provide a title, this is created automatically as: \code{main= paste(var.type, "at", sname, sep=" ")}, 
}
  \item{xlab}{
A title for the x axis. See \code{\link[graphics]{plot}}. 
}
  \item{ylab}{
A title for the y axis. See \code{\link[graphics]{plot}}. 
}
  \item{win.len1}{
number of days for being used in the computation of the first moving average. A value equal to zero indicates that this moving average is not going to be computed.
}
  \item{win.len2}{
number of days for being used in the computation of the second moving average. A value equal to zero indicates that this moving average is not going to be computed.
}
  \item{tick.tstep}{
Character indicating the time step that have to be used for putting the ticks on the time axis. Valid values are: \cr
-) \kbd{days},    \cr
-) \kbd{months},  \cr
-) \kbd{years} 
}
  \item{lab.tstep}{
Character indicating the time step that have to be used for putting the labels on the time axis. Valid values are: \cr
-) \kbd{days},   \cr
-) \kbd{months}, \cr
-) \kbd{years} 
}
  \item{lab.fmt}{
Character indicating with the format to be used for the label of the axis. See \code{format} in \code{\link[base]{as.Date}}. If not specified, it will try \kbd{"\%Y-\%m-\%d"} when \code{lab.tstep=="days"}, \kbd{"\%b"} when \code{lab.tstep=="month"}, and \kbd{"\%Y"} when \code{lab.tstep=="year"}. 
}
  \item{cex}{
A numerical value giving the amount by which plotting text and symbols should be magnified relative to the default. (See \code{\link[graphics]{par}}).
}
  \item{cex.main}{
The magnification to be used for main titles relative to the current setting of \code{cex} (See \code{\link[graphics]{par}}).
}
  \item{cex.lab}{
The magnification to be used for x and y labels relative to the current setting of \code{cex} (See \code{\link[graphics]{par}}).
}
  \item{cex.axis}{
The magnification to be used for axis annotation relative to the current setting of \code{cex} (See \code{\link[graphics]{par}}).
}
  \item{col}{
A character vector with 3 elements, representing the colors to be used for plotting the lines of the ts, the boxplots, and the histograms, respectively. \cr
When \code{pfreq="o"}, only one character element is needed.
See \code{\link[graphics]{plot.default}}).
}
  \item{dates}{
ONLY required when \code{x} is a data frame. It is a numeric, factor or Date object indicating how to obtain the dates corresponding to the \code{sname} station. \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, using the date format  specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of element in \code{x}
}
  \item{date.fmt}{
Character indicating the format in which the dates are stored in \var{dates}, \var{from} and \var{to}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{tz}{
character, with the specification of the time zone used for \code{x}, \code{from}, and \code{to}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is NULL (the default), it is automatically set to the time zone used in \code{time(x)}. \cr

If \code{tz} is provided, it forces \code{time(x)} to be in the tome zone specified by \code{tz}, without modifying the the values (hours, minutes, seconds, etc). \cr

A list of valid time zones can be obtained by calling the base function \code{OlsonNames()}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{from} and \code{to}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{from}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the starting date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{to}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the ending date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{stype}{
 OPTIONAL, only used when \code{pfreq=seasonal}. \cr
character, indicating which weather seasons will be used for computing the output. Possible values are: \cr
-) \kbd{default}          =>  "winter"= DJF  = Dec, Jan, Feb;      "spring"= MAM = Mar, Apr, May;  "summer"= JJA  = Jun, Jul, Aug;       "autumn"= SON = Sep, Oct, Nov  \cr
-) \kbd{FrenchPolynesia}  =>  "winter"= DJFM = Dec, Jan, Feb, Mar; "spring"= AM  = Apr, May;       "summer"= JJAS = Jun, Jul, Aug, Sep;  "autumn"= ON  = Oct, Nov 
}
  \item{season.names}{
 OPTIONAL, only used when \code{pfreq=seasonal}. \cr
character of length 4 indicating the names of each one of the weather seasons defined by \code{stype}.These names are only used for plotting purposes
}
  \item{h}{
OPTIONAL, only used when \code{pfreq=seasonal}, for plotting horizontal lines in each seasonal plot. \cr
numeric, with 1 or 4 elements, with the value used for plotting an horizontal line in each seasonal plot, in the following order: winter (DJF), spring (MAM), summer (JJA), autumn (SON).
}
  \item{\dots}{
further arguments passed to the \code{plot.zoo} and \code{axis} functions or from other methods.
}
}
\details{
Plots of the daily/monthly/annual/seasonal values of the time series given as input. \cr
Depending on the value of \code{pfreq}, daily, monthly, annual and/or seasonal time series plots, boxplots and histograms are produced. \cr
Depending on the value of \code{ptype}, time series plots, boxplots and/or histograms are produced.
}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{sname2ts}}
}
\examples{
#############
## Loading daily streamflows at the station Oca en Ona (Ebro River basin, Spain) ##
data(OcaEnOnaQts)

## 3 ts, 3 boxplots and 3 histograms
hydroplot(OcaEnOnaQts, FUN=mean, ylab= "Q", var.unit = "m3/s")

## only the original time series
hydroplot(OcaEnOnaQts, pfreq="o")

## only the year 1962 of the original time series
hydroplot(OcaEnOnaQts, pfreq="o", from="1962-01-01", to="1962-12-31")

\dontrun{
## seasonal plots
hydroplot(OcaEnOnaQts, pfreq="seasonal", FUN=mean, stype="default")

## custom season names (let's assume to be in the Southern Hemisphere)
hydroplot(OcaEnOnaQts, pfreq="seasonal", FUN=mean, 
          stype="default", season.names=c("Summer","Autumn", "Winter","Spring"))


#############
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

op <- par(mar=c(5.1, 4.1, 4.1, 2.1))
on.exit(par(op))

## Plotting seasonal precipitation at station "P9001"
sname2plot(EbroPPtsMonthly, sname="P9001", FUN=sum, dates=1, pfreq="seasonal", 
           stype="default")

## Plotting the monthly and annual values of precipitation at station "P9001", 
## stored in 'EbroPPtsMonthly'.
sname2plot(EbroPPtsMonthly, sname="P9001", var.type="Precipitation", dates=1, 
           pfreq="ma")
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}
\keyword{manip}

%%File cmv.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2023-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{cmv}
\Rdversion{1.1}
\alias{cmv}
\alias{cmv.default}
\alias{cmv.zoo}
\alias{cmv.data.frame}
\alias{cmv.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Counting Missing Values
}
\description{
Generic function for counting the percentage/amount of missing values in a zoo object, using a user-defined temporal scale.
}
\usage{
cmv(x, ...)

\method{cmv}{default}(x, tscale=c("hourly", "daily", "weekly", "monthly", 
            "quarterly", "seasonal", "annual"),
            out.type=c("percentage", "amount"), dec=3, 
            start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
            start.month=1, ...)

\method{cmv}{zoo}(x, tscale=c("hourly", "daily", "weekly", "monthly", 
            "quarterly", "seasonal", "annual"),
            out.type=c("percentage", "amount"), dec=3, 
            start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
            start.month=1, ...)

\method{cmv}{data.frame}(x, tscale=c("hourly", "daily", "weekly", "monthly", 
            "quarterly", "seasonal", "annual"),
            out.type=c("percentage", "amount"), dec=3, 
            start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
            start.month=1, dates=1, date.fmt="\%Y-\%m-\%d", ...)

\method{cmv}{matrix}(x, tscale=c("hourly", "daily", "weekly", "monthly", 
            "quarterly", "seasonal", "annual"),
            out.type=c("percentage", "amount"), dec=3, 
            start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
            start.month=1, dates=1, date.fmt="\%Y-\%m-\%d", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with the time series to be analised. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represents the time series measured in a gauging statio, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{tscale}{
character with the temporal scale to be used for analysing the mssing data. Valid values are: \cr
-) \kbd{hourly}: the percentage/amount of missing values will be given for each hour and ,therefore, the expected time frequency of \code{x} must be sub-hourly. \cr
-) \kbd{daily}: the percentage/amount of missing values will be given for each day and, therefore, the expected time frequency of \code{x}  must be sub-daily (i.e., hourly or sub-hourly). \cr
-) \kbd{weekly}: the percentage/amount of missing values will be given for each week (starting on Monday) and, therefore, the expected time frequency of \code{x} must be sub-weekly (i.e., daily, (sub)hourly). \cr
-) \kbd{monthly}: the percentage/amount of missing values will be given for each month and, therefore, the expected time frequency of \code{x} must be sub-monthly (i.e., daily, hourly or sub-hourly). \cr
-) \kbd{quarterly}: the percentage/amount of missing values will be given for each quarter and, therefore, the expected time frequency of \code{x} must be sub-quarterly (i.e., monthly, daily, hourly or sub-hourly). \cr
-) \kbd{seasonal}: the percentage/amount of missing values will be given for each weather season (see ?time2season) and, therefore, the expected time frequency of \code{x} must be sub-seasonal (i.e., monthly, daily, hourly or sub-hourly). \cr
-) \kbd{annual}: the percentage/amount of missing values will be given for each year and, therefore, the expected time frequency of \code{x} must be sub-annual (i.e., seasonal, monthly, daily, hourly or sub-hourly). \cr
}
  \item{dec}{
integer indicating the amount of decimal places included in the output. \cr
It is only used when \code{out.type=='percentage'}.
}
  \item{start}{
character, indicating the starting time used for aggregating sub-daily time series into daily ones. It MUST be provided in the format specified by \code{start.fmt}. \cr
This value is used to define the time when a new day begins (e.g., for some rain gauge stations). \cr
-) All the values of \code{x} with a time attribute before \code{start} are considered as belonging to the day before the one indicated in the time attribute of those values. \cr
-) All the values of \code{x} with a time attribute equal to \code{start} are considered to be equal to \code{"00:00:00"} in the output zoo object. \cr
-) All the values of \code{x} with a time attribute after \code{start} are considered as belonging to the same day as the one indicated in the time attribute of those values. \cr

It is useful when the daily values start at a time different from \code{"00:00:00"}. Use with caution. See examples.
}
  \item{start.fmt}{
character indicating the format in which the time is provided in \code{start}, By default \code{date.fmt=\%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.POSIXct}}.
}
  \item{start.month}{
 numeric in [1,..,12], representing the starting month (1:Jan, ..., 12:Dec) to be used in the computation of annual values. By default \code{start.month=1}.
}
  \item{tz}{
character, with the specification of the time zone used in both \code{x} and \code{start}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr
This argument can be used to force using the local time zone or any other time zone instead of UTC as time zone.
}
  \item{dates}{
numeric, factor, POSIXct or POSIXt object indicating how to obtain the dates and times for each column of \code{x} (e.g., gauging station). \cr
If \code{dates} is a number, it indicates the index of the column in \code{x} that stores the date and times. \cr
If \code{dates} is a factor, it is converted into POSIXct class, using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of POSIXct or POSIXt class, this function verifies that the number of elements on it be equal to the number of elements in \code{x}.
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{dates}, By default \code{date.fmt=\%Y-\%m-\%d \%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
character indicating how should be returned the missing values for each temporal scale. Valid values are: \cr
-) \kbd{percentage}: the missing values are returned as an real value, representing the percentage of missing values in each temporal scale. \cr
-) \kbd{amount}: the missing values are returned as an integer value, representing the absolute amount of missing values in each temporal scale.
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
\details{
The amount of missing values in each temporal scale is computed just by counting the amount of NAs in each hour / day / week / month / quarter / season / year, while the percentage of missing values in each temporal scale is computed by dividing the previous number by the total number of data elements in each hour / day / week / month / quarter / season / year. \cr

This function was developed to allow the selective removal of values when agregting from a high temporal resolution into a lower temporal resolution (e.g., from hourly to daily or from daily to monthly), using any of the temporal aggregation functions available int his package (e.g., \code{hourly2daily}, \code{daily2monthly})
}
\value{
a zoo object with the percentage/amount of missing values for each temporal scale selected by the user.
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{dwi}}, \code{\link{subhourly2hourly}}, \code{\link{subdaily2daily}}, \code{\link{daily2monthly}}, \code{\link{daily2annual}}, \code{\link{monthlyfunction}}, \code{\link{izoo2rzoo}}
}
\examples{
######################
## Ex1: Loading the DAILY precipitation data at SanMartino (25567 daily values)
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting to only the 1981-1990 temporal period
x <- window(x, start="1981-01-01", end="1990-12-31")

## Transforming into NA the 10% of values in 'x'
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

# Getting the amount of NAs in 'x' for each week (starting on Monday)
cmv(x, tscale="weekly")

# Getting the amount of NAs in 'x' for each month
cmv(x, tscale="monthly")

# Getting the amount of NAs in 'x' for each quarter
cmv(x, tscale="quarterly")

# Getting the amount of NAs in 'x' for each weather season
cmv(x, tscale="seasonal")

# Getting the amount of NAs in 'x' for each year
cmv(x, tscale="annual")
######################
## Ex2: Loading the time series of HOURLY streamflows for the station 
## Karamea at Gorge (52579 hourly values)
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

# Subsetting to only the first two years (1981-1982)
x <- window(x, end=as.POSIXct("1982-12-31 23:59:00", tz="UTC") )

## Transforming into NA the 30% of values in 'x'
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

# Getting the amount of NAs in 'x' for each day
cmv(x, tscale="daily")

# Getting the amount of NAs in 'x' for each weather season
cmv(x, tscale="seasonal")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File fdcu.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2012 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{fdcu}
\Rdversion{1.1}
\alias{fdcu.data.frame}
\alias{fdcu.default}
\alias{fdcu.matrix}
\alias{fdcu.zoo}
\alias{fdcu}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Flow Duration Curve with uncertainty bounds.
}
\description{
Computes and plots the Flow Duration Curve (\acronym{FDC}) for the streamflows given by \code{x} and for two uncertainty bounds, with the possibility of plotting an additional FDC representing simulated streamflows for \code{x}, in order to compare them.
}
\usage{
fdcu(x, lband, uband, ...)

\method{fdcu}{default}(x, lband, uband, sim=NULL, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
     main="Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
     ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
     col=c("black", "red"), pch=c(1, 15), lwd=c(1, 0.8), lty=c(1, 3), cex=0.2, 
     cex.axis=1.2, cex.lab=1.2, leg.txt= c("Qobs", "Qsim", "95PPU"), 
     leg.cex=1, leg.pos="auto", verbose= TRUE, thr.shw=TRUE, border=NA, 
     bands.col="lightcyan", bands.density=NULL, bands.angle=45, new=TRUE, ...)

\method{fdcu}{matrix}(x, lband, uband, sim=NULL, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
     main="Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
     ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
     col=matrix(c(rep("black", ncol(x)), 
     palette("default")[2:(ncol(x)+1)]), byrow=FALSE, ncol=2), 
     pch=matrix(rep(c(1, 15), ncol(x)), byrow=TRUE, ncol=2),
     lwd=matrix(rep(c(1, 0.8), ncol(x)), byrow=TRUE, ncol=2),
     lty=matrix(rep(c(1, 3), ncol(x)), byrow=TRUE, ncol=2),                        
     cex=rep(0.1, ncol(x)), cex.axis=1.2, cex.lab=1.2, 
     leg.txt=c("OBS", colnames(x), "95PPU"),  leg.cex=1, leg.pos="auto", 
     verbose= TRUE,  thr.shw=TRUE, border=rep(NA, ncol(x)), 
     bands.col=rep("lightcyan", ncol(x)), bands.density=rep(NULL, ncol(x)), 
     bands.angle=rep(45, ncol(x)), new=TRUE, ...)

\method{fdcu}{data.frame}(x, lband, uband, sim=NULL, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
     main="Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
     ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
     col=matrix(c(rep("black", ncol(x)), 
     palette("default")[2:(ncol(x)+1)]), byrow=FALSE, ncol=2),
     pch=matrix(rep(c(1, 15), ncol(x)), byrow=TRUE, ncol=2),
     lwd=matrix(rep(c(1, 0.8), ncol(x)), byrow=TRUE, ncol=2),
     lty=matrix(rep(c(1, 3), ncol(x)), byrow=TRUE, ncol=2),                        
     cex=rep(0.1, ncol(x)), cex.axis=1.2, cex.lab=1.2,
     leg.txt=c("OBS", colnames(x), "95PPU"), leg.cex=1, leg.pos="auto", 
     verbose= TRUE, thr.shw=TRUE, border=rep(NA, ncol(x)), 
     bands.col=rep("lightcyan", ncol(x)), bands.density=rep(NULL, ncol(x)), 
     bands.angle=rep(45, ncol(x)), new=TRUE, ...)

%% \method{fdcu}{zoo}(x, lband, uband, sim=NULL, lQ.thr=0.7, hQ.thr=0.2, plot=TRUE, log="y", 
%%      main="Flow Duration Curve", xlab="\% Time flow equalled or exceeded", 
%%      ylab="Q, [m3/s]", ylim, yat=c(0.01, 0.1, 1), xat=c(0.01, 0.025, 0.05), 
%%      col=matrix(c(rep("black", NCOL(x)), 
%%      palette("default")[2:(NCOL(x)+1)]), byrow=FALSE, ncol=2),
%%      pch=matrix(rep(c(1, 15), NCOL(x)), byrow=TRUE, ncol=2),
%%      lwd=matrix(rep(c(1, 0.8), NCOL(x)), byrow=TRUE, ncol=2),
%%      lty=matrix(rep(c(1, 3), NCOL(x)), byrow=TRUE, ncol=2),                        
%%      cex=rep(0.1, NCOL(x)), cex.axis=1.2, cex.lab=1.2,
%%      leg.txt=c("OBS", colnames(x), "95PPU"), leg.cex=1, leg.pos="auto", 
%%      verbose= TRUE, thr.shw=TRUE, border=rep(NA, NCOL(x)), 
%%      bands.col=rep("lightcyan", NCOL(x)), bands.density=rep(NULL, NCOL(x)), 
%%      bands.angle=rep(45, NCOL(x)), new=TRUE, ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
numeric, zoo, data.frame or matrix object with the observed streamflows for which the flow duration curve have to be computed. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter). When \code{x} is a matrix or data.frame, the flow duration curve is computed for each column.
}
  \item{lband}{
numeric, zoo, data.frame or matrix object with the streamflows representing the the lower uncertainty bound of \code{x}, for which the flow duration curve have to be computed. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object. When \code{lband} is a matrix or data.frame, the flow duration curve is computed for each column.
}
  \item{uband}{
numeric, zoo, data.frame or matrix object with the streamflows representing the the upper uncertainty bound of \code{x}, for which the flow duration curve have to be computed. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object. When \code{uband} is a matrix or data.frame, the flow duration curve is computed for each column.
}
  \item{sim}{
OPTIONAL. \cr
numeric, zoo, data.frame or matrix object with the streamflows simulated for \code{x}, for which the flow duration curve have to be computed. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object. When \code{sim} is a matrix or data.frame, the flow duration curve is computed for each column.
}
\item{lQ.thr}{
numeric, low flows separation threshold. If this value is different from 'NA', a vertical line is drawn in this value, and all the values to the left of it are deemed low flows.
}
\item{hQ.thr}{
numeric, high flows separation threshold. If this value is different from 'NA', a vertical line is drawn in this value, and all the values to the right of it are deemed high flows
}
  \item{plot}{
logical. Indicates if the flow duration curve should be plotted or not.
}
  \item{log}{
character, indicates which axis has to be plotted with a logarithmic scale. Default value is \kbd{y}. 
}
  \item{main}{
See \code{\link[graphics]{plot}}. An overall title for the plot: see \code{\link[graphics]{title}}.
}
  \item{xlab}{
See \code{\link[graphics]{plot}}. A title for the x axis: see \code{\link[graphics]{title}}.
}
  \item{ylab}{
See \code{\link[graphics]{plot}}. A title for the y axis: see \code{\link[graphics]{title}}.
}
  \item{ylim}{
See \code{\link[graphics]{plot.default}}. The y limits of the plot.
}
  \item{yat}{
Only used when \code{log="y"}. \cr
numeric, with points at which tick-marks will try to be drawn in the Y axis, in addition to the defaults computed by R. See the \code{at} argument in \code{\link[graphics]{Axis}}. 
}
  \item{xat}{
Only used when \code{log="x"}. \cr
numeric, with points at which tick-marks will try to be drawn in the x axis, in addition to the defaults computed by R. See the \code{at} argument in \code{\link[graphics]{Axis}}. 
}
  \item{col}{
See \code{\link[graphics]{plot.default}}. The colors for lines and points.  Multiple colors can be specified so that each point can be given its own color.  If there are fewer colors than points they are recycled in the standard fashion. Lines will all be plotted in the first colour specified.
}
  \item{pch}{
See \code{\link[graphics]{plot.default}}. A vector of plotting characters or symbols: see \code{\link[graphics]{points}}.
}
  \item{lwd}{
See \code{\link[graphics]{plot.default}}. The line width, see \code{\link[graphics]{par}}.
}
  \item{lty}{
See \code{\link[graphics]{plot.default}}. The line type, see \code{\link[graphics]{par}}. 
}
  \item{cex}{
See \code{\link[graphics]{plot.default}}. A numerical vector giving the amount by which plotting characters and symbols should be scaled relative to the default.  
This works as a multiple of \env{par("cex")}. 'NULL' and 'NA' are equivalent to '1.0'. Note that this does not affect annotation.
}
  \item{cex.axis}{
magnification of axis annotation relative to 'cex'.
}
  \item{cex.lab}{
Magnification to be used for x and y labels relative to the current setting of 'cex'. See '?par'.
}
  \item{leg.txt}{
vector with the names that have to be used for each column of \code{x}.
}
  \item{leg.cex}{
numeric, indicating the character expansion factor for the legend, *relative* to current \code{par("cex")}. Default value = 1
}
  \item{leg.pos}{
keyword to be used to position the legend. One of the list \sQuote{"bottomright", "bottom", "bottomleft", "left", "topleft", "top", "topright", "right", "center"}. This places the legend on the inside of the plot frame at the given location. See \code{\link[graphics]{legend}}. \cr
When \code{leg.pos="auto"}, the legend provided by \code{leg.txt} is located on the \sQuote{bottomleft} when \code{log="y"} and on the \sQuote{topright} otherwise
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{thr.shw}{
logical, indicating if the streamflow values corresponding to the user-defined thresholds \code{lQ.thr} and \code{hQ.thr} have to be shown in the plot. \cr
When \code{leg.pos="auto"}, the legend with the threshold values is located on the \sQuote{topright} when \code{log="y"} and on the \sQuote{bottomleft} otherwise
}
  \item{border}{
See \code{\link[graphics]{polygon}}. The color to draw the border of the polygon with the uncertainty bounds. The default, 'NA', means to omit borders. 
}
  \item{bands.col}{
See \code{\link[graphics]{polygon}}. The color for filling the polygon. The default, 'NA', is to leave polygons unfilled, unless \code{bands.density} is specified. If \code{bands.density} is specified with a positive value this gives the color of the shading lines.
}
  \item{bands.density}{
See \code{\link[graphics]{polygon}}. The density of shading lines for the polygon with the uncertainty bounds, in lines per inch. The default value of 'NULL' means that no shading lines are drawn. A zero value of \code{bands.density} means no shading nor filling whereas negative values (and 'NA') suppress shading (and so allow color filling).
}
  \item{bands.angle}{
See \code{\link[graphics]{polygon}}. The slope of shading lines for the polygon with the uncertainty bounds, given as an angle in degrees (counter-clockwise). 
}
  \item{new}{
logical, if TRUE, a new plotting window is created. 
}
  \item{\dots}{
further arguments passed to or from other methods (to the plotting functions)
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
\references{
\cite{Vogel, R., and N. M. Fennessey (1994), Flow duration curves I: A new interpretation and confidence intervals, ASCE, Journal of Water Resources Planning and Management, 120(4)}. \cr

\cite{Vogel, R., and N. Fennessey (1995), Flow duration curves II: A review of applications in water resources planning, Water Resources Bulletin, 31(6), 1029-1039, doi:10.1111/j.1752-1688.1995.tb03419.x}. \cr

\cite{Yilmaz, K. K., H. V. Gupta, and T. Wagener  (2008), A process-based diagnostic approach to model evaluation: Application to the NWS distributed hydrologic model, Water Resour. Res., 44, W09417, doi:10.1029/2007WR006716}.
}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
If you do not want to use logarithmic scale for the streamflow axis, you can do it by passing the \code{log=" "} to the \code{...} argument.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{fdc}}
}
\examples{
## Loading daily streamflows at the station Oca en Ona (Ebro River basin, Spain) ##
data(OcaEnOnaQts)
q <- OcaEnOnaQts

# Creating a fictitious lower uncertainty band
lband <- q - min(q, na.rm=TRUE)

# Giving a fictitious upper uncertainty band
uband <- q + mean(q, na.rm=TRUE)

# Plotting the flow duration curve corresponding to 'q', with two uncertainty bounds
fdcu(q, lband, uband)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File dip.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{dip}
\Rdversion{1.1}
\alias{dip}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Days in Period
}
\description{
Given any starting and ending dates, it generates:  \cr
1) a vector of class Date with all the days between \code{from} and \code{to} (both of them included), OR \cr
2) the amount of days between the two dates 
}
\usage{
dip(from, to, date.fmt = "\%Y-\%m-\%d", out.type = "seq")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{from}{
Character indicating the starting date for creating the sequence. It has to be in the format indicated by \code{date.fmt}.
}
  \item{to}{
Character indicating the ending date for creating the sequence. It has to be in the format indicated by \code{date.fmt}.
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
Character indicating the type of result that is given by this function. Valid values are: \cr
1) \kbd{seq} : a vector of class Date with all the days between the two dates, OR \cr
2) \kbd{nmbr}: a single numeric value with the amount of days between the two dates.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
Depending on the value of \code{out.type}, it returns: \cr
1) a vector of class Date with all the days between \code{from} and \code{to} (both of them included), OR \cr
2) the amount of days between the two dates 
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{mip}}, \code{\link{yip}}, \code{\link{hip}}, \code{\link{diy}}
}
\examples{
## Sequence of daily dates between "1961-01-01" and "1961-12-31" ##
dip("1961-01-01", "1961-12-31")

## Number of days between "1961-01-01" and "1965-06-30", 
## but using "\%d-\%m-\%Y" as date format.
dip("01-01-1961", "30-06-1965", date.fmt= "\%d-\%m-\%Y", out.type = "nmbr")

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File vector2zoo.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{vector2zoo}
\Rdversion{1.1}
\alias{vector2zoo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Vector -> Zoo
}
\description{
Transform a numeric vector and its corresponding dates into a zoo object.
}
\usage{
vector2zoo(x, dates, date.fmt = "\%Y-\%m-\%d")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
numeric vector
}
  \item{dates}{
character, factor, Date or POSIXct object with the dates corresponding to each element of \code{x}. Valid object class for \code{dates} are: \code{character, factor, Date, POSIXct}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \sQuote{Details} section in \code{\link[base]{strptime}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="character"}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
%%  ~Describe the value returned
a zoo object, with values given by \code{x} and time stamps given by \code{dates}
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[zoo]{zoo}}, \code{\link{izoo2rzoo}}, \code{\link{dip}}, \code{\link{mip}}, \code{\link{yip}}
}
\examples{
##
## Example1: creating daily data

# Generating a numeric variable (e.g., read from the outputs of an hydrological model)
x <- 1:31

# Generating the dates corresponding to the previous values
dates <- dip("1961-01-01", "1961-01-31")

## Transforming from 'numeric' to 'zoo' class
z <- vector2zoo(x, dates)

##
## Example2: creating hourly data

# Generating a numeric variable
x <- rnorm(7)

# Generating hourly times since 17:00:00 up to 23:00:00 for 2012-Oct-15
dates <- ISOdatetime(2012, 10, 15, 17:23, 00, 0)

## Transforming from 'numeric' to 'zoo' class
z <- vector2zoo(x, dates)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File climograph.Rd
%% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%%                                 http://www.rforge.net/hydroTSM/ ; 
%% Copyright 2008-2022 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{climograph}
\Rdversion{1.1}
\alias{climograph}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Climograph
}
\description{
Function to draw a climograph based on precipitation and air temperature data, with several options for customisation.
}
\usage{
climograph(pcp, tmean, tmx, tmn, na.rm=TRUE,  
           from, to, date.fmt="\%Y-\%m-\%d", 
           main="Climograph", pcp.label="Precipitation, [mm]", 
           tmean.label="Air temperature, [\U00B0 C]", start.month=1, pcp.solid.thr,
           pcp.ylim, temp.ylim,pcp.col="lightblue", pcp.solid.col="skyblue2", 
           tmean.col="darkred", tmn.col="blue", tmx.col="red",
           pcp.labels=TRUE, 
           tmean.labels=TRUE, tmx.labels=TRUE, tmn.labels=TRUE,
           pcp.labels.cex=0.8, temp.labels.cex=0.8,
           pcp.labels.dx=c(rep(ifelse(plot.pcp.probs, -0.25,  0.0),6), 
                           rep(ifelse(plot.pcp.probs, -0.25,  0.0),6)),
           pcp.labels.dy=rep(2, 12),
           temp.labels.dx=c(rep(-0.2,6), rep(0.2,6)), temp.labels.dy=rep(-0.4, 12),
           plot.pcp.probs=TRUE, pcp.probs=c(0.25, 0.75),
           plot.temp.probs=TRUE, temp.probs=c(0.25, 0.75), 
           temp.probs.col=c("#3399FF", "#FF9966", "#FFCC66"),
           temp.probs.alpha=0.3,
           lat, lon
           )

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{pcp}{
variable of type zoo with monthly, daily or subdaily precipitation data.
}
  \item{tmean}{
variable of type 'zoo' with monthly, daily or subdaily mean temperature data.
}
  \item{tmx}{
variable of type 'zoo' with monthly, daily or subdaily maximum temperature data. \cr
ONLY used (together with \code{tmn}) when \code{tmean} is missing.
}
  \item{tmn}{
variable of type 'zoo' with monthly, daily or subdaily minimum temperature data. 
ONLY used (together with \code{tmx}) when \code{tmean} is missing.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the monthly values  are computed considering only those values different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the resulting average monthly value is NA .
}
  \item{from}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the starting date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{to}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the ending date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{date.fmt}{
Character indicating the format in which the dates are stored in \var{dates}, \var{from} and \var{to}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{main}{
Character representing the main title of the plot. 
}
  \item{pcp.label}{
Character used in the legend to represent the monthly average precipitation (mostly thought for languages different from English). 
}
  \item{tmean.label}{
Character used in the legend to represent the monthly average temperature (mostly thought for languages different from English). 
}
  \item{start.month}{
[OPTIONAL]. Only used when the (hydrological) year of interest is different from the calendar year.

numeric in [1:12] indicating the starting month of the (hydrological) year. Numeric values in [1, 12] represents months in [January, December]. By default \code{start.month=1}.
}
  \item{pcp.solid.thr}{
[OPTIONAL]. Only used when using (sub)daily precipitation and temperature are gives as input data. \cr
numeric, indicating the temperature, in degrees Celsius, used to discriminate between solid and liquid precipitation. \cr

When daily \code{tmean <= pcp.solid.thr} the precipitation for that day is considered as solid precipitation.
}
  \item{pcp.ylim}{
[OPTIONAL] numeric of length 2 with the the range used for the precipitation axis. The second value should be larger than the first one.
}
  \item{temp.ylim}{
[OPTIONAL] numeric of length 2 with the the range used for the secondary temperature axis. The second value should be larger than the first one.
}
  \item{pcp.col}{
Color used in the legend to represent the monthly average precipitation. 
}
  \item{pcp.solid.col}{
Color used in the legend to represent the monthly average solid precipitation. 
}
  \item{tmean.col}{
Color used in the legend to represent the monthly average temperature. 
}
  \item{tmn.col}{
Color used in the legend to represent the monthly minimum temperature. 
}
  \item{tmx.col}{
Color used in the legend to represent the monthly maximum temperature. 
}
  \item{pcp.labels}{
logical. Should monthly precipitation values to be shown above the bars?. By default \code{pcp.labels=TRUE}.
}
  \item{tmean.labels}{
logical. Should monthly mean temperature values to be shown above the lines?. By default \code{tmean.labels=TRUE}.
}
  \item{tmx.labels}{
logical. Should monthly maximum temperature values to be shown above the lines?. By default \code{tmx.labels=TRUE}.
}
  \item{tmn.labels}{
logical. Should monthly minimum temperature values to be shown above the lines?. By default \code{tmn.labels=TRUE}.
} 
  \item{pcp.labels.cex}{
numeric giving the amount by which plotting characters used to show the numeric values of monthly precipitation values are scaled relative to the default. \cr
It is only used when \code{pcp.labels=TRUE}.
}
  \item{temp.labels.cex}{
numeric giving the amount by which plotting characters used to show the numeric values of monthly air temperature values (mean, maximum, minimum) are scaled relative to the default.   \cr
It is only used when \code{tmean.labels=TRUE} or \code{tmx.labels=TRUE} or \code{tmn.labels=TRUE}.
}
  \item{pcp.labels.dx}{
numeric of length 12 giving the amount of horizontal coordinate positions that have to be used to shift the labels of monthly precipitation values. \cr
It is only used when \code{pcp.labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{pcp.labels.dy}{
numeric of length 12 giving the amount of vertical coordinate positions that have to be used to shift the labels of monthly precipitation values. \cr
It is only used when \code{pcp.labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{temp.labels.dx}{
numeric of length 12 giving the amount of horizontal coordinate positions that have to be used to shift the labels of monthly air temperature values (mean, maximum, minimum). \cr
It is only used when \code{tmean.labels=TRUE} or \code{tmx.labels=TRUE} or \code{tmn.labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{temp.labels.dy}{
numeric of length 12 giving the amount of vertical coordinate positions that have to be used to shift the labels of monthly air temperature values (mean, maximum, minimum). \cr
It is only used when \code{tmean.labels=TRUE} or \code{tmx.labels=TRUE} or \code{tmn.labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{plot.pcp.probs}{
logical used to decide whether to show uncertainty values around the monthly mean precipitation values. By default \code{plot.pcp.probs=TRUE}. \cr 
When \code{plot.pcp.probs=TRUE} the \code{pcp.probs} argument is used to define the values of the lower an upper uncertainty bounds.
} 
  \item{pcp.probs}{
numeric of length 2. It defines the quantile values used to compute the lower an upper uncertainty bounds for each one of the 12 monthly precipitation values.\cr 
By default \code{pcp.probs=c(0.25, 0.75)}, which indicates that the quantiles 0.25 and 0.75 are used to compute the lower an upper uncertainty bounds for each one of the 12 monthly precipitation values. 
If \code{pcp} is a (sub)daily zoo object, it is first aggregated into monthly values using \code{mean}, and then the \code{pcp.probs} quantiles are computed over all the monthly values belonging to a calendar month.
} 
  \item{plot.temp.probs}{
logical used to decide whether to show uncertainty values around the monthly mean temperature values. By default \code{plot.temp.probs=TRUE}. \cr 
When \code{plot.temp.probs=TRUE} the \code{temp.probs} argument is used to define the values of the lower an upper uncertainty bounds.
} 
  \item{temp.probs}{
numeric of length 2. It is used to define quantile values used to compute the lower an upper uncertainty bounds for each one of the 12 monthly mean temperature values.\cr 
If \code{tmx} and \code{tmn} are provided, then \code{temp.probs} are used to compute the lower an upper uncertainty bounds for each one of the 12 monthly maximum/minimum temperature values. \cr
By default \code{temp.probs=c(0.25, 0.75)}, which indicates that the quantiles 0.25 and 0.75 are used to compute the lower an upper uncertainty bounds for each one of the 12 monthly mean(maximum/minimum) values. 
If \code{tmx/tmn} is provided and is a (sub)daily zoo object, it is first aggregated into monthly values using \code{mean}, and then the \code{temp.probs} quantiles are computed over all the monthly values belonging to a calendar month.
} 
  \item{temp.probs.col}{
character of length 3, with the colors used to for plotting the uncertainty bands around the average monthly values of the minimum, mean and maximum air temperature, respectively. \cr
If \code{tmx} and \code{tmn} are not provided by the user, the second element of \code{temp.probs.col} will still be used to define the color of the  uncertainty band around the mean monthly values of air temperature.
} 
  \item{temp.probs.alpha}{
numeric of length 1, with the factor used to modify the opacity of \code{temp.probs.col}. Typically in [0,1], with 0 indicating a completely transparent colour and 1 indicating no transparency.
} 
  \item{lat}{
[OPTIONAL] numeric or character used to show the latitude for which the climograph was plotted for.
} 
  \item{lon}{
[OPTIONAL] numeric or character used to show the longitude for which the climograph was plotted for.
} 
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
If the output climograph present some mixed or not legible characters, you might try resizing the graphical window and run climograph again with the new size, until you get the climograph in the way you want to.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{monthlyfunction}}
}
\examples{
######################
## Ex1: Loading the DAILY precipitation, maximum and minimum air temperature at 
##      station Maquehue Temuco Ad (Chile)
data(MaquehueTemuco)

# Subsetting to only the 1981-1990 temporal period
MaquehueTemuco <- window(MaquehueTemuco, start="1981-01-01", end="1990-12-31")

pcp <- MaquehueTemuco[, 1]
tmx <- MaquehueTemuco[, 2]
tmn <- MaquehueTemuco[, 3]

## Plotting a full climograph
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)

\dontrun{
## Plotting a climograph with uncertainty bands around mean values, 
## but with no labels for tmx and tmn
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, tmx.labels=FALSE, tmn.labels=FALSE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)

## Plotting a climograph with uncertainty bands around mean values, but with no labels for 
##  tmx, tmn and pcp
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, 
                pcp.labels=FALSE, tmean.labels=FALSE, tmx.labels=FALSE, tmn.labels=FALSE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)

## Plotting a climograph with no uncertainty bands around mean values
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, plot.pcp.probs=FALSE, plot.temp.probs=FALSE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)

## Plotting the most basic climograph: only mean values of precipiation and air temperature
m <- climograph(pcp=pcp, tmean=0.5*(tmn+tmx), na.rm=TRUE, plot.pcp.probs=FALSE, 
                plot.temp.probs=FALSE, main="Maquehue Temuco Ad (Chile)", 
                lat=-38.770, lon=-72.637)


## Plotting a full climograph, starting in April (start.month=4) instead of January (start.month=1),
## to better represent the hydrological year in Chile (South America)
m <- climograph(pcp=pcp, tmx=tmx, tmn=tmn, na.rm=TRUE, 
                start.month=4, temp.labels.dx=c(rep(-0.2,4), rep(0.2,6),rep(-0.2,2)),
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)


## Plotting a full climograph with monthly data
pcp.m <- daily2monthly(pcp, FUN=sum)
tmx.m <- daily2monthly(tmx, FUN=mean)
tmn.m <- daily2monthly(tmn, FUN=mean)
m <- climograph(pcp=pcp.m, tmx=tmx.m, tmn=tmn.m, na.rm=TRUE, 
                main="Maquehue Temuco Ad (Chile)", lat=-38.770, lon=-72.637)
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File baseflow.Rd
%% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%%                                 http://www.rforge.net/hydroTSM/ ; 
%%                                 https://cran.r-project.org/package=hydroTSM
%% Copyright 2023-2025 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{baseflow}
\Rdversion{1.1}
\alias{baseflow}
\alias{baseflow.zoo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  Baseflow
 }
\description{
Given a complete (without missing values) series of streamflow, this function computes the baseflow using the filter proposed by Arnold and Allen (1999). 
}
\usage{
baseflow(x, ...)

\method{baseflow}{zoo}(x, beta=0.925, from=start(x), to=end(x),  n.pass=3L,
         pass.start=c("forward", "backward"), date.fmt, tz, 
         na.fill=c("none", "linear", "spline"), out.type=c("last", "all"), 
         plot=TRUE, xcol="black", bfcol=c("blue", "darkcyan", "darkorange3"),
         pch=15, cex=0.3, ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo or numeric object with streamflow records. The suggested time frequency should be hourly or daily, but the algorithm will work with any time frequency.
}
  \item{beta}{
numeric representing the filter parameter. Default value is 0.925 as recommended by Arnold and Allen (1999)
}
  \item{from}{
Character indicating the starting date for subsetting \code{x}. It has to be in the format indicated by \code{date.fmt}. \cr
The default value corresponds to the date of the first element of \code{x}.
}
  \item{to}{
Character indicating the ending date for subsetting \code{x}. It has to be in the format indicated by \code{date.fmt}. \cr
The default value corresponds to the date of the last element of \code{x}.
}
  \item{n.pass}{
Integer indicating the number times the filter will be passed over the streamflow data stored in \code{x}. Default is 3.
}
  \item{pass.start}{
Character indicating the direction of the first pass of the filter. Subsequent passes alternate direction (e.g., forward-backward). Valid values are: \cr
-) \kbd{forward}  => the first pass of the filter is applied forward \cr
-) \kbd{backward} => the first pass of the filter is applied backward 
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. See \sQuote{Details} section in \code{\link[base]{strptime}}.
By default, \code{date.fmt} is missing, and it is automatically set to \kbd{\%Y-\%m-\%d} when \code{time(x)} is \code{Date} object, and set to \kbd{\%Y-\%m-\%d \%H:\%M:\%S} when \code{x} is a sub-daily zoo object.
}
  \item{tz}{
character, with the specification of the time zone used for \code{from}, \code{to}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{from} and \code{to}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{na.fill}{Character indicating how to fill any NA present in \code{x}. Valid values are: \cr
-) \kbd{remove}      => NAs are not plotted                                                         \cr
-) \kbd{linear}      => NAs are removed by linear interpolation, using \code{\link[zoo]{na.approx}} \cr 
-) \kbd{spline}      => NAs are removed by spline interpolation, using \code{\link[zoo]{na.spline}}
}
  \item{out.type}{
Character indicating the type of result that is given by this function. Valid values are: \cr
-) \kbd{last} => only the baseflow computed after the third pass of the filter is returned. \cr
-) \kbd{all}  => the 3 baseflows computed after each pass of the filter are returned in a matrix or zoo object.
}
  \item{plot}{
logical. Indicates if the baseflow should be plotted or not. If plotted, the original \code{x} values are plotted as well.
}
  \item{xcol}{
character, representing the color to be used for ploting the streamflow time series. Only used when \code{plot=TRUE}.
}
  \item{bfcol}{
character of lenght 3, representing the color(s) to be used for ploting the baseflow time series. The first, second and third element are used to represent the baseflow after the third, second and first pass of the filter, respectively. Only used when \code{plot=TRUE}.
}
  \item{pch}{
numeric, representing the symbols used for ploting the streamflow time series (both, the original series and the baseflow). Only used when \code{plot=TRUE}.
}
  \item{cex}{
a numerical vector giving the amount by which plotting characters and symbols should be scaled relative to the default. This works as a multiple of \code{par("cex")}. See \code{\link[graphics]{plot.default}}. Only used when \code{plot=TRUE}.
}
  \item{\dots}{
further arguments passed to or from other methods. Not used yet.
}
}
 \details{
Although most procedures to separate baseflow from total streamflow are based on physical reasoning, some elements of all separation techniques are subjective.

The digital filter technique (Nathan and McMahon, 1990) implemented in this function was originally proposed by Lyne and Hollick (1979) for signal analysis and processing. Although this technique has no true physical meaning, it is objective and reproducible. 

The equation of the filter is: 

q(t) = Beta*q(t-1) + [ (1+Beta)/2 ]*[ Q(t) - Q(t-1) ]

where q(t) is the filtered surface runoff (quick response) at the time step t (one day), Q is the original streamflow, and Beta is the filter parameter (Beta=0.925). The value Beta=0.925 was obtained by Nathan and McMahon (1990) and Arnold et al. (1995) to give realistic results when compared to manual separation techniques.

Baseflow b(t) is then computed as:

b(t) = Q(t) - q(t)

The filter can be passed over the streamflow data three times (forward, backward, and forward), depending on the user' selected estimates of baseflow from pilot studies. In general, each pass will result in less baseflow as a percentage of total streamflow. 
}
\value{
%%  If it is a LIST, use
  \item{If \code{out.type="last"}}{(default value), only the baseflow computed after the third pass of the filter is returned.}
  \item{If \code{out.type="all"}}{the 3 baseflows computed after each pass of the filter are returned in a matrix or zoo object.}
%% ...
}
\references{
\cite{Arnold, J. G., Allen, P. M., Muttiah, R., Bernhardt, G. (1995). Automated base flow separation and recession analysis techniques. Groundwater, 33(6), 1010--1018. doi:10.1111/j.1745-6584.1995.tb00046.x}.

\cite{Arnold, J. G., Allen, P. M. (1999). Automated methods for estimating baseflow and ground water recharge from streamflow records. JAWRA Journal of the American Water Resources Association, 35(2), 411--424. doi:10.1111/j.1752-1688.1999.tb03599.x}.

\cite{Lyne, V., Hollick, M. (1979). Stochastic time-variable rainfall-runoff modelling. Proceedings of the Hydrology and Water Resources Symposium, Perth, 10--12 September. Institution of Engineers National Conference Publication, No. 79/10, 89--92}.

\cite{Nathan, R. J., & McMahon, T. A. (1990). Evaluation of automated techniques for base flow and recession analyses. Water resources research, 26(7), 1465--1473. doi:10.1029/WR026i007p01465}.

\cite{Ladson, A. R., Brown, R., Neal, B., & Nathan, R. (2013). A standard approach to baseflow separation using the Lyne and Hollick filter. Australasian Journal of Water Resources, 17(1), 25-34. doi:10.7158/13241583.2013.11465417}.

}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail} ; Hector Garces-Figueroa
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{plot_pq}}, \code{\link{hydroplot}}, \code{\link{fdc}}, \code{\link{fdcu}}
}

\examples{
######################
## Ex1: Computing and plotting the baseflows for the full time period
##      of a given time series of streamflows.

## First, we load the daily Q time series for the Cauquenes en 
## El Arrayan catchment, where Q, [m3/s] are stored in the sixth column.
data(Cauquenes7336001)
q <- Cauquenes7336001[, 6]

## Computing the daily baseflow for the full time period
#baseflow(q) # it can not run due to NA values in 'x'

# filling the NA values using spline interpolation
baseflow(q, na.fill="spline") 

## Computing and plotting the daily baseflow for the full time period
baseflow(q, na.fill="spline", plot=TRUE)


######################
## Ex2: Computing and plotting the daily baseflow only for a 
##      specific time period, from April to December 2000.
baseflow(q, na.fill="spline", from="2000-04-01", to="2000-12-31")


######################
## Ex3: Computing and plotting the three daily baseflows (one for each pass 
##      of the filter) only for a specific time period, from April to December
##      2000.
baseflow(q, na.fill="spline", from="2000-04-01", to="2000-12-31", 
         out.type="all", plot=TRUE)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
\keyword{graphs}

%% File matrixplot.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2026 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{matrixplot}
\Rdversion{1.1}
\alias{matrixplot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Matrixplot
}
\description{
Plots a color matrix, representing the values stored in \code{x}. \cr
Originally, it was thought to represent the amount of days with information per year in a set of gauging stations, but it can be used for plotting the information stored in any two dimensional matrix.
}
\usage{
matrixplot(x, ColorRamp="Days", ncolors = 10, main = "", 
           cuts,  cuts.dec=2, cuts.labels, 
           cuts.style=c("equal", "pretty", "fixed", "sd", "quantile", 
                        "kmeans", "bclust", "fisher"), 
           legend.cex=0.8,  legend.title="",  legend.title.cex=1.5, 
           legend.fontsize=15, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
matrix to be plotted. Originally: \cr
-) Each column of \code{x} represent a different gauging station, and it stores the values measured on it \cr
-) Each row of \code{x} represent the years, and they stores the amount of days with information in each station
}
  \item{ColorRamp}{
Character or function defining a personalized color ramp for plotting the maps. \cr
Valid character values are in \kbd{c("Days", "Precipitation", "Temperature", "PCPAnomaly", "PCPAnomaly2" "TEMPAnomaly", "TEMPAnomaly2", "TEMPAnomaly3")}.
}
  \item{ncolors}{
numeric, indicating the number of color intervals that will be used for representing the information content of \code{x}.
}
  \item{main}{
Main title for the plot
}
\item{cuts}{
Numeric, indicating the values used to divide the range of 'x' in the legend of colours. If not provided, it is automatically selected as a function of 'lenght(ColorRamp)'.
}
\item{cuts.dec}{
Number of decimal places used to present the numbers that divide the range of 'x' in the legend of colours.
}
\item{cuts.labels}{
Character indicating the label to be used in the ccolour legend for each one of the values defined by 'cuts'. If not provided, as.character(cuts)' is used.
}
\item{cuts.style}{
character, indicating how to finding class intervals for continuous numerical variables for choosing colours to be used in the figure. See \code{\link[classInt]{classIntervals}}
}
\item{legend.cex}{
character expansion factor *relative* to current \code{par("cex")} used for the legend text.
}
\item{legend.title}{
text to be displayed above the legned of colours (e.g., showing the measurement units of the raster being displayed).
}
\item{legend.title.cex}{
expansion factor(s) for the legend title. Currently it is not used. See \code{legend.fontsize} instead.
}
\item{legend.fontsize}{
size of text (in points) used in the legend title (e.g., showing the measurement units of the raster being displayed).
}
  \item{\dots}{
further arguments passed to \code{\link[lattice]{levelplot}} function (\pkg{lattice} package) or from other methods
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
Adapted from a not available web page (http://www2.warwick.ac.uk/fac/sci/moac/currentstudents/peter_cock/r/matrix_contour/)
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{dwi}}, \code{\link[classInt]{classIntervals}}
}
\examples{
## Loading the SanMartino precipitation data
data(SanMartinoPPts)

# Selecting only the values up to Dec/1960
x <- window(SanMartinoPPts, end=as.Date("1960-12-31"))

## Daily zoo to monthly zoo
m <- daily2monthly(x, FUN=sum, na.rm=TRUE)

# Creating a data.frame with monthly values per year in each column
M <- matrix(m, ncol=12, byrow=TRUE)
colnames(M) <- month.abb
rownames(M) <- unique(format(time(m), "\%Y"))

# Plotting the monthly precipitation values from 1921 to 1960.
# Useful for identifying dry/wet months
matrixplot(M, ColorRamp="Precipitation", 
           main="Monthly precipitation at San Martino st., [mm/month]")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}

%%File daily2weekly.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%% Copyright 2023-2026 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{subdaily2weekly}
\Rdversion{1.1}
\alias{subdaily2weekly}
\alias{subdaily2weekly.default}
\alias{subdaily2weekly.zoo}
\alias{subdaily2weekly.data.frame}
\alias{subdaily2weekly.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Subdaily -> Weekly
}
\description{
Generic function for transforming a DAILY (or sub-daily) regular time series into a WEEKLY one
}
\usage{
subdaily2weekly(x, ...)

\method{subdaily2weekly}{default}(x, FUN, na.rm=TRUE, na.rm.max=0,
           start="00:00:00", start.fmt= "\%H:\%M:\%S", tz, ...)

\method{subdaily2weekly}{zoo}(x, FUN, na.rm=TRUE, na.rm.max=0,
           start="00:00:00", start.fmt= "\%H:\%M:\%S", tz, ...)

\method{subdaily2weekly}{data.frame}(x, FUN, na.rm=TRUE, na.rm.max=0,
           start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
           dates=1, date.fmt = "\%Y-\%m-\%d \%H:\%M:\%S",  
           out.fmt="zoo", verbose=TRUE, ...)

\method{subdaily2weekly}{matrix}(x, FUN, na.rm=TRUE, na.rm.max=0,
           start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
           dates=1, date.fmt = "\%Y-\%m-\%d \%H:\%M:\%S",  
           out.fmt="zoo", verbose=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with (sub)daily time series. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represents the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that have to be applied for transforming from daily to weekly time step (e.g., for precipitation \code{FUN=sum} and for temperature and streamflow ts \code{FUN=mean}). \cr

\code{FUN} MUST accept the \code{na.rm} argument, because \code{na.rm} is passed to \code{FUN}.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the weekly values are computed only for weeks with a percentage of missing values less than \code{na.rm.max} \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the corresponing weekly values in the output object will be \code{NA}.
}
  \item{na.rm.max}{
Numeric in [0, 1]. It is used to define the maximum percentage of missing values allowed in each week to keep the weekly aggregated value in the output object of this function. In other words, if the percentage of missing values in a given week is larger than \code{na.rm.max} the corresponding weekly value will be \code{NA}.
}
  \item{start}{
character, indicating the starting time used for aggregating sub-daily time series into daily ones. It MUST be provided in the format specified by \code{start.fmt}. \cr
This value is used to define the time when a new day begins (e.g., for some rain gauge stations). \cr
-) All the values of \code{x} with a time attribute before \code{start} are considered as belonging to the day before the one indicated in the time attribute of those values. \cr
-) All the values of \code{x} with a time attribute equal to \code{start} are considered to be equal to \code{"00:00:00"} in the output zoo object. \cr
-) All the values of \code{x} with a time attribute after \code{start} are considered as belonging to the same day as the one indicated in the time attribute of those values. \cr

It is useful when the daily values start at a time different from \code{"00:00:00"}. Use with caution. See examples.
}
  \item{start.fmt}{
character indicating the format in which the time is provided in \code{start}, By default \code{date.fmt=\%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.POSIXct}}.
}
  \item{tz}{
character, with the specification of the time zone used in both \code{x} and \code{start}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr
This argument can be used to force using the local time zone or any other time zone instead of UTC as time zone.
}
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates for each gauging station \cr
If \code{dates} is a number (default), it indicates the index of the column in \var{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of Date class, the code verifies that the number of days on it be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the DateTime objects are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d \%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.POSIXct}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.fmt}{
OPTIONAL. Only used when \code{x} is a matrix or data.frame object /cr
character, for selecting if the result will be a matrix/data.frame or a zoo object. Valid values are: \kbd{numeric}, \kbd{zoo}.
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
arguments additional to \code{na.rm} passed to \code{FUN}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
a zoo object with weekly time frequency
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{cmv}}, \code{\link{subhourly2hourly}}, \code{\link{daily2monthly}}, \code{\link{daily2annual}}, \code{\link{subdaily2daily}}, \code{\link{weeklyfunction}}, \code{\link{hydroplot}}, \code{\link{vector2zoo}}, \code{\link{izoo2rzoo}}, \code{\link{as.Date}}
}
\examples{

######################
## Ex1: Computation of WEEKLY values from HOURLY ts, only when the percentage of NAs in 
#       each week is lower than a user-defined percentage (10% in this example).

## Loading the HOURLY streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

# Subsetting 'x' to its first three weeks 
# (31-December-1979 to 31-March-1980)
x <- window(x, end="1980-03-31 23:59:00")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Daily to Weekly, only for weeks with less than 10% of missing values
( w2 <- subdaily2weekly(x, FUN=sum, na.rm=TRUE, na.rm.max=0.1) )

# Verifying that the second and third month of 'x' had 10% or more of missing values
cmv(x, tscale="weekly")

\dontrun{
######################
## Ex2: Computation of WEEKLY values from HOURLY ts, removing any missing value in 'x'
#       Loading the HOURLY streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

# Sub-daily to weekly ts
subdaily2weekly(x, FUN=mean, na.rm=TRUE)
}
}



% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%%File mip.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{mip}
\Rdversion{1.1}
\alias{mip}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Months in Period
}
\description{
 Given any starting and ending dates, it generates: \cr      
1) a vector of class 'Date' with all the months between the two dates (both of them included), OR \cr
2) the amount of months between the two dates 
}
\usage{
mip(from, to, date.fmt = "\%Y-\%m-\%d", out.type = "seq")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{from}{
Character indicating the starting date for creating the sequence. It has to be in the format indicated by \code{date.fmt}.
}
  \item{to}{
Character indicating the ending date for creating the sequence. It has to be in the format indicated by \code{date.fmt}.
}
  \item{date.fmt}{
Character indicating the format in which the dates are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}.
}
  \item{out.type}{
character indicating the type of result that is given by this function. Valid values are: \cr
-) \kbd{seq} : a vectorial sequence with all the months within the given year \cr
-) \kbd{nmbr}: the number of days in the vectorial sequence with all the months within the given year
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
Depending on the value of \code{out.type}, it returns: \cr
1) a vector of class Date with all the months between \code{from} and \code{to} (both of them included), OR \cr
2) a single numeric value with the amount of months between the two dates. 
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{dip}}, \code{\link{diy}}, \code{\link{hip}}, \code{\link{yip}}
}
\examples{
# Sequence of monthly dates between "1961-01-01" and "1961-12-31" ##
mip("1961-01-01", "1961-12-31")

## Computing the number of months between "1961-01-01" and "1965-06-30", 
## with the date format  "\%d-\%m-\%Y" ##
mip("01-01-1961", "30-06-1965", date.fmt= "\%d-\%m-\%Y", out.type = "nmbr")

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File SanMartinoPPts.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{SanMartinoPPts}
\docType{data}
\alias{SanMartinoPPts}
\title{
San Martino, ts of daily precipitation.
}
\description{
Daily time series of precipitation, at station San Martino di Castrozza, Trento Province, Italy, with data from 01/Jan/1921 to 31/Dec/1990.
}
\usage{
data(SanMartinoPPts)
}

%%\details{
%%
%%}
\format{
zoo object.
}
\source{
Provided by MeteoTrentino, Trento, Italy (via prof. Riccardo Rigon). \cr
These data are intended to be used for research purposes only, being distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
}

%%\references{
%% ~put references to the literature/web site here ~
%%}
\keyword{datasets}

%% File annualfunction.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{annualfunction}
\Rdversion{1.1}
\alias{annualfunction}
\alias{annualfunction.default}
\alias{annualfunction.zoo}
\alias{annualfunction.data.frame}
\alias{annualfunction.matrix}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Annual Function
}
\description{
Generic function for obtaining a SINGLE annual value of a zoo object, by applying any R function to ALL the values in \code{x} belonging to the same year, and then applying the same function to ALL the previously computed annual values (e.g., for computing the average annual precipitation or the mean annual streamflow of a long-term time series).
}
\usage{
annualfunction(x, FUN, na.rm = TRUE, ...)

\method{annualfunction}{default}(x, FUN, na.rm = TRUE, ...)

\method{annualfunction}{zoo}(x, FUN, na.rm = TRUE, ...)

\method{annualfunction}{data.frame}(x, FUN, na.rm = TRUE, dates=1, date.fmt = "\%Y-\%m-\%d", 
        verbose = TRUE, ...)
        
\method{annualfunction}{matrix}(x, FUN, na.rm = TRUE, dates=1, date.fmt = "\%Y-\%m-\%d", 
        verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, xts, data.frame or matrix object, with daily/monthly/annual time series. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that will be used to compute the final annual value (e.g., \code{FUN} may be some of \code{mean}, \code{sum}, \code{max}, \code{min}, \code{sd})  .
}
  \item{na.rm}{
Logical. Should missing values be removed?. \cr
-) \kbd{TRUE} : the annual values are computed considering only those values different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a year, the resulting annual value will be NA 
}
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates corresponding to each gauging station. \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it have to be converted into Date class, using the date format specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{verbose}{
Logical; if TRUE, progress messages are printed. 
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
When \code{x} is a time series, a single annual value is returned. \cr
For a data frame, a named vector with the appropriate method being applied column by column.
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
\code{FUN} is first applied to all the values of \code{x} belonging to the same year and then it is applied to all the previously computed annual values to get the final result.
Its result will depend on the sampling frequency of \code{x} and the type of function provided by \code{FUN} (\bold{special attention have to be put when \code{FUN=sum}})
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{monthlyfunction}}, \code{\link{daily2annual}}, \code{\link{monthly2annual}}, \code{\link{yip}}
}
\examples{
## Loading the SanMartino daily precipitation data (1921-1990)
data(SanMartinoPPts)
x <- SanMartinoPPts

# Amount of years in 'x' (needed for computing the average)
nyears <- length( seq(from=time(x[1]), to=time(x[length(x)]), by="years") )

## Average annual precipitation for the 70 years period. 
# It is necessary to divide by the amount of years to obtain the average annual value, 
# otherwise it will give the total precipitation for all the 70 years.
annualfunction(x, FUN=sum, na.rm=TRUE) / nyears


#####################
### verification ####
# Daily to annual
a <- daily2annual(x, FUN=sum, na.rm=TRUE)

# Mean annual value
mean(a)

##############################
##############################
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)
x <- EbroPPtsMonthly

## Dates of 'x'
dates <- as.Date(x[,1])


## Computation of the average annual precipitation
\dontrun{

## Transforming 'x' into a zoo object
z <- zoo( x[, 2:ncol(x)], dates)

# Amount of years in 'x' (needed for computing the average)
nyears <- yip(from=start(z), to=end(z), out.type="nmbr" )

## Average annual precipitation, for the first 5 stations in 'x'
annualfunction(z[ ,1:5], FUN=sum)/nyears
 
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File dwi.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{dwi}
\Rdversion{1.1}
\alias{dwi}
\alias{dwi.default}
\alias{dwi.zoo}
\alias{dwi.data.frame}
\alias{dwi.matrix}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Days with Information
}
\description{
This function generates a table indicating the number of days with information (<>NA) within a zoo object, aggregated by year, month or month per year.
}
\usage{
dwi(x, ...)

\method{dwi}{default}(x, out.unit = "years", from = start(x), to = end(x), 
     date.fmt = "\%Y-\%m-\%d", tstep="days", ...)
     
\method{dwi}{zoo}(x, out.unit = "years", from = start(x), to = end(x), 
     date.fmt = "\%Y-\%m-\%d", tstep="days", ...)

\method{dwi}{data.frame}(x, out.unit = "years", from, to, date.fmt = "\%Y-\%m-\%d", tstep="days", 
     dates = 1, verbose = TRUE, ...)
     
\method{dwi}{matrix}(x, out.unit = "years", from, to, date.fmt = "\%Y-\%m-\%d", tstep="days", 
     dates = 1, verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with daily/monthly/annual time series. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{out.unit}{
aggregation time for the computation of the amount of days with information. Valid values are: \cr
-) \kbd{months}: monthly;  \cr
-) \kbd{years} : annual;   \cr
-) \kbd{mpy}   : month per year (not available for data.frames)
}
  \item{from}{
Character indicating the starting date for the computations. It has to be in the format indicated by \code{date.fmt}.\cr
When \code{x} is a data.frame and this value is not provided, the date corresponding to the first row of \code{x} is used
}
  \item{to}{
Character indicating the ending date for the computations. It has to be in the format indicated by \code{date.fmt}. \cr
When \code{x} is a data.frame and this value is not provided, the date corresponding to the last row of \code{x} is used
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}. 
}
  \item{tstep}{
Time step used for storing the values in \code{x}. Valid values are: \kbd{days}, \kbd{months}, \kbd{years}.
Since the version 0.3-0 of hydroTSM, this argument is not required any more, because it is not used any longer.
}
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates for each column of \code{x}\cr
If \code{dates} is a number, it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, using the date format  specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of element in \code{x}
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{matrixplot}}
}
\examples{
## Loading the SanMartino precipitation data
data(SanMartinoPPts)
x <- SanMartinoPPts

\dontrun{
## Days with information per year
dwi(x)

## Days with information per month per year.
dwi(x, out.unit="mpy")
}

###########
\dontrun{
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

## Months with information per year in the 9 first stations of 'EbroPPtsMonthly'
a <- dwi(EbroPPtsMonthly[,1:10], out.unit="years", dates=1)

## Before plotting the results in 'a', and just for obtaining a more interesting
## plot, 70 random numbers (between 1 and 11) are introduced in 'a'
a[sample(length(a), size = 70)] <- rep(1:11, length=70)

## Plotting the amount of months with information per year in each station
matrixplot(a, var.type="Days", main="Number of months with info per year")
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File infillxy.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{infillxy}
\Rdversion{1.1}
\alias{infillxy}
\alias{infillxy.matrix}
\alias{infillxy.default}
\alias{infillxy.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Infills NA values
}
\description{
Infill all the missing values (NA) in \code{x} with the corresponding values in \code{sim}. 
}
\usage{
infillxy(x, ...)
\method{infillxy}{default}(x, sim, ...)
\method{infillxy}{matrix}(x, sim, ...)
\method{infillxy}{data.frame}(x, sim, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
numeric, data.frame or matrix in which some values are missing (NA).
}
  \item{sim}{
numeric, data.frame or matrix, with the same dimension of \code{x}, which contains the values that will be used for infilling the missing (NA) values in \code{x}
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
\details{
It gives as a result an object of the same dimension of \code{x}, in which all the NA values were infilled with the corresponding values of \code{sim}.
}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%%}
\examples{
obs <- c(1, NA, 3, 4, NA, 5)
sim <- rep(2, 6)

## Filling in the missing values in 'x' with the corresponding values in 'sim'
infillxy(x=obs, sim)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File time2season.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{time2season}
\Rdversion{1.1}
\alias{time2season}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Date/DateTime character -> Seasonal character
}
\description{
This function transforms a character vector of Dates or DateTimes into a character vector of seasons (summer, winter, autumn, spring), depending on the value of \code{type}:  \cr

When \code{type=default}
-) winter = DJF: December, January, February   \cr              
-) spring = MAM: March, April, May             \cr             
-) summer = JJA: June, July, August            \cr            
-) autumn = SON: September, October, November  \cr

When \code{type=FrenchPolynesia}
-) winter = DJFM: December, January, February, March  \cr              
-) spring = AM  : April, May                          \cr             
-) summer = JJAS: June, July, August, September       \cr            
-) autumn = ON  : October, November
}
\usage{
time2season(x, out.fmt = "months", type="default")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
vector with the dates that have to be transformed. class(x) must be Date
}
  \item{out.fmt}{
character, indicating the format of the output seasons. Possible values are: \cr
-) \kbd{seasons} =>  c("winter", "spring",  "summer", autumn")  \cr
-) \kbd{months}  =>  c("DJF", "MAM",  "JJA", SON") or c("DJFM", "AM",  "JJAS", ON")
}
  \item{type}{
character, indicating which weather seasons will be used for computing the output. Possible values are: \cr
-) \kbd{default}          =>  "winter"= Dec, Jan, Feb;      "spring"= Mar, Apr, May;  "summer"=Jun, Jul, Aug;       "autumn"= Sep, Oct, Nov  \cr
-) \kbd{FrenchPolynesia}  =>  "winter"= Dec, Jan, Feb, Mar; "spring"= Apr, May;       "summer"=Jun, Jul, Aug, Sep;  "autumn"= Oct, Nov 
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
character vector with the weather season to which each date in \code{x} belongs
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
  Weather seasons corresponding to French Polynesia were defined following a comment from Lydie Sichoix
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{dm2seasonal}}, \code{\link{seasonalfunction}}, \code{\link{extract}}, \code{\link{dip}}, \code{\link{mip}}
}
\examples{
## Sequence of daily dates between "1961-01-01" and "1961-12-31"
t <- dip("1961-01-01", "1961-12-31")
time2season(t)

## Sequence of monthly dates between "1961-01-01" and "1961-12-31"
t <- mip("1961-01-01", "1961-12-31")
time2season(t)
time2season(t, out.fmt="seasons")

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File hip.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2022 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{hip}
\Rdversion{1.1}
\alias{hip}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Hours in Period
}
\description{
Given any starting and ending date/time objects, it generates:  \cr
1) a vector of class c("POSIXct" "POSIXt") with all the hours between the two date/time objects (both of them included), OR \cr
2) the amount of hours between the two date/time objects 
}
\usage{
hip(from, to, date.fmt="\%Y-\%m-\%d \%H", out.type = "seq", tz="UTC")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{from}{
Character or POSIXct object indicating the starting date/time for creating the sequence. It has to be in the format indicated by \code{date.fmt}.
}
  \item{to}{
Character indicating the ending date/time for creating the sequence. It has to be in the format indicated by \code{date.fmt}.
}
  \item{date.fmt}{
character indicating the format in which the date/time objects are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d \%H:\%M}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
Character indicating the type of result that is given by this function. Valid values are: \cr
1) \kbd{seq} : a vector of class Date with all the days between the two dates, OR \cr
2) \kbd{nmbr}: a single numeric value with the amount of days between the two dates.
}
  \item{tz}{
specification of the desired time zone yo be used. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} (the default value) is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
This argument can be used when working with subdaily zoo objects to force using the local time zone instead of GMT as time zone.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
Depending on the value of \code{out.type}, it returns: \cr
1) a vector of class \code{c("POSIXct" "POSIXt")} with all the hours between \code{from} and \code{to} (both of them included), OR \cr
2) the amount of hours between the two date/time objects
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{dip}}, \code{\link{mip}}, \code{\link{yip}}, \code{\link{diy}}, \code{\link[xts]{timeBasedSeq}}
}
\examples{
## Sequence of hours between "1961-01-01 00:00" and "1961-01-10 00:00", giving the
## starting and ending date/time objects with hours and skipping the minutes (default)
hip("1961-01-01 00", "1961-12-31 00")

## Sequence of hours between "1961-01-01 00:00" and "1961-01-10 00:00", giving the
## starting and ending date/time objects only with hours and minutes(skipping the minutes)
hip("1961-01-01 00:00", "1961-12-31 00:00", date.fmt="\%Y-\%m-\%d \%H:\%M")

## Number of hours between the 10:00 AM of "1961-Jan-02" and the 11:00 AM of "1961-Jan-01", 
## using "\%d/\%m/\%Y" as date/time format.
hip("01/01/1961 10", "02/01/1961 11", date.fmt= "\%d/\%m/\%Y \%H", out.type = "nmbr")

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File daily2annual.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%% Copyright 2008-2025 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{(sub)daily2annual}
\Rdversion{1.1}
\alias{daily2annual}
\alias{daily2annual.default}
\alias{daily2annual.zoo}
\alias{daily2annual.data.frame}
\alias{daily2annual.matrix}
\alias{monthly2annual}
\alias{monthly2annual.default}
\alias{monthly2annual.zoo}
\alias{monthly2annual.data.frame}
\alias{monthly2annual.matrix}
\alias{subdaily2annual}
\alias{subdaily2annual.default}
\alias{subdaily2annual.zoo}
\alias{subdaily2annual.data.frame}
\alias{subdaily2annual.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
(sub)Daily/Monthly -> Annual 
}
\description{
Generic function for transforming a (sub)DAILY/MONTHLY (weekly and quarterly) regular time series into an ANNUAL one.
}
\usage{
daily2annual(x, ...)
subdaily2annual(x, ...)
monthly2annual(x, ...)

\method{daily2annual}{default}(x, FUN, na.rm=TRUE, na.rm.max=0, 
        out.fmt=c("\%Y", "\%Y-\%m-\%d"), start.month=1, ...)

\method{daily2annual}{zoo}(x, FUN, na.rm=TRUE, na.rm.max=0, 
        out.fmt=c("\%Y", "\%Y-\%m-\%d"), start.month=1, ...) 

\method{daily2annual}{data.frame}(x, FUN, na.rm=TRUE, na.rm.max=0, 
        out.fmt=c("\%Y", "\%Y-\%m-\%d"), start.month=1, dates=1, 
        date.fmt = "\%Y-\%m-\%d", out.type = "data.frame", verbose = TRUE, ...)

\method{daily2annual}{matrix}(x, FUN, na.rm = TRUE, na.rm.max=0, 
        out.fmt=c("\%Y", "\%Y-\%m-\%d"), start.month=1, 
        dates=1, date.fmt = "\%Y-\%m-\%d", out.type = "data.frame", 
        verbose = TRUE,  ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with (sub)daily/monthly time series. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represents the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that have to be applied for aggregating from (sub)daily/monthly into annual time step (e.g., for precipitation \code{FUN=sum} and for temperature and streamflows ts \code{FUN=mean}). \cr

\code{FUN} MUST accept the \code{na.rm} argument, because \code{na.rm} is passed to \code{FUN}. \cr

When \code{FUN=max} or \code{FUN=min} the date(time) where the maximum/minimum value actually occurs is returned in the output object, otherwise, a generic 1st of january for each year is returned.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the annual values are computed only for years with a percentage of missing values less than \code{na.rm.max} \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a year, the corresponing annual values in the output object will be \code{NA}.
}
  \item{na.rm.max}{
Numeric in [0, 1]. It is used to define the maximum percentage of missing values allowed in each year to keep the yearly aggregated value in the output object of this function. In other words, if the percentage of missing values in a given year is larger than \code{na.rm.max} the corresponding annual value will be \code{NA}.
}
  \item{out.fmt}{
Character indicating the date format for the output zoo object. See \code{format} in \code{\link[base]{as.Date}}. Possible values are: \cr
-) \kbd{\%Y}        : only the year will be used for the time. Default option. (e.g., "1961" "1962"...) \cr
-) \kbd{\%Y-\%m-\%d}: a complete date format will be used for the time. (e.g., "1961-01-01" "1962-01-01"...). See Details.
}
  \item{start.month}{
 numeric in [1,..,12], representing the starting month (1:Jan, ..., 12:Dec) to be used in the computation of annual values. By default \code{start.month=1}.
 }
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates for corresponding to each gauging station \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, using the date format specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days on it be equal to the number of element in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
Character that defines the desired type of output. Valid values are: \cr
-) \kbd{data.frame}: a data.frame, with as many columns as stations are included in \code{x}, and row names indicating the Year \cr
-) \kbd{db}        : a data.frame, with 3 columns will be produced. \cr
                     The first column (StationID) will store the ID of the station \cr
                     The second column (Year) will store the year, \cr
                     The third column (Value) will contain the annual value corresponding to the two previous columns.
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
arguments additional to \code{na.rm} passed to \code{FUN}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
When \code{FUN!=max} and \code{FUN!=min} the output is a zoo object with annual time frequency, where the time attribute has the format defined in \code{out.fmt}. \cr
When \code{FUN!=max} and \code{FUN!=min} and \code{out.fmt="\%Y-\%m-\%d"} the time attribute of the output zoo object will use the 1st of January of each year to create a full Date object from the corresponding year of each element of the output object (e.g., fi the year is \kbd{2022}, the time attribute will be \kbd{2022-01-01}). The only exception occurrs when \code{FUN=max} or \code{FUN=min}, where the time attribute of each element will correspond to the actual date where the annual maximum/minimum occurs (which is very useful for identifying the date of the annual maximum or the annual minimum of a time series). \cr

When \code{FUN=max} or \code{FUN=min} and \code{x} is a single time series, the output is a zoo object with annual time frequency, where the time attribute has the same class than \code{time(x)}, and the date(time) value corresponds to the date(time) where the maximum/minimum value actually occurs. \cr
When \code{FUN=max} or \code{FUN=min} and \code{x} has two or more time series, the output is a list object where each element has an annual time frequency. The time attribute of each list element has the same class than \code{time(x)}, and the date(time) value of each list element corresponds to the date(time) where the maximum/minimum value actually occurs. \cr
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{subhourly2hourly}}, \code{\link{daily2monthly}}, \code{\link{monthly2annual}}, \code{\link{hydroplot}}, \code{\link{annualfunction}}, \code{\link{vector2zoo}}, \code{\link{as.Date}}
}
\examples{
######################
## Ex1: Computation of annual values, removing any missing value in 'x'

# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting 'x' to its first three months (Jan/1921 - Mar/1921)
x <- window(x, end="1921-03-31")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Agreggating from Daily to Annual, removing any missing value in 'x'
( a <- daily2annual(x, FUN=sum, na.rm=TRUE) )

######################
## Ex2: Compuation of annual values only when the percentage of NAs in each
#       year is lower than a user-defined percentage (10% in this example).

# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting 'x' to its first three months (Jan/1921 - Mar/1921)
x <- window(x, end="1921-12-31")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Daily to annual, only for months with less than 10% of missing values
( a2 <- daily2annual(x, FUN=sum, na.rm=TRUE, na.rm.max=0.1) )

# Verifying that the second and third month of 'x' had 10% or more of missing values
cmv(x, tscale="annual")


######################
## Ex3: Getting the annual maximum value, including the date where this annual 
##      maximum actually occurs
daily2annual(x, FUN=max)


######################
## Ex4: Monthly to Annual, allowing a maximum of 20% of missing values in each month 
m <- daily2monthly(x, FUN=sum, na.rm=TRUE, na.rm.max=0.2)
monthly2annual(m, FUN=sum, na.rm=TRUE)


######################
## Ex5: Loading the time series of HOURLY streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

# Sub-daily to monthly ts
subdaily2annual(x, FUN=mean, na.rm=TRUE)

############
## Ex6: Loading the monthly time series of precipitation within the Ebro River basin
data(EbroPPtsMonthly)

# computing the annual values for the first 10 gauging stations in 'EbroPPtsMonthly'
a <- monthly2annual(EbroPPtsMonthly[,1:11], FUN=sum, dates=1)

# same as before, but with a nicer format of years
a <- monthly2annual(EbroPPtsMonthly[,1:11], FUN=sum, dates=1, out.fmt="\%Y")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File rm1stchar.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{rm1stchar}
\Rdversion{1.1}
\alias{rm1stchar}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Remove First Character(s) 
}
\description{
Deletes the first \code{n} character(s) of a character object.
}
\usage{
rm1stchar(x, n = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
Character, e.g, each element may represent the name of a single gauging station.
}
  \item{n}{
numeric, indicating the number of characters that have to be removed from the beginning of \code{x}
}
}
%%\details{

%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
character object of the same length as \code{x}.
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{substr}}
}
\examples{
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

# Getting the name of each gauging station.
names <- colnames(EbroPPtsMonthly)

# Removing the initial letter 'P' of the name of each gauging station.
rm1stchar(names)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File hydroTSM-package.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2024 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{hydroTSM-package}
\Rdversion{1.1}
\alias{hydroTSM-package}
\alias{hydroTSM}
\docType{package}
\title{
Management, analysis, and plot of hydrological time series, with focus on hydrological modelling
}
\description{
S3 functions for management, analysis, interpolation and plotting of time series used in hydrology and related environmental sciences. In particular, this package is highly oriented to hydrological modelling tasks. 
The focus of this package has been put in providing a collection of tools useful for the daily work of hydrologists (although an effort was made to optimise each function as much as possible, functionality has had priority over speed). Bugs / comments / questions / collaboration of any kind are very welcomed, and in particular, datasets that can be included in this package for academic purposes.

}
\details{
\tabular{ll}{
Package: \tab hydroTSM\cr
Type: \tab Package\cr
Version: \tab 0.8-0\cr
Date: \tab 2026-04-27\cr
License: \tab GPL >= 2 \cr
LazyLoad: \tab yes\cr
Packaged: \tab Mon Apr 27 08:57:54 -04 2026 ; MZB \cr
BuiltUnder: \tab R version 4.6.0 (2026-04-24) -- "Because it was There" ; aarch64-apple-darwin23 \cr
}

--------------------------------------------------------------------------------------------------------------------------- \cr
Datasets:

  \tabular{ll}{
    \code{\link{Cauquenes7336001}}  Hydrometeorological time series for "Cauquenes en El Arrayan" catchment.\cr
    \code{\link{EbroPPtsMonthly}}   Ebro Monthly Precipitation Time Series. \cr
    \code{\link{KarameaAtGorgeQts}} Karamea at Gorge, time series of hourly streamflows. \cr
    \code{\link{MaquehueTemuco}}    San Martino, ts of daily precipitation. \cr
    \code{\link{OcaEnOnaQts}}       Oca in "Ona" (Q0931), time series of daily streamflows \cr
    \code{\link{SanMartinoPPts}}    San Martino, ts of daily precipitation. \cr
--------------------------------------------------------------------------------------------------------------------------- \cr
}

Temporal aggregation: 

  \tabular{ll}{
    \code{\link{annualfunction}}    single representative annual value of a zoo object.\cr
    \code{\link{weeklyfunction}}    single representative weekly value of a zoo object. \cr
    \code{\link{monthlyfunction}}   single representative monthly value of a zoo object. \cr
    \code{\link{seasonalfunction}}  representative values of each weather season of a zoo object. \cr
    \code{\link{daily2annual}}      Aggregation from daily to annual \cr
    \code{\link{subdaily2annual}}   Aggregation from subdaily to annual. \cr
    \code{\link{monthly2annual}}    Aggregation from monthly to annual. \cr
    \code{\link{daily2monthly}}     Aggregation from daily to monthly. \cr
    \code{\link{subdaily2monthly}}  Aggregation from subdaily to monthly. \cr
    \code{\link{daily2weekly}}      Aggregation from daily to weekly. \cr
    \code{\link{dm2seasonal}}       Aggregation from daily or monthly to seasonal. \cr
    \code{\link{subdaily2seasonal}} Aggregation from subdaily to seasonal. \cr
    \code{\link{subdaily2daily}}    Aggregation from subdaily to daily. \cr
    \code{\link{subdaily2weekly}}   Aggregation from subdaily to weekly. \cr
    \code{\link{subhourly2hourly}}  Aggregation from subhourly to hourly. \cr
    \code{\link{subhourly2nminutes}}Aggregation from subhourly to n-minutes. \cr
--------------------------------------------------------------------------------------------------------------------------- \cr
}

Temporal manipulation: 

  \tabular{ll}{
    \code{\link{dip}}         Days in period.\cr
    \code{\link{diy}}         Days in year. \cr
    \code{\link{hip}}         Hours in period. \cr
    \code{\link{mip}}         Months in period. \cr
    \code{\link{yip}}         Years in period. \cr
    \code{\link{izoo2rzoo}}   Irregular zoo object to regular zoo objectl. \cr
    \code{\link{time2season}} Time to weather season. \cr
    \code{\link{vector2zoo}}  Numeric and date/times to zoo object. \cr
--------------------------------------------------------------------------------------------------------------------------- \cr
}

Hydrological functions: 

  \tabular{ll}{
    \code{\link{baseflow}}   Baseflow computation.\cr
    \code{\link{climograph}} Climograph \cr
    \code{\link{dwdays}}     Dry and wet days. \cr
    \code{\link{fdc}}        Flow duration curve. \cr
    \code{\link{fdcu}}       Flow duration curve with uncertainty bounds. \cr
    \code{\link{hydroplot}}  Exploratory figure for hydrological time series. \cr
    \code{\link{sname2plot}} Hydrological time series plotting and extraction. \cr
    \code{\link{plot_pq}}    Plot precipitation and streamflow time series in the same figure. \cr
    \code{\link{si}}         Seasonality Index for precipitation. \cr
    \code{\link{sname2ts}}   Station Name -> Time Series. \cr
    \code{\link{zoo2RHtest}} Zoo object -> RHTest. \cr
--------------------------------------------------------------------------------------------------------------------------- \cr
}

Miscelaneous functions:

  \tabular{ll}{
    \code{\link{calendarHeatmap}} Calendar heat map.\cr
    \code{\link{cmv}}             Counting missing values. \cr
    \code{\link{drawxaxis}}       Draw a temporal horizontal axis. \cr
    \code{\link{dwi}}             Days with information. \cr
    \code{\link{extract}}         Extract a subset of a zoo object. \cr
    \code{\link{hydropairs}}      Visual correlation matrix. \cr
    \code{\link{infillxy}}        Infills NA values. \cr
    \code{\link{istdx}}           Inverse standarization. \cr
    \code{\link{ma}}              Moving average. \cr
    \code{\link{matrixplot}}      2D color matrix. \cr
    \code{\link{rm1stchar}}       Remove first character. \cr
    \code{\link{sfreq}}           Sampling frequency. \cr
    \code{\link{smry}}            Improved summary function. \cr
    \code{\link{stdx}}            Standarization. \cr
--------------------------------------------------------------------------------------------------------------------------- \cr
}

}
\author{
Mauricio Zambrano-Bigiarini

Maintainer: Mauricio Zambrano-Bigiarini <mzb.devel@gmail>
}
%%\references{
%%~~ Literature or other references for background information ~~
%%}

\keyword{ package }
\seealso{
\url{https://github.com/hzambran/hydroGOF}. \cr
\url{https://github.com/hzambran/hydroPSO}. \cr
\url{https://gitlab.com/rmarinao/hydroMOPSO}. \cr
}
\examples{
## Loading the monthly time series (10 years) of precipitation for the Ebro River basin.
data(EbroPPtsMonthly)

#######
## Ex1) Graphical correlation among the ts of monthly precipitation of the first 
##      3 stations in 'EbroPPtsMonthly' (its first column stores the dates).
hydropairs(EbroPPtsMonthly[,2:4])

#######
## Ex2) Annual values of precipitation at the station "P9001"
sname2ts(EbroPPtsMonthly, sname="P9001", dates=1, var.type="Precipitation", 
         tstep.out="annual")

#######
## Ex3) Monthly and annual plots
sname2plot(EbroPPtsMonthly, sname="P9001", var.type="Precipitation", pfreq="ma")


#######
## Ex5)  Mean monthly values of streamflows
## Loading daily streamflows (3 years) at the station 
## Oca en Ona (Ebro River basin, Spain)
data(OcaEnOnaQts)
monthlyfunction(OcaEnOnaQts, FUN=mean, na.rm=TRUE)

}

%%File subhourly2nminutes.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2022-2022 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{subhourly2nminutes}
\Rdversion{1.1}
\alias{subhourly2nminutes}
\alias{subhourly2nminutes.default}
\alias{subhourly2nminutes.zoo}
\alias{subhourly2nminutes.data.frame}
\alias{subhourly2nminutes.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Sub-hourly -> n-minutes
}
\description{
Generic function for aggregating a sub-hourly time series into a "n-minutes' one.
}
\usage{
subhourly2nminutes(x, ...)

\method{subhourly2nminutes}{default}(x, nminutes, FUN, na.rm=TRUE, 
                                     from=start(x), to=end(x), ...)

\method{subhourly2nminutes}{zoo}(x, nminutes, FUN, na.rm=TRUE, 
                                 from=start(x), to=end(x), tz, ...)

\method{subhourly2nminutes}{data.frame}(x, nminutes, FUN, na.rm=TRUE, 
        from=start(x), to=end(x), dates=1, date.fmt="\%Y-\%m-\%d \%H:\%M:\%S", 
        out.fmt="zoo", verbose= TRUE, ...)

\method{subhourly2nminutes}{matrix}(x, nminutes, FUN, na.rm=TRUE, 
        from=start(x), to=end(x), dates=1, date.fmt="\%Y-\%m-\%d \%H:\%M:\%S", 
        out.fmt="zoo", verbose= TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with sub-hourly time series. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} represent the ID of each station.
}
  \item{nminutes}{
numeric, defining the amount of minutes to be used for aggregating \code{x}.
\code{nminutes} must be larger than the amount of minutes between each \code{x} value (computed as \code{time(x)[2]-time(x)[1]}).
}
  \item{FUN}{
Function that have to be applied for aggregating from sub-hourly into n-minutes time step. (e.g., for precipitation \code{FUN=sum} and for temperature and streamflow ts, \code{FUN=mean}). \cr
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the hourly values  are computed considering only those values different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA sub-hourly value within a day, the corresponding hourly value(s) will be NA as well
}
  \item{from}{
POSIX object indicating the starting time used to carry out the temporal aggregation.  \cr
When \code{from > start(x)} then \code{x} is cut in time to the starting DateTime defined by \code{from}. \cr
When \code{from < start(x)} then \code{x} is extended backward with NAs to the starting DateTime defined by \code{from}.
}
  \item{to}{
POSIX object indicating the ending time used to carry out the temporal aggregation.  \cr
When \code{to < end(x)} then \code{x} is cut in time to the ending DateTime defined by \code{to}. \cr
When \code{to > end(x)} then \code{x} is extended forward with NAs to the ending DateTime defined by \code{from}.
}
  \item{tz}{
character, with the specification of the time zone used for \code{x}, \code{from}, and \code{to}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr

If \code{tz} is provided, it forces \code{time(x)} to be in the tome zone specified by \code{tz}, without modifying the the values (hours, minutes, seconds, etc). \cr

A list of valid time zones can be obtained by calling the base function \code{OlsonNames()}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{from} and \code{to}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{dates}{
numeric, factor, POSIXct or POSIXt object indicating how to obtain the dates and times for each column of \code{x} (e.g., gauging station) \cr
If \code{dates} is a number, it indicates the index of the column in \code{x} that stores the date and times \cr
If \code{dates} is a factor, it is converted into POSIXct class, using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of POSIXct or POSIXt class, the code verifies that the number of elements on it be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{dates}, By default \code{date.fmt=\%Y-\%m-\%d \%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.fmt}{
OPTIONAL. Only used when \code{x} is a matrix or data.frame object /cr
character, for selecting if the result will be a matrix/data.frame or a zoo object. Valid values are: \kbd{numeric}, \kbd{zoo} (default)
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
  a zoo object with hourly time series
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{subhourly2hourly}}, \code{\link{subdaily2daily}}, \code{\link{subdaily2monthly}}, \code{\link{subdaily2annual}}, \code{\link{subdaily2seasonal}}, \code{\link{as.POSIXct}}, \code{\link{dm2seasonal}}, \code{\link{monthlyfunction}}, \code{\link{seasonalfunction}}, \code{\link{hydroplot}}, \code{\link{vector2zoo}}, \code{\link{izoo2rzoo}}
}
\examples{
## Creating a 5-min time sequence and counting its length
dt  <- seq( from=as.POSIXct("2021-06-30 00:00"), to=as.POSIXct("2021-06-30 23:55"), by="5 min" )
ndt <- length(dt)

## Creating a dummy 5-min zoo object, with 1 as the only value in each time period
x <- zoo( rep(1, ndt), dt)


## Aggregation from 5-minute single ts into 10-minute ts
h1 <- subhourly2nminutes(x, nminutes= 10, FUN=sum, na.rm=TRUE)

## Aggregation of 3 ts with 5-minute time frequency (i.e., a zoo matrix) 
## into a 30-minute zoo object.
X  <- cbind(x, x, x)
h2 <- subhourly2nminutes(X, nminutes= 30, FUN=sum, na.rm=TRUE)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%%File KarameaAtGorgeQts.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{KarameaAtGorgeQts}
\docType{data}
\alias{KarameaAtGorgeQts}
\title{
Karamea at Gorge, time series of hourly streamflows
}
\description{
Time series with hourly streamflows for the Karamea River(New Zealand) measured at the gauging station "Gorge", for the period 01/Jan/1980 to 31/Dec/1985.

Station Number: 95102, Easting Coordinate (NZMG): 2444629.0, Northing Coordinate (NZMG): 5994427.0, Catchment Area (km2): 1160.0.

In November 25th, 2023 (for hydroTSM v0.7-0), the time zone of this data sete was changed from "none" (i.e., your local time zone was used every time you loaded this dataset)
to GMT+12, in order to avoid missing datetimes at times where daylight saving time ocurred. 
However, due to CRAN requirements, in January 2024 (for hydroTSM v0.7-1), the time zone of this data sete was changed from "GMT+12" to UTM, in order to avoid missing datetimes at times where daylight saving time ocurred. 
After the previous change, the initial DateTime of this dataset changed 
from "1980-01-01 08:15:00 -03" (in my local computer) to "1979-12-31 20:15:00 UTC" (everywhere). 

Only two NA elements were removed from the original dataset: the first one and the last one.
}
\usage{
data(KarameaAtGorgeQts)
}

%%\details{
%%
%%}
\format{
zoo object.
}
\source{
Provided by the National Institute of Water and Atmospheric Research \url{https://niwa.co.nz/}, thanks to the gentle collaboration of Shailesh Singh \cr

These data are intended to be used for research purposes only, being distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
}

%%\references{
%% ~put references to the literature/web site here ~
%%}
\keyword{datasets}

%%File stdx.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{stdx}
\Rdversion{1.1}
\alias{stdx}
\alias{stdx.default}
\alias{stdx.matrix}
\alias{stdx.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Standarization
}
\description{
Standarizes a vector or matrix, i.e., scales all the values in a way that the transformed values will be within the range [0, 1]. 
}
\usage{
stdx(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
vector, matrix or data.frame to be scaled
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
\details{

\deqn{z = \frac{x - x_{min}}{x_{max}-x_{min}}}{%
z = scale(x) = [ x - xmin ] / [ xmax - xmin ] }
}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{scale}}, \code{\link{istdx}}
}
\examples{
############
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

# Standarizing only some values of 'EbroPPtsMonthly'
stdx(as.matrix(EbroPPtsMonthly[1:70,10:13]))
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File hydropairs.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2012 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{hydropairs}
\Rdversion{1.1}
\alias{hydropairs}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Visual Correlation Matrix
}
\description{
Visualization of a correlation matrix. On top the (absolute) value of the correlation plus the result of the \code{\link[stats]{cor.test}} as stars. On bottom, the bivariate scatterplots, with a fitted line. On the diagonal, an histogram of each variable.
}
\usage{
hydropairs(x, dec = 3, use = "pairwise.complete.obs", method = "pearson",...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
data.frame or matrix object with measurements at several locations. Each column of \code{x} represent values measured at different locations.
}
  \item{dec}{
decimal places to be used for showing the correlation values
}
  \item{use}{
See \code{\link[stats]{cor}}. An optional character string giving a method for computing covariances in the presence of missing values. This must be (an abbreviation of) one of the strings "everything", "all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs".
}
  \item{method}{
See \code{\link[stats]{cor}}. A character string indicating which correlation coefficient (or covariance) is to be computed. One of "pearson" (default), "kendall", or "spearman", can be abbreviated
}
  \item{\dots}{
further arguments passed to or from other methods, in particular it is used in the \code{\link[graphics]{pairs}} function.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
\item{On top}{ the (absolute) value of the correlation plus the result of the cor.test as points}
\item{On bottom}{the bivariate scatterplots, with a fitted line}
\item{On diagonal}{histograms (from \code{\link[graphics]{pairs}})}
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
Original idea taken from the R Graph Gallery (nowadays not available on its original link: http://addictedtor.free.fr/graphiques/graphcode.php?graph=137). \cr

Histogram panel was taken from the R help of the original \code{\link[graphics]{pairs}} function
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{cor}}, \code{\link{pairs}}
}
\examples{
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

## Visualizing the correlation among the monthly precipitation values 
## of the first 3 gauging stations in 'EbroPPtsMonthly'. 
## The first column of 'EbroPPtsMonthly' has the dates.
hydropairs(EbroPPtsMonthly[,2:4])
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{math}
\keyword{graphs}

%%File weeklyfunction.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2023-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{weeklyfunction}
\Rdversion{1.1}
\alias{weeklyfunction}
\alias{weeklyfunction.zoo}
\alias{weeklyfunction.default}
\alias{weeklyfunction.matrix}
\alias{weeklyfunction.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Weekly Function
}
\description{
Generic function for obtaining 52 weekly values of a zoo object, by applying any R function to ALL the values in the object belonging to each one of the 52 calendar weeks (starting on Monday).
}
\usage{
weeklyfunction(x, ...)

\method{weeklyfunction}{default}(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                                 start.fmt= "\%H:\%M:\%S", tz, ...)

\method{weeklyfunction}{zoo}(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                             start.fmt= "\%H:\%M:\%S", tz, ...)

\method{weeklyfunction}{data.frame}(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                        start.fmt= "\%H:\%M:\%S", tz, dates=1, date.fmt="\%Y-\%m-\%d",
                        out.type="data.frame", verbose=TRUE,...)
             
\method{weeklyfunction}{matrix}(x, FUN, na.rm=TRUE, na.rm.max=0, start="00:00:00", 
                        start.fmt= "\%H:\%M:\%S", tz, dates=1, date.fmt="\%Y-\%m-\%d",
                        out.type="data.frame", verbose=TRUE,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, xts, data.frame or matrix object, with daily or monthly time series. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that will be applied to ALL the values in \code{x} belonging to each one of the 12 months of the year (e.g., FUN can be some of \code{mean}, \code{sum}, \code{max}, \code{min}, \code{sd}).
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the monthly values and FUN are computed considering only those values in \code{x} different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the corresponding monthly value will be NA 
}
  \item{na.rm.max}{
Numeric in [0, 1]. It is used to define the maximum percentage of missing values allowed in each month to keep the weekly aggregated value in the output object of this function. In other words, if the percentage of missing values in a given month is larger or equal than \code{na.rm.max} the corresponding weekly value will be \code{NA}.
}
  \item{start}{
character, indicating the starting time used for aggregating sub-daily time series into daily ones. It MUST be provided in the format specified by \code{start.fmt}. \cr
This value is used to define the time when a new day begins (e.g., for some rain gauge stations). \cr
-) All the values of \code{x} with a time attribute before \code{start} are considered as belonging to the day before the one indicated in the time attribute of those values. \cr
-) All the values of \code{x} with a time attribute equal to \code{start} are considered to be equal to \code{"00:00:00"} in the output zoo object. \cr
-) All the values of \code{x} with a time attribute after \code{start} are considered as belonging to the same day as the one indicated in the time attribute of those values. \cr

It is useful when the daily values start at a time different from \code{"00:00:00"}. Use with caution. See examples.
}
  \item{start.fmt}{
character indicating the format in which the time is provided in \code{start}, By default \code{date.fmt=\%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.POSIXct}}.
}
  \item{tz}{
character, with the specification of the time zone used in both \code{x} and \code{start}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr
This argument can be used to force using the local time zone or any other time zone instead of UTC as time zone.
}
  \item{dates}{
It is only used when \code{x} is not a zoo object. \cr
numeric, factor, Date indicating how to obtain the dates. \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into 'Date' class, using the date format specified by \code{date.fmt} \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
It is only used when \code{x} is not a zoo object. \cr
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
It is only used when \code{x} is a matrix or data.frame. \cr
Character defining the desired type of output. Valid values are: \cr
-) \kbd{data.frame}: a data.frame, with 12 columns representing the months, and as many rows as gauging stations are included in \code{x} \cr
-) \kbd{db}        : a data.frame, with 4 columns will be produced. Useful for a posterior boxplot \cr
                 The first column ('StationID') will store the ID of the station,  \cr
                 The second column ('Year') will store the year, \cr
                 The third column ('Month') will store month,  \cr
                 The fourth column ('Value') will contain the monthly value corresponding to the three previous columns.
}
  \item{verbose}{
Logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
When \code{x} is a zoo object, a numeric vector with 12 elements representing the computed monthly value for each month. \cr
When \code{x} is a data.frame which columns represent measurements at different gauging stations, the resulting object is a data.frame with 12 columns and as many rows as gauging stations are in \code{x}, each row storing the computed 12 monthly value for each gauging station.
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
\note{
Due to the fact that \code{FUN} is applied over all the elements in \code{x} belonging to a given calendar month, its result will depend on the sampling frequency of \code{x} and the type of function provided by \code{FUN} (\bold{special attention have to be put when \code{FUN=sum}})
}

\seealso{
\code{\link{annualfunction}}, \code{\link{seasonalfunction}}, \code{\link{dm2seasonal}}, \code{\link{daily2monthly}}, \code{\link{daily2annual}}
}
\examples{
## Ex1: Computation of mean WEEKLY values from DAILY ts, removing any missing value in 'x'

# Loading DAILY streamflows (3 years) at the station 
# Oca en Ona (Ebro River basin, Spain)
data(OcaEnOnaQts)
x <- OcaEnOnaQts

## Mean WEEKLY streamflows at station 'x'
weeklyfunction(x, FUN=mean, na.rm=TRUE)

######################
## Ex2: Computation of mean WEEKLY values from HOURLY ts, removing any missing value in 'x'

# Loading HOURLY streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

## Mean WEEKLY streamflows at station 'x'. Each day starts at 00:00:00
weeklyfunction(x, FUN=mean, na.rm=TRUE)

######################
## Ex3: Computation of mean WEEKLY values from HOURLY ts, removing any missing value in 'x'
##      and starting each day at 08:00:00

# Loading HOURLY streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

## Mean WEEKLY streamflows at station 'x'. Each day starts at 00:00:00
weeklyfunction(x, FUN=mean, na.rm=TRUE, start="00:00:00")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File izoo2rzoo.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2026 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{izoo2rzoo}
\Rdversion{1.1}
\alias{izoo2rzoo}
\alias{izoo2rzoo.default}
\alias{izoo2rzoo.zoo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
 Irregular Zoo -> Regular Zoo
}
\description{
It takes an irregular zoo object (with non-existing values for some dates/times) and converts it into a regularly spaced zoo object within the time period defined by \code{from} and \code{to}, by filling the missing dates with \sQuote{NA}
}
\usage{
izoo2rzoo(x, ...)

\method{izoo2rzoo}{default}(x, from= start(x), to= end(x), 
                   date.fmt, tstep, tz, 
                   na.action=c("keep", "linear", "spline"), ... )
     
\method{izoo2rzoo}{zoo}(x, from= start(x), to= end(x), 
                   date.fmt, tstep, tz, 
                   na.action=c("keep", "linear", "spline"), ... )

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
irregular zoo object (vector or matrix) representing a time series (very likely read with some user-defined procedure, and with some missing values for particular days/months/years)
}
  \item{from}{
Character indicating the starting date for creating the regularly spaced zoo object. The default value corresponds to the date of the first element of \code{x} \cr
It has to be in the format indicated by \code{date.fmt}. 
}
  \item{to}{
Character indicating the ending date for creating the regularly spaced zoo object. The default value corresponds to the date of the last element of \code{x} \cr
It has to be in the format indicated by \code{date.fmt}.
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. See \sQuote{Details} section in \code{\link[base]{strptime}}.
By default, \code{date.fmt} is missing, and it is automatically set to \kbd{\%Y-\%m-\%d} when \code{time(x)} is \code{Date} object, and set to \kbd{\%Y-\%m-\%d \%H:\%M:\%S} when \code{x} is a sub-daily zoo object.
}
  \item{tstep}{
character, indicating the time step used for creating the time sequence going from \code{from} to \code{to} that will be used as \code{time(x)} \cr
Valid values are (but not limited to) \kbd{hours}, \kbd{days}, \kbd{months}, \kbd{years}.
By default, \code{tstep} is missing, and it is automatically set to \kbd{"minutes"} when \code{sfreq(x)} is \kbd{min}, to \kbd{"hours"} when \code{sfreq(x)} is \kbd{hourly}, to \kbd{"days"} when \code{sfreq(x)} is \kbd{daily}, to \kbd{"weeks"} when \code{sfreq(x)} is \kbd{weekly}, to \kbd{"months"} when \code{sfreq(x)} is \kbd{monthly}, to \kbd{"quarters"} when \code{sfreq(x)} is \kbd{quarterly}, and to \kbd{"years"} when \code{sfreq(x)} is \kbd{annual}.
}
  \item{tz}{
character, with the specification of the time zone used for \code{x}, \code{from}, and \code{to}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr

If \code{tz} is provided, it forces \code{time(x)} to be in the tome zone specified by \code{tz}, without modifying the the values (hours, minutes, seconds, etc). \cr

A list of valid time zones can be obtained by calling the base function \code{OlsonNames()}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{from} and \code{to}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{na.action}{
character indicating whether to keep 'NA' values in \code{x} (default) or interpolate them with linear or spline interpolation using the \code{\link[zoo]{na.approx}} or the \code{\link[zoo]{na.spline}} function of the zoo package.
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
\details{
If the full time period of \code{x} is a subset of the time period defined by \code{from} and \code{to}, the time period of the resulting zoo is the one defined by \code{from} and \code{to}, assigning 'NA' to all the dates in which \code{x} does not have a value.
}
\value{
a regularly spaced zoo object, with values given by \code{x} and time stamps going from \code{from} to \code{to} at intervals defined by \code{tsteps}.
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[zoo]{zoo}}, \code{\link{vector2zoo}}, \code{\link[base]{as.POSIXct}}, \code{\link[base]{Sys.timezone}}
}

\examples{
##
## Example 1: Adding NA for February 29th to an existing zoo object

# dummy values and dates (February 29th is not present !)
x <- 1:9
dates <- c("1964-02-25", "1964-02-26", "1964-02-27", "1964-02-28", "1964-03-01", 
           "1964-03-02", "1964-03-03", "1964-03-04", "1964-03-05")

# From 'character' to 'Date' class
dates <- as.Date(dates)

## From 'numeric' to 'zoo' class
( x <- zoo(x, dates) ) # Feb 29th is still not present in 'x'
## checking the length of 'x'
length(x) # 9 elements (there is no data for Feb 29th)

## Adding a missing value (NA in this case) for Feb 29th
( y <- izoo2rzoo(x) )

## checking the new length
length(y) # 1 element more than the original 'x' (thre is an NA value in Feb 29th)


##
## Example 2: Extending the original 'x' object from February 1st to the end of March, 
#             assigning 'NA' to the days in which 'x' do not have a value.
( y <- izoo2rzoo(x, from="1964-02-01", to="1964-03-31") )


##
## Example 3: Working with a zoo matrix with two identical 'x' time series, 
##            from 1964-02-25 to 1964-03-05
( Y <- cbind(x,x) )

# Adding a missing value (NA in this case) for Feb 29th in all the columns of Y
( rY <- izoo2rzoo(Y) )


##
## Example 4: Working with hourly data, from 01:00 to 10:00 UTC on 12th December 2000
dates  <- ISOdatetime(year=2000, month=12, day=12, hour=1:10, min=0, sec=0, tz="UTC")
values <- 1:10
x      <- zoo(values, dates)

# removing four values in 'x', from 02:00 to 05:00, i.e., they will not be present 
# anymore in 'x' at all, not even NA !)
x <- x[-c(2:5)]
time(x)
length(x)

# Adding missing values (NA in this case) from 02:00 to 05:00
y <-  izoo2rzoo(x)
time(y)
length(y)


##
## Example 5: Extending hourly data to a DateTime before 'start(x)', 
##            specifying only the date.
##            Time of 'x' is in local time zone (tz="") instead of UTC
dt <- hip("2021-01-01 00:00:00", "2021-01-01 20:00:00", tz="")
x  <- zoo(0:20, dt)
(y  <- izoo2rzoo(x, from="2020-12-31") )# 00:00:00 is ommited
(time(y))


##
## Example 6: Extending hourly data to a DateTime before 'start(x)', 
##            specifying date and time.
##            Time of 'x' is in local time zone (tz="") instead of UTC
dt <- hip("2021-01-01 00:00:00", "2021-01-01 20:00:00", tz="")
x  <- zoo(0:20, dt)
( y  <- izoo2rzoo(x, from="2020-12-31 20:00:00") )


##
## Example 7: Extending hourly data to a DateTime before 'start(x)', 
##            specifying date and time, and forcing UTC to be the time zone.
##            Time of 'x' is in local time zone (tz="") instead of UTC, but
##            it will be treated as UTC by using the 'tz' argument
dt <- hip("2021-01-01 00:00:00", "2021-01-01 20:00:00", tz="")
x  <- zoo(0:20, dt)
(time(x))
(y  <- izoo2rzoo(x, from="2020-12-31 20:00:00", tz="UTC") )# 00:00:00 is ommited
(time(y))

##
## Example 8: Extending hourly data to a DateTime after 'end(x)', 
##            specifying date and time.
##            Time of 'x' is in local time zone (tz="") instead of UTC
dt <- hip("2021-01-01 00:00:00", "2021-01-01 20:00:00", tz="")
x  <- zoo(0:20, dt)
( y  <- izoo2rzoo(x, to="2021-01-02 12:00:00") )


##
## Example 9: Extending hourly data to a DateTime before 'start(x)'.
##            Note that the 'tz' argument can be ommited in the 'hip' function, 
##            because by default it assumes UTC as time zone
dt <- hip("2021-01-01 00:00:00", "2021-01-01 20:00:00", tz="UTC")
x  <- zoo(0:20, dt)
( y  <- izoo2rzoo(x, from="2020-12-31 20:00:00", tz="UTC") )


##
## Example 10: Extending hourly data to a date before 'start(x)'. However, hourly 'x'
##            values are given at HH:15:00 hours instead of HH:00:00 hours.

## Loading the time series of hourly streamflows for the station Karamea at Gorge
## Time Zone for 'KarameaAtGorgeQts' data is 'UTC' (see ?KarameaAtGorgeQts), but it will
## be tr4ated as 'Pacific/Auckland' (Zealand Standard Time) for this example
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

# Subsetting 'x' to its first day only
# (01/Jan/1980 08:15:00 - 01/Jan/1980 23:15:00)
x <- window(x, end="1980-01-01 23:59:00")

# Adding NA hourly data since 1979-12-31 21:15:00
izoo2rzoo(x, from="1979-12-31 21:15:00", tz="Pacific/Auckland")
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%%File Cauquenes7336001.Rd
%% Part of the hydroPSO R package, https://github.com/hzambran/hydroTSM
%%                                 http://cran.r-project.org/web/packages/hydroTSM
%%                                 http://www.rforge.net/hydroTSM/
%% Copyright 2023-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{Cauquenes7336001}
\docType{data}
\alias{Cauquenes7336001}
\title{
Hydrometeorological time series for "Cauquenes en El Arrayan" catchment
}
\description{
Daily time series of precipitation, maximum air temperature, minimum air temperature, potential evapotranspiration, and observed streamflows for the catchment draining into the 'Cauquenes en El Arrayan' streamflow station (Cod.BNA: 7336001, Lat:-36.02, Lon:-72.38). This catchment is located in El Maule Region in Chile, with a pluvial regime, a total drainage area of 622.1 km2, and elevations ranging from 134 to 736 m a.s.l. Data were downloaded from the CAMELS-CL dataset (\url{https://camels.cr2.cl}) from 01/Jan/1979 to 31/Dec/2019 (including some missing values).
}
\usage{
data(Cauquenes7336001)
}

%%\details{
%%
%%}
\format{
zoo matrix with 5 time series: \cr
-) \var{P_mm}: Spatially-averaged mean daily values of precipitation computed based on the CR2met dataset, [mm/day]. \cr
-) \var{Tmx_degC}: Spatially-averaged maximum daily values of air temperature computed based on the CR2met dataset, [degree Celsius]. \cr
-) \var{Tmin_degC}: Spatially-averaged minimum daily values of air temperature computed based on the CR2met dataset, [degree Celsius]. \cr
-) \var{PET_mm}: Spatially-averaged mean daily values of precipitation computed based on the Hargreaves-Samani equation and daily maximum and minimum air temperatures obtained from the CR2met dataset, [mm/day]. \cr 
-) \var{Qobs_mm}: Daily sreamflows, [mm], measured at the "Cauquenes en El Arrayan" (7336001) station. \cr 
-) \var{Qobs_m3s}: Daily sreamflows, [m3/s], measured at the "Cauquenes en El Arrayan" (7336001) station. \cr 
}
\source{
Provided by Center for Climate and Resilience Research, Universidad de Chile, Santiago, Chile (\url{https://camels.cr2.cl/}, last accessed [Nov 2023]). \cr
These data are intended to be used for research purposes only, being distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
}

\references{
 \cite{Alvarez-Garreton, C., Mendoza, P. A., Boisier, J. P., Addor, N., Galleguillos, M., Zambrano-Bigiarini, M., Lara, A., Puelma, C., Cortes, G., Garreaud, R., McPhee, J., and Ayala, A (2018). The CAMELS-CL dataset: catchment attributes and meteorology for large sample studies-Chile dataset. Hydrology and Earth System Sciences, 22(11), 5817-5846. doi:10.5194/hess-22-5817-2018.}
}
\keyword{datasets}

%%File sname2ts.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{sname2ts}
\Rdversion{1.1}
\alias{sname2ts}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Station Name -> Time Series
}
\description{
This function takes a data.frame whose columns contains the time series of several gauging stations, along with a character representing the name of one gauging station, and extracts the time series corresponding to that station.
}
\usage{
sname2ts(x, sname, dates=1, date.fmt = "\%Y-\%m-\%d", var.type, 
         tstep.out = "daily", FUN, na.rm = TRUE, from, to)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
data.frame containing the complete times series of all the stations. \cr
It may also contain 1 column with the dates of the measurements, or they can be provided in a different way (see \code{dates} below).
}
  \item{sname}{
Character representing the name of a station, which have to correspond to one column name in \code{x}
}
  \item{dates}{
numeric, factor, Date object indicating how to obtain the dates corresponding to the \code{sname} station. \cr
-) If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
-) If \code{dates} is a factor, it is converted into Date class, using the date format specified by \code{date.fmt} \cr
-) If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of element in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{var.type}{
character representing the type of variable being plotted. Used for determining the function used for computing the monthly or/and annual values when \code{FUN} is missing. Valid values are: \cr
-) \kbd{Precipitation} => \code{FUN=sum}  \cr
-) \kbd{Temperature}   => \code{FUN= mean} \cr
-) \kbd{Flow}          => \code{FUN= mean}
}
  \item{tstep.out}{
character that defines the time step of the desired output time series. Valid values are: \cr
-) \kbd{daily}  : daily time series   \cr
-) \kbd{monthly}: monthly time series \cr
-) \kbd{annual} : annual time series 
}
  \item{FUN}{
ONLY required when \code{var.type} is missing and \code{tstep} is one of \code{monthly} or \code{annual}. \cr
Function that have to be applied for transforming from daily to monthly or annual time step (e.g., for precipitation \code{FUN=sum} and for temperature and flow ts, \code{FUN=mean}) 
}
  \item{na.rm}{
a logical value indicating whether 'NA' values should be stripped before the computation proceeds.
}
  \item{from}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the starting date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
  \item{to}{
OPTIONAL, used for extracting a subset of values. \cr
Character indicating the ending date for the values to be extracted. It must be provided in the format specified by \code{date.fmt}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
zoo object
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{sname2plot}}
}
\examples{
## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

## Annual values of temperature at the station "T9105", stored in 'EbroPPtsMonthly'.
sname2ts(EbroPPtsMonthly, sname="P9001", dates=1, FUN=sum, tstep.out="annual")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File drawTimeAxis.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2016 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{drawTimeAxis}
\Rdversion{1.1}
\alias{drawTimeAxis}
\alias{drawxaxis}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Customized Time Axis
}
\description{
For a nice time series plot, this function draws a customized time axis, with annual, monthly, daily and sub-daily time marks and labels.
}
\usage{
drawxaxis(x, tick.tstep = "auto", lab.tstep = "auto", 
          lab.fmt=NULL, cex.axis=1, mgp=c(3, 2, 0), ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
time series that will be plotted using the X axis that will be draw class(x) must be ts or zoo
}
  \item{tick.tstep}{
Character indicating the time step that have to be used for putting the ticks on the time axis. Valid values are: \kbd{auto}, \kbd{years}, \kbd{quarters}, \kbd{months},\kbd{weeks}, \kbd{days}, \kbd{hours}, \kbd{minutes}, \kbd{seconds}.
}
  \item{lab.tstep}{
Character indicating the time step that have to be used for putting the labels on the time axis. Valid values are: \kbd{auto}, \kbd{years}, \kbd{quarters}, \kbd{months},\kbd{weeks}, \kbd{days}, \kbd{hours}, \kbd{minutes}, \kbd{seconds}.
}
  \item{lab.fmt}{
Character indicating the format to be used for the label of the axis. See \code{format} in \code{\link[base]{as.Date}}. If not specified (\code{lab.fmt=NULL}), it will try to use: \cr
-) \kbd{"\%Y-\%m-\%d"} when \code{lab.tstep=="days"}, \cr
-) \kbd{"\%b-\%Y"} when \code{lab.tstep=="year"} or \code{lab.tstep=="month"}. 
}
  \item{cex.axis}{
magnification of axis annotation relative to \code{cex} (See \code{\link[graphics]{par}}).
}
  \item{mgp}{
The margin line (in \code{mex} units) for the axis title, axis labels and axis line (See \code{\link[graphics]{par}}). Default value is \code{mgp = c(3, 2, 0)}.
}
  \item{\dots}{
further arguments passed to the \code{axis} function or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\section{Note}{
From version 0.3-0 it changed its name from \code{drawxaxis} to \code{drawTimeAxis}, in order to have a more intuitive name. The old \code{drawxaxis} function is deprecated, but still be kept for compatibility reasons. 
}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%%}
\examples{
## Loading the SanMartino precipitation data
data(SanMartinoPPts)
x <- window(SanMartinoPPts, end=as.Date("1930-12-31"))

## Plotting the daily ts only, and then automatic 'x' axis
plot(x, xaxt = "n", xlab="Time")
drawTimeAxis(x) 

## Plotting the daily ts only, and then monthly ticks in the 'x' axis, 
## with annual labels.
plot(x, xaxt = "n", xlab="Time")
drawTimeAxis(x, tick.tstep="months", lab.tstep="years") 
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%%\keyword{ ~kwd1 }
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File isComplete.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2025-2025 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{isComplete}
\Rdversion{1.1}
\alias{isComplete}
\alias{isComplete.zoo}


%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Is Complete?
}
\description{
Generic function for identifying whether a zoo object has a regular time frequency without missing values from the first one to the last one.
}

\usage{
isComplete(x, ...)

\method{isComplete}{zoo}(x, tz, out.type=c("single", "all"), verbose=TRUE, ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo object with a time attribute that should have a regular time frequency (e.g., hourly, daily, monthly, annual).
}
 \item{tz}{
character, indicating the time zone in which the date of \code{x} is located. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr

If \code{tz} is provided, it forces \code{time(x)} to be in the tome zone specified by \code{tz}, without modifying the the values (hours, minutes, seconds, etc). \cr

A list of valid time zones can be obtained by calling the base function \code{OlsonNames()}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{from} and \code{to}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{out.type}{
Character indicating the type of result that is given by this function. Valid values are: \cr
-) \kbd{single} => only a logical output is returned. See details in the \kbd{Value} section. \cr
-) \kbd{all}  => a list with 3 elements is returned. See details in the \kbd{Value} section.
}
  \item{verbose}{
logical; if TRUE, informative messages are shown in the screen.

When \code{verbose=TRUE} and \code{x} has some missing temporal layers, the DateTime(s) with missing layer(s) are shown in the screen.
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
%%  If it is a LIST, use
  \item{If \code{out.type="single"}}{(default value), it returns a logical value identifying whether \code{x} has a regular time frequency with complete time layers from its first layer to the last one.}
  \item{If \code{out.type="all"}}{it returns a list with the following three elements:

    -) \kbd{isComplete}: logical value identifying whether \code{x} has a regular time frequency with complete time layers from its first layer to the last one.

    -) \kbd{NumberMissingDT}: integer indicating the amount of missing layers from the first layer of \code{x} to the last one.

    -) \kbd{missingDateTimes}: Numeric, Date, or POSIXct vector showing the DateTime of the missing layers in \code{x}. 
    }
}
%% ...

%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail.com}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{window}}
}
\examples{

###########
# EXAMPLE 1: Hourly zoo object, with a POSIXct time attribute
###########

# Loading the HOURLY streamflow data for Karamea at Gorge
data(KarameaAtGorgeQts)
x.h <- KarameaAtGorgeQts

# Checking whether 'x' is complete or not
isComplete(x.h)

###########
## EXAMPLE 2: Daily object, with a Date time attribute
###########
     
# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x.d <- SanMartinoPPts

# Checking whether 'x' is complete or not
isComplete(x.d)

# Removing two values and testing it again
x.d <-x.d[-c(3,5)]
isComplete(x.d)


###########
# EXAMPLE 3: Weekly zoo object, with a Date time attribute
###########

# Creating a dummy weekly object with a Date time attribute
x.w           <- x.d[1:12]
time(x.w) <-  seq(from=as.Date("2001-01-01"), to=as.Date("2001-03-19"), 
                  by="1 week")

# Checking whether 'x.w' is complete or not
isComplete(x.w)

# Removing two values and testing it again
x.w2 <- x.w[-c(3,5)]
isComplete(x.w2)


###########
# EXAMPLE 4: Monthly zoo object, with a yearmon time attribute
###########

# Transforming the daily object into a monthly one
x.m1 <- daily2monthly(x.d, FUN=sum)

# Checking whether 'x.m1' is complete or not
isComplete(x.m1)

# Removing two values and testing it again
x.m12 <- x.m1[-c(3,5)]
isComplete(x.m12)


###########
# EXAMPLE 5: Monthly zoo object, with a Date time attribute
###########

# Creating a monthly object with a Date time attribute
x.m2       <- x.m1
time(x.m2) <-  as.Date( paste0( format(zoo::as.yearmon( time(x.m1) ), "\%Y-\%m"), "-01") )

# Checking whether 'x.m2' is complete or not
isComplete(x.m2)

# Removing two values and testing it again
x.m22 <- x.m2[-c(3,5)]
isComplete(x.m22)


###########
# EXAMPLE 6: Annual zoo object, with a Date time attribute
###########

# Creating a dummy annual object with Date time attribute
x.a1       <- x.m1[1:12]
time(x.a1) <- hydroTSM::yip("2001-01-01", "2012-01-01")

# Checking whether 'x.a1' is complete or not
isComplete(x.a1)

# Removing two values and testing it again
x.a12 <- x.a1[-c(3,5)]
isComplete(x.a12)


###########
# EXAMPLE 7: 3-hourly zoo object, with a POSIXct time attribute
###########

# Creating a dummy 3-hourly object with a POSIXct time attribute
x.3h       <- x.m1[1:12]
time(x.3h) <- seq(from=as.POSIXct("2001-01-01 00:00:00", tz="UTC"), 
                  to=as.POSIXct("2001-01-02 09:00:00", tz="UTC"), 
                  by="3 hours")

# Checking whether 'x.3h' is complete or not
isComplete(x.3h)

# Removing two values and testing it again
x.3h2 <- x.3h[-c(3,5)]
isComplete(x.3h2)


############
# EXAMPLE 8: 8-day zoo object, with a Date time attribute
############

# Creating a dummy 8-day object with a Date time attribute
x.8d       <- x.m1[1:12]
time(x.8d) <- seq(from=as.Date("2001-01-01"), to=as.Date("2001-03-31"), by="8 days")

# Checking whether 'x.8d' is complete or not
isComplete(x.8d)

# Removing two values and testing it again
x.8d2 <- x.8d[-c(3,5)]
isComplete(x.8d2)


############
# EXAMPLE 9: 3-monthly zoo object, with a Date time attribute
############

# Creating a dummy 3-monthly object with a Date time attribute
x.3m       <- x.m1[1:12]
time(x.3m) <- seq(from=as.Date("2001-01-01"), to=as.Date("2003-12-01"), by="3 months")

# Checking whether 'x.3m' is complete or not
isComplete(x.3m)

# Removing two values and testing it again
x.3m2 <- x.3m[-c(3,5)]
isComplete(x.3m2)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%%File subhourly2hourly.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2022 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{subhourly2hourly}
\Rdversion{1.1}
\alias{subhourly2hourly}
\alias{subhourly2hourly.default}
\alias{subhourly2hourly.zoo}
\alias{subhourly2hourly.data.frame}
\alias{subhourly2hourly.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Sub-hourly -> Hourly
}
\description{
Generic function for transforming a sub-HOURLY time series into an HOURLY one
}
\usage{
subhourly2hourly(x, ...)

\method{subhourly2hourly}{default}(x, FUN, na.rm=TRUE, na.rm.max=0, ...)

\method{subhourly2hourly}{zoo}(x, FUN, na.rm=TRUE, na.rm.max=0, tz, ...)

\method{subhourly2hourly}{data.frame}(x, FUN, na.rm=TRUE, na.rm.max=0,
           dates=1, date.fmt="\%Y-\%m-\%d \%H:\%M:\%S", out.fmt="zoo", 
           verbose= TRUE, ...)

\method{subhourly2hourly}{matrix}(x, FUN, na.rm=TRUE, na.rm.max=0,
           dates=1, date.fmt="\%Y-\%m-\%d \%H:\%M:\%S", out.fmt="zoo", 
           verbose= TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with sub-hourly time series. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} represent the ID of each station.
}
  \item{FUN}{
Function that have to be applied for transforming from sub-hourly to hourly time step. (e.g., for precipitation \code{FUN=sum} and for temperature and streamflow ts, \code{FUN=mean}). \cr
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the hourly values  are computed considering only those values different from NA \cr
-) \kbd{FALSE}: if there is AT LEAST one NA sub-hourly value within a day, the corresponding hourly value(s) will be NA as well
}
  \item{na.rm.max}{
Numeric in [0, 1]. It is used to define the maximum percentage of missing values allowed in each hour to keep the hourly aggregated value in the output object of this function. In other words, if the percentage of missing values in a given hour is larger than \code{na.rm.max} the corresponding hourly value will be \code{NA}.
}
  \item{tz}{
character, with the specification of the time zone used for \code{x}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr

If \code{tz} is provided, it forces \code{time(x)} to be in the tome zone specified by \code{tz}, without modifying the the values (hours, minutes, seconds, etc). \cr

A list of valid time zones can be obtained by calling the base function \code{OlsonNames()}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{x}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{dates}{
numeric, factor, POSIXct or POSIXt object indicating how to obtain the dates and times for each column of \code{x} (e.g., gauging station) \cr
If \code{dates} is a number, it indicates the index of the column in \code{x} that stores the date and times \cr
If \code{dates} is a factor, it is converted into POSIXct class, using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of POSIXct or POSIXt class, the code verifies that the number of elements on it be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{dates}, By default \code{date.fmt=\%Y-\%m-\%d \%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.fmt}{
OPTIONAL. Only used when \code{x} is a matrix or data.frame object /cr
character, for selecting if the result will be a matrix/data.frame or a zoo object. Valid values are: \kbd{numeric}, \kbd{zoo} (default)
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
  a zoo object with hourly time series
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{subhourly2nminutes}},, \code{\link{subdaily2daily}}, \code{\link{subdaily2monthly}}, \code{\link{subdaily2annual}}, \code{\link{subdaily2seasonal}}, \code{\link{as.POSIXct}}, \code{\link{dm2seasonal}}, \code{\link{monthlyfunction}}, \code{\link{seasonalfunction}}, \code{\link{hydroplot}}, \code{\link{vector2zoo}}, \code{\link{izoo2rzoo}}
}
\examples{
## Creating a 15-min time sequence and counting its length
dt  <- seq( from=as.POSIXct("2021-06-30 00:15"), to=as.POSIXct("2021-06-30 23:45"), by="15 min" )
ndt <- length(dt)

## Creating a dummy 15-min zoo object, with 1 as the only value in each time period
x <- zoo( rep(1, ndt), dt)


## sub-hourly to hourly
h1 <- subhourly2hourly(x, FUN=sum, na.rm=TRUE)

## Aggregation of 3 sub-hourly ts (i.e., a zoo matrix) into an hourly one
X  <- cbind(x, x, x)
h2 <- subhourly2hourly(X, FUN=sum, na.rm=TRUE)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%%File subdaily2daily.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%% Copyright 2008-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{subdaily2daily}
\Rdversion{1.1}
\alias{subdaily2daily}
\alias{subdaily2daily.default}
\alias{subdaily2daily.zoo}
\alias{subdaily2daily.data.frame}
\alias{subdaily2daily.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Sub-daily -> Daily
}
\description{
Generic function for transforming a Sub-DAILY time series into a DAILY one
}
\usage{
subdaily2daily(x, ...)

\method{subdaily2daily}{default}(x, FUN, na.rm = TRUE, na.rm.max=0,
                       start="00:00:00", start.fmt= "\%H:\%M:\%S", tz, ...)

\method{subdaily2daily}{zoo}(x, FUN, na.rm = TRUE, na.rm.max=0,
                       start="00:00:00", start.fmt= "\%H:\%M:\%S", tz, ...)

\method{subdaily2daily}{data.frame}(x, FUN, na.rm = TRUE, na.rm.max=0,
           start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
           dates=1, date.fmt="\%Y-\%m-\%d \%H:\%M:\%S", out.fmt="zoo", 
           verbose= TRUE, ...)

\method{subdaily2daily}{matrix}(x, FUN, na.rm = TRUE, na.rm.max=0,
           start="00:00:00", start.fmt= "\%H:\%M:\%S", tz,
           dates=1, date.fmt="\%Y-\%m-\%d \%H:\%M:\%S", out.fmt="zoo", 
           verbose= TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with sub-daily time series. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represents the time series measured in each gauging station, and the column names of \code{x} should correspond to the ID of each station (starting by a letter).
}
 \item{FUN}{
Function that have to be applied for aggregating from sub-daily into daily time step (e.g., for precipitation \code{FUN=sum} and for temperature and streamflows ts \code{FUN=mean}). \cr

\code{FUN} MUST accept the \code{na.rm} argument, because \code{na.rm} is passed to \code{FUN}.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the daily values are computed only for days with a percentage of missing values less than \code{na.rm.max} \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a day, the corresponing daily value in the output object will be \code{NA}.
}
  \item{na.rm.max}{
Numeric in [0, 1]. It is used to define the maximum percentage of missing values allowed in each day to keep the daily aggregated value in the output object of this function. In other words, if the percentage of missing values in a given day is larger than \code{na.rm.max} the corresponding daily value will be \code{NA}.
}
  \item{start}{
character, indicating the starting time used for aggregating sub-daily time series into daily ones. It MUST be provided in the format specified by \code{start.fmt}. \cr
This value is used to define the time when a new day begins (e.g., for some rain gauge stations). \cr
-) All the values of \code{x} with a time attribute before \code{start} are considered as belonging to the day before the one indicated in the time attribute of those values. \cr
-) All the values of \code{x} with a time attribute equal to \code{start} are considered to be equal to \code{"00:00:00"} in the output zoo object. \cr
-) All the values of \code{x} with a time attribute after \code{start} are considered as belonging to the same day as the one indicated in the time attribute of those values. \cr

It is useful when the daily values start at a time different from \code{"00:00:00"}. Use with caution. See examples.
}
  \item{start.fmt}{
character indicating the format in which the time is provided in \code{start}, By default \code{date.fmt=\%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.POSIXct}}.
}
  \item{tz}{
character, with the specification of the time zone used in both \code{x} and \code{start}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr
If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(x)}. \cr
This argument can be used to force using the local time zone or any other time zone instead of UTC as time zone.
}
  \item{dates}{
numeric, factor, POSIXct or POSIXt object indicating how to obtain the dates and times for each column of \code{x} (e.g., gauging station) \cr
If \code{dates} is a number, it indicates the index of the column in \code{x} that stores the date and times \cr
If \code{dates} is a factor, it is converted into POSIXct class, using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of POSIXct or POSIXt class, the code verifies that the number of elements on it be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{dates}, By default \code{date.fmt=\%Y-\%m-\%d \%H:\%M:\%S}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.fmt}{
OPTIONAL. Only used when \code{x} is a matrix or data.frame object /cr
character, for selecting if the result will be a matrix/data.frame or a zoo object. Valid values are: \kbd{numeric}, \kbd{zoo} (default)
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
arguments additional to \code{na.rm} passed to \code{FUN}.
}
}
\details{
 This function assumes that \code{x} has a strictly regular time frequency (see \code{\link[zoo]{is.regular}}) and raise a warning otherwise.
}
\value{
  a zoo object with daily time series
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{subhourly2hourly}}, \code{\link{subdaily2monthly}}, 
\code{\link{subdaily2annual}}, \code{\link{subdaily2seasonal}}, 
\code{\link{as.POSIXct}}, \code{\link{dm2seasonal}}, \code{\link{monthlyfunction}}, 
\code{\link{seasonalfunction}}, \code{\link{hydroplot}}, 
\code{\link{vector2zoo}}, \code{\link{izoo2rzoo}}
}
\examples{
## Ex1: Computation of daily values, removing any missing value in 'x'

## Loading the time series of hourly streamflows for the station Karamea at Gorge
data(KarameaAtGorgeQts)
x <- KarameaAtGorgeQts

# Plotting the hourly streamflow values
plot(x)

# Subsetting 'x' to its first three days (01/Jan/1980 - 03/Jan/1980)
x <- window(x, end="1980-01-03 23:59:00")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Agreggating from Sub-Daily to Daily, removing any missing value in 'x'
( d1 <- subdaily2daily(x, FUN=mean, na.rm=TRUE) )


## Ex2: Computation of daily values, removing any missing value in 'x' and
##      considering that the new day starts at 08:00:00 local time
( d2 <- subdaily2daily(x, FUN=mean, na.rm=TRUE, start="08:00:00") )

## Ex3: Computation of daily values, removing any missing value in 'x' and
##      considering that the new day starts at 08:00:00, and forcing 
#       UTC both for 'x' and 'start'
( d3 <- subdaily2daily(x, FUN=mean, na.rm=TRUE, start="08:00:00", tz="UTC") )

######################
## Ex4: Compuation of daily values only when the percentage of NAs in each
#       day is lower than a user-defined percentage (10% in this example).
( d4 <- subdaily2daily(x, FUN=mean, na.rm=TRUE, na.rm.max=0.1) )
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

%% File diy.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2012 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{diy}
\Rdversion{1.1}
\alias{diy}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Days in Year
}
\description{
 Given a single numeric value representing a year, it generates: \cr           
1) a vector of dates with all the days within the year, OR \cr
2) the amount of days in the specified year  
}
\usage{
diy(year, out.type = "seq")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{year}{
numeric, the year for which the sequence of days will be generated
}
  \item{out.type}{
Character indicating the type of result that is given by this function. Valid values are: \cr
-) \kbd{seq} : a vectorial sequence with all the days within the given year \cr
-) \kbd{nmbr}: the number of days in the vectorial sequence with all the days within the given year
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{hip}}, \code{\link{dip}}, \code{\link{mip}}, \code{\link{yip}}
}
\examples{
## Sequence of daily dates for the year 1961
diy(1961)

## Computing the number of days between in 1961
diy(1961, out.type = "nmbr")

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%%File plot_pq.Rd
%% Part of the hydroTSM R package, https://github.com/hzambran/hydroTSM ; 
%%                                 http://www.rforge.net/hydroTSM/ ; 
%%                                 https://cran.r-project.org/package=hydroTSM
%% Copyright 2023-2026 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{plot_pq}
\Rdversion{1.1}
\alias{plot_pq}
\alias{plot_pq.zoo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
 Plot precipitation and streamflow time series in the same figure.
}
\description{
Given a time series of precipitation and streamflow, this function plots the two time series in the same figure, streamflows as a normal time series and precipitation as bars coming from the upper part of the plotting window. 
}
\usage{
plot_pq(p, ...)

\method{plot_pq}{zoo}(p, q, ptype=c("original", "monthly"),
                na.fill=c("remove", "linear", "spline"), 
                from=start(p), to=end(p), date.fmt=NULL, tz=NULL,
                main=ifelse(ptype=="original", "Precipitation and Streamflows", 
                            "Monthly Precipitation and Streamflows"),
                xlab=ifelse(ptype=="original", "Time", "Month"), 
                ylab=c("P, [mm]", "Q, [m3/s]"), 
                p.col=ifelse(ptype=="original", "blue", "lightblue"),
                q.col=ifelse(ptype=="original", "black", "blue"), 
                leg.title="", leg.text=c("P", "Q"),
                q.pch=16, q.cex=0.3,
                            
                start.month=1, 
                plot.p.probs=TRUE, p.probs=c(0.25, 0.75), 
                p.alpha=0.8,
                plot.q.probs=TRUE, q.probs=c(0.25, 0.75), 
                q.probs.col="lightskyblue1", q.probs.alpha=0.8, 
                labels=TRUE, labels.cex=0.8,
                labels.p.dy=NULL,
                labels.q.dx=c(rep(-0.2,6), rep(0.2,6)),
                labels.q.dy=rep(median(q, na.rm=TRUE)*1.3, 12),
                
                ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{p}{
zoo object with precipitation time series, with any time frequency.
}
  \item{q}{
zoo object with streamflow time series, with any time frequency.
}
  \item{ptype}{
Character indicating the type of plot to be produced. Valid values are: \cr
-) \kbd{original} => a time series plot with precipitation in the upper panel (as bars from the time axis) and streamflows in the lower panel (as dotted lines). \cr
-) \kbd{monthly}  => a plot with mean monthly values of precipitation in the upper panel (as bars from top to bottom) and streamflows in the lower panel (as bars from bottom to up). Quantiles of precipitation and streamflows are also plotted depending on the values defined in \code{p.probs} and \code{q.probs}, respectively.
}
  \item{na.fill}{
Character indicating how to fill any NA present in \code{p} or \code{q}. \cr
-) \kbd{remove} => NAs are not plotted
-) \kbd{linear} => NAs are removed by linear interpolation, using \code{\link[zoo]{na.approx}} \cr 
-) \kbd{spline} => NAs are removed by spline interpolation, using \code{\link[zoo]{na.spline}}
}
  \item{from}{
Character indicating the starting date for subsetting \code{p} and \code{q}. The default value corresponds to the date of the first element of \code{p} \cr
It has to be in the format indicated by \code{date.fmt}. 
}
  \item{to}{
Character indicating the ending date for subsetting \code{p} and \code{q}. The default value corresponds to the date of the last element of \code{p} \cr
It has to be in the format indicated by \code{date.fmt}.
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \code{from} and \code{to}, e.g. \kbd{\%Y-\%m-\%d}. See \sQuote{Details} section in \code{\link[base]{strptime}}.
By default, \code{date.fmt} is missing, and it is automatically set to \kbd{\%Y-\%m-\%d} when \code{time(p)} is \code{Date} object, and set to \kbd{\%Y-\%m-\%d \%H:\%M:\%S} when \code{x} is a sub-daily zoo object.
}
  \item{tz}{
character, with the specification of the time zone used for \code{from}, \code{to}. System-specific (see time zones), but \code{""} is the current time zone, and \code{"GMT"} is UTC (Universal Time, Coordinated). See \code{\link[base]{Sys.timezone}} and \code{\link[base]{as.POSIXct}}. \cr

If \code{tz} is missing (the default), it is automatically set to the time zone used in \code{time(p)}. \cr

This argument can be used when working with sub-daily zoo objects to force using  time zones other than the local time zone for \code{from} and \code{to}. It should be used with caution, being well aware of the time zone of the data. See examples.
}
  \item{main}{
The main title (on top) of the figure.
}
  \item{xlab}{
a title for the \code{x} axis. See \code{\link[graphics]{title}}.
}
  \item{ylab}{
a two-element title for the \code{y} axis: see \code{\link[graphics]{title}}. \cr
The first element is used for the right \code{y} axis (i.e., for precipitation). The second element is used for the left \code{y} axis (i.e., for streamflows).
}
  \item{p.col}{
character, representing the colors to be used for plotting the precipitation time series.  
}
  \item{q.col}{
character, representing the colors to be used for plotting the streamflow time series. 
}
  \item{leg.title}{
a character string or length-one expression giving a title to be placed at the top of the legend. \code{\link[graphics]{legend}}.
}
  \item{leg.text}{
a two-element character to appear in the legend placed at the bottom of the figure. \cr
The first element is used for precipitation and the second element is used for streamflows.
}
  \item{q.pch}{
numeric, representing the symbols used for plotting the streamflow time series.
}
  \item{q.cex}{
a numerical vector giving the amount by which plotting characters and symbols should be scaled relative to the default. This works as a multiple of \code{par("cex")}. See \code{\link[graphics]{plot.default}}
}
  \item{start.month}{
numeric in [1:12] indicating the starting month for the monthlycurve. Numeric values in [1, 12] represents months in [January, December]. By default \code{start.month=1}.
}
  \item{plot.p.probs}{
logical used to decide whether to show lower and upper uncertainty bounds for each one of the 12 monthly precipitation values. By default \code{plot.p.probs=TRUE}. \cr 
When \code{plot.p.probs=TRUE} the \code{p.probs} argument is used to define the values of the lower and upper uncertainty bounds.
} 
  \item{p.probs}{
numeric of length 2. It defines the quantile values used to compute the lower and upper uncertainty bounds for each one of the 12 monthly precipitation values. This uncertainty bounds are drawn as vertical lines over the bars used to plot the 12 monthly precipitation values. \cr 
By default \code{p.probs=c(0.25, 0.75)}, which indicates that the quantiles 0.25 and 0.75 are used to compute the lower and upper uncertainty bounds for each one of the 12 monthly precipitation values. 
If \code{p} is a (sub)daily zoo object, it is first aggregated into monthly values using \code{mean}, and then the \code{p.probs} quantiles are computed over all the monthly values belonging to a calendar month.
} 
  \item{p.alpha}{
numeric of length 1, with the factor used to modify the opacity of \code{p.col}. Typically in [0,1], with 0 indicating a completely transparent colour and 1 indicating no transparency.
} 
  \item{plot.q.probs}{
logical used to decide whether to show uncertainty bands around each one of the 12 monthly average or median streamflow values. By default \code{plot.q.probs=TRUE}. \cr 
When \code{plot.q.probs=TRUE} the \code{q.probs} argument is used to define the values of the lower and upper uncertainty bands.
} 
  \item{q.probs}{
numeric of length 2. It is used to define quantile values used to compute the lower and upper uncertainty bands around each one of the 12 monthly average or median streamflow. \cr 
If \code{q} is a (sub)daily zoo object, it is first aggregated into monthly values using \code{FUN}, and then the \code{q.probs} quantiles are computed over all the monthly values belonging to a calendar month.. \cr
By default \code{q.probs=c(0.25, 0.75)}, which indicates that the quantiles 0.25 and 0.75 are used to compute the lower and upper uncertainty bounds for each one of the 12 monthly average or median values. 
If \code{q} is provided and is a (sub)daily zoo object, it is first aggregated into monthly values using \code{FUN}, and then the \code{q.probs} quantiles are computed over all the monthly values belonging to a calendar month.
}
  \item{q.probs.col}{
character with the color used to plot the uncertainty bands around the average or median streamflow values. \cr
} 
  \item{q.probs.alpha}{
numeric of length 1, with the factor used to modify the opacity of \code{q.probs.col}. Typically in [0,1], with 0 indicating a completely transparent colour and 1 indicating no transparency.
} 
  \item{labels}{
logical. Should monthly streamflow values to be shown above the lines?. By default \code{labels=TRUE}.
} 
  \item{labels.cex}{
numeric giving the amount by which plotting characters used to show the numeric values of monthly streamflow values are scaled relative to the default. 
}
  \item{labels.p.dy}{
numeric of length 12 giving the amount of vertical coordinate positions that have to be used to vertically shift the labels of monthly precipitation values. \cr
It is only used when \code{labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{labels.q.dx}{
numeric of length 12 giving the amount of horizontal coordinate positions that have to be used to horizontally shift the labels of monthly streamflow values. \cr
It is only used when \code{labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{labels.q.dy}{
numeric of length 12 giving the amount of vertical coordinate positions that have to be used to vertically shift the labels of monthly streamflow values. \cr
It is only used when \code{labels=TRUE}. \cr
Lengths smaller than 12 are recycled and larger lengths are not used.
}
  \item{\dots}{
further arguments passed to or from other methods. Not used yet.
}
}
\details{
Given a time series of precipitation and streamflow, this function plots the two time series in the same figure, streamflows as a normal time series and precipitation as bars coming from the upper part of the plotting window. 
}
\value{
A figure with the two time series in the same graphical area, streamflows as a normal time series and precipitation as bars coming from the upper part of the plotting window. 
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
 \code{\link{hydroplot}}, \code{\link{climograph}}, \code{\link{fdc}}, \code{\link{fdcu}}, \code{\link{monthlyfunction}}
}
\examples{
######################
## Ex1: Plotting precipitation and streamflows for the full time period of both
##      time series.
##      First, we load the daily P and Q time series for the Cauquenes en 
##      El Arrayan catchment. P, [mm] is the first column and Q, [mm] is the 
##      fifth column.

data(Cauquenes7336001)

# Subsetting to only the 1981-1990 temporal period
Cauquenes7336001 <- window(Cauquenes7336001, start="1981-01-01", end="1990-12-31")

p <- Cauquenes7336001[, 1]
q <- Cauquenes7336001[, 5]

## Plotting P and Q for the full time period of both time series
plot_pq(p=p, q=q)

\dontrun{
  ######################
  ## Ex2: Plotting precipitation and streamflows only for a specific time period,
  ##      from April to December 2000.
  plot_pq(p, q, from="1985-04-01", to="1985-12-31")

  ######################
  ## Ex3: Plotting monthly values of precipitation and streamflows for the 
  ##      full time period of both time series.
  plot_pq(p, q, ptype="monthly")

  ######################
  ## Ex4: Plotting monthly values of precipitation and streamflows for the 
  ##      full time period of both time series, but using a hydrologic year
  ##      starting on April
  plot_pq(p, q, ptype="monthly", start.month=4)
}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}
%%\keyword{ ~kwd2 }

%%File daily2weekly.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 https://CRAN.R-project.org/package=hydroTSM
%% Copyright 2023-2023 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{daily2weekly}
\Rdversion{1.1}
\alias{daily2weekly}
\alias{daily2weekly.default}
\alias{daily2weekly.zoo}
\alias{daily2weekly.data.frame}
\alias{daily2weekly.matrix}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Daily -> Weekly
}
\description{
Generic function for transforming a DAILY (or sub-daily) regular time series into a WEEKLY one
}
\usage{
daily2weekly(x, ...)

\method{daily2weekly}{default}(x, FUN, na.rm=TRUE, na.rm.max=0, ...)

\method{daily2weekly}{zoo}(x, FUN, na.rm=TRUE, na.rm.max=0, ...)

\method{daily2weekly}{data.frame}(x, FUN, na.rm=TRUE, na.rm.max=0, dates=1, 
        date.fmt = "\%Y-\%m-\%d", out.type = "data.frame", out.fmt="numeric", 
        verbose=TRUE, ...)

\method{daily2weekly}{matrix}(x, FUN, na.rm=TRUE, na.rm.max=0, dates=1, 
        date.fmt = "\%Y-\%m-\%d", out.type = "data.frame", out.fmt="numeric", 
        verbose=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, with (sub)daily time series. \cr
Measurements at several gauging stations can be stored in a data.frame or matrix object, and in that case, each column of \code{x} represents the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{FUN}{
Function that have to be applied for transforming from daily to weekly time step (e.g., for precipitation \code{FUN=sum} and for temperature and streamflow ts \code{FUN=mean}). \cr

\code{FUN} MUST accept the \code{na.rm} argument, because \code{na.rm} is passed to \code{FUN}.
}
  \item{na.rm}{
Logical. Should missing values be removed? \cr
-) \kbd{TRUE} : the weekly values are computed only for weeks with a percentage of missing values less than \code{na.rm.max} \cr
-) \kbd{FALSE}: if there is AT LEAST one NA within a month, the corresponing weekly values in the output object will be \code{NA}.
}
  \item{na.rm.max}{
Numeric in [0, 1]. It is used to define the maximum percentage of missing values allowed in each week to keep the weekly aggregated value in the output object of this function. In other words, if the percentage of missing values in a given week is larger than \code{na.rm.max} the corresponding weekly value will be \code{NA}.
}
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates for each gauging station \cr
If \code{dates} is a number (default), it indicates the index of the column in \var{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, using the date format specified by \code{date.fmt}  \cr
If \code{dates} is already of Date class, the code verifies that the number of days on it be equal to the number of elements in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{out.type}{
Character that defines the desired type of output. Valid values are:  \cr
-) \kbd{data.frame}: a data.frame, with as many columns as stations are included in \code{x}, and row names indicating the month and year for each value. \cr
-) \kbd{db}        : a data.frame, with 4 columns will be produced. \cr
                 The first column (StationID) stores the ID of the station, \cr
                 The second column (Year) stores the year \cr
                 The third column (Month) stores the Month  \cr
                 The fourth column (Value) stores the numerical values corresponding to the values specified in the three previous columns. \cr
}
  \item{out.fmt}{
OPTIONAL. Only used when \code{x} is a matrix or data.frame object /cr
character, for selecting if the result will be a matrix/data.frame or a zoo object. Valid values are: \kbd{numeric}, \kbd{zoo}.
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
arguments additional to \code{na.rm} passed to \code{FUN}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
a zoo object with weekly time frequency
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{cmv}}, \code{\link{subhourly2hourly}}, \code{\link{daily2monthly}}, \code{\link{daily2annual}}, \code{\link{subdaily2daily}}, \code{\link{weeklyfunction}}, \code{\link{hydroplot}}, \code{\link{vector2zoo}}, \code{\link{izoo2rzoo}}, \code{\link{as.Date}}
}
\examples{
######################
## Ex1: Computation of weekly values, removing any missing value in 'x'

# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting 'x' to its first three weeks (Jan/1921 - Mar/1921)
x <- window(x, end="1921-03-31")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Agreggating from Daily to Weekly, removing any missing value in 'x'
w <- daily2weekly(x, FUN=sum, na.rm=TRUE)

######################
## Ex2: Computation of Weekly values only when the percentage of NAs in each
#       week is lower than a user-defined percentage (10% in this example).

# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting 'x' to its first three weeks (Jan/1921 - Mar/1921)
x <- window(x, end="1921-03-31")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Daily to Weekly, only for weeks with less than 10% of missing values
w2 <- daily2weekly(x, FUN=sum, na.rm=TRUE, na.rm.max=0.1)

# Verifying that the weeks 01, 02, 06, 08, 10, 11, 12 of 'x' had 10% or more of missing values
cmv(x, tscale="weekly")

######################
## Ex3: Computation of Weekly values in a two-column zoo object, 
##      only when the percentage of NAs in each week is lower than a user-defined 
##      percentage (10% in this example).

# Loading the DAILY precipitation data at SanMartino
data(SanMartinoPPts)
x <- SanMartinoPPts

# Subsetting 'x' to its first three weeks (Jan/1921 - Mar/1921)
x <- window(x, end="1921-03-31")

## Transforming into NA the 10% of values in 'x'
set.seed(10) # for reproducible results
n           <- length(x)
n.nas       <- round(0.1*n, 0)
na.index    <- sample(1:n, n.nas)
x[na.index] <- NA

## Creating a two-column zoo object
X <- cbind(x, y=x)

## Daily to Weekly, only for weeks with less than 10% of missing values
w2 <- daily2weekly(X, FUN=sum, na.rm=TRUE, na.rm.max=0.1)

# Verifying that the weeks 01, 02, 06, 08, 10, 11, 12 of 'x' had 10% or more of missing values
cmv(X, tscale="weekly")
}



% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

\name{hydroTSM-internal}
\title{Internal hydroTSM objects}
\alias{.che2zoo}
\alias{.CHE_daily_analysis}
\alias{.DailyTS2DB}
\alias{.dwi.che}
\alias{.hydroplotboxplot}
\alias{.hydroplothist}
\alias{.hydroplotts}
\alias{.northarrow}
\alias{.plot.daily.analysis}
\alias{.Qcsv2zoo}
\alias{.scalebar}
\alias{.dmc}
\alias{.drty.dcs2zoo}
\alias{.interpol1}
\alias{check_from_and_to_and_subset}
\description{Internal hydroTSM objects.}
\details{These are not to be called by the user.}
\keyword{internal}

%% File istdx.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2009-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{istdx}
\Rdversion{1.1}
\alias{istdx}
\alias{istdx.default}
\alias{istdx.matrix}
\alias{istdx.data.frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Inverse Standarization
}
\description{
This function back transforms a standarized vector/matrix \code{z} into their original values, i.e., re-scales all the values in the [0,1] interval to the original range of values  \code{z = re-scale(x) = x*[ xmax - xmin ] + xmin}.       
}
\usage{
istdx(x, ...)
\method{istdx}{default}(x, xmin, xrange, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
standarized vector or matrix to be re-scaled, all the values have to be in the range [0,1]
}
  \item{xmin}{
numeric with the minimum value(s) in the original \code{x} \cr
-) if \code{x} is a vector, \code{xmin} has to be a real \cr
-) if \code{x} is a matrix/data.frame, \code{xmin} has to be a vector, with the minimum values for each column of the original \code{x}. In this case, the vector of minimums can be obtained as: \code{xmin <-  apply(x, 2, min, na.rm=TRUE)}
}
  \item{xrange}{
numeric with the range of value(s) in the original \code{x} \cr
-) if \code{x} is a vector, \code{xrange} has to be a real \cr
-) if \code{x} is a matrix/data.frame, \code{xrange} has to be a vector, with the range of values for each column of the original \code{x}.
   In this case, the vector of ranges can be obtained as: \cr
   \code{xrange <-  apply(x, 2,range, na.rm=TRUE)} \cr
   \code{xrange <-  apply(xrange, 2, diff, na.rm=TRUE)} 
}
  \item{\dots}{
further arguments passed to or from other methods
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{stdx}}, \code{\link{scale}}
}
\examples{
## Loading daily streamflows at the station Oca en Ona (Ebro River basin, Spain) ##
data(OcaEnOnaQts)
x <- OcaEnOnaQts

## Computing xmin and the range of 'x'
xmin <- min(x, na.rm=TRUE)
r <- diff(range(x, na.rm=TRUE))

## Standarized variable
s <- stdx(x)

## Inverse of the standarized variable
si <- istdx(s, xmin, xrange=r)

## 'si' and 'x' should be the same
summary(x-si)

###########
### Standarizing a subset of the stations 9 to 12 in 'EbroPPtsMonthly'

## Loading the monthly time series of precipitation within the Ebro River basin.
data(EbroPPtsMonthly)

pp <- EbroPPtsMonthly[1:70,10:13]
xmin   <- apply(pp, 2, min, na.rm=TRUE)
xrange <- apply(pp, 2, range, na.rm=TRUE)
xrange <- apply(xrange, 2, diff, na.rm=TRUE)

## Standarized variable
s <- stdx(as.matrix(pp))

## Inverse of the standarized variable
si <- istdx(s, xmin, xrange)

## 'si' and 'pp' should be the same
summary(pp - si)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

%% File dwdays.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2010-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{dwdays}
\Rdversion{1.1}
\alias{dwdays.data.frame}
\alias{dwdays.matrix}
\alias{dwdays.default}
\alias{dwdays}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Amount of dry/wet days in a time series
}
\description{
Given a daily time series (usually precipitation), this function computes the average amount of wet/dry days in each month.
}
\usage{
dwdays(x, ...)

\method{dwdays}{default}(x, thr=0, type="wet", na.rm=TRUE, ... )

\method{dwdays}{data.frame}(x, thr=0, type="wet", na.rm=TRUE, 
        dates=1, date.fmt="\%Y-\%m-\%d", verbose=TRUE,...)

\method{dwdays}{matrix}(x, thr=0, type="wet", na.rm=TRUE, 
        dates=1, date.fmt="\%Y-\%m-\%d", verbose=TRUE,...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
zoo, data.frame or matrix object, usually with daily time series of precipitation. \cr
Measurements at several gauging stations can be stored in a data.frame of matrix object, and in that case, each column of \code{x} represent the time series measured in each gauging station, and the column names of \code{x} have to correspond to the ID of each station (starting by a letter).
}
  \item{thr}{
numeric. Value of daily precipitation used as threshold for classifying a day as dry/wet or not. Days with a precipitation value larger to \code{thr} are classified as \var{wet days}, whereas precipitation values lower to \code{thr} are classified as \var{dry days}. 
}
  \item{type}{
character, indicating if the daily values have to be classified as dry or wet days. It works linked to the values specified in \code{thr}. Valid values are: \kbd{wet}, \kbd{dry}.
}
  \item{na.rm}{
Logical. Should missing values be removed before counting?
}
  \item{dates}{
numeric, factor or Date object indicating how to obtain the dates \cr
If \code{dates} is a number (default), it indicates the index of the column in \code{x} that stores the dates \cr
If \code{dates} is a factor, it is converted into Date class, using the date format  specified by \code{date.fmt}  \cr
If \code{dates} is already of Date class, the code verifies that the number of days in \code{dates} be equal to the number of element in \code{x}
}
  \item{date.fmt}{
character indicating the format in which the dates are stored in \var{dates}, e.g. \kbd{\%Y-\%m-\%d}. See \code{format} in \code{\link[base]{as.Date}}. \cr
ONLY required when \code{class(dates)=="factor"} or \code{class(dates)=="numeric"}.
}
  \item{verbose}{
logical; if TRUE, progress messages are printed 
}
  \item{\dots}{
further arguments passed to or from other methods.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
%%\references{
%% ~put references to the literature/web site here ~
%%}
\author{
Mauricio Zambrano-Bigiarini, \email{mzb.devel@gmail}
}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%%\code{\link{vector2zoo}}, \code{\link{as.Date}}, \code{\link{daily2annual}}
%%}

\examples{
## Loading the SanMartino precipitation data
data(SanMartinoPPts)
x <- SanMartinoPPts

## Average amount of wet days in each month (for this example, this means days 
## with precipitation larger than 0.1mm) 
dwdays(x, thr=0.1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
%%\keyword{ ~kwd2 }

\name{EbroPPtsMonthly}
\docType{data}
\alias{EbroPPtsMonthly}
\title{
Ebro Monthly Precipitation Time Series
}
\description{
Time series of monthly precipitation on 331 stations of the Ebro River basin (NW Spain), for the period January/1961 to December/1963.
}
\usage{
data(EbroPPtsMonthly)
}

\details{
Monthly time series of precipitation on 331 stations of the Ebro River basin (Spain), where some data were in-filled by using the MOSS method and from daily time series used in the technical report "Estudio de Recursos de la Cuenca del Ebro".
}
\format{
A data.frame with 331 monthly time series, plus the first field storing the dates corresponding to each row.
}
\source{
Downloaded from the web site of the Confederacion Hidrografica del Ebro (CHE) \url{http://www.chebro.es/} (original link http://oph.chebro.es/DOCUMENTACION/PrecipitacionMensualRelleno/PrecipitacionMensualRelleno.html, last accessed [March 2010]). \cr
These data are intended to be used for research purposes only, being distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
}

%%\references{
%% ~put references to the literature/web site here ~
%%}
\keyword{datasets}

%%File MaquehueTemuco.Rd
%% Part of the hydroTSM R package, http://www.rforge.net/hydroTSM/ ; 
%%                                 http://cran.r-project.org/web/packages/hydroTSM/
%% Copyright 2008-2013 Mauricio Zambrano-Bigiarini
%% Distributed under GPL 2 or later

\name{MaquehueTemuco}
\docType{data}
\alias{MaquehueTemuco}
\title{
San Martino, ts of daily precipitation.
}
\description{
Daily time series of precipitation, maximum and minimun air temperature at station Maquehue Temuco Ad. (DMC_ID:380013), Araucania Region, Chile (Lat:-38.770, Lon:-72.637), with data from 01/Jan/1950 to 31/Dec/2015 (including some gaps).
}
\usage{
data(MaquehueTemuco)
}

%%\details{
%%
%%}
\format{
zoo matrix with 3 columns: \cr
-) \var{pcp}: daily precipitation, [mm/day]. \cr
-) \var{tmx}: daily maximum air temperature, [degree Celsius]. \cr
-) \var{tmn}: daily minimum air temperature, [degree Celsius]. \cr 
}
\source{
Provided by Center for Climate and Resilience Research, Universidad de Chile, Santiago, Chile (CR2, \url{https://dataclima.cr2.cl/bases_de_datos?database_db_type_id=5}. Original data available at the Chilean Meteorological Organization (DMC, \url{https://climatologia.meteochile.gob.cl/application/diariob/visorDeDatosEma/380013}). Last accessed [May 2025]). \cr
These data are intended to be used for research purposes only, being distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
}

%%\references{
%% ~put references to the literature/web site here ~
%%}
\keyword{datasets}
